!{\src2tex{textfont=tt}}
!!****f* ABINIT/psp2in
!! NAME
!! psp2in
!!
!! FUNCTION
!! Initialize pspcod=2 pseudopotentials (GTH format):
!! continue to read the file, then compute the corresponding
!! local and non-local potentials.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (DCA, XG, GMR, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!  lmax=value of lmax mentioned at the second line of the psp file
!!  lmnmax=if useylm=1, max number of (l,m,n) comp. over all type of psps
!!        =if useylm=0, max number of (l,n)   comp. over all type of psps
!!  lnmax=max. number of (l,n) components over all type of psps
!!  mpsang= 1+maximum angular momentum for nonlocal pseudopotentials
!!  mqgrid_ff=dimension of q (or G) grid.
!!  mqgrid_vl=dimension of q (or G) grid or r grid (if vlspl_recipSpace = .false.).
!!  qgrid_ff(mqgrid_ff)=values of q (or G) on grid from 0 to qmax (bohr^-1)
!!  qgrid_vl(mqgrid_vl)=values of q (or G) on grid from 0 to qmax (bohr^-1)
!!                if vlspl_recipSpace is .true. else values of r on grid from
!!                0 to 2pi / qmax * mqgrid_ff (bohr).
!!  useylm=governs the way the nonlocal operator is to be applied:
!!         1=using Ylm, 0=using Legendre polynomials
!!  vlspl_recipSpace= .true. if computation of vlspl is done in reciprocal space
!!  dvlspl(mqgrid_vl,2)=dVloc(r)/dr and second derivatives from spline fit (only
!!                      allocated if vlspl_recipSpace is false.
!!  zion=nominal valence of atom as specified in psp file
!!
!! OUTPUT
!!  ekb(lnmax)=Kleinman-Bylander energy,
!!             {{\ \begin{equation}
!!               \frac{\int_0^\infty [Rl(r)^2 (Vl(r)-Vloc(r))^2 dr]}
!!             {\int_0^\infty [Rl(r)^2 (Vl(r)-Vloc(r))   dr]}
!!              \end{equation} }}
!!             for each (l,n)
!!  epsatm=$ (4\pi)\int_0^\infty [r^2 (V(r)+Zv/r) dr]$ (hartree)
!!  ffspl(mqgrid,2,lnmax)=Kleinman-Bylander form factor f_l(q) and
!!   second derivative from spline fit for each angular momentum and
!!   each projector
!!  indlmn(6,i)= array giving l,m,n,ln,lm,s for i=ln  (if useylm=0)
!!                                           or i=lmn (if useylm=1)
!!  nproj(mpsang)=number of projection functions for each angular momentum
!!  vlspl(mqgrid,2)=q^2 Vloc(q) and second derivatives from spline fit
!!
!! SIDE EFFECTS
!!  Input/output
!!  lmax : at input =value of lmax mentioned at the second line of the psp file
!!    at output= 1
!!
!! PARENTS
!!      pspatm
!!
!! CHILDREN
!!      leave_new,psp2lo,psp2nl,spline,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine psp2in(ekb,epsatm,ffspl,indlmn,lmax,lmnmax,lnmax,&
&                  mpsang,mqgrid_ff,mqgrid_vl,nproj,qgrid_ff,qgrid_vl,useylm,vlspl,dvlspl,vlspl_recipSpace,zion)

 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01managempi
 use interfaces_02psp, except_this_one => psp2in
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: lmax,lmnmax,lnmax,mpsang,mqgrid_ff,mqgrid_vl,useylm
 real(dp),intent(in) :: zion
 real(dp),intent(out) :: epsatm
 logical,intent(in) :: vlspl_recipSpace
!arrays
 integer,intent(out) :: indlmn(6,lmnmax),nproj(mpsang)
 real(dp),intent(in) :: qgrid_ff(mqgrid_ff),qgrid_vl(mqgrid_vl)
 real(dp),intent(out) :: dvlspl(mqgrid_vl,2),ekb(lnmax)
 real(dp),intent(out) :: ffspl(mqgrid_ff,2,lnmax),vlspl(mqgrid_vl,2)

!Local variables-------------------------------
!scalars
 integer :: iln,index,ipsang,kk,ll,mm
 real(dp) :: cc1,cc2,cc3,cc4,h1p,h1s,h2s,rloc,rrp,rrs,yp1,ypn
 character(len=500) :: message
!arrays
 real(dp),allocatable :: qgrid_work(:),work_space(:),work_spl(:)

! ***************************************************************************

!Set various terms to 0 in case not defined below
!GTH values
 rloc=0.d0
 cc1=0.d0
 cc2=0.d0
 cc3=0.d0
 cc4=0.d0
 rrs=0.d0
 h1s=0.d0
 h2s=0.d0
 rrp=0.d0
 h1p=0.d0
 nproj(1:mpsang)=0

! DEBUG
!write(message,'(a)') ' psp2in : enter '
!call wrtout(06,  message,'COLL')
! ENDDEBUG

! Read and write different lines of the pseudopotential file
 read (tmp_unit,*) rloc,cc1,cc2,cc3,cc4
 write(message, '(a,f12.7)' ) ' rloc=',rloc
 call wrtout(ab_out,message,'COLL')
 call wrtout(06,  message,'COLL')
 write(message, '(a,f12.7,a,f12.7,a,f12.7,a,f12.7)' )&
&  '  cc1=',cc1,'; cc2=',cc2,'; cc3=',cc3,'; cc4=',cc4
 call wrtout(ab_out,message,'COLL')
 call wrtout(06,  message,'COLL')

 read (tmp_unit,*) rrs,h1s,h2s
 write(message, '(a,f12.7,a,f12.7,a,f12.7)' )&
&  '  rrs=',rrs,'; h1s=',h1s,'; h2s=',h2s
 call wrtout(ab_out,message,'COLL')
 call wrtout(06,  message,'COLL')

 read (tmp_unit,*) rrp,h1p
 write(message, '(a,f12.7,a,f12.7)' )&
&  '  rrp=',rrp,'; h1p=',h1p
 call wrtout(ab_out,message,'COLL')
 call wrtout(06,  message,'COLL')

 if (abs(h1s)>1.d-08) nproj(1)=1
 if (abs(h2s)>1.d-08) nproj(1)=2

 if (abs(h1p)>1.d-08) then
  if(mpsang<2)then
   write(message, '(a,a,a,a,es12.4,a,a,a,i2,a)' ) ch10,&
&   ' psp2in : BUG -',ch10,&
&   '  With non-zero h1p (=',h1p,&
&                  '), mpsang should be at least 2,',ch10,&
&   '  while mpsang=',mpsang,'.'
   call wrtout(ab_out,message,'COLL')
   call wrtout(06,  message,'COLL')
   call leave_new('COLL')
  end if
  nproj(2)=1
  if (lmax<1) then
   write(message, '(a,a,a,a,i5,a,e12.4,a,a,a,a)' ) ch10,&
&   ' psp2in : ERROR -',ch10,&
&   '  Input lmax=',lmax,' disagree with input h1p=',h1p,'.',&
&   '  Your pseudopotential is incoherent.',ch10,&
&   '  Action : correct your pseudopotential file.'
   call wrtout(ab_out,message,'COLL')
   call wrtout(06,  message,'COLL')
   call leave_new('COLL')
  end if
 end if

!Initialize array indlmn array giving l,m,n,ln,lm,s for i=lmn
 index=0;iln=0;indlmn(:,:)=0
 do ipsang=1,lmax+1
  if(nproj(ipsang)>0)then
   ll=ipsang-1
   do kk=1,nproj(ipsang)
    iln=iln+1
    do mm=1,2*ll*useylm+1
     index=index+1
     indlmn(1,index)=ll
     indlmn(2,index)=mm-ll*useylm-1
     indlmn(3,index)=kk
     indlmn(4,index)=ll*ll+(1-useylm)*ll+mm
     indlmn(5,index)=iln
     indlmn(6,index)=1
    end do
   end do
  end if
 end do

!First, the local potential --
!compute q^2V(q) or V(r)
 call psp2lo(cc1,cc2,cc3,cc4,dvlspl(:,1),epsatm,mqgrid_vl,qgrid_vl,&
& vlspl(:,1),rloc,vlspl_recipSpace,yp1,ypn,zion)

! DEBUG
!write(6,*)' psp2in : after psp2lo '
!stop
! ENDDEBUG

!Fit spline to (q^2)V(q) or V(r) (Numerical Recipes subroutine)
 allocate(work_space(mqgrid_vl),work_spl(mqgrid_vl))
 call spline (qgrid_vl,vlspl(:,1),mqgrid_vl,yp1,ypn,work_spl,work_space)
 vlspl(:,2)=work_spl(:)
 if (.not.vlspl_recipSpace) then
   call spline (qgrid_vl,dvlspl(:,1),mqgrid_vl,yp1,ypn,work_spl,work_space)
   dvlspl(:,2)=work_spl(:)
 end if
 deallocate(work_space,work_spl)


!Second, compute KB energies and form factors and fit splines
 ekb(:)=0.0d0
!First check if any nonlocal projectors are being used
 if (maxval(nproj(1:lmax+1))>0) then
  call psp2nl(ekb,ffspl,h1p,h1s,h2s,lnmax,mqgrid_ff,qgrid_ff,rrp,rrs)
 end if

end subroutine psp2in
!!***
