!{\src2tex{textfont=tt}}
!!****f* ABINIT/psp3in
!! NAME
!! psp3in
!!
!! FUNCTION
!! Initialize pspcod=3 pseudopotentials (HGH psps PRB58,3641(1998)):
!! continue to read the file, then compute the corresponding
!! local and non-local potentials.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (DCA, XG, GMR, FD)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!  lmnmax=if useylm=1, max number of (l,m,n) comp. over all type of psps
!!        =if useylm=0, max number of (l,n)   comp. over all type of psps
!!  lnmax=max. number of (l,n) components over all type of psps
!!  mpsang= 1+maximum angular momentum for nonlocal pseudopotentials
!!  mpssoang= 1+maximum (spin*angular momentum) for nonlocal pseudopotentials
!!  pspso=spin-orbit characteristics, govern the content of ffspl and ekb
!!   if =1 : this input requires NO spin-orbit characteristics of the psp
!!   if =2 : this input requires HGH characteristics of the psp
!!   if =3 : this input requires HFN characteristics of the psp
!!  mqgrid=dimension of q (or G) grid for arrays.
!!  qgrid(mqgrid)=values of q (or |G|) on grid from 0 to qmax
!!  useylm=governs the way the nonlocal operator is to be applied:
!!         1=using Ylm, 0=using Legendre polynomials
!!  zion=nominal valence of atom as specified in psp file
!!
!! OUTPUT
!!  ekb(lnmax)=Kleinman-Bylander energy,
!!             {{\ \begin{equation}
!!               \frac{\int_0^\infty [Rl(r)^2 (Vl(r)-Vloc(r))^2 dr]}
!!             {\int_0^\infty [Rl(r)^2 (Vl(r)-Vloc(r))   dr]}
!!              \end{equation} }}
!!             for each (l,n)
!!             if any, spin-orbit components begin at l=mpsang+1
!!  epsatm=$ (4\pi)\int_0^\infty [r^2 (V(r)+\frac{Zv}{r}) dr]$ (hartree)
!!  ffspl(mqgrid,2,lnmax)=Kleinman-Bylander form factor f_l(q) and
!!   second derivative from spline fit for each angular momentum and
!!   each projector; if any, spin-orbit components begin at l=mpsang+1
!!  indlmn(6,i)= array giving l,m,n,ln,lm,s for i=ln  (if useylm=0)
!!                                           or i=lmn (if useylm=1)
!!  nproj(mpssoang)=number of projection functions for each angular momentum
!!  vlspl(mqgrid,2)=q^2 Vloc(q) and second derivatives from spline fit
!!
!! SIDE EFFECTS
!!  Input/output
!!  lmax : at input =value of lmax mentioned at the second line of the psp file
!!    at output= 1
!!
!! PARENTS
!!      pspatm
!!
!! CHILDREN
!!      psp2lo,psp3nl,spline,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine psp3in(ekb,epsatm,ffspl,indlmn,lmax,lmnmax,lnmax,&
&     mpsang,mpssoang,mqgrid,nproj,pspso,qgrid,useylm,&
&     vlspl,zion)

 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01managempi
 use interfaces_02psp, except_this_one => psp3in
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: lmnmax,lnmax,mpsang,mpssoang,mqgrid,pspso,useylm
 integer,intent(inout) :: lmax
 real(dp),intent(in) :: zion
 real(dp),intent(out) :: epsatm
!arrays
 integer,intent(out) :: indlmn(6,lmnmax),nproj(mpssoang)
 real(dp),intent(in) :: qgrid(mqgrid)
 real(dp),intent(out) :: ekb(lnmax),ffspl(mqgrid,2,lnmax),vlspl(mqgrid,2)

!Local variables-------------------------------
!scalars
 integer :: iln,iln0,index,ipsang,jj,kk,ll,mm,mproj,nn,nso
 real(dp) :: cc1,cc2,cc3,cc4,h11d,h11f,h11p,h11s,h22d,h22p,h22s,h33d,h33p,h33s
 real(dp) :: k11d,k11f,k11p,k22d,k22p,k33d,k33p,rloc,rrd,rrf,rrp,rrs,yp1,ypn
 character(len=500) :: message
!arrays
 real(dp),allocatable :: dvlspl(:),ekb_so(:,:),ekb_sr(:,:),ffspl_so(:,:,:,:)
 real(dp),allocatable :: ffspl_sr(:,:,:,:),work_space(:),work_spl(:)

! ***************************************************************************

!DEBUG
! if(pspso/=1)then
!  write(6,*)' psp3in : pspso=',pspso,' is not yet allowed .'
! end if
!ENDDEBUG

!Set various terms to 0 in case not defined below
!GTH values
 rloc=0.0d0 ; rrs=0.0d0  ; h11p=0.0d0 ; k33p=0.0d0 ; k11d=0.0d0;
 cc1=0.0d0  ; h11s=0.0d0 ; h22p=0.0d0 ; rrd=0.0d0  ; k22d=0.0d0;
 cc2=0.0d0  ; h22s=0.0d0 ; h33p=0.0d0 ; h11d=0.0d0 ; k33d=0.0d0;
 cc3=0.0d0  ; h33s=0.0d0 ; k11p=0.0d0 ; h22d=0.0d0 ; h11f=0.0d0;
 cc4=0.0d0  ; rrp=0.0d0  ; k22p=0.0d0 ; h33d=0.0d0 ; k11f=0.0d0;
 rrf=0.0d0
 nproj(1:mpssoang)=0

!DEBUG
!write(6,*) ' psp3in : enter '
!ENDDEBUG

!Read and write different lines of the pseudopotential file

 read (tmp_unit,*) rloc,cc1,cc2,cc3,cc4
 write(message, '(a,f12.7)' ) ' rloc=',rloc
 call wrtout(ab_out,message,'COLL')
 call wrtout(06,  message,'COLL')
 write(message, '(a,f12.7,a,f12.7,a,f12.7,a,f12.7)' )&
&  ' cc1 =',cc1,'; cc2 =',cc2,'; cc3 =',cc3,'; cc4 =',cc4
 call wrtout(ab_out,message,'COLL')
 call wrtout(06,  message,'COLL')

!For the time being, the s state line must be present and is read,
!even for local pseudopotentials (zero must appear)
 read (tmp_unit,*) rrs,h11s,h22s,h33s
 write(message, '(a,f12.7,a,f12.7,a,f12.7,a,f12.7)' )&
&  ' rrs =',rrs,'; h11s=',h11s,'; h22s=',h22s,'; h33s=',h33s
 call wrtout(ab_out,message,'COLL')
 call wrtout(06,  message,'COLL')

 if (lmax > 0) then

  read (tmp_unit,*) rrp,h11p,h22p,h33p
  write(message, '(a,f12.7,a,f12.7,a,f12.7,a,f12.7)' )&
&  ' rrp =',rrp,'; h11p=',h11p,'; h22p=',h22p,'; h33p=',h33p
  call wrtout(ab_out,message,'COLL')
  call wrtout(06,  message,'COLL')

  read (tmp_unit,*) k11p,k22p,k33p
  write(message, '(a,f12.7,a,f12.7,a,f12.7)' )&
&  '                    k11p=',k11p,'; k22p=',k22p,'; k33p=',k33p
  call wrtout(ab_out,message,'COLL')
  call wrtout(06,  message,'COLL')

 end if

 if (lmax > 1) then

  read (tmp_unit,*) rrd,h11d,h22d,h33d
  write(message, '(a,f12.7,a,f12.7,a,f12.7,a,f12.7)' )&
&  ' rrd =',rrd,'; h11d=',h11d,'; h22d=',h22d,'; h33d=',h33d
  call wrtout(ab_out,message,'COLL')
  call wrtout(06,  message,'COLL')

  read (tmp_unit,*) k11d,k22d,k33d
  write(message, '(a,f12.7,a,f12.7,a,f12.7)' )&
&  '                    k11d=',k11d,'; k22d=',k22d,'; k33d=',k33d
  call wrtout(ab_out,message,'COLL')
  call wrtout(06,  message,'COLL')

 end if

 if (lmax > 2) then

  read (tmp_unit,*) rrf,h11f
  write(message, '(a,f12.7,a,f12.7)' )' rrf =',rrf,'; h11f=',h11f
  call wrtout(ab_out,message,'COLL')
  call wrtout(06,  message,'COLL')

  read (tmp_unit,*) k11f
  write(message, '(a,f12.7)' )'                    k11f=',k11f
  call wrtout(ab_out,message,'COLL')
  call wrtout(06,  message,'COLL')

 end if

 if (abs(h11s)>1.d-08) nproj(1)=1
 if (abs(h22s)>1.d-08) nproj(1)=2
 if (abs(h33s)>1.d-08) nproj(1)=3

 if (abs(h11p)>1.d-08) then
  nproj(2)=1
  if (lmax<1) then
   write(message, '(a,a,a,a,i5,a,e12.4,a,a)' ) ch10,&
&   ' psp3in : COMMENT -',ch10,&
&   '  input lmax=',lmax,'  does not agree with input h11p=',h11p,ch10,&
&   '  setting lmax to 1'
   call wrtout(ab_out,message,'COLL')
   call wrtout(06,  message,'COLL')
   lmax=1
  end if
 end if

 if (abs(h22p)>1.d-08) then
  nproj(2)=2
  if (lmax<1) then
   write(message, '(a,a,a,a,i5,a,e12.4,a,a)' ) ch10,&
&   ' psp3in : COMMENT -',ch10,&
&   '  input lmax=',lmax,' does not agree with input h22p=',h22p,ch10,&
&   '  setting lmax to 1'
   call wrtout(ab_out,message,'COLL')
   call wrtout(06,  message,'COLL')
  lmax=1
  end if
 end if


 if (abs(h33p)>1.d-08) then
  nproj(2)=3
   if (lmax<1) then
   write(message, '(a,a,a,a,i5,a,e12.4,a,a)' ) ch10,&
&   ' psp3in : COMMENT -',ch10,&
&   '  input lmax=',lmax,' does not agree with input h33p=',h33p,ch10,&
&   '  setting lmax to 1'
   call wrtout(ab_out,message,'COLL')
   call wrtout(06,  message,'COLL')
   lmax=1
  end if
 end if

 if (abs(h11d)>1.d-08) then
  nproj(3)=1
   if (lmax<2) then
    write(message, '(a,a,a,a,i5,a,e12.4,a,a)' ) ch10,&
&    ' psp3in : COMMENT -',ch10,&
&    '  input lmax=',lmax,'  does not agree with input h11d=',h11d,ch10,&
&    '  setting lmax to 2'
    call wrtout(ab_out,message,'COLL')
    call wrtout(06,  message,'COLL')
   lmax=2
  end if
 end if

 if (abs(h22d)>1.d-08) then
  nproj(3)=2
  if (lmax<2) then
   write(message, '(a,a,a,a,i5,a,e12.4,a,a)' ) ch10,&
&   ' psp3in : COMMENT -',ch10,&
&   '  input lmax=',lmax,'  does not agree with input h22d=',h22d,ch10,&
&   '  setting lmax to 2'
   call wrtout(ab_out,message,'COLL')
   call wrtout(06,  message,'COLL')
   lmax=2
  end if
 end if

 if (abs(h33d)>1.d-08) then
  nproj(3)=3
  if (lmax<2) then
   write(message, '(a,a,a,a,i5,a,e12.4,a,a)' ) ch10,&
&   ' psp3in : COMMENT -',ch10,&
&   '  input lmax=',lmax,' does not agree with input h33d=',h33d,ch10,&
&   '  setting lmax to 2'
   call wrtout(ab_out,message,'COLL')
   call wrtout(06,  message,'COLL')
   lmax=2
  end if
 end if

 if (abs(h11f)>1.d-08) then
  nproj(4)=1
  if (lmax<3) then
   write(message, '(a,a,a,a,i5,a,e12.4,a,a)' ) ch10,&
&   ' psp3in : COMMENT -',ch10,&
&   '  input lmax=',lmax,' does not agree with input h11f=',h11f,ch10,&
&   '  setting lmax to 3'
   call wrtout(ab_out,message,'COLL')
   call wrtout(06,  message,'COLL')
   lmax=3
  end if
 end if

 if(pspso/=1) then

  if (abs(k11p)>1.d-08) then
   nproj(mpsang+1)=1
   if (lmax<1) then
    write(message, '(a,a,a,a,i5,a,e12.4,a,a)' ) ch10,&
&    ' psp3in : COMMENT -',ch10,&
&    '  input lmax=',lmax,'  does not agree with input k11p=',k11p,ch10,&
&    '  setting lmax to 1'
    call wrtout(ab_out,message,'COLL')
    call wrtout(06,  message,'COLL')
    lmax=1
   end if
  end if

  if (abs(k22p)>1.d-08) then
   nproj(mpsang+1)=2
   if (lmax<1) then
    write(message, '(a,a,a,a,i5,a,e12.4,a,a)' ) ch10,&
&    ' psp3in : COMMENT -',ch10,&
&    '  input lmax=',lmax,' does not agree with input k22p=',k22p,ch10,&
&    '  setting lmax to 1'
    call wrtout(ab_out,message,'COLL')
    call wrtout(06,  message,'COLL')
   lmax=1
   end if
  end if


  if (abs(k33p)>1.d-08) then
   nproj(mpsang+1)=3
    if (lmax<1) then
    write(message, '(a,a,a,a,i5,a,e12.4,a,a)' ) ch10,&
&    ' psp3in : COMMENT -',ch10,&
&    '  input lmax=',lmax,' does not agree with input k33p=',k33p,ch10,&
&    '  setting lmax to 1'
    call wrtout(ab_out,message,'COLL')
    call wrtout(06,  message,'COLL')
    lmax=1
   end if
  end if

  if (abs(k11d)>1.d-08) then
   nproj(mpsang+2)=1
    if (lmax<2) then
     write(message, '(a,a,a,a,i5,a,e12.4,a,a)' ) ch10,&
&     ' psp3in : COMMENT -',ch10,&
&     '  input lmax=',lmax,'  does not agree with input k11d=',k11d,ch10,&
&     '  setting lmax to 2'
     call wrtout(ab_out,message,'COLL')
     call wrtout(06,  message,'COLL')
    lmax=2
   end if
  end if

  if (abs(k22d)>1.d-08) then
   nproj(mpsang+2)=2
   if (lmax<2) then
    write(message, '(a,a,a,a,i5,a,e12.4,a,a)' ) ch10,&
&    ' psp3in : COMMENT -',ch10,&
&    '  input lmax=',lmax,'  does not agree with input k22d=',k22d,ch10,&
&    '  setting lmax to 2'
    call wrtout(ab_out,message,'COLL')
    call wrtout(06,  message,'COLL')
    lmax=2
   end if
  end if

  if (abs(k33d)>1.d-08) then
   nproj(mpsang+2)=3
   if (lmax<2) then
    write(message, '(a,a,a,a,i5,a,e12.4,a,a)' ) ch10,&
&    ' psp3in : COMMENT -',ch10,&
&    '  input lmax=',lmax,' does not agree with input k33d=',k33d,ch10,&
&    '  setting lmax to 2'
    call wrtout(ab_out,message,'COLL')
    call wrtout(06,  message,'COLL')
    lmax=2
   end if
  end if

  if (abs(k11f)>1.d-08) then
   nproj(mpsang+3)=1
   if (lmax<3) then
    write(message, '(a,a,a,a,i5,a,e12.4,a,a)' ) ch10,&
&    ' psp3in : COMMENT -',ch10,&
&    '  input lmax=',lmax,' does not agree with input k11f=',k11f,ch10,&
&    '  setting lmax to 3'
    call wrtout(ab_out,message,'COLL')
    call wrtout(06,  message,'COLL')
    lmax=3
   end if
  end if

 end if

!Initialize array indlmn array giving l,m,n,ln,lm,s for i=lmn
 nso=1;if(pspso/=1) nso=2
 index=0;iln=0;indlmn(:,:)=0
 do nn=1,nso
  do ipsang=1+(nn-1)*(lmax+1),nn*lmax+1
   if (nproj(ipsang)>0) then
    ll=ipsang-(nn-1)*lmax-1
    do kk=1,nproj(ipsang)
     iln=iln+1
     do mm=1,2*ll*useylm+1
      index=index+1
      indlmn(1,index)=ll
      indlmn(2,index)=mm-ll*useylm-1
      indlmn(3,index)=kk
      indlmn(4,index)=ll*ll+(1-useylm)*ll+mm
      indlmn(5,index)=iln
      indlmn(6,index)=nn
     end do
    end do
   end if
  end do
 end do

!First, the local potential --  compute on q grid and fit spline
 call psp2lo(cc1,cc2,cc3,cc4,dvlspl,epsatm,mqgrid,qgrid,&
&  vlspl(:,1),rloc,.true.,yp1,ypn,zion)

! DEBUG
! write(6,*)' psp3in : after psp2lo '
! ENDDEBUG

!Fit spline to q^2 V(q) (Numerical Recipes subroutine)
 allocate(work_space(mqgrid),work_spl(mqgrid))
 call spline (qgrid,vlspl(:,1),mqgrid,yp1,ypn,work_spl,work_space)
 vlspl(:,2)=work_spl(:)
 deallocate(work_space,work_spl)

!Second, compute KB energies and form factors and fit splines
 ekb(:)=0.0d0

!Check if any nonlocal projectors are being used
 mproj=maxval(nproj(1:lmax+1))
 if (mproj>0) then

  allocate(ekb_sr(mpsang,mproj),ffspl_sr(mqgrid,2,mpsang,mproj))
  allocate(ekb_so(mpsang,mproj),ffspl_so(mqgrid,2,mpsang,mproj))

  call psp3nl(ekb_sr,ffspl_sr,h11s,h22s,h33s,h11p,h22p,h33p,h11d,h22d,&
&             h33d,h11f,mproj,mpsang,mqgrid,qgrid,rrd,rrf,rrp,rrs)
  if(pspso/=1) &
&  call psp3nl(ekb_so,ffspl_so,0.0d0,0.0d0,0.0d0,k11p,k22p,k33p,k11d,&
&              k22d,k33d,k11f,mproj,mpsang,mqgrid,qgrid,rrd,rrf,rrp,rrs)


! Convert ekb and ffspl
  iln0=0;do jj=1,lmnmax
   iln=indlmn(5,jj)
   if (iln>iln0) then
    iln0=iln
    if (indlmn(6,jj)<=1) then
     ekb(iln)=ekb_sr(1+indlmn(1,jj),indlmn(3,jj))
     ffspl(:,:,iln)=ffspl_sr(:,:,1+indlmn(1,jj),indlmn(3,jj))
    else
     ekb(iln)=ekb_so(1+indlmn(1,jj),indlmn(3,jj))
     ffspl(:,:,iln)=ffspl_so(:,:,1+indlmn(1,jj),indlmn(3,jj))
    end if
   end if
  end do

  deallocate(ekb_sr,ffspl_sr)
  deallocate(ekb_so,ffspl_so)

 end if

! DEBUG
!write(6,*)' psp3in : exit '
!stop
! ENDDEBUG

end subroutine psp3in
!!***
