!{\src2tex{textfont=tt}}
!!****f* ABINIT/sbf8
!! NAME
!! sbf8
!!
!! FUNCTION
!! Computes set of spherical bessel functions using accurate algorithm
!! based on downward recursion in order and normalization sum.
!! Power series used at small arguments.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (DRH)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!  nm=maximum angular momentum wanted + one
!!  xx=argument of sbf
!!
!! OUTPUT
!!  sb_out(nm)=values of spherical bessel functions for l=0,nm-1
!!
!! PARENTS
!!      psp8nl
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine sbf8(nm,xx,sb_out)

 use defs_basis

 implicit none

!Arguments----------------------------------------------------------
!scalars
 integer,intent(in) :: nm
 real(dp),intent(in) :: xx
!arrays
 real(dp),intent(out) :: sb_out(nm)

!Local variables-------------------------------
!scalars
 integer :: nlim,nn
 real(dp) :: fn,sn,xi,xn,xs
!arrays
 real(dp),allocatable :: sb(:)

! *************************************************************************

 if(xx<= 1.0d-36) then
! zero argument section
  sb_out(:)=0.0d0
  sb_out(1)=1.0d0
 else if(xx<1.d-3) then
! small argument section
  xn=1.0d0
  xs=0.5d0*xx**2
  do nn=1,nm
   sb_out(nn)=xn*(1.0d0 - xs*(1.0d0 - xs/(4*nn+6))/(2*nn+1))
   xn=xx*xn/(2*nn+1)
  end do
 else
! recursion method
  if(xx<1.0d0) then
   nlim=nm+idint(15.0d0*xx)+1
  else
   nlim=nm+idint(1.36d0*xx)+15
  end if
  allocate(sb(nlim+1))
  nn=nlim
  xi=1.0d0/xx
  sb(nn+1)=0.0d0
  sb(nn)=1.d-18
  sn=dble(2*nn-1)*1.d-36
  do nn=nlim-1,1,-1
   sb(nn)=dble(2*nn+1)*xi*sb(nn+1) - sb(nn+2)
  end do
  do nn=1,nlim-1
   sn=sn + dble(2*nn-1)*sb(nn)*sb(nn)
  end do
  fn=1.d0/sqrt(sn)
  sb_out(:)=fn*sb(1:nm)
  deallocate(sb)
 end if
end subroutine sbf8
!!***
