!{\src2tex{textfont=tt}}
!!****f* ABINIT/memory
!! NAME
!! memory
!!
!! FUNCTION
!! Estimation of the memory needed for a ground-state job.
!! According to the value of the option variable,
!! might also try to allocate this amount of memory, and if it fails,
!! might estimate the available memory.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (XG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!  getcell=if non-zero, the values of acell and rprim are taken from
!!   the output of another dataset
!!  idtset=number of the current dataset
!!  icoultrtmt=0 for periodic Fourier calculation of Hartree potential; 1 for isolated system using Poisson solver.
!!  intxc=control xc quadrature
!!  ionmov=control force calculations
!!  iout=unit number for output of formatted data.
!!  iprcel=govern the choice of preconditioner for the SCF cycle
!!  iscf=governs the choice of SCF algorithm, or non-SCF calculation.
!!  ixc=exchange-correlation choice parameter.
!!  jdtset=index of the current dataset
!!  lmnmax=max. number of (l,m,n) components over all type of psps
!!  lnmax =max. number of (l,n)   components over all type of psps
!!  mband =maximum number of bands
!!  mffmem =governs the number of FFT arrays which are fit in core memory
!!  mgfft =maximum single fft dimension
!!  mgfftdiel =maximum single fft dimension for susceptibility and dielectric
!!   matrices.
!!  mkmem =maximum number of k points which can fit in core memory
!!  mpi_enreg=informations about MPI parallelization
!!  mpssang is 1+maximum angular momentum for nonlocal pseudopotential
!!  mpssoang is 1+maximum (spin*angular momentum) for nonlocal pseudopotential
!!  mpw   =maximum number of planewaves in basis sphere (large number)
!!  mqgrid=maximum dimension of grid of q values for psp representations
!!  natom =number of atoms in unit cell
!!  nband(nkpt*nsppol)=number of bands at each k point, for each polarization
!!  nfft  =number of fft grid points for wavefunctions and density
!!  nfftdiel  =maximum number of fft grid points for susceptibility
!!    and dielectric matrices
!!  ngfft(18)=contain all needed information about 3D FFT, see ~abinit/doc/input_variables/vargs.htm#ngfft
!!  ngfftdiel(18)=contain all needed information about 3D FFT, dielectric case,
!!                 see ~abinit/doc/input_variables/vargs.htm#ngfft
!!    for susceptibility and dielectric matrices
!!  nkpt  =number of k points
!!  npwdiel=number of plane wave for susceptibility and dielectric matrix
!!  nloalg(5)=governs the choice of the algorithm for non-local operator.
!!  nspden=number of spin-density components
!!  nspinor=number of spinorial components of the wavefunctions
!!  nsppol=number of channels for spin-polarization (1 or 2)
!!  nstep =number of scf steps
!!  nsym  =number of symmetry elements in space group
!!  ntypat =number of types of atoms
!!  n1xccc=dimension of xccc1d ; 0 if no XC core correction is used
!!  occopt=option for occupation numbers. If 3<=occopt<=7, varying occupation
!!  optforces=1 if forces are computed during run
!!  option : if 0 , no test of available memory
!!           if 1 , the routine tries to allocate the estimated memory, for testing
!!                    purposes, and if a failure occurs, the routine stops.
!!           if 2 , like 1, but before stopping, the routine will provide
!!                    an estimation of the available memory.
!!  optstress=1 if stresses are computed during run
!!  prtvol=control print volume
!!  usepaw= 0 for non paw calculation; =1 for paw calculation
!!  useylm=governs the way the nonlocal operator is to be applied:
!!         1=using Ylm, 0=using Legendre polynomials
!!
!! OUTPUT
!!  (only writing)
!!
!! NOTES
!! for the estimation, it is only taken into account those
!! arrays that have some probability of being larger than 1000*8 bytes :
!! - All the arrays that have large numbers as one of their dimensions
!! (mqgrid, mpw, nfft, ngfft(4)*ngfft(5)*ngfft(6),
!!                     ngfftdiel(4)*ngfftdiel(5)*ngfftdiel(6), n1xccc
!!                                      or a constant larger than 1000)
!! - All the arrays that have a product of two moderately large numbers
!! (potential size above 30  : mband, mgfft, mkmem, natom, nkpt, nsym,
!!  or a constant larger than 30)
!! After this estimation, an amount of (176 + 55 + 6*natom) Kbytes is added
!! to take into account the static arrays declared
!! in rhohxc and daughter routines (at maximum 22*1000 dp numbers),
!! as well as other arrays like
!! character(len=500) :: message (present in about 100 routines), or the different
!! arrays allocated in move.f, brdmin.f, gstate.f (xf array) or pspini.f
!! In the case 3<=occopt<=7 this amount is increased by 760 Kbytes
!! to take into account the arrays smdfun, occfun, entfun, workfun and xgrid,
!! declared in getnel
!!
!! The current version takes into account
!! 1) and 2) the "main chain" in its two slightly different versions :
!! driver - gstate - (move or brdmin) - scfcv - vtorho - vtowfk -
!!     cgwf - getghc - fourwf or (nonlop+opernl)
!! 3) the xc chain :
!! driver - gstate - (move or brdmin) - scfcv - (vresfo) - rhohxc - xcden
!! 4) the mkrho chain :
!! driver - gstate - (move or brdmin) - scfcv - vtorho - mkrho
!! 5) the fourdp chain :
!! driver - gstate - (move or brdmin) - scfcv - vtorho
!!         ( + ftofr - fourdp - symrhg )
!! 6) the parallel k-point chain :
!! driver - gstate - (move or brdmin) - scfcv - vtorho - MPI_ALLREDUCE
!! 7) the newvtr chain :
!! driver - gstate - (move or brdmin) - scfcv - newvtr
!! 8) the susceptibility chain :
!! driver - gstate - (move or brdmin) - scfcv - vtorho - suscep - suskmm
!! 9) the dielectric chain :
!! driver - gstate - (move or brdmin) - scfcv - vtorho - dielmt
!! 10) the tddft chain :
!! driver - gstate - (move or brdmin) - scfcv - vtorho - tddft
!!
!! It is valid for all values of iscf, but not for nstep=0 (when the chain
!!     goes through energy instead of vtorho).
!!
!! Also, it is assumed that the potentials are non-local, even if there
!!     are local ! It would be necessary to update this routine
!!     now that the beginning of psp files is read before
!!     the present call (XG 980502)
!!
!! One might also estimate if there must be a chain arriving at :
!!  strnps , mkffnl, mkcore, mklocl, mkrho, prcpot, irrzg, initro,
!!  clnup1.
!! This is because there are allocated arrays in these routines.
!!
!! PARENTS
!!      invars2m
!!
!! CHILDREN
!!      memana,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine memory(n1xccc,getcell,idtset,icoultrtmt,intxc,ionmov,iout,iprcel,&
& iscf,ixc,jdtset,lmnmax,lnmax,&
& mband,mffmem,mgfft,mgfftdiel,mkmem,mpi_enreg,mpsang,mpssoang,mpw,mqgrid,&
& natom,nband,nfft,nfftdiel,ngfft,ngfftdiel,&
& nkpt,nloalg,npwdiel,nspden,nspinor,nsppol,nstep,nsym,ntypat,&
& occopt,optforces,option,optstress,prtvol,usepaw,useylm)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01managempi
 use interfaces_03iovars, except_this_one => memory
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: getcell,icoultrtmt,idtset,intxc,ionmov,iout,iprcel,iscf
 integer,intent(in) :: ixc,jdtset,lmnmax,lnmax,mband,mffmem,mgfft,mgfftdiel
 integer,intent(in) :: mkmem,mpsang,mpssoang,mpw,mqgrid,n1xccc,natom,nfft
 integer,intent(in) :: nfftdiel,nkpt,npwdiel,nspden,nspinor,nsppol,nstep,nsym
 integer,intent(in) :: ntypat,occopt,optforces,option,optstress,prtvol,usepaw
 integer,intent(in) :: useylm
 type(MPI_type),intent(in) :: mpi_enreg
!arrays
 integer,intent(in) :: nband(nkpt*nsppol),ngfft(18),ngfftdiel(18),nloalg(5)

!Local variables-------------------------------
!marrays=maximal number of arrays to be monitored (or group of arrays)
!cmpw(marrays)=count of blocks of size mpw bytes
!cfft(marrays)=number of blocks of size nfft bytes
!cadd(marrays)=additional storage needed (in bytes)
!dttyp(marrays)=datatype of the array : 4 for integers, 8 for real(dp)
!nchain=number of different chains of routines
!chain(marrays,nchain)=different chains of routines
 ! The cfoo arrays are used to store the allocated memory in the different
 ! routines of the program. Each stack of the program can allocate some
 ! memory and the amount is estimated and stored in cfoo(i). The lower i,
 ! the higher routine. cfft is memory used by FFT handling, cmpw for
 ! plane waves storage and cadd for miscellaneous memory occupation.
 ! The unit is the multiplier of the size of nfft for cfft, the multiplier
 ! of mpw for cmpw and the actually allocated memory for cadd.
 ! This array stores the size of each chunk of memory (8 for double
 ! floating point precision, 4 for integers and so on).
 ! This array defines if the chain defined above allocate or not the
 ! memory (depending on options).
!scalars
 integer,parameter :: marrays=150,nchain=10
 integer :: fftalgb,matblk,mblk,mincat,mu,n_fftgr,narr_fourdp,nbnd_in_blk
 integer :: ndiel4,ndiel456,ndiel5,ndiel6,ngrad,nprocwf,nspgrad
 real(dp) :: mbcg,mbdiskpd,mbdiskwf,mbf_fftgr
 character(len=500) :: message
!arrays
 integer :: cadd(marrays),cfft(marrays),cmpw(marrays),dttyp(marrays)
 logical :: chain(marrays,nchain)

! **************************************************************************
 if(option<0 .or. option>2)then
  write(message, '(A,A,A,A,I0,A)') ch10,&
&  ' memory : BUG -',ch10,&
&  '  option=',option,' while the only allowed values are 0, 1, or 2.'
  call wrtout(06,message,'COLL')
 end if

 cmpw(:)=0 ; cfft(:)=0 ; cadd(:)=0 ; dttyp(:)=0

 write(message,*)'memory : analysis of memory needs '
 call wrtout(06,message,'COLL')

 if(jdtset/=0)then
  write(message,'(80a,a,a,i3,a)')('=',mu=1,80),ch10,&
&  ' Values of the parameters that define the memory need for DATASET',jdtset,'.'
 else
  write(message,'(80a,a,a)')('=',mu=1,80),ch10,&
&  ' Values of the parameters that define the memory need of the present run '
 end if
 call wrtout(iout,message,'COLL')
 call wrtout(06,message,'COLL')

 write(message,'( 4(a,i10),a,4(a,i10) )' ) &
&  '   intxc =',intxc  ,'  ionmov =',ionmov,&
&  '    iscf =',iscf   ,'     ixc =',ixc   , ch10,&
&  '  lmnmax =',lmnmax ,'   lnmax =',lnmax,&
&  '   mband =',mband ,'  mffmem =',mffmem
 call wrtout(iout,message,'COLL')
 call wrtout(06,message,'COLL')

 write(message,'(3(4(a,i10),a),4(a,i10))')&
&  'P  mgfft =',mgfft    ,'   mkmem =',mkmem,&
&  ' mpssoang=',mpssoang ,'     mpw =',mpw   ,ch10,&
&  '  mqgrid =',mqgrid   ,'   natom =',natom ,&
&  '    nfft =',nfft     ,'    nkpt =',nkpt  ,ch10,&
&  '  nloalg =',nloalg(1),'  nspden =',nspden,&
&  ' nspinor =',nspinor  ,'  nsppol =',nsppol,ch10,&
&  '    nsym =',nsym     ,'  n1xccc =',n1xccc,&
&  '  ntypat =',ntypat   ,'  occopt =',occopt
 call wrtout(iout,message,'COLL')
 call wrtout(06,message,'COLL')

!Additional information needed for the susceptibility and dielectric matrices
 if((iprcel>=20.and.iprcel<70) .or. iscf==-1)then

! Compute the number of bands in blocks (nbnd_in_blk) from mband (see suskmm.f)
! Consider that if the number of bands is large, there are at most 8 blocks
  if(mband>=48)then
   mblk=8
   nbnd_in_blk=(mband-1)/mblk+1
! If the number of bands is medium, place 6 bands per block
  else if(mband>=12)then
   nbnd_in_blk=6
! Otherwise, must have at least 2 blocks
  else
   mblk=2
   nbnd_in_blk=(mband-1)/mblk+1
  end if

  write(message, '(a,a,a,i10,a,i6,a,i10,a,i10)' )&
&  ' For the susceptibility and dielectric matrices, or tddft :',ch10,&
&  '   mgfft =',mgfftdiel,'  nbnd_in_blk=',nbnd_in_blk,'    nfft =',nfftdiel,&
&  '     npw =',npwdiel
   call wrtout(iout,message,'COLL')
   call wrtout(06,message,'COLL')
  ndiel4=ngfftdiel(4) ; ndiel5=ngfftdiel(5) ; ndiel6=ngfftdiel(6)
  ndiel456=ndiel4*ndiel5*ndiel6

 end if

 write(message,'(80a)') ('=',mu=1,80)
 call wrtout(iout,message,'COLL')
 call wrtout(06,message,'COLL')

 if(getcell>0 .or. (getcell<0 .and. idtset+getcell>0) )then
  write(message,'(a,a,a,a,a,a,i3,a,i3,a,a,a,a,a,a)' )ch10,&
&  ' memory : COMMENT -',ch10,&
&  '  The determination of memory needs at this stage is meaningless,',ch10,&
&  '  since getcell = ',getcell,' is non-zero, while idtset=',idtset,'.',ch10,&
&  '  The following numbers are obtained by supposing that acell and rprim',ch10,&
&  '  are NOT taken from a previous dataset. You cannot rely on them.',ch10
  call wrtout(iout,message,'COLL')
  call wrtout(06,message,'COLL')
 end if

 n_fftgr=1
 if(iscf==1)            n_fftgr=5
 if(iscf==2.or.iscf==3) n_fftgr=4
 if(iscf==5.or.iscf==6) n_fftgr=10

!work1 and work2 in fourdp : take into account approximately fftalgb
 fftalgb=mod(ngfft(7),100)/10
 if(fftalgb==0)narr_fourdp=2*2
 if(fftalgb==1)narr_fourdp=2

 ngrad=1
 if(ixc>=11 .and. ixc<=16)ngrad=2

! (1)                     in main, driver, gstate and brdmin ----------------
!                         in move, nothing interesting is allocated.
!   kg (gstate)
   cmpw(1)=3*mkmem               ; dttyp(1)=4
!   indsym (gstate)
   cadd(3)=4*nsym*natom          ; dttyp(3)=4
!   irrzon  (gstate)
 if(nsym/=1)then
   cfft(4)=2*(nspden/nsppol)     ; dttyp(4)=4
 end if
!   ylm (gstate)
   cmpw(5)=mkmem*mpsang*mpsang*useylm ; dttyp(5)=8
!
!   rhor,rhog (gstate)
   cfft(5)=nspden+2              ; dttyp(5)=8
!   cg (gstate)
   cmpw(6)=2*nspinor*mband*mkmem*nsppol  ; dttyp(6)=8
!   eigen,resid,occ (occ is initialized in abinit, and not in driver)
   cadd(7)=3*mband*nkpt*nsppol   ; dttyp(7)=8
!   qgrid,vlspl,ffspl
   cadd(8)=mqgrid*(1+2*ntypat*(1+lnmax))   &
&                                ; dttyp(8)=8
!   ph1d (actually allocated in scfcv3 !!)
   cadd(9)=2*3*(2*mgfft+1)*natom ; dttyp(9)=8
!   phnons (in gstate)
 if(nsym/=1)then
   cfft(10)=2*(nspden/nsppol)    ; dttyp(10)=8
 end if
!   xccc1d (in driver)
   cadd(11)=n1xccc*6*ntypat      ; dttyp(11)=8

!   hessin in brdmin
 if(ionmov==2)then
   cadd(15)=3*natom*3*natom      ; dttyp(15)=8
 end if

! (2)                     in scfcv----------------------------------------

!   vhartr,vpsp,vtrial,vxc
   cfft(21)=2+2*nspden           ; dttyp(21)=8
 if(iscf>0)then
!   f_fftgr
   cfft(22)=nspden*n_fftgr*mffmem; dttyp(22)=8
 end if
 if( iscf>0 .and. (iprcel>=20.and.iprcel<70))then
!   dielinv, susmat
   cadd(23)=4*(npwdiel*nspden)**2; dttyp(23)=8
 end if
 if( (iscf>0 .and. iprcel>=20 .and. iprcel<70) .or. iscf==-1 )then
!  kg_diel
  cadd(25)=3*npwdiel             ; dttyp(25)=4
  if(nsym/=1)then
!  irrzondiel
  cadd(27)=2*nfftdiel*(nspden/nsppol)   ; dttyp(27)=4
!  phnonsdiel
  cadd(28)=2*nfftdiel*(nspden/nsppol)   ; dttyp(28)=8
  end if
 end if
 if(n1xccc/=0)then
!   xccc3d
   cfft(29)=1                    ; dttyp(29)=8
 end if
! Kernel of Poisson's solver
 if (icoultrtmt == 1) then
  cadd(24) = ngfft(4)*ngfft(5)*ngfft(6) ; dttyp(24) = 8
 end if

! (3)                     in rhohxc, xcden -------------------------------

 if(ixc/=0)then
  if(n1xccc/=0)then
!   rhocorval
   cfft(31)=nspden               ; dttyp(31)=8
  end if
!   dnexcdn, rhonow
   nspgrad=nspden*ngrad
   if(nspden==2 .and. ngrad==2)nspgrad=5
   cfft(32)=nspden*ngrad*ngrad+nspgrad  ; dttyp(32)=8
  if(intxc==1 .or. ngrad==2)then
!   wkcmpx,work in xcden +work1,work2 in fourdp
   cfft(33)=3+narr_fourdp        ; dttyp(33)=8
   cadd(33)=narr_fourdp*(ngfft(4)*ngfft(5)*ngfft(6)-nfft)
  end if
  if(ngrad==2)then
!   workgr in xcden
   cfft(34)=2                    ; dttyp(34)=8
  end if
 end if
 if(iscf>0)then
! In this case, rhohxc is called from vresfo also,
! for which vresid was allocated in scfcv
! vresid
   cfft(35)=nspden               ; dttyp(35)=8
 end if
! Poisson's solver with zero padding
 if (icoultrtmt == 1) then
  cfft(36) = 8                   ; dttyp(36) = 8
  cadd(36) = ngfft(4) * ngfft(5) * ngfft(6) - nfft
 end if

! Note : in hartre, called by rhohxc, one uses
!   2 dp arrays of total size 3*nfft,
!   and 2 arrays of total size 4*n4*n5*n6 for fourdp
!   This will be smaller than the total use for symrhg

! (4)                     in newvtr --------------------------------------

 if(iscf>0)then
!   vresid (allocated in scfcv) and vrespc
   cfft(41)=2*nspden             ; dttyp(41)=8
  if(mffmem==0)then
!   f_fftgr_disk
   cfft(42)=nspden*n_fftgr       ; dttyp(42)=8
  end if
 end if

! (5-6)                   in vtorho-----------------------------------------

! Note : (5) is for the arrays inside the spin and k-point loop
!                   they belong to the main chain
!        (6) is for the arrays after the spin and k-point loop
!        (6a) is for the arrays after that loop, for the parallel k-point chain
!        (6b) is for the arrays in mkrho, for the mkrho chain
!        (6c) is for the arrays in symrhg, for the fourdp chain
!        (6d) is for the arrays in suscep, for the suscep chain, see (10)
!        (6e) is for the arrays in dielmt, for the dielmt chain, see (11)

!   eknlk, enlnk, grnlnk
   cadd(51)=(11+3*natom)*mband*nkpt*nsppol &
&                                ; dttyp(51)=8
!   kg_k
   cmpw(52)=3                    ; dttyp(52)=4
!   rhoaug, vlocal
   cfft(53)=2                    ; dttyp(53)=8
   cadd(53)=2*(ngfft(4)*ngfft(5)*ngfft(6)-nfft)
 if(mkmem==0)then
!   cg_disk
   cmpw(54)=2*nspinor*mband      ; dttyp(54)=8
 end if
!   eig_k, ek_k, enl_k, grnl_k, occ_k, resid_k
   cadd(56)=(14+3*natom)*mband   ; dttyp(56)=8
!   ylm_k
   cmpw(58)=mpsang*mpsang*useylm ; dttyp(58)=8

! (6)                     in vtorho----------------------------------------

!   doccde
   cadd(60)=mband*nkpt*nsppol    ; dttyp(60)=8

! (6a)                    in vtorho----------------------------------------
 if(mpi_enreg%paral_compil_kpt==1)then
! Parallel case
! buffer1
! buffer2
  if(occopt>=3 .and. occopt <=7) then
                                   dttyp(61)=8
    if(nsppol*nfft >= (13+3*natom)*mband*nkpt*nspden)then
   cfft(61)=2*nspden
    else
   cadd(61)=(13+3*natom)*mband*nkpt*nspden
    end if
  else
   cfft(61)=2*nspden             ; dttyp(61)=8
   cadd(61)=9+3*natom+2+2*mband*nkpt*nspden
  end if
 end if

! (6b)                    in mkrho, called by vtorho--------------------------
 if(occopt>=3 .and. occopt <=7)then
  if(mkmem==0)then
!   cg_disk
   cmpw(62)=2*nspinor*mband      ; dttyp(62)=8
  end if
!   cwavef
   cmpw(65)=2*nspinor            ; dttyp(64)=8

!   rhoaug, wfraug, work1 in fourwf
   cfft(66)=5                    ; dttyp(66)=8
   cadd(66)=5*(ngfft(4)*ngfft(5)*ngfft(6)-nfft)
 end if

! (6c)                    in symrhg, called by vtorho--------------------------
 if(iscf>0)then
   cfft(67)=narr_fourdp          ; dttyp(67)=8
   cadd(67)=narr_fourdp*(ngfft(4)*ngfft(5)*ngfft(6)-nfft)
  if(nsym>1)then
!   work1  in symrhg
   cfft(68)=2                    ; dttyp(68)=8
   cadd(68)=2*(ngfft(4)*ngfft(5)*ngfft(6)-nfft)
  end if
 end if

! (6d) and (6e)           in suscep and dielmt, called by vtorho,
!                         see (10) and (11) -------------------------------

! (7)                     in vtowfk----------------------------------------

!   evec
   cadd(71)=2*mband*mband        ; dttyp(71)=8
!   subham
!   subvnl
   cadd(72)=2*mband*(mband+1)    ; dttyp(72)=8
!   gkpsq
   cmpw(73)=1                    ; dttyp(73)=8
!   ffnl
   cmpw(74)=2*ntypat*lmnmax      ; dttyp(74)=8
!   ph3d
   matblk=nloalg(4)
   if(nloalg(1)<=0)matblk=natom
   cmpw(75)=2*matblk             ; dttyp(75)=8
!   ghc
!   gvnlc
   cmpw(76)=2*2*nspinor          ; dttyp(76)=8
!   Note : matvnl and mat1 do not belong to a chain defined until now
!
 if(occopt<3 .and. iscf>0)then
!   cwavef
   cmpw(77)=2*nspinor            ; dttyp(77)=8
!   wfraug
   cfft(78)=2                    ; dttyp(78)=8
   cadd(78)=2*(ngfft(4)*ngfft(5)*ngfft(6)-nfft)
!   work1 in fourwf
   cfft(79)=2                    ; dttyp(79)=8
   cadd(79)=2*(ngfft(4)*ngfft(5)*ngfft(6)-nfft)
 end if

! (8)                     in cgwf------------------------------------------

!   conjgr, cwavef, direc, gh_direc, gvnl_direc
   cmpw(81)=2*5*nspinor          ; dttyp(81)=8

! (9a)                    in getghc and fourwf----------------------------

!   work (in getghc)
   cfft(91)=2                    ; dttyp(91)=8
   cadd(92)=2*(ngfft(4)*ngfft(5)*ngfft(6)-nfft)
!   work1 (in fourwf)
   cfft(92)=2                    ; dttyp(92)=8
   cadd(92)=2*(ngfft(4)*ngfft(5)*ngfft(6)-nfft)

! (9b)                    in getghc, nonlop and opernl--------------------
   mincat=min(nloalg(4),natom-ntypat+1)
   if (useylm==0) then                          ! ===== nonlop_pl
!    gxa  (in nonlop)
    cadd(94)=2*20*mincat*2       ; dttyp(94)=8
!    dgxdt  (in nonlop)
    if (optforces>0) then
     cadd(95)=2*3*20*mincat*2    ; dttyp(95)=8
    end if
!    dgxds  (in nonlop)
    if (optstress>0) then
     cadd(96)=2*56*mincat*2      ; dttyp(96)=8
    end if
!    teffv (in opernl4 - no distinction is made for opernl, opernl2 or opernl3)
!    kpgx, ffkg
!    here, evaluate an upper value, with nproj=2, p,d and f orbitals, but not
!    considering the stress, since it will be called outside of the main chain
    cadd(97)=nloalg(3)*40        ; dttyp(97)=8
!    kpg if nloalg(5)=1
    cadd(98)=3*mpw*nloalg(5)     ; dttyp(98)=8
   else                                        ! ===== nonlop_ylm
!    gx + gxfac
    cadd(94)=2* 2*mpw*lmnmax*mincat    ; dttyp(94)=8
!    dgxdt + dgxdtfac
    if (optforces>0.and.optstress==0) cadd(95)=4*2*mpw*lmnmax*mincat
    if (optstress>0) cadd(95)=7*2*mpw*lmnmax*mincat
    dttyp(95)=8
!    kpg
    cadd(96)=3*mpw       ; dttyp(96)=8
!    miscelaneous: indlmn_typ, ffnl_typ
    cadd(97)=lmnmax*(6+mpw*(1+optstress)); dttyp(97)=8
!    opernla_ylm: scalar,scali,scalarr,scalari
    cadd(98)=2*mpw+2*mpw
    if (optstress>0) cadd(98)=cadd(98)+3*2*mpw
    dttyp(98)=8
   end if

! (10)                    in suscep and suskmm ----------------------------

 if(iprcel>=20.and.iprcel<70)then
!   Variables allocated in suscep
  if(mkmem==0)then
!   cg_disk
   cmpw(101)=2*mband             ; dttyp(101)=8
  end if
  if(occopt>=3)then
!   drhode
   cadd(103)=2*npwdiel*nsppol    ; dttyp(103)=8
  end if
!   rhoextrap (always included, although it appears only when extrap==1)
   cadd(104)=ndiel456            ; dttyp(104)=8

!   Variables allocated in suskmm
!   cwavef
   cmpw(106)=2                   ; dttyp(106)=8
!   rhoaug, wfraug
   cadd(107)=3*ndiel456          ; dttyp(107)=8
!   wfprod
   cadd(108)=2*npwdiel           ; dttyp(108)=8
!   wfrspa1, wfrspa2
   cadd(109)=4*ndiel456*nbnd_in_blk ; dttyp(109)=8

 end if

! (11)                    in dielmt ---------------------------------------

 if(iprcel>=20.and.iprcel<70)then
!   dielh,dielvec,eig_diel,zhpev1,zhpev2
   cadd(111)=3*npwdiel*npwdiel                   &
&           +9*npwdiel           ; dttyp(111)=8
 end if

! (12)                    in tddft  ---------------------------------------

 if(iscf==-1)then
  if(mkmem/=0)then
!    cg_disk
    cmpw(121)=2*mband            ; dttyp(121)=8
  end if
!   cwavef
   cmpw(124)=2*mband             ; dttyp(124)=8
!   rhoaug,wfraug,wfrspa
   cadd(125)=(2+mband)*ndiel456  ; dttyp(125)=8
 end if

! --------------------------------------------------------------------------

 chain(:,:)=.true.

!Define the main chain version a (fourwf)
 chain(31:50,1)=.false.
 chain(60:70,1)=.false.
 chain(77:80,1)=.false.
 chain(93:100,1)=.false.
 chain(101:marrays,1)=.false.

!Define the main chain version b (nonlop+opernl)
 chain(31:50,2)=.false.
 chain(60:70,2)=.false.
 chain(77:80,2)=.false.
 chain(91:92,2)=.false.
 chain(101:marrays,2)=.false.

!Define the XC chain ( 31:40 belong only to this chain)
 chain(41:marrays,3)=.false.

!Define the mkrho chain ( 62:66 and 76:77 belong only to this chain)
!                         is it sure that they have to be summed ?)
 chain(31:50,4)=.false.
 chain(51:59,4)=.false.
 chain(61   ,4)=.false.
 chain(67:70,4)=.false.
 chain(71:marrays,4)=.false.
 chain(77:80,4)=.true.

!Define the fourdp chain ( 67:70 belong only to this chain)
 chain(31:50,5)=.false.
 chain(51:66,5)=.false.
 chain(60   ,5)=.true.
 chain(71:marrays,5)=.false.

!Define the parallel k-point chain ( 61 belong only to this chain )
 chain(31:50,6)=.false.
 chain(51:59,6)=.false.
 chain(62:70,6)=.false.
 chain(71:marrays,6)=.false.

!Define the newvtr chain ( 41:50 belong only to this chain)
 chain(31:40,7)=.false.
 chain(51:marrays,7)=.false.

!Define the suscep chain ( 101:110 belong only to this chain)
 chain(31:marrays,8)=.false.
 chain(60    ,8)=.true.
 chain(101:110,8)=.true.

!Define the dielmt chain ( 111:120 belong only to this chain)
 chain(31:marrays,9)=.false.
 chain(60    ,9)=.true.
 chain(111:120,9)=.true.

!Define the tddft chain ( 121:130 belong only to this chain)
 chain(31:marrays,10)=.false.
 chain(60    ,10)=.true.
 chain(121:130,10)=.true.

!The memory needed for each chain has been computed
!-------------------------------------------------------------------------
!Still need some auxiliary data : estimate the disk space
!or the maximum segment size.

!XG030513 : MPIWF need to multiply mbdiskwf by the number of processors
! in the WF group. For the time being, nprocwf=1
 nprocwf=mpi_enreg%nproc_fft

 mbdiskwf=(8*2*mpw*nprocwf*sum(nband(1:nkpt*nsppol)))/1024.d0**2 + 0.002d0
 mbdiskpd=(8*nfft*nsppol)/1024.d0**2 + 0.002d0

!Determine the largest array out of cg (cg_disk) or f_fftgr (f_fftgr_disk)
 if(mkmem==0)then
  mbcg=(8*2*mpw*mband)/1024.d0**2 + 0.002d0
 else
  mbcg=(8*2*mpw*mband*mkmem*nsppol)/1024.d0**2 + 0.002d0
 end if
 if(mffmem==0)then
  mbf_fftgr=(8*nfft*n_fftgr)/1024.d0**2 + 0.002d0
 else
  mbf_fftgr=(8*nfft*n_fftgr*nsppol*mffmem)/1024.d0**2 + 0.002d0
 end if

!---------------------------------------------------------------------
!Now, analyze the data

 call memana(cadd,cfft,chain,cmpw,dttyp,iout,iprcel,iscf,&
& marrays,mbcg,mbdiskpd,mbdiskwf,mbf_fftgr,mffmem,&
& mkmem,mpi_enreg,mpw,natom,nchain,nfft,nfftdiel,occopt,option,prtvol,usepaw)

!DEBUG
!write(6,*)' memory : exit '
!ENDDEBUG

end subroutine memory
!!***
