!{\src2tex{textfont=tt}}
!!****f* ABINIT/getcprj
!! NAME
!! getcprj
!!
!! FUNCTION
!! 1) Compute <Proj_i|Cnk> for every wave function |Cnk> expressed
!!    in reciprocal space.
!!    |Proj_i> is a non-local projector (for each atom and each l,m,n)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!  cwavef(2,nspinor*npw_k)=input cmplx wavefunction coefficients <G|Cnk>
!!  dimekb1,dimekb2=dimensions of ekb
!!  dimffnl=second dimension of ffnl
!!  ekb(dimekb1,dimekb2)= Kleinman-Bylander energies (hartree) (useful here only in non-PAW case)
!!  ffnl(npw_k,dimffnl,lmnmax,ntypat)=nonlocal form factors to be used for the application of the nl operator
!!  indlmn(6,i,ntypat)= array giving l,m,n,ln,lm,s for i=ln  (if useylm=0)
!!                                                  or i=lmn (if useylm=1)
!!  istwf_k=option parameter that describes the storage of wfs
!!  kg_k(3,npw_k)=reduced planewave coordinates
!!  kpoint(3)=k point in terms of recip. translations
!!  lmnmax=max. number of (l,m,n) components over all types of atoms
!!  matblk=dimension of the array ph3d
!!  mgfft=maximum size of 1D FFTs
!!  mpi_enreg=informations about MPI parallelization
!!  mpj=total number of non-local projectors
!!  natom=number of atoms in cell
!!  nattyp(ntypat)=number of atoms of each type
!!  ngfft(18)=contain all needed information about 3D FFT, see ~ABINIT/Infos/vargs.htm#ngfft
!!  nloalg(5)=governs the choice of the algorithm for nonlocal operator
!!  npw_k=number of planewaves for given k point
!!  nspinor=number of spinorial components of the wavefunctions
!!  ntypat=number of types of atoms in unit cell
!!  phkxred(2,natom)=phase factors exp(2 pi kpoint.xred)
!!  ph1d(2,3*(2*mgfft+1)*natom)=1-dim structure factor phase information
!!  ph3d(2,npw_k,matblk)=3D structure factors, for each atom and plane wave
!!   Note : is declared intent(inout), because it can be computed inside ph1d3d, and some compiler declare
!!   a conflict with the normal intent(in) of the present argument
!!  usepaw= 0 for non paw calculation; =1 for paw calculation
!!  useylm=governs the way the nonlocal operator is to be applied
!!
!! OUTPUT
!!  cwavep(2,mpj*nspinor)=projected input wave function <Proj_i|Cnk> with all NL projectors
!!
!! PARENTS
!!      cgwf,ctocprj
!!
!! CHILDREN
!!      leave_new,opernla_ylm,ph1d3d,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

 subroutine getcprj(cwavef,cwavep,dimekb1,dimekb2,dimffnl,ekb,ffnl,indlmn,istwf_k,kg_k,kpoint,&
&                   lmnmax,matblk,mgfft,mpi_enreg,mpj,natom,nattyp,ngfft,nloalg,npw_k,nspinor,ntypat,&
&                   phkxred,ph1d,ph3d,usepaw,useylm,ucvol_dum)

 use defs_basis
 use defs_datatypes
#          if defined NETCDF
 use netcdf
#          endif

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01managempi
 use interfaces_03nonlocal, except_this_one => getcprj
#endif
!End of the abilint section

 implicit none

!Arguments -------------------------------
!scalars
 integer :: dimekb1,dimekb2,dimffnl,istwf_k,lmnmax,matblk,mgfft,mpj,natom,npw_k
 integer :: nspinor,ntypat,usepaw,useylm
 type(MPI_type),intent(inout) :: mpi_enreg
!arrays
 integer,intent(in) :: indlmn(6,lmnmax,ntypat),kg_k(3,npw_k),nattyp(ntypat)
 integer,intent(in) :: ngfft(18),nloalg(5)
 real(dp),intent(in) :: cwavef(2,npw_k*nspinor),ekb(dimekb1,dimekb2)
 real(dp),intent(in) :: ffnl(npw_k,dimffnl,lmnmax,ntypat),kpoint(3)
 real(dp),intent(in) :: ph1d(2,3*(2*mgfft+1)*natom)
 real(dp),intent(in) :: phkxred(2,natom)
 real(dp),intent(out) :: cwavep(2,mpj*nspinor*natom)
 real(dp),intent(inout) :: ph3d(2,npw_k,matblk)

!Local variables-------------------------------
!scalars
 integer :: ia,ia1,ia2,ia3,ia4,ia5,il,ilmn,ishift1,ishift2,ishift3,ispinor
 integer :: itypat,mincat,nincat,nkpg,nlmn
 real(dp) :: cwavpi,cwavpr,ucvol_dum
 logical :: testnl
 character(len=500) :: message
!arrays
 integer,allocatable :: indlmn_typ(:,:)
 real(dp),parameter :: ili(8)=(/0.0d0,1.d0,0.d0,-1.d0,0.d0,1.d0,0.d0,-1.d0/)
 real(dp),parameter :: ilr(8)=(/1.0d0,0.d0,-1.d0,0.d0,1.d0,0.d0,-1.d0,0.d0/)
 real(dp),allocatable :: cwav(:,:),cwavp(:,:,:),cwavp_dum(:,:,:)
 real(dp),allocatable :: d2gxdt_dum(:,:,:,:),dgxdt_dum(:,:,:,:),ffnl_typ(:,:,:)
 real(dp),allocatable :: kpg_dum(:,:)

! *********************************************************************

!DEBUG
!write(6,*)' getcprj : enter'
!ENDDEBUG

 if (useylm==0) then
  write(message, '(4a)' ) ch10,&
&  ' getcprj : ERROR -',ch10,&
&  '  Not yet available for useylm=0 !'
  call wrtout(06,message,'PERS')
  call leave_new('PERS')
 end if

 mincat=min(nloalg(4),maxval(nattyp));nkpg=0
 allocate(cwav(2,npw_k))

!Loop over spinorial components
 ishift1=0
 do ispinor=1,nspinor
  cwav(:,:)=cwavef(:,1+ishift1:npw_k+ishift1)

! Loop over atom types
  ia1=1;ishift2=ishift1
  do itypat=1,ntypat
   ia2=ia1+nattyp(itypat)-1;ia5=0
   if (useylm==0) nlmn=maxval(indlmn(4,:,itypat))
   if (useylm==1) nlmn=count(indlmn(3,:,itypat)>0)

!  Test on local part
   testnl=(usepaw>0)
   if (usepaw==0) then
    do ilmn=1,nlmn
     if (abs(ekb(indlmn(5,ilmn,itypat),itypat))>tol10) testnl=.true.
    end do
   end if
   if (.not.testnl) then
    cwavep(:,1+ishift1:nlmn*nattyp(itypat)+ishift1)=zero
   else

!   Retrieve some data for this type of atom
    allocate(indlmn_typ(6,nlmn),ffnl_typ(npw_k,dimffnl,nlmn))
    indlmn_typ(:,1:nlmn)=indlmn(:,1:nlmn,itypat)
    ffnl_typ(:,:,1:nlmn)=ffnl(:,:,1:nlmn,itypat)

!   Loop on blocks of atoms inside type
    do ia3=ia1,ia2,mincat
     ia4=min(ia2,ia3+mincat-1);nincat=ia4-ia3+1
     allocate(cwavp(2,nlmn,nincat),cwavp_dum(2,nlmn,nincat))

!    Prepare the phase factors if they were not already computed
     if (nloalg(1)<=0) then
      call ph1d3d(ia3,ia4,kg_k,kpoint,matblk,natom,npw_k,&
&                                  ngfft(1),ngfft(2),ngfft(3),phkxred,ph1d,ph3d)
     endif

!    Compute <P|c> elements for this |Cnk> and this block of atoms
     if (useylm==1) then
      allocate (d2gxdt_dum(2,0,nlmn,nincat),dgxdt_dum(2,0,nlmn,nincat))
      call opernla_ylm(1,dimffnl,d2gxdt_dum,dgxdt_dum,ffnl,cwavp_dum,ia3,0,&
&                       indlmn_typ,istwf_k,kpg_dum,matblk,mpi_enreg,0,0,&
&                       nincat,nkpg,nlmn,nloalg,npw_k,ph3d,1,cwavef)
     end if
!VR
!     do ilmn=1,nlmn
!       il=indlmn_typ(1,ilmn)+1
!       cwavpr=cwavp_dum(1,ilmn,ia3)*ilr(il)-cwavp_dum(2,ilmn,ia3)*ili(il)
!       cwavpi=cwavp_dum(1,ilmn,ia3)*ili(il)+cwavp_dum(2,ilmn,ia3)*ilr(il)
!       cwavp(1,ilmn,ia3)=cwavpr
!       cwavp(2,ilmn,ia3)=cwavpi
!     enddo
     ishift3=nlmn*ia5+ishift2
     do ia=1,nincat
      do ilmn=1,nlmn
       il=indlmn_typ(1,ilmn)+1
       cwavpr=cwavp_dum(1,ilmn,ia3)*ilr(il)-cwavp_dum(2,ilmn,ia3)*ili(il)
       cwavpi=cwavp_dum(1,ilmn,ia3)*ili(il)+cwavp_dum(2,ilmn,ia3)*ilr(il)
       cwavp(1,ilmn,ia)=cwavpr
       cwavp(2,ilmn,ia)=cwavpi
      end do
      cwavep(1:2,1+nlmn*(ia-1)+ishift3:nlmn*ia+ishift3)=cwavp(1:2,1:nlmn,ia)
     end do

!    End loop inside block of atoms
     ia5=ia5+nincat
     deallocate(cwavp,cwavp_dum)
    end do
!  End IF nonlocal part exists
    deallocate(indlmn_typ,ffnl_typ,d2gxdt_dum,dgxdt_dum)
   end if
! End loop over atom types
  ia1=ia1+nattyp(itypat)
  ishift2=ishift2+nlmn*nattyp(itypat)
  end do

!End loop over spinorial components
  ishift1=ishift1+npw_k
 end do

 deallocate(cwav)

!DEBUG
!write(6,*)' getcprj : exit'
!ENDDEBUG

 end subroutine getcprj
!!***
