!{\src2tex{textfont=tt}}
!!****f* ABINIT/nonlop
!! nonlop
!!
!! NAME
!! nonlop
!!
!! FUNCTION
!! This routine is a driver to compute:
!! * Application of a nonlocal operator Vnl in order to get:
!!    - contracted elements (energy, forces, stresses, ...), if signs=1
!!    - a function in reciprocal space (|out> = Vnl|in>), if signs=2
!! * Optionnaly, in case of PAW calculation:
!!   - Contributions to Rhoij quantities (augmentation oocupancies)
!!     and their various derivatives.
!!   - Application of the overlap matrix in reciprocal
!!     space (<in|S|in> or (I+S)|in>).
!! According to user s choice, the routine calls a subroutine, computing all quantities:
!!   - using Legendre Polynomials Pl (Norm-conserving psps only)
!!   - using Spherical Harmonics Ylm (N-conserving or PAW ; compulsory for PAW)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.
!!
!! INPUTS
!!  atindx1(natom)=index table for atoms, inverse of atindx
!!  choice: chooses possible output:
!!    choice=1 => a non-local energy contribution
!!          =2 => a gradient with respect to atomic position(s)
!!          =3 => a gradient with respect to strain(s)
!!          =23=> a gradient with respect to atm. pos. and strain(s)
!!          =4 => a gradient and 2nd derivative with respect to atomic pos.
!!          =5 => a gradient with respect to k wavevector
!!          =6 => 2nd derivatives with respect to strain and atm. pos.
!!  dimenl1,dimenl2=dimensions of enl (see enl)
!!  dimffnlin=second dimension of ffnlin (1+number of derivatives)
!!  dimffnlout=second dimension of ffnlout (1+number of derivatives)
!!  enl(dimenl1,dimenl2)=
!!  ->Norm conserving : ==== when paw_opt=0 ====
!!                      (Real) Kleinman-Bylander energies (hartree)
!!                      dimenl1=lmnmax  -  dimenl2=ntypat
!!  ->PAW :             ==== when paw_opt=1 or 4 ====
!!                      (Real, symmetric) Dij coefs to connect projectors
!!                      dimenl1=lmnmax*(lmnmax+1)/2  -  dimenl2=natom
!!  ffnlin(npwin,dimffnlin,lmnmax,ntypat)=nonlocal form factors to be used
!!          for the application of the nonlocal operator to the |in> vector
!!  ffnlout(npwout,dimffnlout,lmnmax,ntypat)=nonlocal form factors to be used
!!          for the application of the nonlocal operator to the |out> vector
!!  gmet(3,3)=metric tensor for G vecs (in bohr**-2)
!!  gprimd(3,3)=dimensional reciprocal space primitive translations
!!  idir=direction of the - atom to be moved in the case (choice=2,signs=2),
!!                        - k point direction in the case (choice=5,signs=2)
!!                        - strain component (1:6) in the case (choice=2,signs=2) or (choice=6,signs=1)
!!  indlmn(6,i,ntypat)= array giving l,m,n,ln,lm,s for i=ln  (if useylm=0)
!!                                                  or i=lmn (if useylm=1)
!!  istwf_k=option parameter that describes the storage of wfs
!!  kgin(3,npwin)=integer coords of planewaves in basis sphere, for the |in> vector
!!  kgout(3,npwout)=integer coords of planewaves in basis sphere, for the |out> vector
!!  kpgin(npw,npkgin)= (k+G) components and related data, for the |in> vector  (only if useylm=1)
!!  kpgout(npw,nkpgout)=(k+G) components and related data, for the |out> vector (only if useylm=1)
!!  kptin(3)=k point in terms of recip. translations, for the |in> vector
!!  kptout(3)=k point in terms of recip. translations, for the |out> vector
!!  lmnmax=max. number of (l,m,n) components over all types of atoms
!!  matblk=dimension of the arrays ph3din and ph3dout
!!  mgfft=maximum size of 1D FFTs
!!  mpi_enreg=informations about MPI parallelization
!!  mpsang= 1+maximum angular momentum for nonlocal pseudopotentials
!!  mpssoang= 1+max(spin*angular momentum) for nonlocal pseudopotentials
!!  natom=number of atoms in cell
!!  nattyp(ntypat)=number of atoms of each type
!!  ngfft(18)=contain all needed information about 3D FFT, see ~abinit/doc/input_variables/vargs.htm#ngfft
!!  nkpgin,nkpgout=second sizes of arrays kpgin/kpgout
!!  nloalg(5)=governs the choice of the algorithm for nonlocal operator
!!  nnlout=dimension of enlout (when signs=1):
!!         ==== if paw_opt=0 or 1 ====
!!         choice=1=>nnlout=1   choice=2=>nnlout=3*natom
!!         choice=3=>nnlout=6   choice=4=>nnlout=6*natom
!!         choice=5=>nnlout=1   choice=6=>nnlout=6*(3*natom+6)
!!         choice=23=>nnlout=6+3*natom
!!         ==== if paw_opt=2 ====
!!         choice=1 =>nnlout=Sum_over_atoms(1*lmn_size)
!!         choice=2 =>nnlout=Sum_over_atoms(4*lmn_size)
!!         choice=3 =>nnlout=Sum_over_atoms(7*lmn_size)
!!         choice=23=>nnlout=Sum_over_atoms(10*lmn_size)
!!         ==== if paw_opt=3 ====
!!         choice=1 =>nnlout=1
!!  npwin=number of planewaves for given k point, for the |in> vector
!!  npwout=number of planewaves for given k point, for the |out> vector
!!  nspinor=number of spinorial components of the wavefunctions
!!  ntypat=number of types of atoms in cell
!!  paw_opt= define the nonlocal operator concerned with:
!!           paw_opt=0 : Norm-conserving Vnl (use of Kleinman-Bylander ener.)
!!           paw_opt=1 : PAW nonlocal part of H (use of Dij coeffs)
!!           paw_opt=2 : PAW augmentation occuppancies (rhoij)
!!           paw_opt=3 : PAW overlap matrix (Sij)
!!           paw_opt=4 : both PAW nonlocal part of H (Dij) and overlap matrix (Sij)
!!  phkxredin(2,natom)=phase factors exp(2 pi kptin.xred)
!!  phkxredout(2,natom)=phase factors exp(2 pi kptout.xred)
!!  ph1d(2,3*(2*mgfft+1)*natom)=1D structure factors phase information
!!  ph3din(2,npwin,matblk)=3D structure factors, for each atom and plane wave (in)
!!  ph3dout(2,npwout,matblk)=3-dim structure factors, for each atom and plane wave (out)
!!  pspso(ntypat)=spin-orbit characteristic for each atom type
!!  signs= if 1, get contracted elements (energy, forces, stress, ...)
!!         if 2, applies the non-local operator to a function in reciprocal space
!!  sij(dimenl1,ntypat*(paw_opt/3))=overlap matrix components (only if paw_opt=3 or 4)
!!  tim_nonlop=timing code of the calling routine (can be set to 0 if not attributed)
!!  ucvol=unit cell volume (bohr^3)
!!  useylm=governs the way the nonlocal operator is to be applied:
!!         1=using Ylm, 0=using Legendre polynomials
!!  vectin(2,nspinor*npwin)=input cmplx wavefunction coefficients <G|Cnk>
!!
!! OUTPUT
!!  ==== if (signs==1) ====
!!  enlout(nnlout)=
!!    if paw_opt==0 or 1: contribution of this state to the nl part of various properties
!!    if paw_opt==2 :     contribution of this state to various PAW Rhoij quantities
!!    if paw_opt==3:      contribution of this state to <c|S|c>  (where S=overlap when PAW)
!!  ==== if (signs==2) ====
!!    if paw_opt==0, 1 or 4:
!!       vectout(2,nspinor*npwout)=result of the aplication of the nl operator
!!                                 or one of its derivative to the input vect.
!!    if paw_opt==3 or 4:
!!       svectout(2,nspinor*npwout*(paw_opt/3))=result of the aplication of (I+S)
!!                                  to the input vect. (where S=overlap when PAW)
!!
!! NOTES
!! * See nonlop_pl and nonlop_ylm to have more comments...
!! * In the case signs=1, the array vectout is not used, nor modified
!!   so that the same array as vectin can be used as a dummy argument;
!!   the same is true for the pairs npwin-npwout, ffnlin-ffnlout,
!!   kgin-kgout, ph3din-ph3dout, phkredin-phkxredout).
!!
!! PARENTS
!!      cgwf3,dyfnl3,eig1fixed,eltfrnl3,energy,forstrnps,getghc,getgsc
!!      lobpcgIIwf,lobpcgccIIwf,lobpcgccwf,lobpcgwf,nstwf3,nstwf4,outkss
!!      resp3dte,vtowfk,vtowfk_htor
!!
!! CHILDREN
!!      leave_new,nonlop_pl,nonlop_ylm,timab,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine nonlop(atindx1,choice,dimenl1,dimenl2,dimffnlin,dimffnlout,&
&                  enl,enlout,ffnlin,ffnlout,gmet,gprimd,idir,indlmn,&
&                  istwf_k,kgin,kgout,kpgin,kpgout,kptin,kptout,lmnmax,matblk,mgfft,&
&                  mpi_enreg,mpsang,mpssoang,natom,nattyp,ngfft,nkpgin,nkpgout,nloalg,&
&                  nnlout,npwin,npwout,nspinor,ntypat,paw_opt,phkxredin,&
&                  phkxredout,ph1d,ph3din,ph3dout,pspso,signs,sij,svectout,&
&                  tim_nonlop,ucvol,useylm,vectin,vectout)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_00basis
 use interfaces_01managempi
 use interfaces_03nonlocal, except_this_one => nonlop
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: choice,dimenl1,dimenl2,dimffnlin,dimffnlout,idir,istwf_k
 integer,intent(in) :: lmnmax,matblk,mgfft,mpsang,mpssoang,natom,nkpgin,nkpgout
 integer,intent(in) :: nnlout,npwin,npwout,nspinor,ntypat,paw_opt,signs
 integer,intent(in) :: tim_nonlop,useylm
 real(dp),intent(in) :: ucvol
 type(MPI_type),intent(inout) :: mpi_enreg
!arrays
 integer,intent(in) :: atindx1(natom),indlmn(6,lmnmax,ntypat),kgin(3,npwin)
 integer,intent(in) :: kgout(3,npwout),nattyp(ntypat),ngfft(18),nloalg(5)
 integer,intent(in) :: pspso(ntypat)
 real(dp),intent(in) :: enl(dimenl1,dimenl2)
 real(dp),intent(in) :: ffnlin(npwin,dimffnlin,lmnmax,ntypat)
 real(dp),intent(in) :: ffnlout(npwout,dimffnlout,lmnmax,ntypat),gmet(3,3)
 real(dp),intent(in) :: gprimd(3,3),kpgin(npwin,nkpgin*useylm)
 real(dp),intent(in) :: kpgout(npwout,nkpgout*useylm),kptin(3),kptout(3)
 real(dp),intent(in) :: ph1d(2,3*(2*mgfft+1)*natom),phkxredin(2,natom)
 real(dp),intent(in) :: phkxredout(2,natom),sij(dimenl1,ntypat*(paw_opt/3))
 real(dp),intent(inout) :: ph3din(2,npwin,matblk),ph3dout(2,npwout,matblk)
 real(dp),intent(inout) :: vectin(2,nspinor*npwin)
 real(dp),intent(out) :: enlout(nnlout),svectout(2,nspinor*npwout*(paw_opt/3))
 real(dp),intent(out) :: vectout(2,nspinor*npwout)

!Local variables-------------------------------
!scalars
 character(len=500) :: message
!arrays
 real(dp) :: tsec(2)

! **********************************************************************

!DEBUG
!write(6,*) 'nonlop: enter'
!ENDDEBUG

!Keep track of time spent in this routine. Note the selection of
!different slots for different choices.
 call timab(220+tim_nonlop,1,tsec)

!Error on bad input
 if (useylm==0.and.paw_opt>0) then
  write(message, '(4a)' ) ch10,&
&   ' nonlop : BUG -',ch10,&
&   '  useylm=0 when paw_opt>0 !'
  call wrtout(06,message,'PERS')
  call leave_new('PERS')
 end if

 if (useylm==0) then
  call nonlop_pl (choice,dimenl1,dimenl2,dimffnlin,dimffnlout,&
&                 enl,enlout,ffnlin,ffnlout,gmet,gprimd,idir,&
&                 indlmn,istwf_k,kgin,kgout,kpgin,kpgout,kptin,kptout,lmnmax,matblk,&
&                 mgfft,mpi_enreg,mpsang,mpssoang,natom,nattyp,ngfft,nkpgin,nkpgout,&
&                 nloalg,nnlout,npwin,npwout,nspinor,ntypat,phkxredin,&
&                 phkxredout,ph1d,ph3din,ph3dout,pspso,signs,ucvol,vectin,vectout)
 else
  call nonlop_ylm(atindx1,choice,dimenl1,dimenl2,dimffnlin,dimffnlout,&
&                 enl,enlout,ffnlin,ffnlout,gmet,gprimd,idir,indlmn,&
&                 istwf_k,kgin,kgout,kpgin,kpgout,kptin,kptout,lmnmax,matblk,mgfft,&
&                 mpi_enreg,natom,nattyp,ngfft,nkpgin,nkpgout,nloalg,nnlout,&
&                 npwin,npwout,nspinor,ntypat,paw_opt,phkxredin,phkxredout,ph1d,&
&                 ph3din,ph3dout,pspso,signs,sij,svectout,ucvol,vectin,vectout)
 end if

 call timab(220+tim_nonlop,2,tsec)

!DEBUG
!write(6,*) 'nonlop: exit'
!ENDDEBUG

end subroutine nonlop
!!***
