!{\src2tex{textfont=tt}}
!!****f* ABINIT/opernlb_ylm
!! NAME
!! opernlb_ylm
!!
!! FUNCTION
!! * Operate with the non-local part of the hamiltonian,
!!   from projected scalars to reciprocal space.
!! * Operate with the non-local projectors and the overlap matrix,
!!   from projected scalars to reciprocal space.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.
!!
!! INPUTS
!!  choice=chooses possible output (see below)
!!  dgxdtfac(2,ndgxdtfac,nlmn,nincat)= gradients of gxfac
!!  dimffnl=second dimension of ffnl
!!  ffnl(npw,dimffnl,nlmn)= nonlocal quantities containing nonlocal form factors
!!  gxfac(2,nlmn,nincat)= reduced projected scalars related to Vnl (NL operator)
!!  gxfac_sij(2,nlmn,nincat*(paw_opt/3))= reduced projected scalars related to Sij (overlap)
!!  ia3=gives the number of the first atom in the subset presently treated
!!  idir=direction of the - atom to be moved in the case (choice=2,signs=2),
!!                        - k point direction in the case (choice=5,signs=2)
!!                        - strain component (1:6) in the case (choice=2,signs=2) or (choice=6,signs=1)
!!  kpg(npw,nkpg)=(k+G) components (if nkpg=3)
!!  matblk=dimension of the array ph3d
!!  ndgxdtfac=second dimension of dgxdtfac
!!  nincat=number of atoms in the subset here treated
!!  nkpg=second dimension of array kpg (0 or 3)
!!  nlmn=number of (l,m,n) numbers for current type of atom
!!  nloalg(5)=governs the choice of the algorithm for non-local operator.
!!  npw=number of plane waves in reciprocal space
!!  paw_opt= define the nonlocal operator concerned with:
!!           paw_opt=0 : Norm-conserving Vnl (use of Kleinman-Bylander ener.)
!!           paw_opt=1 : PAW nonlocal part of H (use of Dij coeffs)
!!           paw_opt=2 : PAW augmentation occuppancies (rhoij)
!!           paw_opt=3 : PAW overlap matrix (Sij)
!!           paw_opt=4 : both PAW nonlocal part of H (Dij) and overlap matrix (Sij)
!!  ph3d(2,npw,matblk)=three-dimensional phase factors
!!
!! OUTPUT
!!  (see side effects)
!!
!! SIDE EFFECTS
!!  if (paw_opt=0, 1 or 4)
!!    vect(2*npw)=final vector in reciprocal space:
!!      if (choice=1) <G|V_nonlocal|vect_start>
!!      if (choice=2) <G|dV_nonlocal/d(atm coord)|vect_start>
!!      if (choice=3) <G|dV_nonlocal/d(strain)|vect_start>
!!      if (choice=5) <G|dV_nonlocal/dk|vect_start>
!!  if (paw_opt/=3 or 4)
!!    svect(2*npw*(paw_opt/3))=final vector in reciprocal space:
!!      if (choice=1) <G|I+S|vect_start>  (S= overlap matrix)
!!
!! NOTES
!! Operate for one type of atom, and within this given type of atom,
!! for a subset of at most nincat atoms.
!!
!! PARENTS
!!      nonlop_ylm
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine opernlb_ylm(choice,dgxdtfac,dimffnl,ffnl,gxfac,gxfac_sij,ia3,idir,kpg,&
&                       matblk,ndgxdtfac,nincat,nkpg,nlmn,nloalg,npw,paw_opt,ph3d,svect,vect)

 use defs_basis

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: choice,dimffnl,ia3,idir,matblk,ndgxdtfac,nincat,nkpg
 integer,intent(in) :: nlmn,npw,paw_opt
!arrays
 integer,intent(in) :: nloalg(5)
 real(dp),intent(in) :: dgxdtfac(2,ndgxdtfac,nlmn,nincat)
 real(dp),intent(in) :: ffnl(npw,dimffnl,nlmn),gxfac(2,nlmn,nincat)
 real(dp),intent(in) :: gxfac_sij(2,nlmn,nincat*(paw_opt/3)),kpg(npw,nkpg)
 real(dp),intent(in) :: ph3d(2,npw,matblk)
 real(dp),intent(inout) :: svect(2,npw*(paw_opt/3)),vect(2,npw)

!Local variables-------------------------------
!scalars
 integer :: ia,iaph3d,ilmn,ipw
 real(dp) :: ai,ar,bi,br,ffnl1,ffnl2,ph3d1,ph3d2

! *************************************************************************

!Nothing to do when choice=4
 if (paw_opt==2.or.choice==4.or.choice==6.or.choice==23) return

!Loops (blocking)
!$OMP PARALLEL DEFAULT(PRIVATE) &
!$OMP&SHARED(choice,nincat,nloalg,ia3,npw,nlmn)
!$OMP&SHARED(ffnl,gxfac,dgxdt,dgxdtfax,kpg,ph3d,vect)
!$OMP DO

!Loop on atoms
 do ia=1,nincat
  iaph3d=ia;if (nloalg(1)>0) iaph3d=ia+ia3-1

! Loop on plane waves
  do ipw=1,npw

   ph3d1=ph3d(1,ipw,iaph3d);ph3d2=ph3d(2,ipw,iaph3d)

   if (paw_opt/=3) then

    if (choice==1) then
     ar=zero;ai=zero
     do ilmn=1,nlmn
      ffnl1=ffnl(ipw,1,ilmn)
      ar=ar+gxfac(1,ilmn,ia)*ffnl1
      ai=ai+gxfac(2,ilmn,ia)*ffnl1
     end do

    elseif (choice==2) then
     ar=zero;ai=zero;br=zero;bi=zero
     do ilmn=1,nlmn
      ffnl1=ffnl(ipw,1,ilmn)
      ar=ar+dgxdtfac(1,1,ilmn,ia)*ffnl1
      ai=ai+dgxdtfac(2,1,ilmn,ia)*ffnl1
      br=br+gxfac(2,ilmn,ia)*ffnl1
      bi=bi-gxfac(1,ilmn,ia)*ffnl1
     end do
     ar=ar+br*two_pi*kpg(ipw,idir)
     ai=ai+bi*two_pi*kpg(ipw,idir)

    elseif (choice==3) then
     ar=zero;ai=zero
     do ilmn=1,nlmn
      ffnl1=ffnl(ipw,1,ilmn)
      ffnl2=ffnl(ipw,2,ilmn)
      if (idir<=3) then
       ar=ar+dgxdtfac(1,1,ilmn,ia)*ffnl1-gxfac(1,ilmn,ia)*(ffnl2+ffnl1)
       ai=ai+dgxdtfac(2,1,ilmn,ia)*ffnl1-gxfac(2,ilmn,ia)*(ffnl2+ffnl1)
      else
       ar=ar+dgxdtfac(1,1,ilmn,ia)*ffnl1-gxfac(1,ilmn,ia)*ffnl2
       ai=ai+dgxdtfac(2,1,ilmn,ia)*ffnl1-gxfac(2,ilmn,ia)*ffnl2
      end if
     end do

    elseif (choice==5) then
     ar=zero;ai=zero
     do ilmn=1,nlmn
      ffnl1=ffnl(ipw,1,ilmn)
      ffnl2=ffnl(ipw,2,ilmn)
      ar=ar+dgxdtfac(1,1,ilmn,ia)*ffnl1+gxfac(1,ilmn,ia)*ffnl2
      ai=ai+dgxdtfac(2,1,ilmn,ia)*ffnl1+gxfac(2,ilmn,ia)*ffnl2
     end do
    end if

    vect(1,ipw)=vect(1,ipw)+ar*ph3d1+ai*ph3d2
    vect(2,ipw)=vect(2,ipw)+ai*ph3d1-ar*ph3d2
   end if

   if (paw_opt>2.and.choice==1) then
    ar=zero;ai=zero
    do ilmn=1,nlmn
     ffnl1=ffnl(ipw,1,ilmn)
     ar=ar+gxfac_sij(1,ilmn,ia)*ffnl1
     ai=ai+gxfac_sij(2,ilmn,ia)*ffnl1
    end do
    svect(1,ipw)=svect(1,ipw)+ar*ph3d1+ai*ph3d2
    svect(2,ipw)=svect(2,ipw)+ai*ph3d1-ar*ph3d2
   end if

! End loop on planewaves
  end do

!End loop on atoms
 end do
!$OMP END DO
!$OMP END PARALLEL

end subroutine opernlb_ylm
!!***
