!{\src2tex{textfont=tt}}
!!****f* ABINIT/opernlc_ylm
!! NAME
!! opernlc_ylm
!!
!! FUNCTION
!! * Operate with the non-local part of the hamiltonian,
!!   in order to reduce projected scalars
!! * Operate with the non-local projectors and the overlap matrix,
!!   in order to reduce projected scalars
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.
!!
!! INPUTS
!!  atindx1(natom)=index table for atoms (gives the absolute index of
!!                 an atom from its rank in a block of atoms)
!!  natom=number of atoms in cell
!!  dgxdt(2,ndgxdt,nlmn,nincat)=grads of projected scalars (only if optder>0)
!!  dimenl1,dimenl2=dimensions of enl (see enl)
!!  enl(dimenl1,dimenl2)=
!!  ->Norm conserving : ==== when paw_opt=0 ====
!!                      (Real) Kleinman-Bylander energies (hartree)
!!                      dimenl1=lmnmax  -  dimenl2=ntypat
!!  ->PAW :             ==== when paw_opt=1 or 4 ====
!!                      (Real, symmetric) Dij coefs to connect projectors
!!                      dimenl1=lmnmax*(lmnmax+1)/2  -  dimenl2=natom
!!  gx(2,nlmn,nincat*abs(enl_opt))= projected scalars
!!  iatm=absolute rank of first atom of the current block of atoms
!!  indlmn(6,nlmn)= array giving l,m,n,ln,lm,s for i=lmn
!!  itypat=type of atoms
!!  ndgxdt=second dimension of dgxdt
!!  ndgxdtfac=second dimension of dgxdtfac
!!  nincat=number of atoms in the subset here treated
!!  nlmn=number of (l,m,n) numbers for current type of atom
!!  optder=0=only gxfac is computed, 1=both gxfac and dgxdtfac are computed
!!  paw_opt= define the nonlocal operator concerned with:
!!           paw_opt=0 : Norm-conserving Vnl (use of Kleinman-Bylander ener.)
!!           paw_opt=1 : PAW nonlocal part of H (use of Dij coeffs)
!!           paw_opt=2 : PAW augmentation occuppancies (rhoij)
!!           paw_opt=3 : PAW overlap matrix (Sij)
!!           paw_opt=4 : both PAW nonlocal part of H (Dij) and overlap matrix (Sij)
!!  sij((paw_opt/3)*nlm*(nlmn+1)/2)=overlap matrix components (only if paw_opt=3 or 4)
!!  wt=normalization factor (wt=four_pi**2/ucvol)
!!
!! OUTPUT
!!  if (paw_opt=0, 1 or 4)
!!    gxfac(2,nlmn,nincat)= reduced projected scalars related to Vnl (NL operator)
!!  if (paw_opt=3 or 4)
!!    gxfac_sij(2,nlmn,nincat*(paw_opt/3))= reduced projected scalars related to Sij (overlap)
!!  if (optder==1.and.paw_opt=0, 1 or 4)
!!    dgxdtfac(2,ndgxdtfac,nlmn,nincat)= gradients of gxfac related to Vnl (NL operator)
!!
!! NOTES
!! Operate for one type of atom, and within this given type of atom,
!! for a subset of at most nincat atoms.
!!
!! PARENTS
!!      nonlop_ylm
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine opernlc_ylm(atindx1,dgxdt,dgxdtfac,dimenl1,dimenl2,enl,gx,&
&                       gxfac,gxfac_sij,iatm,indlmn,itypat,natom,ndgxdt,&
&                       ndgxdtfac,nincat,nlmn,optder,paw_opt,sij,wt)

 use defs_basis

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: dimenl1,dimenl2,iatm,itypat,natom,ndgxdt,ndgxdtfac
 integer,intent(in) :: nincat,nlmn,optder,paw_opt
 real(dp),intent(in) :: wt
!arrays
 integer,intent(in) :: atindx1(natom),indlmn(6,nlmn)
 real(dp),intent(in) :: dgxdt(2,ndgxdt,nlmn,nincat),enl(dimenl1,dimenl2)
 real(dp),intent(in) :: gx(2,nlmn,nincat),sij((paw_opt/3)*nlmn*(nlmn+1)/2)
 real(dp),intent(out) :: dgxdtfac(2,ndgxdtfac,nlmn,nincat),gxfac(2,nlmn,nincat)
 real(dp),intent(out) :: gxfac_sij(2,nlmn,nincat*(paw_opt/3))

!Local variables-------------------------------
!scalars
 integer :: ia,ijlmn,ilm,ilmn,iln,index_enl,j0lmn,jjlmn,jl,jlm,jlmil,jlmn,mu
 real(dp) :: enli,enlr,gxfii,gxfir,gxii,gxir,gxji,gxjr,siji,sijr
!no_abirules
!Constant: i^l (-3<=l<=3)
 real(dp),parameter :: ili(7)=(/1.d0,0.d0,-1.d0,0.d0,1.d0,0.d0,-1.d0/)
 real(dp),parameter :: ilr(7)=(/0.d0,-1.d0,0.d0,1.d0,0.d0,-1.d0,0.d0/)
 real(dp),allocatable :: gxfji(:),gxfjr(:)

! *************************************************************************

 if (paw_opt==2) return

!Accumulate gxfac related to nonlocal operator (Norm-conserving)
!-------------------------------------------------------------------
 if (paw_opt==0) then                  ! Enl is E(Kleinman-Bylander)
  do ia=1,nincat
   do ilmn=1,nlmn
    iln=indlmn(5,ilmn)
    gxfac(:,ilmn,ia)=enl(iln,itypat)*gx(:,ilmn,ia)
   end do
  end do
  gxfac(:,1:nlmn,1:nincat)=wt*gxfac(:,1:nlmn,1:nincat)
 end if

!Accumulate gxfac related to nonlocal operator (PAW)
!-------------------------------------------------------------------
 if (paw_opt==1.or.paw_opt==4) then        ! Enl is psp strenght Dij
  gxfac(:,1:nlmn,1:nincat)=zero
  do ia=1,nincat
   index_enl=atindx1(iatm+ia)
   do jlmn=1,nlmn
    j0lmn=jlmn*(jlmn-1)/2
    jjlmn=j0lmn+jlmn
    jl=indlmn(1,jlmn)+4
    enlr=enl(jjlmn,index_enl)
    gxjr=gx(1,jlmn,ia);gxji=gx(2,jlmn,ia)
    gxfac(1,jlmn,ia)=gxfac(1,jlmn,ia)+enlr*gxjr
    gxfac(2,jlmn,ia)=gxfac(2,jlmn,ia)+enlr*gxji
    do ilmn=1,jlmn-1
     ijlmn=j0lmn+ilmn
     jlmil=jl-indlmn(1,ilmn)
     enlr=enl(ijlmn,index_enl)*ilr(jlmil)
     enli=enl(ijlmn,index_enl)*ili(jlmil)
     gxir=gx(1,ilmn,ia);gxii=gx(2,ilmn,ia)
     gxfac(1,ilmn,ia)=gxfac(1,ilmn,ia)+(enlr*gxjr-enli*gxji)
     gxfac(2,ilmn,ia)=gxfac(2,ilmn,ia)+(enlr*gxji+enli*gxjr)
     gxfac(1,jlmn,ia)=gxfac(1,jlmn,ia)+(enlr*gxir+enli*gxii)
     gxfac(2,jlmn,ia)=gxfac(2,jlmn,ia)+(enlr*gxii-enli*gxir)
    end do
   end do
  end do
  gxfac(:,1:nlmn,1:nincat)=wt*gxfac(:,1:nlmn,1:nincat)
 end if

!Accumulate gxfac related to overlap (Sij) (PAW)
!-------------------------------------------------------------------
 if (paw_opt>2) then
  gxfac_sij(:,1:nlmn,1:nincat)=zero
  do ia=1,nincat
   do jlmn=1,nlmn
    j0lmn=jlmn*(jlmn-1)/2
    jjlmn=j0lmn+jlmn
    jl=indlmn(1,jlmn)+4
    jlm=indlmn(4,jlmn)
    sijr=sij(jjlmn)
    gxjr=gx(1,jlmn,ia);gxji=gx(2,jlmn,ia)
    gxfac_sij(1,jlmn,ia)=gxfac_sij(1,jlmn,ia)+sijr*gxjr
    gxfac_sij(2,jlmn,ia)=gxfac_sij(2,jlmn,ia)+sijr*gxji
    do ilmn=1,jlmn-1
     ilm=indlmn(4,ilmn)
     if (ilm==jlm) then
      ijlmn=j0lmn+ilmn
      jlmil=jl-indlmn(1,ilmn)
      sijr=sij(ijlmn)*ilr(jlmil)
      siji=sij(ijlmn)*ili(jlmil)
      gxir=gx(1,ilmn,ia);gxii=gx(2,ilmn,ia)
      gxfac_sij(1,ilmn,ia)=gxfac_sij(1,ilmn,ia)+(sijr*gxjr-siji*gxji)
      gxfac_sij(2,ilmn,ia)=gxfac_sij(2,ilmn,ia)+(sijr*gxji+siji*gxjr)
      gxfac_sij(1,jlmn,ia)=gxfac_sij(1,jlmn,ia)+(sijr*gxir+siji*gxii)
      gxfac_sij(2,jlmn,ia)=gxfac_sij(2,jlmn,ia)+(sijr*gxii-siji*gxir)
     end if
    end do
   end do
  end do
  gxfac_sij(:,1:nlmn,1:nincat)=wt*gxfac_sij(:,1:nlmn,1:nincat)
 end if

! Accumulate dgxdtfac related to nonlocal operator (Norm-conserving)
!-------------------------------------------------------------------
 if (optder==1.and.paw_opt==0) then    ! Enl is E(Kleinman-Bylander)
  do ia=1,nincat
   do ilmn=1,nlmn
    iln=indlmn(5,ilmn)
    do mu=1,ndgxdtfac
     dgxdtfac(:,mu,ilmn,ia)=enl(iln,itypat)*dgxdt(:,mu,ilmn,ia)
    end do
   end do
  end do
  dgxdtfac(:,:,1:nlmn,1:nincat)=wt*dgxdtfac(:,:,1:nlmn,1:nincat)
 end if

! Accumulate dgxdtfac related to nonlocal operator (PAW)
!-------------------------------------------------------------------
 if (optder==1.and.(paw_opt==1.or.paw_opt==4)) then  ! Enl is psp strenght Dij
  allocate(gxfji(ndgxdtfac),gxfjr(ndgxdtfac))
  dgxdtfac(:,:,1:nlmn,1:nincat)=zero
  do ia=1,nincat
   index_enl=atindx1(iatm+ia)
   do jlmn=1,nlmn
    j0lmn=jlmn*(jlmn-1)/2
    jjlmn=j0lmn+jlmn
    jl=indlmn(1,jlmn)+4
    enlr=enl(jjlmn,index_enl)
    do mu=1,ndgxdtfac
     gxfjr(mu)=dgxdt(1,mu,jlmn,ia);gxfji(mu)=dgxdt(2,mu,jlmn,ia)
     dgxdtfac(1,mu,jlmn,ia)=dgxdtfac(1,mu,jlmn,ia)+enlr*gxfjr(mu)
     dgxdtfac(2,mu,jlmn,ia)=dgxdtfac(2,mu,jlmn,ia)+enlr*gxfji(mu)
    end do
    do ilmn=1,jlmn-1
     ijlmn=j0lmn+ilmn
     jlmil=jl-indlmn(1,ilmn)
     enlr=enl(ijlmn,index_enl)*ilr(jlmil)
     enli=enl(ijlmn,index_enl)*ili(jlmil)
     do mu=1,ndgxdtfac
      gxfir=dgxdt(1,mu,ilmn,ia);gxfii=dgxdt(2,mu,ilmn,ia)
      dgxdtfac(1,mu,ilmn,ia)=dgxdtfac(1,mu,ilmn,ia)+(enlr*gxfjr(mu)-enli*gxfji(mu))
      dgxdtfac(2,mu,ilmn,ia)=dgxdtfac(2,mu,ilmn,ia)+(enlr*gxfji(mu)+enli*gxfjr(mu))
      dgxdtfac(1,mu,jlmn,ia)=dgxdtfac(1,mu,jlmn,ia)+(enlr*gxfir+enli*gxfii)
      dgxdtfac(2,mu,jlmn,ia)=dgxdtfac(2,mu,jlmn,ia)+(enlr*gxfii-enli*gxfir)
     end do
    end do
   end do
  end do
  deallocate(gxfji,gxfjr)
  dgxdtfac(:,:,1:nlmn,1:nincat)=wt*dgxdtfac(:,:,1:nlmn,1:nincat)
  end if

end subroutine opernlc_ylm
!!***
