!{\src2tex{textfont=tt}}
!!****f* ABINIT/opernld_ylm
!! NAME
!! opernld_ylm
!!
!! FUNCTION
!! * Operate with the non-local part of the hamiltonian,
!!   in order to get contributions to energy/forces/stress/dyn.matrix/elst tens.
!!   from projected scalars
!! * Operate with the non-local projectors and the overlap matrix Sij
!!   in order to get contributions to <c|S|c>
!!   from projected scalars
!! * Operate with the non-local projectors
!!   in order to get contributions to rhoij PAW quantities
!!   from projected scalars
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.
!!
!! INPUTS
!!  atindx1(natom)=index table for atoms (gives the absolute index of
!!                 an atom from its rank in a block of atoms)
!!  choice=chooses possible output
!!  d2gxdt(2,nd2gxdt,nlmn,nincat)=2nd gradients of projected scalars
!!  dgxdt(2,ndgxdt,nlmn,nincat)=gradients of projected scalars
!!  dgxdtfac(2,ndgxdtfac,nlmn,nincat)=gradients of reduced projected scalars
!!  gx(2,nlmn,nincat)= projected scalars
!!  gxfac(2,nlmn,nincat)= reduced projected scalars related to Vnl (NL operator)
!!  gxfac_sij(2,nlmn,nincat*(paw_opt/3))= reduced projected scalars related to Sij (overlap)
!!  ia3=gives the absolute number of the first atom in the subset presently treated
!!  ia5=gives the rank in the type of the first atom of the subset presently treated
!!  iatm=absolute rank of first atom of the current block of atoms
!!  indlmn(6,nlmn)= array giving l,m,n,ln,lm,s for i=lmn
!!  lmnmax=max. number of (l,m,n) numbers over all types of atom
!!  natom=number of atoms in cell
!!  nd2gxdt=second dimension of d2gxdt
!!  ndgxdt=second dimension of dgxdt
!!  ndgxdtfac=second dimension of dgxdtfac
!!  nincat=number of atoms in the subset here treated
!!  nlmn=number of (l,m,n) numbers for current type of atom
!!  nnlout=dimension of enlout
!!  paw_opt= define the nonlocal operator concerned with:
!!           paw_opt=0 : Norm-conserving Vnl (use of Kleinman-Bylander ener.)
!!           paw_opt=1 : PAW nonlocal part of H (use of Dij coeffs)
!!           paw_opt=2 : PAW augmentation occuppancies (rhoij)
!!           paw_opt=3 : PAW overlap matrix (Sij)
!!           paw_opt=4 : both PAW nonlocal part of H (Dij) and overlap matrix (Sij)
!!  rhoijshift= shift used to access into rhoij array (pawopt=2 only)
!!  ucvol=unit cell volume (bohr^3)
!!  wt=normalization factor (wt=four_pi**2/ucvol)
!!
!! OUTPUT
!!  (see side effects)
!!
!! SIDE EFFECTS
!! --If (paw_opt==0, 1)
!!    enlout(nnlout)= contribution to the non-local part of the following properties:
!!      if choice=1 : enlout(1)               -> the energy
!!      if choice=2 : enlout(1:3*natom)       -> the forces
!!      if choice=3 : enlout(1:6)             -> the stresses
!!      if choice=23: enlout(1:6+3*natom)     -> the forces and the stresses
!!      if choice=4 : enlout(1:6*natom)       -> the frozen wf part of dyn. mat.
!!      if choice=6 : enlout(1:6*(3*natom+6)) -> the frozen wf part of elastic tensor
!!    if (choice==3.or.choice==6.or.choice==23)
!!      enlk=contribution to the non-local part of the energy
!!    if (choice==6)
!!      fnlk(3*natom)=contribution to the non-local part of the forces
!!      strnlk(6)=contribution to the non-local part of the stresses
!! --If (paw_opt==2)
!!    enlout(nnlout)= contribution to rhoij PAW quantities (and derivatives)
!!                    related the following properties:
!!      if choice=1 : enlout(:)   -> energy
!!      if choice=2 : enlout(:)   -> energy + forces
!!      if choice=3 : enlout(:)   -> energy + stresses
!!      if choice=23: enlout(:)   -> energy + stresses + forces
!! --If (paw_opt==3 or 4)
!!    if choice=1 : enlout(nnlout)= contribution to <c|S|c>  (nnlout=1)
!!
!! NOTES
!! Operate for one type of atom, and within this given type of atom,
!! for a subset of at most nincat atoms.
!!
!! PARENTS
!!      nonlop_ylm
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine opernld_ylm(atindx1,choice,d2gxdt,dgxdt,dgxdtfac,enlk,enlout,fnlk,&
&                       gx,gxfac,gxfac_sij,ia3,ia5,indlmn,lmnmax,natom,nd2gxdt,&
&                       ndgxdt,ndgxdtfac,nincat,nlmn,nnlout,paw_opt,rhoijshift,&
&                       strnlk,ucvol,wt)

 use defs_basis

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: choice,ia3,ia5,lmnmax,natom,nd2gxdt,ndgxdt,ndgxdtfac
 integer,intent(in) :: nincat,nlmn,nnlout,paw_opt,rhoijshift
 real(dp),intent(in) :: ucvol,wt
 real(dp),intent(out) :: enlk
!arrays
 integer,intent(in) :: atindx1(natom),indlmn(6,nlmn)
 real(dp),intent(in) :: d2gxdt(2,nd2gxdt,nlmn,nincat)
 real(dp),intent(in) :: dgxdt(2,ndgxdt,nlmn,nincat)
 real(dp),intent(in) :: dgxdtfac(2,ndgxdtfac,nlmn,nincat),gx(2,nlmn,nincat)
 real(dp),intent(in) :: gxfac(2,nlmn,nincat)
 real(dp),intent(in) :: gxfac_sij(2,nlmn,nincat*(paw_opt/3))
 real(dp),intent(inout) :: enlout(nnlout),fnlk(3*natom),strnlk(6)

!Local variables-------------------------------
!scalars
 integer :: dimrhoij,ia,iashift,ilmn,ishift,j0lmn,jl,jlmil,jlmn,klmn,mu,mua,mub
 integer :: mushift,nu,nushift,rhoijsize
 real(dp) :: enl1,enl2,enl3,enl4,enl5,enl6,facti,factr,gxfacim,gxfacre,gxii
 real(dp) :: gxir,gxji,gxjr
!arrays
 integer,parameter :: alpha(6)=(/1,2,3,3,3,2/),beta(6)=(/1,2,3,2,1,1/)
 real(dp),parameter :: ili(7)=(/1.d0,0.d0,-1.d0,0.d0,1.d0,0.d0,-1.d0/)
 real(dp),parameter :: ilr(7)=(/0.d0,-1.d0,0.d0,1.d0,0.d0,-1.d0,0.d0/)

! *************************************************************************

 if (paw_opt==0.or.paw_opt==1) then

! ============== Accumulate the non-local energy ===============
  if (choice==1) then
   do ia=1,nincat
    do ilmn=1,nlmn
     enlout(1)=enlout(1)+(gxfac(1,ilmn,ia)*gx(1,ilmn,ia)+ &
&                         gxfac(2,ilmn,ia)*gx(2,ilmn,ia))
    end do
   end do
  end if

! ============ Accumulate the forces contributions =============
  if (choice==2.or.choice==23) then
   ishift=0;if (choice==23) ishift=6
   do ia=1,nincat
    iashift=3*(ia+ia3-2)+ishift
    do ilmn=1,nlmn
     do mu=1,3
      enlout(mu+iashift)=enlout(mu+iashift)+2.0d0*ucvol* &
&                       (gxfac(1,ilmn,ia)*dgxdt(1,mu+ishift,ilmn,ia)&
&                       +gxfac(2,ilmn,ia)*dgxdt(2,mu+ishift,ilmn,ia))
     end do
    end do
   end do
  end if

! ======== Accumulate the stress tensor contributions ==========
  if (choice==3.or.choice==23) then
   enl1=zero ; enl2=zero ; enl3=zero ; enl4=zero ; enl5=zero ; enl6=zero
   do ia=1,nincat
    do ilmn=1,nlmn
     gxfacre=gxfac(1,ilmn,ia) ; gxfacim=gxfac(2,ilmn,ia)
     enlk=enlk+(gxfacre*gx(1,ilmn,ia)+gxfacim*gx(2,ilmn,ia))
     enl1=enl1+two*(gxfacre*dgxdt(1,1,ilmn,ia)+gxfacim*dgxdt(2,1,ilmn,ia))
     enl2=enl2+two*(gxfacre*dgxdt(1,2,ilmn,ia)+gxfacim*dgxdt(2,2,ilmn,ia))
     enl3=enl3+two*(gxfacre*dgxdt(1,3,ilmn,ia)+gxfacim*dgxdt(2,3,ilmn,ia))
     enl4=enl4+two*(gxfacre*dgxdt(1,4,ilmn,ia)+gxfacim*dgxdt(2,4,ilmn,ia))
     enl5=enl5+two*(gxfacre*dgxdt(1,5,ilmn,ia)+gxfacim*dgxdt(2,5,ilmn,ia))
     enl6=enl6+two*(gxfacre*dgxdt(1,6,ilmn,ia)+gxfacim*dgxdt(2,6,ilmn,ia))
    end do
   end do
   enlout(1)=enlout(1)+enl1
   enlout(2)=enlout(2)+enl2
   enlout(3)=enlout(3)+enl3
   enlout(4)=enlout(4)+enl4
   enlout(5)=enlout(5)+enl5
   enlout(6)=enlout(6)+enl6
  end if

! ====== Accumulate the dynamical matrix contributions =========
  if (choice==4) then
   do ia=1,nincat
    iashift=6*(ia+ia3-2)
    do ilmn=1,nlmn
     do mu=1,6
      mua=alpha(mu);mub=beta(mu)
      enlout(mu+iashift)=enlout(mu+iashift)+2.0d0*ucvol*&
&               (gxfac(1,ilmn,ia)*d2gxdt(1,mu,ilmn,ia)&
&               +gxfac(2,ilmn,ia)*d2gxdt(2,mu,ilmn,ia)&
&               +dgxdtfac(1,mub,ilmn,ia)*dgxdt(1,mua,ilmn,ia)&
&               +dgxdtfac(2,mub,ilmn,ia)*dgxdt(2,mua,ilmn,ia))
     end do
    end do
   end do
  end if

! ======= Accumulate the elastic tensor contributions ==========
  if (choice==6) then
   do ia=1,nincat
    iashift=3*(ia+ia3-2)
    do ilmn=1,nlmn
     enlk=enlk+(gxfac(1,ilmn,ia)*gx(1,ilmn,ia)+ &
&               gxfac(2,ilmn,ia)*gx(2,ilmn,ia))
     do mu=1,3
      fnlk(mu+iashift)=fnlk(mu+iashift)+2.0d0* &
&                       (gxfac(1,ilmn,ia)*dgxdt(1,6+mu,ilmn,ia)&
&                       +gxfac(2,ilmn,ia)*dgxdt(2,6+mu,ilmn,ia))
     end do
     do mu=1,6
      strnlk(mu)=strnlk(mu)+2.d0* &
&                (gxfac(1,ilmn,ia)*dgxdt(1,mu,ilmn,ia)&
&                +gxfac(2,ilmn,ia)*dgxdt(2,mu,ilmn,ia))
     end do
     do mub=1,6
      mushift=6*(mub-1);nushift=(3*natom+6)*(mub-1)
      do mua=1,6
       mu=mushift+mua;nu=nushift+mua
       enlout(nu)=enlout(nu)+2.d0* &
&                (gxfac(1,ilmn,ia)*d2gxdt(1,mu,ilmn,ia)&
&                +gxfac(2,ilmn,ia)*d2gxdt(2,mu,ilmn,ia)&
&                +dgxdtfac(1,mua,ilmn,ia)*dgxdt(1,mub,ilmn,ia)&
&                +dgxdtfac(2,mua,ilmn,ia)*dgxdt(2,mub,ilmn,ia))
      end do
      mushift=36+3*(mub-1);nushift=6+iashift+(3*natom+6)*(mub-1)
      do mua=1,3
       mu=mushift+mua;nu=nushift+mua
       enlout(nu)=enlout(nu)+2.d0* &
&                (gxfac(1,ilmn,ia)*d2gxdt(1,mu,ilmn,ia)&
&                +gxfac(2,ilmn,ia)*d2gxdt(2,mu,ilmn,ia)&
&                +dgxdtfac(1,mub,ilmn,ia)*dgxdt(1,6+mua,ilmn,ia)&
&                +dgxdtfac(2,mub,ilmn,ia)*dgxdt(2,6+mua,ilmn,ia))
      end do
     end do
    end do
   end do
  end if
 end if


! ========= Store contributions to rhoij PAW quantities and derivatives ============
 if (paw_opt==2) then
  dimrhoij=1
  if (choice==2.or.choice==23) dimrhoij=dimrhoij+3
  if (choice==3.or.choice==23) dimrhoij=dimrhoij+6
  rhoijsize=dimrhoij*nlmn*(nlmn+1)/2
  do ia=1,nincat
   iashift=1+rhoijshift+rhoijsize*(ia5+ia-2)
   do jlmn=1,nlmn
    j0lmn=jlmn*(jlmn-1)/2
    jl=indlmn(1,jlmn)+4
    do ilmn=1,jlmn
     klmn=j0lmn+ilmn
     jlmil=jl-indlmn(1,ilmn)
     factr=ilr(jlmil);facti=ili(jlmil)
     gxjr= gx(1,jlmn,ia)*factr-gx(2,jlmn,ia)*facti
     gxji= gx(1,jlmn,ia)*facti+gx(2,jlmn,ia)*factr
     if (dimrhoij>1) then
      gxir= gx(1,ilmn,ia)*factr+gx(2,ilmn,ia)*facti
      gxii=-gx(1,ilmn,ia)*facti+gx(2,ilmn,ia)*factr
     end if
!    --- Contribution to rhoij
     enlout(iashift)=wt*(gx(1,ilmn,ia)*gxjr+gx(2,ilmn,ia)*gxji)
!    --- Contribution to derivative of rhoij wrt atm pos./strains/...
     if (dimrhoij>1) then
      do mu=1,dimrhoij-1
       enlout(iashift+mu)=wt* &
&                    (dgxdt(1,mu,ilmn,ia)*gxjr+dgxdt(2,mu,ilmn,ia)*gxji &
&                    +dgxdt(1,mu,jlmn,ia)*gxir+dgxdt(2,mu,jlmn,ia)*gxii)
      end do
     end if
     iashift=iashift+dimrhoij
    end do
   end do
  end do
 end if


! ============== Accumulate contribution to <c|S|c> ===============
 if (paw_opt>2) then
  do ia=1,nincat
   do ilmn=1,nlmn
    enlout(1)=enlout(1)+(gxfac_sij(1,ilmn,ia)*gx(1,ilmn,ia)+ &
&                        gxfac_sij(2,ilmn,ia)*gx(2,ilmn,ia))
   end do
  end do
 end if

end subroutine opernld_ylm
!!***
