!{\src2tex{textfont=tt}}
!!****f* ABINIT/nhatgrid
!! NAME
!! nhatgrid
!!
!! FUNCTION
!! Determine parts of the rectangular (fine) grid that are contained
!! inside spheres around atoms (used to compute n_hat density).
!! Compute also g(r)*Ylm(r) (and -if requested- derivatives) on this grid (g=radial shape function).
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.
!!
!! INPUTS
!!  atindx1(natom)=index table for atoms, inverse of atindx (see scfcv.f)
!!  gmet(3,3)=reciprocal space metric tensor in bohr**-2
!!  natom=number of atoms in cell.
!!  nattyp(ntypat)= # atoms of each type.
!!  nfft=(effective) number of FFT grid points (for this processor)
!!  ngfft(18)=contain all needed information about 3D FFT, see ~abinit/doc/input_variables/vargs.htm#ngfft
!!  ntypat=number of types of atoms in unit cell
!!  optder= 0=only g(r)*Ylm(r) is computed, 1=g(r)*Ylm(r) and derivatives are computed
!!  optrad= 1 if vectors (r-r_atom) on the fine grid around atoms have to be stored
!!  pawfgrtab(natom) <type(pawfgrtab_type)>=atomic data given on fine rectangular grid
!!  pawrad(ntypat) <type(pawrad_type)>=paw radial mesh and related data
!!  pawtab(ntypat) <type(pawtab_type)>=paw tabulated starting data
!!  rprimd(3,3)=dimensional primitive translations in real space (bohr)
!!  typat(natom)=type (integer) for each atom
!!  ucvol=unit cell volume in bohr**3
!!  xred(3,natom)=reduced dimensionless atomic coordinates
!!
!! OUTPUT
!!  pawfgrtab(natom)%gylm(nfgd,l_size**2)= g(r)*ylm(r)
!!                                          on the fine rect. grid around each atom
!!  pawfgrtab(natom)%ifftsph(nfgd)=FFT index (fine grid) of a point
!!                                  in the PAW sphere around each atom
!!  pawfgrtab(natom)%nfgd= number of (fine grid) FFT points
!!                          in the paw spheres around atoms
!!  if (optder==1)
!!    pawfgrtab(natom)%gylmgr(3,nfgd,l_size**2)= derivatives of g(r)*ylm(r)
!!                                                wrt cart. coordinates
!!  if (optrad==1)
!!    pawfgrtab(natom)%rfgd(3,nfgd)= coordinates of r-r_atom
!!                                   on the fine rect. grid around each atom
!!
!! PARENTS
!!      scfcv
!!
!! CHILDREN
!!      deducer0,initylmr,leave_new,nderiv_gen,sort_dp,spline,splint,timab,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine nhatgrid(atindx1,gmet,mpi_enreg,natom,nattyp,nfft,ngfft,ntypat,&
 &optder,optrad,pawfgrtab,pawrad,pawtab,rprimd,typat,ucvol,xred)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_00basis
 use interfaces_01managempi
 use interfaces_01util
 use interfaces_03paw, except_this_one => nhatgrid
#endif
!End of the abilint section

 implicit none

!Arguments ---------------------------------------------
!scalars
 integer,intent(in) :: natom,nfft,ntypat,optder,optrad
 real(dp),intent(in) :: ucvol
 type(MPI_type),intent(in) :: mpi_enreg
!arrays
 integer,intent(in) :: atindx1(natom),nattyp(ntypat),ngfft(18),typat(natom)
 real(dp),intent(in) :: gmet(3,3),rprimd(3,3),xred(3,natom)
 type(pawfgrtab_type),intent(inout) :: pawfgrtab(natom)
 type(pawrad_type),intent(in) :: pawrad(ntypat)
 type(pawtab_type),intent(in) :: pawtab(ntypat)

!Local variables ------------------------------
!scalars
 integer,parameter :: ishift=5
 integer :: i1,i2,i3,iat,iatm,iatom,ic,ifft_local,ir,itypat,ix,iy,iz,izero,l0
 integer :: l_size,lambda,ll,me_fft,meshsz,mm,n1,n1a,n1b,n2,n2a,n2b,n3,n3a,n3b
 integer :: ncmax,nd3,nfftot,nfgd,normchoice,option,shape_type
 real(dp),parameter :: delta=0.99_dp,tol16=tol8**2
 real(dp) :: arg,argl,dg_tmp,difx,dify,difz,dshpfunc1,dshpfunc2,dshpfunc3,g_tmp
 real(dp) :: pi_over_rshp,r2,r2shp,r3shp,rr1,rr2,rr3,rshp,rx,ry,rz,s1,s2,s3
 real(dp) :: shapefunc1,shapefunc2,sigma,yp1,ypn
 character(len=500) :: message
!arrays
 integer,allocatable :: ifftsph_sort(:),ifftsph_tmp(:),isort(:)
 real(dp) :: ss(4),tsec(2)
 real(dp),allocatable :: aa(:,:),d2shpfunc(:,:),d3shpfunc(:,:),dgfact(:,:)
 real(dp),allocatable :: dshpfunc(:,:),dshpfuncnum(:,:),gfact(:,:),gnorm(:)
 real(dp),allocatable :: qq(:,:),rad(:),rnrm(:),rnrm_inv(:),rnrm_sort(:)
 real(dp),allocatable :: rr(:,:),rr_sort(:,:),shapefuncnum(:,:),shpfunc(:,:)
 real(dp),allocatable :: work(:),ylmr(:,:),ylmrgr(:,:,:)
!no_abirules
!Statement functions -----------------------------------
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 integer :: ifromr
#endif
!End of the abilint section

 shapefunc1(arg)= exp(-(arg/sigma)**lambda)
 shapefunc2(arg)= (sin(pi_over_rshp*arg)/(pi_over_rshp*arg))**2
 dshpfunc1(arg)=-lambda/sigma*(arg/sigma)**(lambda-1)*exp(-(arg/sigma)**lambda)
 dshpfunc2(arg)=two*pi_over_rshp*sin(pi_over_rshp*arg)/(pi_over_rshp*arg)**3&
&              *(pi_over_rshp*arg*cos(pi_over_rshp*arg)-sin(pi_over_rshp*arg))

! *************************************************************************

 call timab(559,1,tsec)
 n1=ngfft(1);n2=ngfft(2);n3=ngfft(3);nd3=n3/mpi_enreg%nproc_fft
 nfftot=n1*n2*n3
 me_fft=mpi_enreg%me_fft
!Loop over types of atom
!-------------------------------------------
 iatm=0
 do itypat=1,ntypat

  shape_type=pawtab(itypat)%shape_type
  l_size=pawtab(itypat)%l_size
  rshp=pawtab(itypat)%rshp;r3shp=0.75d0*rshp
  r2shp=1.0000001d0*rshp**2
  meshsz=ifromr(pawrad(itypat),rshp)
  ncmax=1+int(1.2d0*nfftot*four_pi/(3.d0*ucvol)*rshp**3)
  allocate(ifftsph_tmp(ncmax),rnrm(ncmax),rr(3,ncmax))
  allocate(gnorm(l_size));gnorm(:)=pawtab(itypat)%gnorm(:)
  rr1=sqrt(r2shp*gmet(1,1))
  rr2=sqrt(r2shp*gmet(2,2))
  rr3=sqrt(r2shp*gmet(3,3))
  if (shape_type==1) then
   sigma=pawtab(itypat)%shape_sigma;lambda=pawtab(itypat)%shape_lambda
  end if
  if (shape_type==2) pi_over_rshp=pi/rshp

! In case of a "numeric" shape function (shape_type=-1),
! has to store some shape function derivatives
  if (shape_type==-1) then
   allocate(shpfunc(meshsz,l_size),d2shpfunc(meshsz,l_size),rad(meshsz),work(meshsz))
   rad(1:meshsz)=pawrad(itypat)%rad(1:meshsz)
   do ll=1,l_size
    shpfunc(1:meshsz,ll)=pawtab(itypat)%shapefunc(1:meshsz,ll)
    if (pawrad(itypat)%radfact(1)>zero) then
     yp1=1.d0/12.d0/pawrad(itypat)%stepint/pawrad(itypat)%radfact(1) &
&     *(-25.d0*shpfunc(1,ll)+48.d0*shpfunc(2,ll)-36.d0*shpfunc(3,ll) &
&       +16.d0*shpfunc(4,ll)- 3.d0*shpfunc(5,ll))
    else
     yp1=(shpfunc(2,ll)-shpfunc(1,ll))/(rad(2)-rad(1))
    end if
    ypn=1.d0/12.d0/pawrad(itypat)%stepint &
&    *(  3.d0*shpfunc(meshsz-4,ll)-16.d0*shpfunc(meshsz-3,ll) &
&      +36.d0*shpfunc(meshsz-2,ll)-48.d0*shpfunc(meshsz-1,ll) &
&      +25.d0*shpfunc(meshsz  ,ll)) / pawrad(itypat)%radfact(meshsz)
    call spline(rad,shpfunc(:,ll),meshsz,yp1,ypn,d2shpfunc(:,ll),work)
   end do
   if(optder==1) then
    allocate(dshpfunc(pawrad(itypat)%mesh_size,l_size),d3shpfunc(meshsz,l_size))
    do ll=1,l_size
     call nderiv_gen(dshpfunc(:,ll),pawtab(itypat)%shapefunc(:,ll),1,pawrad(itypat))
     if (pawrad(itypat)%radfact(1)>zero) then
      yp1=1.d0/12.d0/pawrad(itypat)%stepint/pawrad(itypat)%radfact(1) &
&      *(-25.d0*dshpfunc(1,ll)+48.d0*dshpfunc(2,ll)-36.d0*dshpfunc(3,ll)&
&        +16.d0*dshpfunc(4,ll)-3.d0*dshpfunc(5,ll))
     else
      yp1=(dshpfunc(2,ll)-dshpfunc(1,ll))/(rad(2)-rad(1))
     end if
     ypn=1.d0/12.d0/pawrad(itypat)%stepint &
&     *(  3.d0*dshpfunc(meshsz-4,ll)-16.d0*dshpfunc(meshsz-3,ll) &
&       +36.d0*dshpfunc(meshsz-2,ll)-48.d0*dshpfunc(meshsz-1,ll) &
&       +25.d0*dshpfunc(meshsz  ,ll)) / pawrad(itypat)%radfact(meshsz)
     call spline(rad,dshpfunc(1:meshsz,ll),meshsz,yp1,ypn,d3shpfunc(:,ll),work)
    end do
   end if
    deallocate(work)
  end if

!Loop over atoms
!-------------------------------------------
  do iat=1,nattyp(itypat)
   iatm=iatm+1;iatom=atindx1(iatm)
   izero=-1

!  Define a "box" around each atom
   n1a=int((xred(1,iatom)-rr1+ishift)*n1+delta)-ishift*n1
   n1b=int((xred(1,iatom)+rr1+ishift)*n1      )-ishift*n1
   n2a=int((xred(2,iatom)-rr2+ishift)*n2+delta)-ishift*n2
   n2b=int((xred(2,iatom)+rr2+ishift)*n2      )-ishift*n2
   n3a=int((xred(3,iatom)-rr3+ishift)*n3+delta)-ishift*n3
   n3b=int((xred(3,iatom)+rr3+ishift)*n3      )-ishift*n3

!------------------------------------------------------------------
!A-Identify the fft indexes of the rectangular grid around the atom
!------------------------------------------------------------------

   nfgd=0
   do i3=n3a,n3b
    iz=mod(i3+ishift*n3,n3)
    difz=dble(i3)/dble(n3)-xred(3,iatom)
    do i2=n2a,n2b
     iy=mod(i2+ishift*n2,n2)
     dify=dble(i2)/dble(n2)-xred(2,iatom)
     do i1=n1a,n1b
      ix=mod(i1+ishift*n1,n1)
      difx=dble(i1)/dble(n1)-xred(1,iatom)
      rx=difx*rprimd(1,1)+dify*rprimd(1,2)+difz*rprimd(1,3)
      ry=difx*rprimd(2,1)+dify*rprimd(2,2)+difz*rprimd(2,3)
      rz=difx*rprimd(3,1)+dify*rprimd(3,2)+difz*rprimd(3,3)
      r2=rx**2+ry**2+rz**2

      if (r2 <= r2shp) then
       if(iz/nd3==me_fft) then !MPIWF
        ifft_local=1+ix+n1*(iy+n2*modulo(iz,nd3))
        nfgd=nfgd+1
        if (nfgd>ncmax) then
         write(message, '(a,a,a,a,i4,a,a,i4,a)' ) ch10,&
&          ' nhatgrid : BUG -',ch10,&
&          '   Number of fft points around atom ',iatom,ch10,&
&          '   exceeds max. allowed (',ncmax,') !'
         call wrtout(06,message,'COLL')
         call leave_new('PERS')
        end if

        rr(1,nfgd)=rx
        rr(2,nfgd)=ry
        rr(3,nfgd)=rz
        rnrm(nfgd)=dsqrt(r2);if (rnrm(nfgd)<=epsilon(one)) izero=nfgd
        ifftsph_tmp(nfgd)=ifft_local
       end if
      end if

     end do
    end do
   end do
!  In case of a "numeric" shape function (shape_type=-1):
!  Has to spline it onto selected radii
   if (shape_type==-1) then
    allocate(ifftsph_sort(nfgd),isort(nfgd),rnrm_sort(nfgd),rr_sort(3,nfgd))
    do ic=1,nfgd;isort(ic)=ic;end do
    rnrm_sort(1:nfgd)=rnrm(1:nfgd)
    rr_sort(:,1:nfgd)=rr(:,1:nfgd)
    ifftsph_sort(1:nfgd)=ifftsph_tmp(1:nfgd)
    call sort_dp(nfgd,rnrm_sort,isort,tol16)
    rnrm(1:nfgd)=rnrm_sort(1:nfgd)
    do ic=1,nfgd
     rr(:,ic)=rr_sort(:,isort(ic))
     ifftsph_tmp(ic)=ifftsph_sort(isort(ic))
    end do
    deallocate(isort,ifftsph_sort,rnrm_sort,rr_sort)
    if (izero>0) izero=1
    allocate(shapefuncnum(nfgd,l_size))
    do ll=1,l_size
     call splint(meshsz,rad,shpfunc(:,ll),d2shpfunc(:,ll),nfgd,rnrm,shapefuncnum(:,ll))
    end do
    if(optder==1) then
     allocate(dshpfuncnum(nfgd,l_size))
     do ll=1,l_size
      call splint(meshsz,rad,dshpfunc(1:meshsz,ll),d3shpfunc(:,ll),nfgd,rnrm,dshpfuncnum(:,ll))
     end do
    end if
   end if

!  Allocate arrays defining sphere (and related data) around current atom
   pawfgrtab(iatom)%nfgd=nfgd
   if (associated(pawfgrtab(iatom)%ifftsph)) deallocate(pawfgrtab(iatom)%ifftsph)
   allocate(pawfgrtab(iatom)%ifftsph(nfgd))
   pawfgrtab(iatom)%ifftsph(1:nfgd)=ifftsph_tmp(1:nfgd)

   if (associated(pawfgrtab(iatom)%gylm)) deallocate(pawfgrtab(iatom)%gylm)
   allocate(pawfgrtab(iatom)%gylm(nfgd,l_size*l_size))

   if (optder==1) then
    if (associated(pawfgrtab(iatom)%gylmgr)) deallocate(pawfgrtab(iatom)%gylmgr)
    allocate(pawfgrtab(iatom)%gylmgr(3,nfgd,l_size*l_size))
   end if

   if (optrad==1) then
    if (associated(pawfgrtab(iatom)%rfgd)) deallocate(pawfgrtab(iatom)%rfgd)
    allocate(pawfgrtab(iatom)%rfgd(3,nfgd))
    pawfgrtab(iatom)%rfgd(1:3,1:nfgd)=rr(1:3,1:nfgd)
   end if

!------------------------------------------------------------------
!B-Calculate g(r)*Ylm(r) for each r around the atom
!------------------------------------------------------------------

!  Ylm(r) calculation
   normchoice=1 ; option=1+2*optder
   allocate(ylmr(l_size**2,nfgd));if(optder==1) allocate(ylmrgr(3,l_size**2,nfgd))
   call initylmr(l_size,normchoice,nfgd,rnrm,option,rr,ylmr,ylmrgr)

   pawfgrtab(iatom)%gylm=zero;if(optder==1) pawfgrtab(iatom)%gylmgr=zero

!  Store temporary factors (depending on shape function type)
!  ==========================================================
   allocate(gfact(nfgd,0:l_size-1))
   if(optder==1) then
    allocate(dgfact(nfgd,0:l_size-1),rnrm_inv(nfgd))
    if (izero>0) rnrm_inv(izero)=zero
    do ic=1,nfgd
     if (ic/=izero) rnrm_inv(ic)=1.d0/rnrm(ic)
    end do
   end if
!  ----- type -1 -----
   if (shape_type==-1) then
    do ll=0,l_size-1
     gfact(1:nfgd,ll)=shapefuncnum(1:nfgd,ll+1)
    end do
    if(optder==1) then
     do ll=0,l_size-1
      dgfact(1:nfgd,ll)=dshpfuncnum(1:nfgd,ll+1)*rnrm_inv(1:nfgd)
     end do
    end if
   end if
!  ----- type 1 -----
   if (shape_type==1) then
    do ic=1,nfgd
     gfact(ic,0)=gnorm(1)*shapefunc1(rnrm(ic))
    end do
    do ll=1,l_size-1
     do ic=1,nfgd
      gfact(ic,ll)=gnorm(ll+1)*shapefunc1(rnrm(ic))*(rnrm(ic)**ll)
     end do
    end do
    if(optder==1) then
     do ic=1,nfgd
      dgfact(ic,0)=gnorm(1)*dshpfunc1(rnrm(ic))*rnrm_inv(ic)
     end do
     do ll=1,l_size-1
      do ic=1,nfgd
       dgfact(ic,ll)=gnorm(ll+1)*rnrm_inv(ic)*(dshpfunc1(rnrm(ic))*rnrm(ic)**ll&
&                   +shapefunc1(rnrm(ic))*ll*rnrm(ic)**(ll-1))
      end do
     end do
    end if
!  ----- type 2 -----
   else if (shape_type==2) then
    do ic=1,nfgd
     if (ic/=izero) gfact(ic,0)=gnorm(1)*shapefunc2(rnrm(ic))
    end do
    if (izero>0) gfact(izero,0)=gnorm(1)
    do ll=1,l_size-1
     do ic=1,nfgd
      if (ic/=izero) gfact(ic,ll)=gnorm(ll+1)*shapefunc2(rnrm(ic))*(rnrm(ic)**ll)
     end do
     if (izero>0) gfact(izero,ll)=zero
    end do
    if(optder==1) then
     do ic=1,nfgd
      if (ic/=izero) dgfact(ic,0)=gnorm(1)*dshpfunc2(rnrm(ic))*rnrm_inv(ic)
     end do
     if (izero>0) dgfact(izero,0)=zero
     do ll=1,l_size-1
      do ic=1,nfgd
       if (ic/=izero) dgfact(ic,ll)=gnorm(ll+1)*rnrm_inv(ic)*(dshpfunc2(rnrm(ic))*rnrm(ic)**ll&
&                   +shapefunc2(rnrm(ic))*ll*rnrm(ic)**(ll-1))
      end do
      if (izero>0) dgfact(izero,ll)=zero
     end do
    end if
   end if

!  Compute g(r)*Ylm(r)
!  ==========================================================
   do ll=0,l_size-1
    l0=ll**2+ll+1
    do mm=-ll,ll
     do ic=1,nfgd
      if (ic==izero.and.shape_type/=-1) cycle
      pawfgrtab(iatom)%gylm(ic,l0+mm)=gfact(ic,ll)*ylmr(l0+mm,ic)
     end do
    end do
   end do
   if (izero>0.and.shape_type/=-1) pawfgrtab(iatom)%gylm(izero,1)=ylmr(1,izero)*gnorm(1)

!  Compute d/dr{g(r)*Ylm(r)}
!  ==========================================================
   if(optder==1) then
    do ll=0,l_size-1
     l0=ll**2+ll+1
     do mm=-ll,ll
      do ic=1,nfgd
       if (ic==izero) cycle
       pawfgrtab(iatom)%gylmgr(:,ic,l0+mm)=gfact(ic,ll)*ylmrgr(:,l0+mm,ic)&
&                                         +dgfact(ic,ll)*rr(:,ic)*ylmr(l0+mm,ic)
      end do
     end do
    end do
    if (izero>0.and.l_size>1) then
     if (shape_type==-1) then
      ss(2)=pawtab(itypat)%shapefunc(2,2)/pawrad(itypat)%rad(2)
      ss(3)=pawtab(itypat)%shapefunc(3,2)/pawrad(itypat)%rad(3)
      ss(4)=pawtab(itypat)%shapefunc(4,2)/pawrad(itypat)%rad(4)
      call deducer0(ss,4,pawrad(itypat))
      g_tmp=ss(1)*gnorm(2)*sqrt(3.d0/four_pi)
      pawfgrtab(iatom)%gylmgr(2,izero,2)=g_tmp
      pawfgrtab(iatom)%gylmgr(3,izero,3)=g_tmp
      pawfgrtab(iatom)%gylmgr(1,izero,4)=g_tmp
     else
      g_tmp=gnorm(2)*sqrt(3.d0/four_pi)
      pawfgrtab(iatom)%gylmgr(2,izero,2)=g_tmp
      pawfgrtab(iatom)%gylmgr(3,izero,3)=g_tmp
      pawfgrtab(iatom)%gylmgr(1,izero,4)=g_tmp
     end if
    end if
   end if

   deallocate(gfact);if(optder==1) deallocate(dgfact,rnrm_inv)
   if (shape_type==-1) then
    deallocate(shapefuncnum)
    if (optder==1) deallocate(dshpfuncnum)
   end if
   if (shape_type==3) deallocate(aa,qq)

!End loops over types/atoms
!-------------------------------------------
   deallocate (ylmr);if (optder==1) deallocate(ylmrgr)
  end do
  deallocate(gnorm,ifftsph_tmp,rnrm,rr)
  if (shape_type==-1) then
   deallocate(shpfunc,d2shpfunc,rad)
   if (optder==1) deallocate(dshpfunc,d3shpfunc)
  end if
 end do
 call timab(559,2,tsec)

end subroutine nhatgrid

!!***
