!{\src2tex{textfont=tt}}
!!****f* ABINIT/pawgrnhat
!!
!! NAME
!! pawgrnhat
!!
!! FUNCTION
!! PAW: Compute GRadients of total energy du to NHAT density
!! -compensation charge density- and non-local rhoij quantities
!! (Also add an overlap contribution)
!! In particular, compute contribution to forces, stresses
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.
!!
!! INPUTS
!!  choice=select then type of gradients to compute.
!!         The meaning of "choice" has been choosen to be the same as
!!         in "nonlop" routine:
!!         choice=2 => gradient with respect to atomic position(s)
!!               =3 => a gradient with respect to strain(s)
!!               =23=> a gradient with respect to atm. pos. and strain(s)
!!  dimnhat=second dimension of array nhat
!!  natom=number of atoms in cell
!!  nfft=(effective) number of FFT grid points (for this processor)
!!  nhat(nfft,dimnhat)=compensation charge density on rectangular grid in real space
!!  nspden=number of spin-density components
!!  nsym=number of symmetries in space group
!!  ntypat=number of types of atoms
!!  paw_ij(natom*usepaw) <type(paw_ij_type)>=paw arrays given on (i,j) channels
!!  pawang <type(pawang_type)>=paw angular mesh and related data
!!  pawfgrtab(natom) <type(pawfgrtab_type)>=atomic data given on fine rectangular grid
!!  pawtab(ntypat) <type(pawtab_type)>=paw tabulated starting data
!!  rprimd(3,3)=dimensional primitive translations in real space (bohr)
!!  symrec(3,3,nsym)=symmetries in reciprocal space, reduced coordinates
!!  typat(natom)=type integer for each atom in cell
!!  vtrial(nfft,nspden)= total potential
!!
!! OUTPUT
!!  ==== if choice==2.or.choice==23 ====
!!   grhat(3,natom)=gradients of energy wrt atomic coordinates du to compensation charge
!!   grnl(3*natom) =gradients of energy wrt atomic coordinates du to rhoij quantities
!!  ==== if choice==3.or.choice==23 ====
!!   nhatstr(6)=gradients of energy wrt strains du to compensation charge
!!   nlstr(6)  =gradients of energy wrt strains du to rhoij quantities
!!
!! PARENTS
!!      etotfor,forstr
!!
!! CHILDREN
!!      metric,strconv,xcomm_init,xsum_mpi
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine pawgrnhat(choice,dimnhat,grhat,grnl,mpi_enreg,natom,nfft,ngfft,&
&                     nhat,nhatstr,nlstr,nspden,nsym,ntypat,paw_ij,pawang,&
&                     pawfgrtab,pawtab,rprimd,symrec,typat,vtrial)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01managempi
 use interfaces_02geometry
#else
 use defs_xfuncmpi
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: choice,dimnhat,natom,nfft,nspden,nsym,ntypat
 type(MPI_type),intent(inout) :: mpi_enreg
 type(pawang_type),intent(in) :: pawang
!arrays
 integer,intent(in) :: ngfft(18),symrec(3,3,nsym),typat(natom)
 real(dp),intent(in) :: nhat(nfft,dimnhat),rprimd(3,3),vtrial(nfft,nspden)
 real(dp),intent(out) :: grhat(3,natom),grnl(3*natom),nhatstr(6),nlstr(6)
 type(paw_ij_type),intent(in) :: paw_ij(natom)
 type(pawfgrtab_type),intent(in) :: pawfgrtab(natom)
 type(pawtab_type),intent(in) :: pawtab(ntypat)

!Local variables-------------------------------
!scalars
 integer :: iatom,ic,ier,ils,ilslm,irhoij,isel,ishift,ispden,itypat,jc,klm,klmn
 integer :: l_size_sq,lm0,lmax,lmin,lmn2_size,mm,mu,mua,mub,mushift,nfftot
 integer :: ngrad,ngrhat,ngrhoij,old_paral_level,spaceComm
 real(dp) :: dij_tmp,dlt_tmp,fact,grhat_tmp,nhat_diag,nlstr_diag,r2,ro,ro_d
 real(dp) :: sij_tmp,ucvol
!arrays
 integer,parameter :: alpha(6)=(/1,2,3,3,3,2/),beta(6)=(/1,2,3,2,1,1/)
 real(dp) :: gmet(3,3),gprimd(3,3),rmet(3,3)
 real(dp),allocatable :: grhat_red(:),grnl_red(:),prod(:,:)

! *************************************************************************

!DEBUG
!write(6,*)' pawgrnhat: enter '
!stop
!ENDDEBUG

!Compute different geometric tensor, as well as ucvol, from rprimd
 call metric(gmet,gprimd,-1,rmet,rprimd,ucvol)

!Initializations and allocations
 ngrad=0;ngrhat=0
 if (choice==2.or.choice==23) then
  ishift=0;if (choice==23) ishift=6
  ngrad=ngrad+3;ngrhat=ngrhat+3*natom
 end if
 if (choice==3.or.choice==23) then
  nlstr=zero;nhatstr=zero;nlstr_diag=zero
  ngrad=ngrad+6;ngrhat=ngrhat+6
 end if

 allocate(grhat_red(ngrhat),grnl_red(ngrhat))
 nfftot=ngfft(1)*ngfft(2)*ngfft(3)
 fact=ucvol/dble(nfftot)

!DEBUG
!write(6,*)' pawgrnhat: before iatom loop '
!stop
!ENDDEBUG

!Loops over atoms and spin components
 do iatom=1,natom
  itypat=typat(iatom)
  l_size_sq=(pawtab(itypat)%l_size)**2
  lmn2_size=paw_ij(iatom)%lmn2_size
  ngrhoij=paw_ij(iatom)%ngrhoij
  allocate(prod(ngrad,l_size_sq))
  grhat_red=zero;grnl_red=zero
  do ispden=1,nspden

!  ----- Compute temporary projected scalars
   prod=zero
!  ==== Contribution to forces ====
   if (choice==2.or.choice==23) then
!DEBUG
!write(6,*)pawfgrtab(iatom)%nfgd
!write(6,*)pawfgrtab(iatom)%ifftsph(:)
!ENDDEBUG
    do ilslm=1,l_size_sq
     do ic=1,pawfgrtab(iatom)%nfgd
      do mu=1,3
       jc=pawfgrtab(iatom)%ifftsph(ic)
       prod(mu+ishift,ilslm)=prod(mu+ishift,ilslm)-&
&      vtrial(jc,ispden)*pawfgrtab(iatom)%gylmgr(mu,ic,ilslm)
      end do
     end do
    end do
   end if
!  ==== Contribution to stresses ====
   if (choice==3.or.choice==23) then
    do ilslm=1,l_size_sq
     do ic=1,pawfgrtab(iatom)%nfgd
      do mu=1,6
       mua=alpha(mu);mub=beta(mu)
       prod(mu,ilslm)=prod(mu,ilslm) &
&                    +0.5d0*vtrial(pawfgrtab(iatom)%ifftsph(ic),ispden) &
&                    *(pawfgrtab(iatom)%gylmgr(mua,ic,ilslm)*pawfgrtab(iatom)%rfgd(mub,ic)&
&                     +pawfgrtab(iatom)%gylmgr(mub,ic,ilslm)*pawfgrtab(iatom)%rfgd(mua,ic))
      end do
     end do
    end do
   end if

!  ---- Compute gradients
 if(mpi_enreg%paral_compil_fft==1)then
  old_paral_level= mpi_enreg%paral_level
  mpi_enreg%paral_level=3
  call xcomm_init(mpi_enreg,spaceComm)
  if(mpi_enreg%mode_para=='b') spaceComm=mpi_enreg%comm_fft
  call xsum_mpi(prod,spaceComm,ier)
  mpi_enreg%paral_level=old_paral_level
 end if
  do klmn=1,lmn2_size
    dij_tmp=paw_ij(iatom)%dij(klmn,ispden)
    sij_tmp=pawtab(itypat)%sij(klmn)
    dlt_tmp=pawtab(itypat)%dltij(klmn)
    ro     =paw_ij(iatom)%grhoij(1,klmn,ispden)

!  ==== Compute gradient du to nhat ====
     if (abs(ro)>tol10) then
     klm =pawtab(itypat)%indklmn(1,klmn)
     lmin=pawtab(itypat)%indklmn(3,klmn)
     lmax=pawtab(itypat)%indklmn(4,klmn)
     ro_d=ro*dlt_tmp
     do ils=lmin,lmax,2
      lm0=ils**2+ils+1
      do mm=-ils,ils
       ilslm=lm0+mm;isel=pawang%gntselect(lm0+mm,klm)
       if (isel>0) then
        grhat_tmp=ro_d*pawtab(itypat)%qijl(ilslm,klmn)
        do mu=1,ngrad
         grhat_red(mu)=grhat_red(mu)+grhat_tmp*prod(mu,ilslm)
        end do
       end if
      end do
     end do
     if (choice==3.or.choice==23) nlstr_diag=nlstr_diag+ro_d*paw_ij(iatom)%dij(klmn,ispden)
    end if

!   ==== Compute gradient du to rhoij ====
    do mu=1,ngrhoij/2-1
     mushift=2*mu+1
     grnl_red(mu)=grnl_red(mu)+dlt_tmp &
&                *(dij_tmp*paw_ij(iatom)%grhoij(mushift  ,klmn,ispden) &
&                 -sij_tmp*paw_ij(iatom)%grhoij(mushift+1,klmn,ispden))
    end do
    if (choice==3.or.choice==23) nlstr_diag=nlstr_diag &
&                               -dlt_tmp*sij_tmp*paw_ij(iatom)%grhoij(2,klmn,ispden)

   end do ! klmn

  end do ! ispden

! Sum atomic contributions
! ==== Forces ====
  if (choice==2.or.choice==23) then
   mushift=3*(iatom-1)
   do mu=1,3
    grhat(mu,iatom)=fact &
&                  *(rprimd(1,mu)*grhat_red(ishift+1)+&
&                    rprimd(2,mu)*grhat_red(ishift+2)+&
&                    rprimd(3,mu)*grhat_red(ishift+3))
    grnl(mu+mushift)=grnl_red(ishift+mu)
   end do
  end if
! ==== Stresses ====
  if (choice==3.or.choice==23) then
   nhatstr(1:6)=nhatstr(1:6)+ grhat_red(1:6)
   nlstr(1:6)  =nlstr(1:6)  + grnl_red (1:6)
  end if

!End loop on atoms
  deallocate(prod)
 end do

!DEBUG
!write(6,*)' pawgrnhat: after iatom loop '
!stop
!ENDDEBUG

!Deallocate memory
 deallocate(grhat_red,grnl_red)

!Convert stresses gradients from reduced to cartesian coordinates
 if (choice==3.or.choice==23) then
! Has to compute int[nhat+*vtrial]
  nhat_diag=zero
  if (nspden==1) then
   do ic=1,nfft
    nhat_diag=nhat_diag+vtrial(ic,1)*nhat(ic,1)
   end do
  else
   do ic=1,nfft
    nhat_diag=nhat_diag+vtrial(ic,1)*nhat(ic,2)+vtrial(ic,2)*(nhat(ic,1)-nhat(ic,2))
   end do
  end if
! Convert nhat contribution
  nhatstr(1:3)=(nhatstr(1:3)+nhat_diag)/dble(nfft)
  nhatstr(4:6)=nhatstr(4:6)/dble(nfft)
!  if (nsym>1) call strsym(gprimd,nsym,nhatstr,symrec) !Not needed
! Convert NL (rhoij) contribution
  call strconv(nlstr,gprimd,nlstr)
  nlstr(1:3)=(nlstr(1:3)-nlstr_diag)/ucvol
  nlstr(4:6)=nlstr(4:6)/ucvol
!  if (nsym>1) call strsym(gprimd,nsym,nlstr,symrec) !Not needed
 end if
!DEBUG
!write(6,*)' pawgrnhat: exit '
!stop
!ENDDEBUG

end subroutine pawgrnhat
!!***
