!{\src2tex{textfont=tt}}
!!****f* ABINIT/pawmknhat
!! NAME
!! pawmknhat
!!
!! FUNCTION
!! PAW only:
!! Compute compensation charge density (and derivatives) on the fine FFT grid
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.
!!
!! INPUTS
!!  ider= 0: nhat(r) is computed
!!        1: cartesian derivatives of nhat(r) are computed
!!        2: nhat(r) and derivatives are computed
!!  izero=if 1, unbalanced components of nhat(g) have to be set to zero
!!  mpi_enreg=informations about MPI parallelization
!!  natom=number of atoms in cell.
!!  nfft=number of point on the rectangular fft grid
!!  ngfft(18)=contain all needed information about 3D FFT, see ~abinit/doc/input_variables/vargs.htm#ngfft
!!  nhatgrdim= -PAW only- 0 if pawgrnhat array is not used ; 1 otherwise
!!  ntypat=number of types of atoms in unit cell.
!!  paw_ij(natom) <type(paw_ij_type)>=paw arrays given on (i,j) channels
!!  pawang <type(pawang_type)>=paw angular mesh and related data
!!  pawfgrtab(natom) <type(pawfgrtab_type)>=atomic data given on fine rectangular grid
!!  pawtab(ntypat) <type(pawtab_type)>=paw tabulated starting data
!!  typat(natom)=type (integer) for each atom
!!  ucvol=volume of the unit cell
!!
!! OUTPUT
!!  === if ider=0 or 2
!!    compch_fft=compensation charge inside spheres computed over fine fft grid
!!    pawnhat(nfft,ispden)=nhat on fine rectangular grid
!!  === if ider=1 or 2
!!    pawgrnhat(nfft,ispden,3)=derivatives of nhat on fine rectangular grid (and derivatives)
!!
!! PARENTS
!!      afterscfloop,energy,outscfcv,scfcv,vtorho
!!
!! CHILDREN
!!      fourdp,leave_new,mean_fftr,wrtout,zerosym
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine pawmknhat(compch_fft,ider,izero,mpi_enreg,natom,nfft,ngfft,nhatgrdim,nspden,&
&          ntypat,paw_ij,pawang,pawfgrtab,pawgrnhat,pawnhat,pawtab,typat,ucvol)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01managempi
 use interfaces_02ffts
 use interfaces_02spacepar
#endif
!End of the abilint section

 implicit none

!Arguments ---------------------------------------------
!scalars
 integer,intent(in) :: ider,izero,natom,nfft,nhatgrdim,nspden,ntypat
 real(dp),intent(in) :: ucvol
 real(dp),intent(out) :: compch_fft
 type(MPI_type),intent(inout) :: mpi_enreg
 type(pawang_type),intent(in) :: pawang
!arrays
 integer,intent(in) :: ngfft(18),typat(natom)
 real(dp),intent(out) :: pawgrnhat(nfft,nspden,3*nhatgrdim)
 real(dp),intent(out) :: pawnhat(nfft,nspden)
 type(paw_ij_type),intent(in) :: paw_ij(natom)
 type(pawfgrtab_type),intent(in) :: pawfgrtab(natom)
 type(pawtab_type),intent(in) :: pawtab(ntypat)

!Local variables ---------------------------------------
!scalars
 integer :: iatom,ic,ils,ilslm,irhoij,ispden,itypat,jc,jspden,klm,klmn,lcutdens
 integer :: lmax,lmin,mm,nfftot
 real(dp) :: ro,ro_ql
 logical :: compute_grad,compute_nhat
 character(len=500) :: message
!arrays
 real(dp) :: tmp_compch_fft(nspden)
 real(dp),allocatable :: work(:,:)

! *************************************************************************

 if(nspden==4) then
  write(message, '(a,a,a,a)' )ch10,&
&   ' pawmknhat : ERROR -',ch10,&
&   '  nspden 4 is not yet allowed.'
  call wrtout(6,message,'COLL')
  call leave_new('PERS')
 end if
 if(ider>0.and.nhatgrdim==0) then
  write(message, '(a,a,a,a)' )ch10,&
&   ' pawmknhat : BUG -',ch10,&
&   '  Gradients of nhat required but not aloocated !'
  call wrtout(6,message,'COLL')
  call leave_new('PERS')
 end if
 compute_nhat=(ider==0.or.ider==2)
 compute_grad=(ider==1.or.ider==2)
 if ((.not.compute_nhat).and.(.not.compute_grad)) return

 if (compute_nhat) pawnhat=zero
 if (compute_grad) pawgrnhat=zero

 lcutdens=pawang%lcutd

!------------------------------------------------------------------------
!----- Loops over atoms and density components
!------------------------------------------------------------------------
 do iatom=1,natom
  itypat=typat(iatom)

  do ispden=1,nspden
   jspden=nspden+1-ispden

!------------------------------------------------------------------------
!----- Loop over ij channels (basis components)
!------------------------------------------------------------------------
   do irhoij=1,paw_ij(iatom)%nrhoijsel(ispden)
    klmn=paw_ij(iatom)%rhoijselect(irhoij,ispden)
    klm =pawtab(itypat)%indklmn(1,klmn)
    lmin=pawtab(itypat)%indklmn(3,klmn)
    lmax=min(lcutdens-1,pawtab(itypat)%indklmn(4,klmn))
    ro=pawtab(itypat)%dltij(klmn)*paw_ij(iatom)%rhoijp(irhoij,ispden)

    if (compute_nhat) then
     do ils=lmin,lmax,2
      do mm=-ils,ils
       ilslm=ils*ils+ils+mm+1
       if (pawang%gntselect(ilslm,klm)>0) then
        ro_ql=ro*pawtab(itypat)%qijl(ilslm,klmn)
        do ic=1,pawfgrtab(iatom)%nfgd
         jc=pawfgrtab(iatom)%ifftsph(ic)
         pawnhat(jc,jspden)=pawnhat(jc,jspden)+ro_ql*pawfgrtab(iatom)%gylm(ic,ilslm)
        end do
       end if
      end do
     end do
    end if

    if (compute_grad) then
      do ils=lmin,lmax,2
      do mm=-ils,ils
       ilslm=ils*ils+ils+mm+1
       if (pawang%gntselect(ilslm,klm)>0) then
        ro_ql=ro*pawtab(itypat)%qijl(ilslm,klmn)
        do ic=1,pawfgrtab(iatom)%nfgd
         jc=pawfgrtab(iatom)%ifftsph(ic)
         pawgrnhat(jc,jspden,1)=pawgrnhat(jc,jspden,1)+ro_ql*pawfgrtab(iatom)%gylmgr(1,ic,ilslm)
         pawgrnhat(jc,jspden,2)=pawgrnhat(jc,jspden,2)+ro_ql*pawfgrtab(iatom)%gylmgr(2,ic,ilslm)
         pawgrnhat(jc,jspden,3)=pawgrnhat(jc,jspden,3)+ro_ql*pawfgrtab(iatom)%gylmgr(3,ic,ilslm)
        end do
       end if
      end do
     end do
    end if

!------------------------------------------------------------------------
!----- End loop over ij channels
!------------------------------------------------------------------------
   end do

!------------------------------------------------------------------------
!----- End loops over iatom and ispden
!------------------------------------------------------------------------
  end do
 end do

!----- Transfer components of density (1=up+down, 2=up)
 if (nspden==2) then
  if (compute_nhat) then
   pawnhat(:,1)=pawnhat(:,1)+pawnhat(:,2)
  end if
  if (compute_grad) then
   pawgrnhat(:,1,1)=pawgrnhat(:,1,1)+pawgrnhat(:,2,1)
   pawgrnhat(:,1,2)=pawgrnhat(:,1,2)+pawgrnhat(:,2,2)
   pawgrnhat(:,1,3)=pawgrnhat(:,1,3)+pawgrnhat(:,2,3)
  end if
 end if

!----- Avoid unbalanced g-components numerical errors
 if (izero==1.and.compute_nhat) then
  allocate(work(2,nfft))
  do ispden=1,min(2,nspden)
   call fourdp(1,work,pawnhat(:,ispden),-1,mpi_enreg,nfft,ngfft,0)
   call zerosym(work,2,mpi_enreg,ngfft(1),ngfft(2),ngfft(3))
   call fourdp(1,work,pawnhat(:,ispden),+1,mpi_enreg,nfft,ngfft,0)
  end do
  deallocate(work)
 end if

!----- Computation of compensation charge over FFT grid
 if (compute_nhat) then
  nfftot=ngfft(1)*ngfft(2)*ngfft(3)
  call mean_fftr(pawnhat,tmp_compch_fft,mpi_enreg,nfft,nfftot,1)
  compch_fft = tmp_compch_fft(1)
  compch_fft=compch_fft*ucvol
 end if

end subroutine pawmknhat
!!***
