!{\src2tex{textfont=tt}}
!!****f* ABINIT/pawmkrhoij
!! NAME
!! pawmkrhoij
!!
!! FUNCTION
!! Calculate the PAW quantities rhoij (augmentation occupancies)
!! for each atom:
!!   * rhoij=Sum_{n,k} {occ(n,k)*rhoij_nk}
!!   * Usefull quantities to compute (later)
!!       rhoij derivatives (wrt atm. pos., strains, ...)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.
!!
!! INPUTS
!!  atindx1(natom)=index table for atoms, inverse of atindx (see scfcv.f)
!!  dimrhoij=1st dimension of array pawrhoijnk
!!  eigen(mband*nkpt*nsppol)=array for holding eigenvalues (hartree)
!!  mband=maximum number of bands
!!  mkmem =number of k points which can fit in memory; set to 0 if use disk
!!  mpi_enreg=informations about MPI parallelization
!!  natom=number of atoms in cell.
!!  nattyp(ntypat)= # atoms of each type.
!!  nband=number of bands for all k points
!!  ngrhoijnk= number of rhoij gradients stored in array pawrhoijnk
!!             only for testing ; if ngrhoijnk >1, some derivatives are required
!!  nkpt=number of k points.
!!  nspden=number of spin-density components
!!  nsppol=1 for unpolarized, 2 for spin-polarized
!!  ntypat=number of types of atoms in unit cell.
!!  occ(mband*nkpt*nsppol)=occupation number for each band for each k
!!  occopt==option for occupancies
!!  optrhoij= 1 if rhoij quantities have to be stored
!!  paw_ij(natom) <type(paw_ij_type)>=paw arrays given on (i,j) channels
!!  pawprtvol=control print volume and debugging output for PAW
!!  pawrhoijnk(dimrhoij,mband*mkmem*nsppol)=rhoij_nk quantities
!!             and (eventually) their derivatives wrt atm. pos., strains, ...
!!  unpaw=unit number for rhoij_nk data (if used)
!!  wtk(nkpt)=weight assigned to each k point
!!
!! OUTPUT
!!  if (optrhoij=1)
!!    paw_ij(iatom)%rhoij_(lmn2_size,nspden)= paw rhoij quantities for each atom (non symetrized)
!!                     Rhoij = Sum_{n,k} {occ(n,k)*rhoij_nk}
!!  if (ngrhoijnk>1)
!!    paw_ij(iatom)%grhoij(ngrhoij,lmn2_size,nspden)= usefull quantities to compute later
!!                                           gradients of rhoij quantities for each atom:
!!                     -------------------------------------
!!                     Sum_{n,k} {occ(n,k)*rhoij_nk}
!!                     Sum_{n,k} {occ(n,k)*eigen(n,k)*rhoij_nk}
!!                     -------------------------------------
!!                     Sum_{n,k} {occ(n,k)*drhoij_nk/dstrain}
!!                     Sum_{n,k} {occ(n,k)*eigen(n,k)*drhoij_nk/dstrain}
!!                     -------------------------------------
!!                     Sum_{n,k} {occ(n,k)*drhoij_nk/dxred}
!!                     Sum_{n,k} {occ(n,k)*eigen(n,k)*drhoij_nk/dxred}
!!
!! PARENTS
!!      forstrnps,vtorho
!!
!! CHILDREN
!!      leave_new,leave_test,print_ij,timab,wrtout,xcomm_init,xme_init,xsum_mpi
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine pawmkrhoij(atindx1,dimrhoij,eigen,mband,mkmem,mpi_enreg,natom,nattyp,nband,ngrhoijnk,&
&                      nkpt,nspden,nsppol,ntypat,occ,occopt,optrhoij,paw_ij,pawprtvol,pawrhoijnk,&
&                      unpaw,wtk)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_00basis
 use interfaces_01managempi
 use interfaces_01util
#else
 use defs_xfuncmpi
#endif
!End of the abilint section

 implicit none

!Arguments ---------------------------------------------
!scalars
 integer,intent(in) :: dimrhoij,mband,mkmem,natom,ngrhoijnk,nkpt,nspden,nsppol
 integer,intent(in) :: ntypat,occopt,optrhoij,pawprtvol,unpaw
 type(MPI_type),intent(inout) :: mpi_enreg
!arrays
 integer,intent(in) :: atindx1(natom),nattyp(ntypat),nband(nkpt*nsppol)
 real(dp),intent(in) :: eigen(mband*nkpt*nsppol),occ(mband*nkpt*nsppol)
 real(dp),intent(in) :: pawrhoijnk(dimrhoij,mband*mkmem*nsppol),wtk(nkpt)
 type(paw_ij_type),intent(inout) :: paw_ij(natom)

!Local variables ---------------------------------------
!scalars
 integer :: bdtot_index,bufdim,dimrhoij0,iat,iatm,iatom,ib,iband,ibg,ierr,ikpt
 integer :: indx_rhoij,ispden,isppol,itypat,jdim,klmn,mband0,me,mu,mushift
 integer :: natinc,nband_k,spaceComm
 real(dp) :: grhoij_tmp,rhoij_tmp,weight
 character(len=500) :: message
!arrays
 integer :: dimlmn(natom)
 integer,allocatable :: idum(:)
 real(dp) :: tsec(2)
 real(dp),allocatable :: buffer1(:),buffer2(:),pawrhoijnk_k(:,:)

!************************************************************************

 if(nspden==4) then
  write(message, '(a,a,a,a)' )ch10,&
&   ' pawmkrhoij : ERROR -',ch10,&
&   '  nspden 4 not allowed !'
  call wrtout(6,message,'PERS')
  call leave_new('PERS')
 end if

 if (ngrhoijnk>1.and.paw_ij(1)%ngrhoij/=2*ngrhoijnk) then
  write(message, '(a,a,a,a)' )ch10,&
&   ' pawmkrhoij : BUG -',ch10,&
&   '  Incorrect  dimension of array grhoij !'
  call wrtout(6,message,'PERS')
  call leave_new('PERS')
 end if

!Init mpi_comm
 call xcomm_init(mpi_enreg,spaceComm)
!BEGIN TF_CHANGES
!Define me
 call xme_init(mpi_enreg,me)
!END TF_CHANGES

!Initialize temporary file
 if (mkmem==0) then
  rewind unpaw;read(unpaw) dimrhoij0,mband0
  if(dimrhoij/=dimrhoij0.or.mband/=mband0) then
   write(message, '(a,a,a,a)' )ch10,&
&    ' pawmkrhoij : BUG -',ch10,&
&    '  _PAW file was not created with the right options !'
   call wrtout(6,message,'PERS')
   call leave_new('PERS')
  end if
 end if

!Initialize output quantities
 if (optrhoij==1) then
  do iatom=1,natom;paw_ij(iatom)%rhoij_=zero;end do
 end if
 if (ngrhoijnk>1) then
  do iatom=1,natom;paw_ij(iatom)%grhoij=zero;end do
 end if
 dimlmn(1:natom)=paw_ij(1:natom)%lmn2_size

!LOOP OVER SPINS
 bdtot_index=0;ibg=0
 do isppol=1,nspden

! LOOP OVER k POINTS
  do ikpt=1,nkpt

   nband_k=nband(ikpt+(isppol-1)*nkpt)

   if(mpi_enreg%paral_compil_kpt==1)then
    if (mpi_enreg%parareel == 0) then
!BEGIN TF_CHANGES
     if(minval(abs(mpi_enreg%proc_distrb(ikpt,1:nband_k,isppol) &
&       -me))/=0) then
!END TF_CHANGES
      bdtot_index=bdtot_index+nband_k
!     Skip the rest of the k-point loop
      cycle
     end if
    else
!BEGIN TF_CHANGES
     if(mpi_enreg%proc_distrb_para(mpi_enreg%ipara,ikpt) &
&       /= me) then
!END TF_CHANGES
      bdtot_index=bdtot_index+nband_k
!     Skip the rest of the k-point loop
      cycle
     end if
    end if
   end if

!  Extract Rhoij_nk quantities according to mkmem
   allocate(pawrhoijnk_k(dimrhoij,nband_k))
   if (mkmem==0) then
    read(unpaw) pawrhoijnk_k(1:dimrhoij,1:nband_k)
   else
    pawrhoijnk_k(1:dimrhoij,1:nband_k)=pawrhoijnk(1:dimrhoij,ibg+1:ibg+nband_k)
   end if

!  LOOP OVER BANDS
   do ib=1,nband_k
    iband=bdtot_index+ib
    if(mpi_enreg%paral_compil_kpt==1)then
!    Skip this band if not the proper processor
     if(mpi_enreg%parareel == 0) then
!BEGIN TF_CHANGES
      if (mpi_enreg%proc_distrb(ikpt,ib,isppol)/= me) cycle
 !END TF_CHANGES
    else
      if (mpi_enreg%proc_distrb_para(mpi_enreg%ipara,ikpt)/= mpi_enreg%me) cycle
     end if
    end if

!   Select occupied bands
    if (abs(occ(iband))>tol8) then
     weight=wtk(ikpt)*occ(iband)

     indx_rhoij=0;iatm=0
     do itypat=1,ntypat
      do iat=1,nattyp(itypat)
       iatm=iatm+1;iatom=atindx1(iatm)
       do klmn=1,dimlmn(iatom)
        rhoij_tmp=weight*pawrhoijnk_k(indx_rhoij+1,ib)
        if (optrhoij==1) paw_ij(iatom)%rhoij_(klmn,isppol)= &
&             paw_ij(iatom)%rhoij_(klmn,isppol) + rhoij_tmp
        if (ngrhoijnk>1) then
         paw_ij(iatom)%grhoij(1,klmn,isppol)= &
&             paw_ij(iatom)%grhoij(1,klmn,isppol) + rhoij_tmp
         paw_ij(iatom)%grhoij(2,klmn,isppol)= &
&             paw_ij(iatom)%grhoij(2,klmn,isppol) + eigen(iband)*rhoij_tmp
         do mu=2,ngrhoijnk
          mushift=2*mu-1
          grhoij_tmp=weight*pawrhoijnk_k(indx_rhoij+mu,ib)
          paw_ij(iatom)%grhoij(mushift,klmn,isppol)= &
&             paw_ij(iatom)%grhoij(mushift,klmn,isppol) + grhoij_tmp
          paw_ij(iatom)%grhoij(mushift+1,klmn,isppol)= &
&             paw_ij(iatom)%grhoij(mushift+1,klmn,isppol) + eigen(iband)*grhoij_tmp
         end do
        end if
        indx_rhoij=indx_rhoij+ngrhoijnk
       end do
      end do
     end do

!END LOOPS
    end if ! abs(occ)>tol8
   end do ! ib
   bdtot_index=bdtot_index+nband_k
   if (mkmem/=0) ibg=ibg+nband_k

   deallocate(pawrhoijnk_k)
  end do ! ikpt
 end do ! isppol

!MPI: need to exchange arrays between procs
!==========================================
 if(mpi_enreg%paral_compil_kpt==1)then
  call timab(66,1,tsec)
  if (mpi_enreg%parareel == 0) then
!BEGIN TF_CHANGES
   call leave_test(mpi_enreg)
!END TF_CHANGES
  end if
  call timab(66,2,tsec)
  call timab(48,1,tsec)

! Exchange rhoij_
  if (optrhoij==1) then
   bufdim=sum(dimlmn)*nspden
   allocate(buffer1(bufdim),buffer2(bufdim))
   jdim=0
   do iatom=1,natom
    do ispden=1,nspden
     buffer1(jdim+1:jdim+dimlmn(iatom))=paw_ij(iatom)%rhoij_(:,ispden)
     jdim=jdim+dimlmn(iatom)
    end do
   end do
   call xsum_mpi(buffer1,buffer2,bufdim,spaceComm,ierr) !Build sum of everything
   jdim=0
   do iatom=1,natom
    do ispden=1,nspden
     paw_ij(iatom)%rhoij_(:,ispden)=buffer2(jdim+1:jdim+dimlmn(iatom))
     jdim=jdim+dimlmn(iatom)
    end do
   end do
   deallocate(buffer1,buffer2)
  end if

! If needed, exchange grhoij
  if (ngrhoijnk>1) then
   bufdim=2*ngrhoijnk*sum(dimlmn)*nspden
   allocate(buffer1(bufdim),buffer2(bufdim))
   jdim=0
   do iatom=1,natom
    do ispden=1,nspden
     do mu=1,2*ngrhoijnk
      buffer1(jdim+1:jdim+dimlmn(iatom))=paw_ij(iatom)%grhoij(mu,1:dimlmn(iatom),ispden)
      jdim=jdim+dimlmn(iatom)
     end do
    end do
   end do
   call xsum_mpi(buffer1,buffer2,bufdim,spaceComm,ierr) !Build sum of everything
   jdim=0
   do iatom=1,natom
    do ispden=1,nspden
     do mu=1,2*ngrhoijnk
      paw_ij(iatom)%grhoij(mu,1:dimlmn(iatom),ispden)=buffer2(jdim+1:jdim+6*dimlmn(iatom))
      jdim=jdim+dimlmn(iatom)
     end do
    end do
   end do
   deallocate(buffer1,buffer2)
  end if

  call timab(48,2,tsec)
 end if ! mpi_enreg%paral_compil_kpt==1

!Print info
 if (pawprtvol>=1.and.optrhoij==1) then
  natinc=1;if(natom>1) natinc=natom-1
  do iatom=1,natom,natinc
   write(message, '(4a,i3,a)') ch10," PAW TEST:",ch10,&
&    ' ====== Values of RHOIJ in pawmkrhoij (iatom=',iatom,') ======'
   call wrtout(6,message,'COLL')
   do ispden=1,nspden
    write(message, '(a,i1,a)') '   Density component= ',ispden,':'
    call wrtout(6,message,'COLL')
    call print_ij(paw_ij(iatom)%rhoij_(:,ispden),paw_ij(iatom)%lmn2_size,paw_ij(iatom)%lmn_size,1,0,idum,1)
   end do
  end do
 end if

end subroutine pawmkrhoij
!!***
