!{\src2tex{textfont=tt}}
!!****f* ABINIT/symrhoij
!! NAME
!! symrhoij
!!
!! FUNCTION
!! Symmetrize rhoij quantities (augmentation occupancies) and/or gradients
!! Compute also rhoij residuals (new-old values of rhoij and gradients)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.
!!
!! INPUTS
!!  choice=select then type of rhoij gradients to symmetrize.
!!         choice=1 => no gradient
!!         choice=2 => gradient with respect to atomic position(s)
!!               =3 => a gradient with respect to strain(s)
!!               =23=> a gradient with respect to atm. pos. and strain(s)
!!  indlmn(6,lmnmax,ntypat)=array giving l,m,n,lm,ln,spin for i=lmn (for each atom type)
!!  indsym(4,nsym,natom)=indirect indexing array for atom labels
!!  lmnmax=maximum number of PAW radial wavefunctions
!!  natom=number of atoms in cell
!!  nsym=number of symmetry elements in space group
!!  ntypat=number of types of atoms in unit cell.
!!  optrhoij= 1 if rhoij quantities have to be symmetrized
!!  paw_ij(natom)%lmn_size=number of (l,m,n) elements for the paw basis
!!  paw_ij(natom)%nspden=number of spin-density components
!!  paw_ij(natom)%rhoij_(lmn2_size,nspden)=non-symetrized paw rhoij quantities
!!  pawang <type(pawang_type)>=angular mesh discretization and related data
!!  pawprtvol=control print volume and debugging output for PAW
!!  symafm(nsym)=(anti)ferromagnetic part of symmetry operations
!!  symrec(3,3,nsym)=symmetries of group in terms of operations on
!!                   reciprocal space primitive translations
!!  typat(natom)=type for each atom
!!
!! OUTPUT
!!  if (optrhoij==1)
!!    paw_ij(natom)%nrhoijsel(nspden)=number of non-zero values of rhoij
!!    paw_ij(natom)%rhoij (lmn2_size,nspden)=symetrized paw rhoij quantities (all values)
!!    paw_ij(natom)%rhoijp(lmn2_size,nspden)=symetrized paw rhoij quantities in PACKED STORAGE (only non-zero values)
!!    paw_ij(natom)%rhoijres(lmn2_size,nspden)=paw rhoij quantities residuals (new values - old values)
!!    paw_ij(natom)%rhoijselect(lmn2_size,nspden)=select the non-zero values of rhoij
!!
!! SIDE EFFECTS
!!  if (paw_ij(:)%ngrhoij>1) (equivalent to choice>1)
!!    At input:
!!    paw_ij(natom)%grhoij(ngrhoij,lmn2_size,nspden)=non-symetrized gradients of rhoij
!!    At output:
!!    paw_ij(natom)%grhoij(ngrhoij,lmn2_size,nspden)=symetrized gradients of rhoij
!!
!! NOTES
!!  This file was initially inspirated by the file SymWij.f
!!  by N.A.W. Holzwarth and A. Tackett for the code pwpaw
!!
!! PARENTS
!!      forstrnps,vtorho
!!
!! CHILDREN
!!      leave_new,print_ij,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine symrhoij(choice,indlmn,indsym,lmnmax,natom,nsym,ntypat,optrhoij,&
&                    paw_ij,pawang,pawprtvol,symafm,symrec,typat)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01managempi
 use interfaces_01util
#endif
!End of the abilint section

 implicit none

!Arguments ---------------------------------------------
!scalars
 integer,intent(in) :: choice,lmnmax,natom,nsym,ntypat,optrhoij,pawprtvol
 type(pawang_type),intent(in) :: pawang
!arrays
 integer,intent(in) :: indlmn(6,lmnmax,ntypat),indsym(4,nsym,natom)
 integer,intent(in) :: symafm(nsym),symrec(3,3,nsym),typat(natom)
 type(paw_ij_type),intent(inout) :: paw_ij(natom)

!Local variables ---------------------------------------
!scalars
 integer :: at_indx,iatom,il,il0,ilmn,iln,iln0,ilpm,indexi,indexii,indexj
 integer :: indexjj,indexjj0,indexk,irot,ishift,ispden,itypat,j0lmn,jj,jl,jl0
 integer :: jlmn,jln,jln0,jlpm,klmn,lmn_size,mi,mj,mu,mua,mub,mushift,natinc
 integer :: ngrhoij,nselect,nsym_used,nu,nushift
 real(dp) :: ro,sum1,sum2,sumrho,sym1,sym2,sym3,syma,zarot2
 character(len=500) :: message
!arrays
 integer,parameter :: alpha(6)=(/1,2,3,3,3,2/),beta(6)=(/1,2,3,2,1,1/)
 integer,allocatable :: idum(:)
 real(dp) :: work1(3,3),work2(3,3)
 real(dp),allocatable :: rotgr(:),sumgr(:)
!no_abirules
 type grhoij_at
  real(dp),pointer :: grhoij(:,:,:)
 end type
 type(grhoij_at) :: grtmp(natom)

! *********************************************************************

!Symetrization occurs only when nsym>1
 if (nsym>1) then

! Test: consistency between choice and ngrhoij
  ngrhoij=paw_ij(1)%ngrhoij
  if ((choice==1.and.ngrhoij/=0) .or.(choice==2.and.ngrhoij/=8).or. &
&     (choice==3.and.ngrhoij/=14).or.(choice==23.and.ngrhoij/=20)) then
   write(message, '(a,a,a,a)' )ch10,&
&    ' symrhoij : BUG -',ch10,&
&    '  Inconsistency between nvariables choice and ngrhoij !'
   call wrtout(6,message,'PERS')
   call leave_new('PERS')
  end if

! Have to make a temporary copy of grhoij
  if (choice>1) then
   allocate(sumgr(ngrhoij),rotgr(ngrhoij))
   do iatom=1,natom
    allocate(grtmp(iatom)%grhoij(ngrhoij,&
&                                paw_ij(iatom)%lmn2_size,&
&                                paw_ij(iatom)%nspden))
    grtmp(iatom)%grhoij(:,:,:)=paw_ij(iatom)%grhoij(:,:,:)
   end do
   ishift=1;if (choice==23) ishift=13
  end if

! Loops over atoms and spin components
! ------------------------------------
  do iatom=1,natom
   itypat=typat(iatom)
   lmn_size=paw_ij(iatom)%lmn_size
   do ispden=1,paw_ij(iatom)%nspden
    nselect=0

!   Loops over (il,im) and (jl,jm)
!   ------------------------------
    jl0=-1;jln0=-1;indexj=1
    do jlmn=1,lmn_size
     jl=indlmn(1,jlmn,itypat)
     jlpm=1+jl+indlmn(2,jlmn,itypat)
     jln=indlmn(5,jlmn,itypat)
     if (jln/=jln0) indexj=indexj+2*jl0+1
     j0lmn=jlmn*(jlmn-1)/2
     il0=-1;iln0=-1;indexi=1
     do ilmn=1,jlmn
      il=indlmn(1,ilmn,itypat)
      ilpm=1+il+indlmn(2,ilmn,itypat)
      iln=indlmn(5,ilmn,itypat)
      if (iln/=iln0) indexi=indexi+2*il0+1
      klmn=j0lmn+ilmn

      nsym_used=0
      sumrho=zero
      if (choice>1) rotgr=zero

!     Loop over symmetries
!     --------------------
      do irot=1,nsym
       if (symafm(irot)/=1) cycle
       nsym_used=nsym_used+1
       at_indx=indsym(4,irot,iatom)

!      Accumulate values over (mi,mj)
!      ------------------------------
       if (choice>1) sumgr=zero
       do mj=1,2*jl+1
        indexjj=indexj+mj;indexjj0=indexjj*(indexjj-1)/2
        do mi=1,2*il+1
         indexii=indexi+mi
         if (indexii<=indexjj) then
          indexk=indexjj0+indexii
         else
          indexk=indexii*(indexii-1)/2+indexjj
         end if
         zarot2=pawang%zarot(mi,ilpm,il+1,irot)&
&              *pawang%zarot(mj,jlpm,jl+1,irot)

         if (optrhoij==1) &
&         sumrho=sumrho+zarot2*paw_ij(at_indx)%rhoij_(indexk,ispden)

         if (choice>1) then
          do mu=1,ngrhoij
           sumgr(mu)=sumgr(mu)+zarot2*grtmp(at_indx)%grhoij(mu,indexk,ispden)
          end do
         end if

        end do
       end do

!      Accumulate values over symmetries
!      ---------------------------------
!      ===== Common contribution to forces and stresses ====
       if (choice>1) rotgr(1:2)=rotgr(1:2)+sumgr(1:2)
!      ===== Contributions to forces ====
       if (choice==2.or.choice==23) then
        do nu=1,3
         nushift=2*nu+ishift
         do mu=1,3
          mushift=2*mu+ishift
          rotgr(mushift)  =rotgr(mushift)  +dble(symrec(mu,nu,irot))*sumgr(nushift)
          rotgr(mushift+1)=rotgr(mushift+1)+dble(symrec(mu,nu,irot))*sumgr(nushift+1)
         end do
        end do
       end if
!      ===== Contributions to stresses ====
       if (choice==3.or.choice==23) then
        work1(1,1)=sumgr(3) ;work2(1,1)=sumgr(4)
        work1(2,2)=sumgr(5) ;work2(2,2)=sumgr(6)
        work1(3,3)=sumgr(7) ;work2(3,3)=sumgr(8)
        work1(2,3)=sumgr(9) ;work2(2,3)=sumgr(10)
        work1(3,2)=sumgr(9) ;work2(3,2)=sumgr(10)
        work1(1,3)=sumgr(11);work2(1,3)=sumgr(12)
        work1(3,1)=sumgr(11);work2(3,1)=sumgr(12)
        work1(1,2)=sumgr(13);work2(1,2)=sumgr(14)
        work1(2,1)=sumgr(13);work2(2,1)=sumgr(14)
        do mu=1,6
         mushift=2*mu+1
         mua=alpha(mu);mub=beta(mu)
         sum1=zero;sum2=zero
         sym1=dble(symrec(mub,1,irot))
         sym2=dble(symrec(mub,2,irot))
         sym3=dble(symrec(mub,3,irot))
         do nu=1,3
          syma=dble(symrec(mua,nu,irot))
          sum1=sum1+syma*(work1(nu,1)*sym1+work1(nu,2)*sym2+work1(nu,3)*sym3)
          sum2=sum2+syma*(work2(nu,1)*sym1+work2(nu,2)*sym2+work2(nu,3)*sym3)
         end do
         rotgr(mushift)  =rotgr(mushift)  +sum1
         rotgr(mushift+1)=rotgr(mushift+1)+sum2
        end do
       end if

      end do ! End loop over symmetries

!     Select non-zero elements of rhoij
!     ---------------------------------
      if (optrhoij==1) then
       ro=sumrho/nsym_used
       if (abs(ro)>tol10) then
        nselect=nselect+1
        paw_ij(iatom)%rhoijselect(nselect,ispden)=klmn
        paw_ij(iatom)%rhoijres(klmn,ispden)=ro-paw_ij(iatom)%rhoij(klmn,ispden)
        paw_ij(iatom)%rhoij(klmn,ispden)=ro
        paw_ij(iatom)%rhoijp(nselect,ispden)=ro
       else
        paw_ij(iatom)%rhoijres(klmn,ispden)=-paw_ij(iatom)%rhoij(klmn,ispden)
        paw_ij(iatom)%rhoij   (klmn,ispden)=zero
       end if
      end if

!     Normalization of gradients
      if (choice>1) then
       do mu=1,ngrhoij
        paw_ij(iatom)%grhoij(mu,klmn,ispden)=rotgr(mu)/nsym_used
       end do
      end if

      il0=il;iln0=iln  ! End loops over (il,im) and (jl,jm)
     end do
     jl0=jl;jln0=jln
    end do

!   Store number of non-zero values of rhoij
    if (optrhoij==1) paw_ij(iatom)%nrhoijsel(ispden)=nselect

   end do  ! End loops over ispden and iatom
  end do

  if (choice>1) then
   do iatom=1,natom
    deallocate(grtmp(iatom)%grhoij)
   end do
   deallocate(sumgr,rotgr)
  end if

 else  ! nsym>1

! *********************************************************************
! If nsym==1, only copy rhoij_ into rhoij
!             also has to fill rhoijselect array
  if (optrhoij==1) then
   do iatom=1,natom
    do ispden=1,paw_ij(iatom)%nspden
     nselect=0
     do klmn=1,paw_ij(iatom)%lmn2_size
      if (abs(paw_ij(iatom)%rhoij_(klmn,ispden))>tol10) then
       nselect=nselect+1
       paw_ij(iatom)%rhoijselect(nselect,ispden)=klmn
       ro=paw_ij(iatom)%rhoij_(klmn,ispden)
       paw_ij(iatom)%rhoijres(klmn,ispden)=ro-paw_ij(iatom)%rhoij(klmn,ispden)
       paw_ij(iatom)%rhoij(klmn,ispden)=ro
       paw_ij(iatom)%rhoijp(nselect,ispden)=ro
      else
       paw_ij(iatom)%rhoijres(klmn,ispden)=-paw_ij(iatom)%rhoij(klmn,ispden)
       paw_ij(iatom)%rhoij(klmn,ispden)=zero
      end if
     end do
     paw_ij(iatom)%nrhoijsel(ispden)=nselect
    end do
   end do
  end if

 end if

!*********************************************************************
!Printing of Rhoij

 if (optrhoij==1) then
  natinc=1;if(natom>1) natinc=natom-1
  do iatom=1,natom,natinc
   if (pawprtvol>=1) then
    write(message, '(4a,i3,a)') ch10," PAW TEST:",ch10,&
&    ' ====== Values of RHOIJ in symrhoij (iatom=',iatom,') ======'
    call wrtout(6,message,'COLL')
   end if
   do ispden=1,paw_ij(iatom)%nspden
    if (pawprtvol>=1) then
     write(message, '(a,i1,a)') '   Density component= ',ispden,':'
    else
     write(message, '(a,a,i3,a,i1,a)') ch10,&
&     ' *********** Rhoij (atom ',iatom,', ispden=',ispden,') **********'
    end if
    call wrtout(6,message,'COLL')
    call print_ij(paw_ij(iatom)%rhoij(:,ispden),paw_ij(iatom)%lmn2_size,paw_ij(iatom)%lmn_size,1,0,idum,1)
   end do
  end do
  message='';;call wrtout(6,message,'COLL')
 end if

end subroutine symrhoij
!!***
