!{\src2tex{textfont=tt}}
!!****f* ABINIT/energy
!! NAME
!! energy
!!
!! FUNCTION
!! Compute electronic energy terms
!! eeig, ek and enl from arbitrary (orthonormal) provided wf,
!! ehart, enxc, and eei from provided density and potential,
!! eeig=Sum of the eigenvalues - Band energy (Hartree)
!! ek=kinetic energy, ehart=Hartree electron-electron energy,
!! enxc,enxcdc=exchange-correlation energies, eei=local pseudopotential energy,
!! enl=nonlocal pseudopotential energy
!! Also, compute new density from provided wfs, after the evaluation
!! of ehart, enxc, and eei.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (DCA, XG, GMR, AR, MB, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.
!!
!! INPUTS
!!  atindx(natom)=index table for atoms (see scfcv.f)
!!  atindx1(natom)=index table for atoms, inverse of atindx (see scfcv.f)
!!  cg(2,mpw*nspinor*mband*mkmem*nsppol)=<G|Cnk>=Fourier coefficients of wavefunction
!!  densymop_gs <type(dens_sym_operator_type)>=the density symmetrization
!!   operator (ground-state symmetries)
!!  dtfil <type(datafiles_type)>=variables related to files
!!  dtset <type(dataset_type)>=all input variables for this dataset
!!  eew=Ewald energy (hartree)
!!  eigen(mband*nkpt*nsppol)=array for holding eigenvalues (hartree)
!!  eii=psp core-core energy
!!  entropy=entropy due to the occupation number smearing (if metal)
!!  epaw=PAW spherical part energy
!!  epawdc=PAW spherical part double-counting energy
!!  gsqcut=G^2 cutoff from gsqcut=ecut/(2 Pi^2)
!!  irrzon(nfft**(1-1/nsym),2,nspden/nsppol)=irreducible zone data
!!  kg(3,mpw*mkmem)=work array for coordinates of G vectors in basis
!!  mband=maximum number of bands
!!  mgfft=maximum size of 1D FFTs
!!  mpi_enreg=informations about MPI parallelization
!!  mpsang= 1+maximum angular momentum for nonlocal pseudopotentials
!!  mpw=maximum dimension for number of planewaves
!!  natom=number of atoms in unit cell
!!  nattyp(ntypat)=array describing how many atoms of each type in cell
!!  nfft=(effective) number of FFT grid points (for this processor)
!!  nfftf= -PAW ONLY- number of FFT grid points for the fine grid
!!         (nfftf=nfft for norm-conserving potential runs)
!!  ngfft(18)=contain all needed information about 3D FFT, see ~abinit/doc/input_variables/vargs.htm#ngfft
!!  ngfftf(18)= -PAW ONLY- contain all needed information about 3D FFT for the fine grid
!!              (ngfftf=ngfft for norm-conserving potential runs)
!!  nhat(nfftf,nspden*usepaw)= compensation charge density  (PAW)
!!  nkpt=number of k points
!!  npwarr(nkpt)=number of planewaves at each k point, and boundary
!!  nspden=number of spin-density components
!!  nspinor=number of spinorial components of the wavefunctions
!!  nsppol=1 for unpolarized, 2 for polarized
!!  nsym=number of symmetry elements in space group (at least 1)
!!  ntypat=number of types of atoms in cell
!!  n3xccc=dimension of the xccc3d array (0 or nfftf).
!!  occ(mband*nkpt*nsppol)=occupation numbers of bands (usually 2) at each k point
!!  occopt=option for occupancies
!!  optene=option for the computation of total energy (direct scheme or double-counting scheme)
!!  paw_ij(natom*usepaw) <type(paw_ij_type)>=paw arrays given on (i,j) channels
!!  pawang <type(pawang_type)>=paw angular mesh and related data
!!  pawfgr(natom) <type(pawfgr_type)>=fine grid parameters and related data
!!  pawfgrtab(natom) <type(pawfgrtab_type)>=atomic data given on fine rectangular grid
!!  pawtab(ntypat*usepaw) <type(pawtab_type)>=paw tabulated starting data
!!  phnons(2,nfft**(1-1/nsym),nspden/nsppol)=nonsymmorphic translation phases
!!  ph1d(2,3*(2*mgfft+1)*natom)=phase information related to structure factor
!!  psps <type(pseudopotential_type)>=variables related to pseudopotentials
!!  rprimd(3,3)=dimensional real space primitive translations (bohr)
!!  tsmear=smearing energy or temperature (if metal)
!!  usexcnhat= -PAW only- flag controling use of compensation density in Vxc
!!  vpsp(nfftf)=local pseudopotential in real space (hartree)
!!  wffnow=structured array giving all information about wavefunction file
!!  xccc3d(n3xccc)=3D core electron density for XC core correction (bohr^-3)
!!  xred(3,natom)=reduced coordinates of atoms (dimensionless)
!!  ylm(mpw*mkmem,mpsang*mpsang*useylm)= real spherical harmonics for each G and k point
!!
!! OUTPUT
!!  eeig=Sum of the eigenvalues - Band energy (Hartree)
!!  ehart=Hartree energy
!!  ek=kinetic energy
!!  enl=nonlocal psp energy
!!  enxc=exchange-correlation energy
!!  etotal=total energy (hartree):
!!    - computed by direct scheme if optene=0 or 2
!!    - computed by double-counting scheme if optene=1 or 3
!!  resid(mband*nkpt*nsppol)=residuals for each band over all k points (hartree^2)
!!  strsxc(6)=exchange-correlation contribution to stress tensor
!!  vhartr(nfftf)=work space to hold Hartree potential in real space (hartree)
!!  vtrial(nfftf,nspden)=total local potential (hartree)
!!  vxc(nfftf,nspden)=work space to hold Vxc(r) in real space (hartree)
!!  ==== if optene==0, 2 or 3
!!    eei=local psp energy (all energies in Hartrees)
!!  ==== if optene==1, 2 or 3
!!    enxcdc=exchange-correlation double-counting energy (hartree)
!!
!! SIDE EFFECTS
!!  rhog(2,nfftf)=work space for rho(G); save intact on return
!!  rhor(nfftf,nspden)=work space for rho(r); save intact on return
!!  nspinor should not be modified in the call of rdnpw
!!
!! NOTES
!!  Be careful to the meaning of nfft (size of FFT grids):
!!   - In case of norm-conserving calculations the FFT grid is the usual FFT grid.
!!   - In case of PAW calculations:
!!     Two FFT grids are used; one with nfft points (coarse grid) for
!!     the computation of wave functions ; one with nfftf points
!!     (fine grid) for the computation of total density.
!!
!!  There is a large amount of overhead in the way this routine do the computation of the energy !
!!  For example, the density has already been precomputed, so why to compute it again here ??
!!
!! PARENTS
!!      scfcv
!!
!! CHILDREN
!!      dotprod_vn,fftpac,hdr_skip,leave_new,leave_test,meanvalue_g,metric
!!      mkffnl,mkkin,mkresi,mkrho,nonlop,pawmknhat,ph1d3d,rdnpw,rhohxc,rwwf
!!      sphereboundary,timab,transgrid,wrtout,xcomm_init,xdefineoff
!!      xmaster_init,xme_init,xsum_mpi
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine energy(atindx,atindx1,cg,densymop_gs,dtfil,dtset,eei,eeig,eew,&
& ehart,eigen,eii,ek,enl,entropy,enxc,enxcdc,epaw,epawdc,etotal,gsqcut,irrzon,&
& kg,mband,mgfft,mkmem,mpi_enreg,mpsang,mpw,natom,nattyp,nfft,nfftf,ngfft,ngfftf,nhat,&
& nkpt,npwarr,nspden,nspinor,nsppol,nsym,ntypat,n3xccc,occ,occopt,optene,paw_ij,pawang,pawfgr,&
& pawfgrtab,pawtab,phnons,ph1d,psps,resid,rhog,rhor,rprimd,strsxc,tsmear,usexcnhat,vhartr,vtrial,&
& vpsp,vxc,wffnow,xccc3d,xred,ylm)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_00basis
 use interfaces_01managempi
 use interfaces_02ffts
 use interfaces_02geometry
 use interfaces_02spacepar
 use interfaces_03nonlocal
 use interfaces_03paw
 use interfaces_03xc
 use interfaces_04iowfdenpot
 use interfaces_04wfs
 use interfaces_05common, except_this_one => energy
#else
 use defs_xfuncmpi
 use defs_interfaces
 use defs_xc
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: mband,mgfft,mkmem,mpsang,mpw,n3xccc,natom,nfft,nfftf
 integer,intent(in) :: nkpt,nspden,nsppol,nsym,ntypat,occopt,optene,usexcnhat
 integer,intent(inout) :: nspinor
 real(dp),intent(in) :: eew,eii,entropy,epaw,epawdc,gsqcut,tsmear
 real(dp),intent(out) :: eei,eeig,ehart,ek,enl,enxc,enxcdc,etotal
 type(MPI_type),intent(inout) :: mpi_enreg
 type(datafiles_type),intent(in) :: dtfil
 type(dataset_type),intent(in) :: dtset
 type(dens_sym_operator_type),intent(in) :: densymop_gs
 type(pawang_type),intent(in) :: pawang
 type(pawfgr_type),intent(in) :: pawfgr
 type(pseudopotential_type),intent(in) :: psps
 type(wffile_type),intent(inout) :: wffnow
!arrays
 integer,intent(in) :: atindx(natom),atindx1(natom)
!no_abirules
!nfft**(1-1/nsym) is 1 if nsym==1, and nfft otherwise
 integer, intent(in) :: irrzon(dtset%nfft**(1-1/nsym),2,nspden/nsppol),kg(3,mpw*mkmem)
 integer, intent(in) :: nattyp(ntypat),ngfft(18),ngfftf(18),npwarr(nkpt)
 real(dp), intent(in) :: cg(2,mpw*nspinor*mband*mkmem*nsppol),eigen(mband*nkpt*nsppol)
 real(dp), intent(in) :: occ(mband*nkpt*nsppol),ph1d(2,3*(2*mgfft+1)*natom)
 real(dp), intent(inout) :: nhat(nfftf,nspden*psps%usepaw)
!nfft**(1-1/nsym) is 1 if nsym==1, and nfft otherwise
 real(dp), intent(in) :: phnons(2,dtset%nfft**(1-1/nsym),nspden/nsppol)
 real(dp), intent(out) :: resid(mband*nkpt*nsppol)
 real(dp), intent(inout) :: rhog(2,nfftf),rhor(nfftf,nspden)
 real(dp), intent(out) :: strsxc(6)
 real(dp), intent(in) :: rprimd(3,3),vpsp(nfftf),xccc3d(n3xccc),xred(3,natom)
 real(dp), intent(out) :: vhartr(nfftf),vtrial(nfftf,nspden),vxc(nfftf,nspden)
 real(dp), intent(in) :: ylm(mpw*mkmem,mpsang*mpsang*psps%useylm)
 type(paw_ij_type), intent(in) :: paw_ij(natom*psps%usepaw)
 type(pawfgrtab_type),intent(in) :: pawfgrtab(natom)
 type(pawtab_type), intent(in)  :: pawtab(ntypat*psps%usepaw)

!Local variables-------------------------------
!scalars
 integer :: bdtot_index,choice,dimdij,dimffnl,formeig,ia,iatom,iband,icg,ider
 integer :: idir,ierr,ifft,ig,igs,ii,ikg,ikpt,ilm,ilmn,index,ipw,ipw1,iresid
 integer :: ispden,ispinor,isppol,istwf_k,master,matblk,mcg,mcg_disk,me,mu,muig
 integer :: n1,n2,n3,n4,n5,n6,nband1,nband_k,nfftotf,nhatgrdim,nkpg,nkxc,nnlout
 integer :: npw_k,nvloc,option,paw_opt,signs,spaceComm,tim_mkrho,tim_nonlop
 integer :: tim_rwwf
 real(dp) :: DDOT,arg,doti,dotr,dum,eeigk,ekk,enlk,ucvol,vxcavg
 character(len=500) :: message
 type(gs_hamiltonian_type) :: gs_hamk
!arrays
 integer,allocatable :: kg_dum(:,:),kg_k(:,:)
 real(dp) :: enlout(1),gmet(3,3),gprimd(3,3),kpoint(3),nonlop_dum(1),rhodum(1)
 real(dp) :: rmet(3,3),tsec(2),ylmgr_dum(1)
 real(dp),allocatable :: buffer(:),cg_disk(:,:),cgrvtrial(:,:),cwavef(:,:)
 real(dp),allocatable :: eig_dum(:),eig_k(:),ffnl(:,:,:,:),kinpw(:)
 real(dp),allocatable :: kpg_dum(:,:),kxc(:,:),nhatgr(:,:,:),occ_dum(:)
 real(dp),allocatable :: occ_k(:),ph3d(:,:,:),resid_k(:),vlocal(:,:,:,:)
 real(dp),allocatable :: vlocal_tmp(:,:,:),ylm_k(:,:)

! *************************************************************************

! DEBUG
!write(6,*)' energy : enter '
!stop
! ENDDEBUG

!Test size of FFT grids (1 grid in norm-conserving, 2 grids in PAW)
 nfftotf=ngfftf(1)*ngfftf(2)*ngfftf(3)
 if ((psps%usepaw==1.and.pawfgr%nfft/=nfftf).or.(psps%usepaw==0.and.nfft/=nfftf)) then
  write(message, '(a,a,a,a)' ) ch10,&
&  ' energy :  BUG -',ch10,&
&  '  wrong values for nfft, nfftf !'
  call wrtout(6,message,'COLL')
  call leave_new('COLL')
 end if

 call timab(59,1,tsec)
!Init mpi_comm
 call xcomm_init(mpi_enreg,spaceComm)
!Init me
 call xme_init(mpi_enreg,me)
!Init master
 call xmaster_init(mpi_enreg,master)

!Compute gmet, gprimd and ucvol from rprimd
 call metric(gmet,gprimd,-1,rmet,rprimd,ucvol)

!Eventually compute "exact" gradients of compensation density (PAW)
 nhatgrdim=0
 if (psps%usepaw==1.and.usexcnhat>0.and.dtset%ixc>=11.and.dtset%ixc<=16) then
  nhatgrdim=1;allocate(nhatgr(nfft,nspden,3))
  call pawmknhat(dum,1,0,mpi_enreg,natom,nfft,ngfft,nhatgrdim,nspden,ntypat,paw_ij,&
&                pawang,pawfgrtab,nhatgr,nhat,pawtab,dtset%typat,ucvol)
 end if

!Compute Hxc potential from density (separate up and down if spin-polarized)
 option=1;nkxc=0;allocate(kxc(1,nkxc))
 call rhohxc(dtset,enxc,gsqcut,psps%usepaw,kxc,mpi_enreg,nfftf,&
&  ngfftf,nhat,psps%usepaw,nhatgr,nhatgrdim,nkxc,nspden,n3xccc,option,rhog,rhor,rprimd,strsxc,&
&  usexcnhat,vhartr,vxc,vxcavg,xccc3d)
 deallocate(kxc)

!Free memory used by compensation density gradients
 if (nhatgrdim>0) deallocate(nhatgr)

!Total local potential (for either spin channel) is
!Hartree + local psp + Vxc(spin), minus its mean
!(Note : this potential should agree with the input vtrial)
 do isppol=1,nsppol
  do ifft=1,nfftf
   vtrial(ifft,isppol)=vhartr(ifft)+vpsp(ifft)+vxc(ifft,isppol)
  end do
 end do

!Compute Hartree energy - use up+down rhor
 call dotprod_vn(1,rhor,ehart ,doti,mpi_enreg,nfftf,nfftotf,1,1,vhartr,ucvol)
 ehart=half*ehart

!Compute local psp energy - use up+down rhor
 if (optene/=1) call dotprod_vn(1,rhor,eei,doti,mpi_enreg,nfftf,nfftotf,1,1,vpsp,ucvol)

!Compute DC-xc energy - use up+down rhor
 if (optene>0) call dotprod_vn(1,rhor,enxcdc,doti,mpi_enreg,nfftf,nfftotf,nspden,1,vxc,ucvol)

 if (mkmem==0) then

! Read wavefunction file header
  call hdr_skip(wffnow,ierr)

! Define offsets, in case of MPI I/O
  formeig=0
  call xdefineOff(formeig,wffnow,mpi_enreg,dtset%nband,npwarr,nspinor,nsppol,nkpt)

  mcg_disk=mpw*nspinor*mband
  allocate(cg_disk(2,mcg_disk))

 end if

 eeig=zero
 ek=zero
 enl=zero
 bdtot_index=0
 icg=0


! DEBUG
!write(6,*)' energy : before loop over spins '
!stop
! ENDDEBUG

 n1=dtset%ngfft(1) ; n2=dtset%ngfft(2) ; n3=dtset%ngfft(3)
 n4=dtset%ngfft(4) ; n5=dtset%ngfft(5) ; n6=dtset%ngfft(6)
 nvloc=1
 if(nspden==4)nvloc=4
 allocate(vlocal(n4,n5,n6,nvloc),kg_k(3,mpw),cwavef(2,mpw*nspinor))

!Allocate the arrays of the Hamiltonian whose dimensions do not depend on k
 allocate(gs_hamk%atindx(natom),gs_hamk%atindx1(natom))
 allocate(gs_hamk%gbound(2*mgfft+8,2))
 allocate(gs_hamk%indlmn(6,psps%lmnmax,ntypat))
 allocate(gs_hamk%nattyp(ntypat))
 allocate(gs_hamk%phkxred(2,natom))
 allocate(gs_hamk%ph1d(2,3*(2*mgfft+1)*natom))
 allocate(gs_hamk%pspso(ntypat))
 allocate(gs_hamk%xred(3,natom))

!Initialize most of the Hamiltonian
 gs_hamk%atindx(:)  =atindx(:)
 gs_hamk%atindx1(:) =atindx1(:)
 gs_hamk%gmet(:,:)  =gmet(:,:)
 gs_hamk%gprimd(:,:)=gprimd(:,:)
 gs_hamk%indlmn(:,:,:)=psps%indlmn(:,:,:)
 gs_hamk%lmnmax     =psps%lmnmax
 gs_hamk%mgfft      =mgfft
 gs_hamk%mpsang     =mpsang
 gs_hamk%mpssoang   =psps%mpssoang
 gs_hamk%natom      =natom
 gs_hamk%nattyp(:)  =nattyp(:)
 gs_hamk%nfft       =nfft
 gs_hamk%ngfft(:)   =dtset%ngfft(:)
 gs_hamk%nloalg(:)  =dtset%nloalg(:)
 gs_hamk%ntypat      =ntypat
 gs_hamk%nvloc      =nvloc
 gs_hamk%n4         =n4
 gs_hamk%n5         =n5
 gs_hamk%n6         =n6
 gs_hamk%usepaw     =psps%usepaw
 gs_hamk%ph1d(:,:)  =ph1d(:,:)
 gs_hamk%pspso(:)   =psps%pspso(:)
 gs_hamk%ucvol      =ucvol
 gs_hamk%useylm     =psps%useylm
 gs_hamk%xred(:,:)  =xred(:,:)

!Special case of array ekb:
!  Not the same meaning for norm-cons. psps and paw
 if (psps%usepaw==1) then
  gs_hamk%dimekb1=psps%dimekb
  gs_hamk%dimekb2=natom
  allocate(gs_hamk%ekb(psps%dimekb,natom))
! ekb will receive Dij coeffs later (cf below)
 else
  gs_hamk%dimekb1=psps%dimekb
  gs_hamk%dimekb2=ntypat
  allocate(gs_hamk%ekb(psps%dimekb,ntypat))
  gs_hamk%ekb(:,:)=psps%ekb(:,:)
 end if

!PAW: Convert sij from pseudo format to nonlop format
 allocate(gs_hamk%sij(gs_hamk%dimekb1,ntypat))
 if (psps%usepaw==1) then
  do ii=1,ntypat
   gs_hamk%sij(1:pawtab(ii)%lmn2_size,ii)=pawtab(ii)%sij(:)
  end do
 end if

!LOOP OVER SPINS
 do isppol=1,nsppol

! PAW: transfer Dij coeffs into array ekb
  if (psps%usepaw==1) then
   do iatom=1,natom
!   Transfer Dij coeffs into array ekb
    dimdij=paw_ij(iatom)%lmn2_size
    do ilmn=1,dimdij
     gs_hamk%ekb(ilmn,iatom)=paw_ij(iatom)%dij(ilmn,isppol)
    end do
    if(dimdij<gs_hamk%dimekb1)then
     do ilmn=dimdij+1,gs_hamk%dimekb1
      gs_hamk%ekb(ilmn,iatom)=zero
     end do
    end if
   end do
  end if

! Rewind kpgsph data file if needed:
  if (mkmem==0) rewind dtfil%unkg
  if (mkmem==0.and.psps%useylm==1) rewind dtfil%unylm
  ikg=0

! Set up local potential vlocal with proper dimensioning, from vtrial
! Also take into account the spin.
  if(nspden/=4)then
   if (psps%usepaw==0) then
    call fftpac(isppol,nspden,n1,n2,n3,n4,n5,n6,dtset%ngfft,vtrial,vlocal,2)
   else
    allocate(cgrvtrial(nfft,nspden))
    call transgrid(mpi_enreg,nspden,-1,0,0,pawfgr,rhodum,rhodum,cgrvtrial,vtrial)
    call fftpac(isppol,nspden,n1,n2,n3,n4,n5,n6,dtset%ngfft,cgrvtrial,vlocal,2)
    deallocate(cgrvtrial)
   end if
  else
   allocate(vlocal_tmp(n4,n5,n6))
   do ispden=1,nspden
    call fftpac(ispden,nspden,n1,n2,n3,n4,n5,n6,dtset%ngfft,vtrial,vlocal_tmp,2)
    vlocal(:,:,:,ispden)=vlocal_tmp(:,:,:)
   end do
   deallocate(vlocal_tmp)
  end if

! Loop over k points
  do ikpt=1,nkpt
   nband_k=dtset%nband(ikpt+(isppol-1)*nkpt)
   istwf_k=dtset%istwfk(ikpt)
   npw_k=npwarr(ikpt)

   if(mpi_enreg%paral_compil_kpt==1)then
!   Skip this k-point if not the proper processor
    if (mpi_enreg%parareel == 0) then
     if(minval(abs(mpi_enreg%proc_distrb(ikpt,1:nband_k,isppol) &
&        -mpi_enreg%me))/=0) then
      resid(1+bdtot_index : nband_k+bdtot_index) = zero
      bdtot_index=bdtot_index+nband_k
      cycle
     end if
    else
     if(mpi_enreg%proc_distrb_para(mpi_enreg%ipara,ikpt) &
&         /= mpi_enreg%me) then
      resid(1+bdtot_index : nband_k+bdtot_index) = zero
      bdtot_index=bdtot_index+nband_k
      cycle
     end if
    end if
   end if

!  Continue to initialize the Hamiltonian
   gs_hamk%istwf_k    =istwf_k
   gs_hamk%npw        =npw_k

   allocate(eig_k(nband_k),occ_k(nband_k),resid_k(nband_k))
   allocate(ylm_k(npw_k,mpsang*mpsang*psps%useylm))
   resid_k(:)=0.0d0
   kpoint(:)=dtset%kptns(:,ikpt)
   gs_hamk%kpoint(:)  =kpoint(:)
   occ_k(:)=occ(1+bdtot_index:nband_k+bdtot_index)
   eig_k(:)=eigen(1+bdtot_index:nband_k+bdtot_index)
   if (minval(eig_k)>1.d100) eig_k=zero

   if (mkmem==0) then

!   Read sphere data centered at k in dtfil%unkg, then k+g data
    call rdnpw(ikpt,isppol,nband_k,npw_k,nspinor,0,dtfil%unkg)
    read (dtfil%unkg) kg_k(1:3,1:npw_k)
    call sphereboundary(gs_hamk%gbound,istwf_k,kg_k,mgfft,npw_k)

!   Eventually read spherical harmonics
    if (psps%useylm==1) then
     read(dtfil%unylm)
     read(dtfil%unylm) ((ylm_k(muig,ilm),muig=1,npw_k),ilm=1,mpsang*mpsang)
    end if

!   Read the wavefunction block for ikpt,isppol
    tim_rwwf=3
    allocate(eig_dum(mband),kg_dum(3,0),occ_dum(mband))
    call rwwf(cg_disk,eig_dum,0,0,0,ikpt,isppol,kg_dum,mband,mcg_disk,nband_k,nband_k,&
&    npw_k,nspinor,occ_dum,-2,0,tim_rwwf,wffnow)
    deallocate(eig_dum,kg_dum,occ_dum)

   else

    kg_k(:,1:npw_k)=kg(:,1+ikg:npw_k+ikg)
    call sphereboundary(gs_hamk%gbound,istwf_k,kg_k,mgfft,npw_k)
    if (psps%useylm==1) then
     do ilm=1,mpsang*mpsang
      ylm_k(1:npw_k,ilm)=ylm(1+ikg:npw_k+ikg,ilm)
     end do
    end if

!  End if for choice governed by mkmem
   end if

!  Compute kinetic energy
   allocate(kinpw(npw_k))
   call mkkin(dtset%ecut,dtset%ecutsm,dtset%effmass,gmet,kg_k,kinpw,kpoint,npw_k)

   index=1+icg
   do iband=1,nband_k
!   Compute kinetic energy of each band

    if(mpi_enreg%paral_compil_kpt==1)then
!    Skip this band if not the proper processor
     if (mpi_enreg%paralbd >1) then
      if(mpi_enreg%proc_distrb(ikpt,iband,isppol)/= mpi_enreg%me) then
       index=index+npw_k*nspinor
       cycle
      end if
     end if
    end if

    if(mkmem/=0)then
     do ig=1,npw_k*nspinor
      cwavef(1,ig)=cg(1,index)
      cwavef(2,ig)=cg(2,index)
      index=index+1
     end do
    else
     do ig=1,npw_k*nspinor
      cwavef(1,ig)=cg_disk(1,index)
      cwavef(2,ig)=cg_disk(2,index)
      index=index+1
     end do
    end if
    call meanvalue_g(ekk,kinpw,0,istwf_k,mpi_enreg,npw_k,nspinor,cwavef)
    ek=ek+dtset%wtk(ikpt)*occ_k(iband)*ekk
   end do

   enlk=zero
   eeigk=zero

!  Compute nonlocal form factors ffnl at all (k+G):
   ider=0;dimffnl=1;nkpg=0
   allocate(ffnl(npw_k,dimffnl,psps%lmnmax,ntypat))
   call mkffnl(psps%dimekb,dimffnl,psps%ekb,ffnl,psps%ffspl,&
&   gmet,gprimd,ider,ider,psps%indlmn,kg_k,kpg_dum,kpoint,psps%lmnmax,&
&   psps%lnmax,mpsang,psps%mqgrid_ff,nkpg,&
&   npw_k,ntypat,psps%pspso,psps%qgrid_ff,rmet,&
&   psps%usepaw,psps%useylm,ylm_k,ylmgr_dum)

!  Allocate the arrays phkxred and ph3d, compute phkxred
!  and eventually ph3d.
   do ia=1,natom
    iatom=atindx(ia)
    arg=two_pi*(kpoint(1)*xred(1,ia)+kpoint(2)*xred(2,ia)+kpoint(3)*xred(3,ia))
    gs_hamk%phkxred(1,iatom)=cos(arg)
    gs_hamk%phkxred(2,iatom)=sin(arg)
   end do
   if(dtset%nloalg(1)<=0)then
!   Only the allocation, not the precomputation.
    matblk=dtset%nloalg(4)
    allocate(ph3d(2,npw_k,matblk))
   else
!   Here, allocation as well as precomputation
    matblk=natom
    allocate(ph3d(2,npw_k,matblk))
    call ph1d3d(1,natom,kg_k,kpoint,matblk,natom,npw_k,n1,n2,n3,&
&               gs_hamk%phkxred,ph1d,ph3d)
   end if
   gs_hamk%matblk=matblk

!  DEBUG
!  write(6,*)' energy : before nonlop '
!  stop
! ENDDEBUG

!   Compute nonlocal psp energy - Norm-conserving only
   if (psps%usepaw==0) then
    do iband=1,nband_k
     if(mpi_enreg%paral_compil_kpt==1)then
!     Skip this band if not the proper processor
      if (mpi_enreg%paralbd >1) then
       if(mpi_enreg%proc_distrb(ikpt,iband,isppol)/= mpi_enreg%me) then
        cycle
       end if
      end if
     end if

     if(mkmem/=0)cwavef(:,1:npw_k*nspinor)=&
&     cg(:,1+(iband-1)*npw_k*nspinor+icg:iband*npw_k*nspinor+icg)
     if(mkmem==0)cwavef(:,1:npw_k*nspinor)=&
&     cg_disk(:,1+(iband-1)*npw_k*nspinor:iband*npw_k*nspinor)

     choice=1 ; signs=1 ; idir=0 ; nnlout=1 ; tim_nonlop=3
     paw_opt=gs_hamk%usepaw
     call nonlop(atindx1,choice,gs_hamk%dimekb1,gs_hamk%dimekb2,dimffnl,dimffnl,&
&     gs_hamk%ekb,enlout,ffnl,ffnl,gmet,gprimd,idir,psps%indlmn,istwf_k,&
&     kg_k,kg_k,kpg_dum,kpg_dum,kpoint,kpoint,psps%lmnmax,matblk,mgfft,mpi_enreg,psps%mpsang,&
&     psps%mpssoang,natom,nattyp,dtset%ngfft,nkpg,nkpg,dtset%nloalg,nnlout,npw_k,npw_k,&
&     nspinor,ntypat,paw_opt,gs_hamk%phkxred,gs_hamk%phkxred,ph1d,&
&     ph3d,ph3d,psps%pspso,signs,nonlop_dum,nonlop_dum,tim_nonlop,ucvol,&
&     psps%useylm,cwavef,cwavef)

     enlk=enlk+occ_k(iband)*enlout(1)
     eeigk=eeigk+occ_k(iband)*eig_k(iband)

    end do

    enl=enl+dtset%wtk(ikpt)*enlk
    eeig=eeig+dtset%wtk(ikpt)*eeigk

!   DEBUG
!   write(6,*)' energy : after nonlop '
!   stop
!  ENDDEBUG

   else

!  In PAW: enl is zero and only eeig has to be computed
    do iband=1,nband_k
!    Compute kinetic energy of each band

     if(mpi_enreg%paral_compil_kpt==1)then
!     Skip this band if not the proper processor
      if (mpi_enreg%paralbd >1) then
       if(mpi_enreg%proc_distrb(ikpt,iband,isppol)/= mpi_enreg%me) then
        index=index+npw_k*nspinor
        cycle
       end if
      end if
     end if

     eeigk=eeigk+occ_k(iband)*eig_k(iband)
    end do

    eeig=eeig+dtset%wtk(ikpt)*eeigk

   end if

!  Compute residual of each band (for informative purposes)
   if(mkmem/=0)then
    mcg=mpw*nspinor*mband*mkmem*nsppol
    call mkresi(cg,dimffnl,eig_k,ffnl,dtfil%filstat,&
&    gs_hamk,icg,ikpt,isppol,&
&    kg_k,kinpw,psps%lmnmax,matblk,mcg,mgfft,mpi_enreg,&
&    mpsang,psps%mpssoang,natom,&
&    nband_k,npw_k,&
&    nspinor,ntypat,nvloc,n4,n5,n6,&
&    ph3d,dtset%prtvol,&
&    resid_k,psps%usepaw,vlocal)
   else if(mkmem==0)then
    mcg=mpw*nspinor*mband
    call mkresi(cg_disk,dimffnl,eig_k,ffnl,dtfil%filstat,&
&    gs_hamk,icg,ikpt,isppol,&
&    kg_k,kinpw,psps%lmnmax,matblk,mcg,mgfft,mpi_enreg,&
&    mpsang,psps%mpssoang,natom,&
&    nband_k,npw_k,&
&    nspinor,ntypat,nvloc,n4,n5,n6,&
&    ph3d,dtset%prtvol,&
&    resid_k,psps%usepaw,vlocal)

   end if
   resid(1+bdtot_index : nband_k+bdtot_index) = resid_k(:)

   deallocate(eig_k,occ_k,resid_k)

!  DEBUG
!  write(6,*)' isppol,ikpt',isppol,ikpt
!  write(6,*)resid(1+bdtot_index:nband_k+bdtot_index)
!  ENDDEBUG

   bdtot_index=bdtot_index+nband_k

   if (mkmem/=0) then
!   Handle case in which kg, cg, are kept in core
    icg=icg+npw_k*nspinor*nband_k
    ikg=ikg+npw_k
   end if

   deallocate(ffnl,kinpw,ph3d)
   deallocate(ylm_k)

! End loops on isppol and ikpt
  end do
 end do

 if(mpi_enreg%paral_compil_kpt==1)then
!BEGIN TF_CHANGES
  call leave_test(mpi_enreg)
!END TF_CHANGES
  write(message,*) 'energy: loop on k-points and spins done in parallel'
  call wrtout(06,message,'COLL')
 end if

 deallocate(gs_hamk%atindx,gs_hamk%atindx1)
 deallocate(gs_hamk%ekb)
 deallocate(gs_hamk%gbound)
 deallocate(gs_hamk%indlmn)
 deallocate(gs_hamk%nattyp)
 deallocate(gs_hamk%phkxred)
 deallocate(gs_hamk%pspso)
 deallocate(gs_hamk%ph1d)
 deallocate(gs_hamk%sij)
 deallocate(gs_hamk%xred)

 if(mkmem==0)then
  deallocate(cg_disk)
 end if

!DEBUG
!write(6,*)' energy : after loop on kpts and spins '
!stop
!ENDDEBUG

 if(mpi_enreg%paral_compil_kpt==1)then
! Accumulate enl and ek on all proc.
  allocate(buffer(3+mband*nkpt*nsppol))
  buffer(1)=enl ; buffer(2)=ek ; buffer(3)=eeig
  do iresid=1,mband*nkpt*nsppol
   buffer(iresid+3)=resid(iresid)
  end do
  call timab(48,1,tsec)
  call xsum_mpi(buffer,spaceComm,ierr)
  call timab(48,2,tsec)
  enl=buffer(1) ; ek=buffer(2) ; eeig=buffer(3)
  do iresid=1,mband*nkpt*nsppol
   resid(iresid)=buffer(iresid+3)
  end do
  deallocate(buffer)
 end if

!Compute total (free) energy
 if (optene==0.or.optene==2) then
  if (psps%usepaw==0) then
   etotal=ek+ehart+enxc+eei+eew+eii+enl
  else
   etotal=ek+ehart+enxc+eei+eew+eii+epaw
  end if
 else if (optene==1.or.optene==3) then
  if (psps%usepaw==0) then
   etotal=eeig-ehart+enxc-enxcdc+eew+eii
  else
   etotal=eeig-ehart+enxc-enxcdc+eew+eii+epawdc
  end if
 end if
 if(occopt>=3 .and. occopt<=7) etotal=etotal-tsmear*entropy

!Compute new charge density based on incoming wf
!Keep rhor and rhog intact for later use e.g. in stress.
!This is not done in case of PAW (to be modified later)
 if (psps%usepaw==0) then
  tim_mkrho=3 ;
  call mkrho(cg,densymop_gs,dtset,irrzon,dtset%istwfk,kg,mband,mgfft,mkmem,&
&  mpi_enreg,mpw,dtset%nband,nfft,dtset%ngfft,&
&  nkpt,npwarr,nspden,nspinor,nsppol,nsym,occ,phnons,rhog,rhor,&
&  dtset%symafm,tim_mkrho,ucvol,dtfil%unkg,wffnow,dtset%wtk)
 end if

 write(message, '(a,a,a,a)' )ch10, &
& ' energy: COMMENT -',ch10,&
& '  New density rho(r) made from input wfs'
 call wrtout(06,message,'COLL')

 call timab(59,2,tsec)

 deallocate(cwavef,kg_k,vlocal)

end subroutine energy
!!***
