!{\src2tex{textfont=tt}}
!!****f* ABINIT/etotfor
!! NAME
!! etotfor
!!
!! FUNCTION
!! This routine is called to compute the total energy and various parts of it.
!! The routine computes -if requested- the forces.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (XG, GMR, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.
!!
!! INPUTS
!!  atindx(natom)=index table for atoms (see scfcv.f)
!!  dtset <type(dataset_type)>=all input variables in this dataset
!!   | berryopt  = 4: electric field is on -> add the contribution of the
!!   |                - \Omega E.P term to the total energy
!!   |          /= 4: electric field is off
!!   | efield = cartesian coordinates of the electric field in atomic units
!!   | iatfix(3,natom)=1 for frozen atom along some direction, 0 for unfrozen
!!   | ionmov=governs the movement of atoms (see help file)
!!   | iprcch=governs the mixed electronic-atomic part of the preconditioner
!!   | natom=number of atoms in cell.
!!   | nconeq=number of atomic constraint equations
!!   | nspden=number of spin-density components
!!   | nsym=number of symmetry elements in space group
!!   | occopt=option for occupancies
!!   | prtvol=integer controlling volume of printed output
!!   | tsmear=smearing energy or temperature (if metal)
!!   | typat(natom)=type integer for each atom in cell
!!   | wtatcon(3,natom,nconeq)=weights for atomic constraints
!!  eei=local psp energy (hartree)
!!  eeig=Sum of the eigenvalues - Band energy (Hartree)
!!  eew=Ewald energy (hartree)
!!  efield_dot = reciprocal lattice coordinates of the electric field
!!  ehart=Hartree part of total energy (hartree units)
!!  eii=psp core-core energy
!!  ek=kinetic energy part of total energy.
!!  enl=nonlocal pseudopotential part of total energy.
!!  entropy=entropy due to the occupation number smearing (if metal)
!!  enxc=exchange-correlation energy (hartree)
!!  enxcdc=exchange-correlation double-counting energy (hartree)
!!  epaw=PAW spherical part energy
!!  epawdc=PAW spherical part double-counting energy
!!  grewtn(3,natom)=grads of Ewald energy (hartree)
!!  gsqcut=cutoff on (k+G)^2 (bohr^-2)
!!  indsym(4,nsym,natom)=indirect indexing array for atom labels
!!  kxc(nfft,nkxc)=exchange-correlation kernel, needed only if nkxc>0
!!  mgfft=maximum size of 1D FFTs
!!  mpi_enreg=informations about MPI parallelization
!!  nattyp(ntypat)=number of atoms of each type
!!  nfft=(effective) number of FFT grid points (for this processor)
!!  ngfft(18)=contain all needed information about 3D FFT, see ~abinit/doc/input_variables/vargs.htm#ngfft
!!  nkxc=second dimension of the array kxc, see rhohxc.f for a description
!!  ntypat=number of types of atoms in unit cell.
!!  nvresid(nfft,nspden)=potential or density residual
!!  n1xccc=dimension of xccc1d ; 0 if no XC core correction is used
!!  n3xccc=dimension of the xccc3d array (0 or nfft).
!!  optene=option for the computation of total energy (0=direct scheme; 1=double-counting scheme)
!!  optforces=option for the computation of forces
!!  optres=0 if residual array (nvresid) contains the potential residual
!!        =1 if residual array (nvresid) contains the density residual
!!  paw_ij(natom) <type(paw_ij_type)>=paw arrays given on (i,j) channels
!!  pawang <type(pawang_type)>=paw angular mesh and related data
!!  pawfgrtab(natom) <type(pawfgrtab_type)>=atomic data given on fine rectangular grid
!!  pawtab(ntypat) <type(pawtab_type)>=paw tabulated starting data
!!  pel = reduced coordinates of the electronic polarization
!!        (pel does not take into account the factor 1/ucvol)
!!  ph1d(2,3*(2*mgfft+1)*natom)=1-dim phase (structure factor) information.
!!  pion = reduced coordinates of the ionic polarization
!!        (pel does not take into account the factor 1/ucvol)
!!  psps <type(pseudopotential_type)>=variables related to pseudopotentials
!!  rhog(2,nfft)=array for Fourier transform of electron density
!!  rhor(nfft,nspden)=array for electron density in electrons/bohr**3
!!  rprimd(3,3)=dimensional primitive translations in real space (bohr)
!!  symrec(3,3,nsym)=symmetry operations in reciprocal space
!!  ucvol = unit cell volume (Bohr**3)
!!  usepaw= 0 for non paw calculation; =1 for paw calculation
!!  vhartr(nfft)=array for holding Hartree potential
!!  vpsp(nfft)=array for holding local psp
!!  vxc(nfft,nspden)=array for holding XC potential
!!  xred(3,natom)=reduced dimensionless atomic coordinates
!!
!! OUTPUT
!!  deltae=change in total energy
!!         between the previous and present SCF cycle
!!  enefield=the term of the energy functional that depends explicitely
!!           on the electric field:  enefield = -ucvol*E*P
!!  etotal=total energy (hartree)
!!  ===== if optforces==1
!!   diffor=maximum absolute change in component of forces between present and previous SCF cycle.
!!   favg(3)=mean of fcart before correction for translational symmetry
!!   fcart(3,natom)=cartesian forces from fred (hartree/bohr)
!!   fred(3,natom)=symmetrized form of grtn (grads of Etot) (hartree)
!!   gresid(3,natom)=forces due to the residual of the density/potential
!!   grhat(3,natom)=-PAW ONLY- Gradients of energy due to compensation charge
!!   grhf(3,natom)=Hellman-Feynman derivatives of the total energy
!!   grxc(3,natom)=d(Exc)/d(xred) derivatives (0 without core charges)
!!   maxfor=maximum absolute value of force
!!   synlgr(3,natom)=symmetrized form of grads of Enl (hartree)
!!
!! SIDE EFFECTS
!! Input/Output:
!!  elast=previous value of the energy,
!!        needed to compute deltae, then updated.
!!  ===== if optforces==1
!!   forold(3,natom)=cartesian forces of previous SCF cycle (hartree/bohr)
!!   grnl(3*natom)=gradients of Etot due to nonlocal contributions
!!                 Input for norm-conserving psps, output for PAW
!!
!! NOTES
!!  In case of PAW calculations:
!!    All computations are done on the fine FFT grid.
!!    All variables (nfft,ngfft,mgfft) refer to this fine FFT grid.
!!    All arrays (densities/potentials...) are computed on this fine FFT grid.
!!  ! Developpers have to be careful when introducing others arrays:
!!      they have to be stored on the fine FFT grid.
!!  In case of norm-conserving calculations the FFT grid is the usual FFT grid.
!!
!! PARENTS
!!      scfcv
!!
!! CHILDREN
!!      forces,pawgrnhat,timab
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine etotfor(atindx,deltae,diffor,dtset,&
&  eei,eeig,eew,efield_dot,ehart,eii,ek,elast,enefield,&
&  enl,entropy,enxc,enxcdc,epaw,epawdc,etotal,favg,fcart,forold,fred,gresid,grewtn,grhat,grhf,grnl,&
&  grxc,gsqcut,indsym,kxc,maxfor,mgfft,mpi_enreg,nattyp,&
&  nfft,ngfft,nkxc,ntypat,nvresid,n1xccc,n3xccc,optene,optforces,optres,&
&  paw_ij,pawang,pawfgrtab,pawtab,pel,ph1d,pion,psps,rhog,rhor,rprimd,symrec,synlgr,&
&  ucvol,usepaw,vhartr,vpsp,vxc,xred)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_00basis
 use interfaces_03paw
 use interfaces_05common, except_this_one => etotfor
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: mgfft,n1xccc,n3xccc,nfft,nkxc,ntypat,optene,optforces
 integer,intent(in) :: optres,usepaw
 real(dp),intent(in) :: eei,eeig,eew,ehart,eii,ek,enl,entropy,enxc,enxcdc,epaw
 real(dp),intent(in) :: epawdc,gsqcut,ucvol
 real(dp),intent(inout) :: elast
 real(dp),intent(out) :: deltae,diffor,enefield,etotal,maxfor
 type(MPI_type),intent(inout) :: mpi_enreg
 type(dataset_type),intent(in) :: dtset
 type(pawang_type),intent(in) :: pawang
 type(pseudopotential_type),intent(in) :: psps
!arrays
 integer,intent(in) :: atindx(dtset%natom),indsym(4,dtset%nsym,dtset%natom)
 integer,intent(in) :: nattyp(ntypat),ngfft(18),symrec(3,3,dtset%nsym)
 real(dp),intent(in) :: efield_dot(3),grewtn(3,dtset%natom),kxc(nfft,nkxc)
 real(dp),intent(in) :: pel(3),ph1d(2,3*(2*mgfft+1)*dtset%natom),pion(3)
 real(dp),intent(in) :: rhog(2,nfft),rhor(nfft,dtset%nspden),rprimd(3,3)
 real(dp),intent(in) :: vhartr(nfft),vpsp(nfft),vxc(nfft,dtset%nspden)
 real(dp),intent(inout) :: forold(3,dtset%natom),grnl(3*dtset%natom)
 real(dp),intent(inout) :: nvresid(nfft,dtset%nspden),xred(3,dtset%natom)
 real(dp),intent(out) :: favg(3),fcart(3,dtset%natom),fred(3,dtset%natom)
 real(dp),intent(out) :: gresid(3,dtset%natom),grhat(3,dtset%natom)
 real(dp),intent(out) :: grhf(3,dtset%natom),grxc(3,dtset%natom)
 real(dp),intent(out) :: synlgr(3,dtset%natom)
 type(paw_ij_type),intent(in) :: paw_ij(dtset%natom)
 type(pawfgrtab_type),intent(in) :: pawfgrtab(dtset%natom)
 type(pawtab_type),intent(in) :: pawtab(ntypat)

!Local variables-------------------------------
!scalars
 integer :: choice,dimnhat,ifft,ispden,option
 real(dp) :: eent,epel,epion
 character(len=500) :: message
!arrays
 real(dp) :: str_dum(6),tsec(2)
 real(dp),allocatable :: nhat_dum(:,:),work(:,:)

! *********************************************************************

!DEBUG
!write(6,*)' etotfor : enter'
!ENDDEBUG

 call timab(80,1,tsec)

!When the finite-temperature VG broadening scheme is used,
!the total entropy contribution "tsmear*entropy" has a meaning,
!and gather the two last terms of Eq.8 of the VG paper
!Warning : might have to be changed for fixed moment calculations
 if(dtset%occopt>=3 .and. dtset%occopt<=7) then
  eent=-dtset%tsmear*entropy
 else
  eent=zero
 end if

!Turn it into an electric enthalpy, by adding both ionic and electronic contributions
 enefield = zero
 if (dtset%berryopt == 4) then
! First, ionic contribution (epion):
  epion = -1_dp*(efield_dot(1)*pion(1) + efield_dot(2)*pion(2) + &
&          efield_dot(3)*pion(3))
  enefield = enefield + epion
! Now, electronic contribution (epel):
  epel = -1_dp*(efield_dot(1)*pel(1) + efield_dot(2)*pel(2) + &
&               efield_dot(3)*pel(3))
  enefield = enefield + epel
 end if

!Compute total (free) energy by direct scheme
 if (optene==0) then
  if (usepaw==0) then
   etotal=ek+ehart+enxc+eei+eew+eii+eent+enefield+enl
  else
   etotal=ek+ehart+enxc+eei+eew+eii+eent+enefield+epaw
  end if
 end if

!Compute total (free) energy by double-counting scheme
 if (optene==1) then
  if (usepaw==0) then
   etotal=eeig-ehart+enxc-enxcdc+eew+eii+eent+enefield
  else
   etotal=eeig-ehart+enxc-enxcdc+eew+eii+eent+enefield+epawdc
  end if
 end if

!Compute energy residual
 deltae=etotal-elast
 elast=etotal
!DEBUG
!write(6,*) 'eeig-ehart+enxc-enxcdc+eew+eii+eent+enefield+epawdc',eeig,ehart,enxc,enxcdc,eew,eii,eent,enefield,epawdc
!ENDEBUG
 call timab(80,2,tsec)

!------Compute forces-----------------------------------------------------

 if (optforces==1) then

! PAW: compute compensation charge (nhat) and rhoij parts of forces
  if (usepaw==1) then
   choice=2;dimnhat=0;allocate(nhat_dum(1,dimnhat),work(nfft,dtset%nspden))
   do ispden=1,min(dtset%nspden,2)
!$OMP PARALLEL DO PRIVATE(ifft) &
!$OMP&SHARED(ispden,nfft,dtset%nspden,vhartr,work,vpsp,vxc)
    do ifft=1,nfft
     work(ifft,ispden)=vhartr(ifft)+vpsp(ifft)+vxc(ifft,ispden)
    end do
! $OMP END PARALLEL DO
   end do
!DEBUG
! write(6,*) 'in etotfor call pawgrnhat',mpi_enreg%paral_level
!ENDDEBUG
   call pawgrnhat(choice,dimnhat,grhat,grnl,mpi_enreg,dtset%natom,nfft,ngfft,nhat_dum,&
&                 str_dum,str_dum,dtset%nspden,dtset%nsym,ntypat,paw_ij,pawang,pawfgrtab,&
&                 pawtab,rprimd,symrec,dtset%typat,work)
   deallocate(nhat_dum,work)
  else
   grhat=zero
  end if

! Note that rho(G) (no core density) must be correct when forces is called below
!DEBUG
!write(6,*) 'in etotfor call forces',mpi_enreg%paral_level
!ENDDEBUG
  call forces(atindx,diffor,dtset,favg,fcart,forold,fred,gresid,grewtn,grhat,&
&             grhf,grnl,grxc,gsqcut,indsym,kxc,maxfor,mgfft,mpi_enreg,&
&             n1xccc,n3xccc,nattyp,nfft,ngfft,nkxc,ntypat,&
&             nvresid,optres,ph1d,psps,rhog,rhor,rprimd,symrec,synlgr,vxc,xred)

! Returned fred are full symmetrized gradients of Etotal
! wrt reduced coordinates xred, d(Etotal)/d(xred)
! Forces are contained in array fcart

 else   ! if optforces==0
  fcart=zero
  fred=zero
  favg=zero
  diffor=zero
  gresid=zero
  grhat=zero
  grhf=zero
  maxfor=zero
  synlgr=zero
 end if

 call timab(80,2,tsec)

!DEBUG
!write(6,*)' etotfor : exit '
!write(6,*) etotal
!call leave_new("COLL")
!stop
!ENDDEBUG

end subroutine etotfor
!!***
