!{\src2tex{textfont=tt}}
!!****f* ABINIT/getgsc
!! NAME
!! getgsc
!!
!! FUNCTION
!! 1) Compute <G|S|C> for every wave function |C> expressed
!!    in reciprocal space.
!!    S is the overlap matrix: H|Psi_i>=E_i.S|Psi_i>
!! 2) Eventually (if option=1):
!!    Orthogonalize wave functions according to overlap matrix
!!    (i.e. <Psi_i|S|Psi_j>=delta_ij).
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (FJ, MT, MB)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!  atindx(natom)=index table for atoms (see scfcv.f)
!!  atindx1(natom)=index table for atoms, inverse of atindx (see scfcv.f)
!!  dtfil <type(datafiles_type)>=variables related to files
!!  dtset <type(dataset_type)>=all input variables for this dataset
!!  gmet(3,3)=reciprocal space metric tensor in bohr**-2
!!  gprimd(3,3)=dimensional reciprocal space primitive translations
!!  kg(3,mpw*mkmem)=reduced planewave coordinates
!!  mband=maximum number of bands
!!  mgfft=maximum size of 1D FFTs
!!  mkmem =number of k points which can fit in memory; set to 0 if use disk
!!  mpi_enreg=informations about MPI parallelization
!!  mpsang= 1+maximum angular momentum for nonlocal pseudopotentials
!!  mpw=maximum dimensioned size of npw
!!  natom=number of atoms in cell
!!  nattyp(ntypat)= # atoms of each type
!!  nkpt=number of k points
!!  npwarr(nkpt)=number of planewaves in basis at this k point
!!  nspinor=number of spinorial components of the wavefunctions
!!  nsppol=1 for unpolarized, 2 for spin-polarized
!!  ntypat=number of types of atoms in unit cell
!!  option=1 if the WF have to be orthogonalized and updated according to S
!!  pawtab(ntypat*usepaw) <type(pawtab_type)>=paw tabulated starting data
!!  ph1d(2,3*(2*mgfft+1)*natom)=1-dim structure factor phase information
!!  psps <type(pseudopotential_type)>=variables related to pseudopotentials
!!  rmet(3,3)=real space metric (bohr**2)
!!  tim_getgsc=timing code of the calling routine
!!  ucvol=unit cell volume in bohr**3
!!  xred(3,natom)=reduced dimensionless atomic coordinates
!!  ylm(mpw*mkmem,mpsang*mpsang)= real spherical harmonics for each G and k point
!!
!! OUTPUT
!!  gsc(2,mpw*nspinor*mband*mkmem*nsppol*usepaw)=matrix elements <G|S|C> (S=overlap matrix)
!!
!! SIDE EFFECTS
!!  cg(2,mpw*nspinor*mband*mkmem*nsppol)=planewave coefficients of wavefunctions
!!  swfnow=struct infos for <g|S|c> disk file
!!  wffnew,wffnow=struct infos for wf disk files
!!
!! NOTES
!!  Source code for compatibility with parareel algorithm has been inserted
!!  but not tested (probably wrong) !
!!
!! PARENTS
!!      gstate,scfcv
!!
!! CHILDREN
!!      hdr_clean,hdr_io,hdr_io_netcdf,ini_wf_netcdf,leave_test,mkffnl,mkkin
!!      nonlop,orthon,ph1d3d,rdnpw,rwwf,sphereboundary,status,timab,wrtout
!!      xcomm_init,xdefineoff,xmaster_init,xme_init
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine getgsc(atindx,atindx1,cg,dtfil,dtset,gmet,gprimd,gsc,&
&  kg,mband,mgfft,mkmem,mpi_enreg,mpsang,mpw,natom,&
&  nattyp,nkpt,npwarr,nspinor,nsppol,ntypat,option,pawtab,ph1d,psps,rmet,&
&  swfnow,tim_getgsc,ucvol,wffnew,wffnow,xred,ylm)

 use defs_basis
 use defs_datatypes
#if defined HAVE_NETCDF
 use netcdf
#endif

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_00basis
 use interfaces_01managempi
 use interfaces_01util
 use interfaces_02ffts
 use interfaces_03nonlocal
 use interfaces_04iowfdenpot
 use interfaces_04wfs
#else
 use defs_interfaces
#endif
!End of the abilint section

 implicit none

!Arguments -------------------------------
!scalars
 integer,intent(in) :: mband,mgfft,mkmem,mpsang,mpw,natom,nkpt,nsppol,ntypat
 integer,intent(in) :: option,tim_getgsc
 integer,intent(inout) :: nspinor
 real(dp),intent(in) :: ucvol
 type(MPI_type),intent(inout) :: mpi_enreg
 type(datafiles_type),intent(in) :: dtfil
 type(dataset_type),intent(in) :: dtset
 type(pseudopotential_type),intent(in) :: psps
 type(wffile_type),intent(inout) :: swfnow,wffnew,wffnow
!arrays
 integer,intent(in) :: atindx(natom),atindx1(natom),kg(3,mpw*mkmem)
 integer,intent(in) :: nattyp(ntypat),npwarr(nkpt)
 real(dp),intent(in) :: gmet(3,3),gprimd(3,3),ph1d(2,3*(2*mgfft+1)*natom)
 real(dp),intent(in) :: rmet(3,3),xred(3,natom),ylm(mpw*mkmem,mpsang*mpsang)
 real(dp),intent(inout) :: cg(2,mpw*nspinor*mband*mkmem*nsppol)
 real(dp),intent(out) :: gsc(2,mpw*nspinor*mband*mkmem*nsppol*psps%usepaw)
 type(pawtab_type),intent(in) :: pawtab(ntypat*psps%usepaw)

!Local variables-------------------------------
!scalars
 integer,parameter :: level=7
 integer :: choice,counter,dimffnl,fform,formeig,ia,iatom,iband,icg,ider,idir
 integer :: ierr,iexit,ig,ii,ikg,ikpt,ilm,index,ipw,isppol,istwf_k,itypat,iwav
 integer :: master,matblk,mcg,mcg_disk,me,muig,n1,n2,n3,nband_k,nkpg,nnlout
 integer :: npw_k,paw_opt,rdwr,signs,spaceComm,spaceComm_old,tim_nonlop
 integer :: tim_rwwf
 real(dp) :: arg
 character(len=500) :: message
 type(hdr_type) :: hdr0
 type(wffile_type) :: wfftmp
!arrays
 integer,allocatable :: gbound(:,:),kg_dum(:,:),kg_k(:,:)
 real(dp) :: enlout(1),kpoint(3),tsec(2),ylmgr_dum(1)
 real(dp),allocatable :: cg_disk(:,:),cwavef(:,:),eig_k(:),enl_dum(:,:)
 real(dp),allocatable :: ffnl(:,:,:,:),gsc_disk(:,:),gsc_k(:,:),kinpw(:)
 real(dp),allocatable :: kpg_dum(:,:),occ_k(:),ph3d(:,:,:),phkxred(:,:)
 real(dp),allocatable :: sij(:,:),ylm_k(:,:)

! *********************************************************************

!DEBUG
!write(6,*)' getgsc : enter'
!ENDDEBUG

 call timab(561+tim_getgsc,1,tsec)
 call status(0,dtfil%filstat,iexit,level,'enter         ')

!Init parallelism
 call xcomm_init(mpi_enreg,spaceComm)
 call xme_init(mpi_enreg,me)
 call xmaster_init(mpi_enreg,master)

!Initialize some variables
 n1=dtset%ngfft(1) ; n2=dtset%ngfft(2) ; n3=dtset%ngfft(3)
 mcg=mpw*nspinor*mband*mkmem*nsppol
 mcg_disk=mpw*nspinor*mband
 icg=0
 gsc(:,:)=zero

!Common parameters for "nonlop" calls
 choice=1 ; signs=2 ; idir=0 ; tim_nonlop=1 ; nnlout=1 ; paw_opt=3

!Prepare wf files if mkmem==0
 if (mkmem==0) then

! Read header info for old wf file
  rdwr=1
  if (wffnow%accesswff /= 2) then
   call hdr_io(fform,hdr0,rdwr,wffnow)
#if defined HAVE_NETCDF
  else if (wffnow%accesswff == 2) then
   call hdr_io_netcdf(fform,hdr0,rdwr,wffnow)
#endif
  end if
! Define offsets, in case of MPI I/O
  formeig=0
  call xdefineOff(formeig,wffnow,mpi_enreg,dtset%nband,npwarr,nspinor,nsppol,nkpt)

! Write header info for new wf file
  if (option==1) then
   rdwr=2 ; fform=2
   if (wffnew%accesswff /= 2) then
    call hdr_io(fform,hdr0,rdwr,wffnew)
#if defined HAVE_NETCDF
   else if (wffnew%accesswff == 2) then
    call hdr_io_netcdf(fform,hdr0,rdwr,wffnew)
    call ini_wf_netcdf(mpw,wffnew%unwff,0)
#endif
   end if
   call hdr_io(fform,hdr0,rdwr,wffnew)
!  Define offsets, in case of MPI I/O
   formeig=0
   call xdefineOff(formeig,wffnew,mpi_enreg,dtset%nband,npwarr,nspinor,nsppol,nkpt)
  end if

! Write header info on temporary file <g|S|c>
  rdwr=2 ; fform=2
  call hdr_io(fform,hdr0,rdwr,swfnow)
! Define offsets, in case of MPI I/O
  formeig=0
  call xdefineOff(formeig,swfnow,mpi_enreg,dtset%nband,npwarr,nspinor,nsppol,nkpt)

! Clean the temporary header
  call hdr_clean(hdr0)

! Allocate the work space
  allocate(cg_disk(2,mcg_disk))
  allocate(gsc_disk(2,mcg_disk));gsc_disk(:,:)=zero

 end if

!Uncompress Sij from pseudo format to nonlop format
 allocate(sij(psps%dimekb,ntypat))
 sij(:,:)=zero
 do itypat=1,ntypat
  sij(1:pawtab(itypat)%lmn2_size,itypat)=pawtab(itypat)%sij(:)
 end do

!LOOP OVER SPINS
 do isppol=1,nsppol

! Rewind temporary files if needed
  if (mkmem==0) rewind dtfil%unkg
  if (mkmem==0) rewind dtfil%unylm

  ikg=0

! BIG FAT k POINT LOOP
  do ikpt=1,nkpt

   counter=100*ikpt+isppol
   call status(counter,dtfil%filstat,iexit,level,'loop ikpt     ')
   nband_k=dtset%nband(ikpt+(isppol-1)*nkpt)
   istwf_k=dtset%istwfk(ikpt)
   npw_k=npwarr(ikpt)

   if(mpi_enreg%paral_compil_kpt==1)then
    if (mpi_enreg%parareel == 0) then
!BEGIN TF_CHANGES
     if(minval(abs(mpi_enreg%proc_distrb(ikpt,1:nband_k,isppol) &
&        -me))/=0) then
!END TF_CHANGES
!     Skip the rest of the k-point loop
      cycle
     end if
    else
     if(mpi_enreg%proc_distrb_para(mpi_enreg%ipara,ikpt) &
&         /= mpi_enreg%me) then
!     Skip the rest of the k-point loop
      cycle
     end if
    end if
   end if

   allocate(gbound(2*mgfft+8,2))
   allocate(ylm_k(npw_k,mpsang*mpsang))
   allocate(kg_k(3,npw_k))

   kpoint(:)=dtset%kptns(:,ikpt)

   if (mkmem==0) then
!   Read (k+G) basis sphere data (same for each spin)
    call rdnpw(ikpt,isppol,nband_k,npw_k,nspinor,0,dtfil%unkg)
!   Read k+g data
    read (dtfil%unkg) kg_k(1:3,1:npw_k)
    call sphereboundary(gbound,istwf_k,kg_k,mgfft,npw_k)
!   Read spherical harmonics
    read(dtfil%unylm)
    read(dtfil%unylm) ((ylm_k(muig,ilm),muig=1,npw_k),ilm=1,mpsang*mpsang)
!   Read the wavefunction block for ikpt,isppol
    call status(counter,dtfil%filstat,iexit,level,'read wfs      ')
    tim_rwwf=1
    allocate(eig_k(mband),kg_dum(3,0),occ_k(mband))
    call rwwf(cg_disk,eig_k,0,0,0,ikpt,isppol,kg_dum,mband,mcg_disk,nband_k,nband_k,&
&    npw_k,nspinor,occ_k,-2,0,tim_rwwf,wffnow)
    deallocate(kg_dum)

   else

    kg_k(:,1:npw_k)=kg(:,1+ikg:npw_k+ikg)
    call sphereboundary(gbound,istwf_k,kg_k,mgfft,npw_k)
    do ilm=1,mpsang*mpsang
     ylm_k(1:npw_k,ilm)=ylm(1+ikg:npw_k+ikg,ilm)
    end do

!  End if for choice governed by mkmem
   end if

!  Allocate the arrays phkxred and ph3d, compute phkxred
!  and eventually ph3d
   allocate(phkxred(2,natom))
   do ia=1,natom
    iatom=atindx(ia)
    arg=two_pi*(kpoint(1)*xred(1,ia)+kpoint(2)*xred(2,ia)+kpoint(3)*xred(3,ia))
    phkxred(1,iatom)=cos(arg)
    phkxred(2,iatom)=sin(arg)
   end do

   if(dtset%nloalg(1)<=0)then
!   Here, only the allocation, not the precomputation
    matblk=dtset%nloalg(4)
    allocate(ph3d(2,npw_k,matblk))
   else
!   Here, allocation as well as precomputation
    matblk=natom
    allocate(ph3d(2,npw_k,matblk))
    call ph1d3d(1,natom,kg_k,kpoint,matblk,natom,npw_k,n1,n2,n3,&
&               phkxred,ph1d,ph3d)
   end if

!  Filter the wavefunctions for large modified kinetic energy
   allocate(kinpw(npw_k))
   call mkkin(dtset%ecut,dtset%ecutsm,dtset%effmass,gmet,kg_k,kinpw,kpoint,npw_k)
   iwav=0
   if (mkmem/=0) iwav=iwav+icg
   do iband=1,nband_k
    do ig=1,npw_k
     if(kinpw(ig)>huge(0.0d0)*1.d-11)then
      if (mkmem==0) then
       cg_disk(1,ig+iwav)=zero
       cg_disk(2,ig+iwav)=zero
      else
       cg(1,ig+iwav)=zero
       cg(2,ig+iwav)=zero
      end if
     end if
    end do
    iwav=iwav+npw_k*nspinor
   end do

!  Compute nonlocal form factors ffnl at all (k+G)
   call status(0,dtfil%filstat,iexit,level,'call mkffnl   ')
   ider=0;dimffnl=1;nkpg=0
   allocate(ffnl(npw_k,dimffnl,psps%lmnmax,ntypat))
   call mkffnl(psps%dimekb,dimffnl,psps%ekb,ffnl,psps%ffspl,&
&   gmet,gprimd,ider,ider,psps%indlmn,kg_k,kpg_dum,kpoint,psps%lmnmax,&
&   psps%lnmax,psps%mpsang,psps%mqgrid_ff,nkpg,npw_k,ntypat,&
&   psps%pspso,psps%qgrid_ff,rmet,psps%usepaw,psps%useylm,ylm_k,ylmgr_dum)

!  Compute <g|S|c> elements for this k point
   allocate(cwavef(2,npw_k*nspinor),gsc_k(2,npw_k*nspinor))
   do iband=1,nband_k
    gsc_k(:,:)=zero
    if (mkmem==0) then
     cwavef(:,:)=cg_disk(:,1+(iband-1)*npw_k*nspinor:iband*npw_k*nspinor)
    else
     cwavef(:,:)=cg(:,1+(iband-1)*npw_k*nspinor+icg:iband*npw_k*nspinor+icg)
    end if
!MOST UGLY PATCH, TO BE DELETED
    if(mpi_enreg%mode_para=='b') then
     spacecomm_old=mpi_enreg%comm_fft
     mpi_enreg%comm_fft=mpi_enreg%commcart
    end if
    call nonlop(atindx1,choice,psps%dimekb,0,dimffnl,dimffnl,&
&               enl_dum,enlout,ffnl,ffnl,gmet,gprimd,idir,psps%indlmn,istwf_k,&
&               kg_k,kg_k,kpg_dum,kpg_dum,kpoint,kpoint,psps%lmnmax,matblk,mgfft,mpi_enreg,&
&               psps%mpsang,psps%mpssoang,natom,nattyp,dtset%ngfft,nkpg,nkpg,dtset%nloalg,&
&               nnlout,npw_k,npw_k,dtset%nspinor,ntypat,paw_opt,phkxred,phkxred,&
&               ph1d,ph3d,ph3d,psps%pspso,signs,sij,gsc_k,tim_nonlop,&
&               ucvol,psps%useylm,cwavef,cwavef)
    if(mpi_enreg%mode_para=='b') then
     mpi_enreg%comm_fft=spacecomm_old
    end if
    if (mkmem==0) then
     gsc_disk(:,1+(iband-1)*npw_k*nspinor:iband*npw_k*nspinor)=gsc_k(:,:)
    else
     gsc(:,1+(iband-1)*npw_k*nspinor+icg:iband*npw_k*nspinor+icg)=gsc_k(:,:)
    end if
   end do

   deallocate(ffnl,gbound,ph3d,phkxred,ylm_k,cwavef,gsc_k)

   if (option==1) then
!   Orthogonalize the wave functions according to new overlap matrix
    if (mpi_enreg%paral_fft == 1) then
      mpi_enreg%num_group_fft=ikpt + (isppol-1)*nkpt
     else
      mpi_enreg%num_group_fft=0
    end if
    if (mkmem==0) then
     call orthon(icg,istwf_k,mcg_disk,mpi_enreg,npw_k*nspinor,&
&                    nband_k,gsc_disk,1,cg_disk)
!    Write new wavefunctions to disk if needed (if mkmem==0)
     tim_rwwf=1
     call rwwf(cg_disk,eig_k,0,0,0,ikpt,isppol,kg_k,mband,mcg_disk,nband_k,&
&              nband_k,npw_k,nspinor,occ_k,2,1,tim_rwwf,wffnew)
    else
     call orthon(icg,istwf_k,mcg,mpi_enreg,npw_k*nspinor,&
&                nband_k,gsc,1,cg)
    end if
   end if

!  Filter <g|S|c> for large modified kinetic energy
   iwav=0
   if (mkmem/=0) iwav=iwav+icg
   do iband=1,nband_k
    do ig=1,npw_k
     if(kinpw(ig)>huge(0.0d0)*1.d-11)then
      if (mkmem==0) then
       gsc_disk(1,ig+iwav)=zero
       gsc_disk(2,ig+iwav)=zero
      else
       gsc(1,ig+iwav)=zero
       gsc(2,ig+iwav)=zero
      end if
     end if
    end do
    iwav=iwav+npw_k*nspinor
   end do
   deallocate(kinpw)

!  Write new <g|S|c> to disk if needed (if mkmem==0)
   if (mkmem==0) then
    tim_rwwf=1
    call rwwf(gsc_disk,eig_k,0,0,0,ikpt,isppol,kg_k,mband,mcg_disk,nband_k,&
&             nband_k,npw_k,nspinor,occ_k,2,1,tim_rwwf,swfnow)
    deallocate(eig_k,occ_k)

   else

!   Shift array memory (if mkmem/=0)
    icg=icg+npw_k*nspinor*nband_k
    ikg=ikg+npw_k
   end if

   deallocate(kg_k)

! End big k point loop
  end do

!End loop over spins
 end do

 if(mpi_enreg%paral_compil_kpt==1)then
  if (mpi_enreg%parareel == 0) then
!BEGIN TF_CHANGES
   call leave_test(mpi_enreg)
!END TF_CHANGES
  end if
  write(message,*) 'getgsc: loop on k-points and spins done in parallel'
  call wrtout(06,message,'COLL')
 end if

!Deallocate temporary storage
 deallocate(sij)
 if(mkmem==0) deallocate(cg_disk,gsc_disk)

!Rotate labels of disk files when wf i/o is used
 if (mkmem==0.and.option==1) then
  wfftmp=wffnow ; wffnow=wffnew ; wffnew=wfftmp
 end if

 call timab(561+tim_getgsc,2,tsec)

!DEBUG
!write(6,*)' getgsc : exit'
!ENDDEBUG

end subroutine getgsc
!!***
