!{\src2tex{textfont=tt}}
!!****f* ABINIT/ionion_realspace
!!
!! NAME
!! ionion_realspace
!!
!! FUNCTION
!! Compute the ion/ion interaction energies and forces in real space
!! case. Use ewald() instead if computations are done in reciprocal
!! space since it also includes the correction for the shift done in
!! potentials calculations and includes replica interactions.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (DCA, XG, GMR)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!! natom=number of atoms in unit cell
!! ntypat=number of type of atoms
!! rmet(3,3)=metric tensor in real space (bohr^2)
!! typat(natom)=integer label of each type of atom (1,2,...)
!! xred(3,natom)=relative coords of atoms in unit cell (dimensionless)
!! zion(ntypat)=charge on each type of atom (real number)
!!
!! OUTPUT
!! eew=final ion/ion energy in hartrees
!! grewtn(3,natom)=grads of ion/ion wrt xred(3,natom), hartrees.
!!
!! PARENTS
!!      setvtr
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine ionion_realSpace(eew,grewtn,natom,ntypat,rprimd,typat,xred,zion)

 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_02geometry
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: natom,ntypat
 real(dp),intent(out) :: eew
!arrays
 integer,intent(in) :: typat(natom)
 real(dp),intent(in) :: rprimd(3,3),zion(ntypat)
 real(dp),intent(inout) :: xred(3,natom)
 real(dp),intent(out) :: grewtn(3,natom)

!Local variables-------------------------------
!scalars
 integer :: ia1,ia2,iatom,igeo
 real(dp) :: r
 character(len=500) :: message
!arrays
 real(dp),allocatable :: grew_cart(:,:),xcart(:,:)

! *************************************************************************

!DEBUG
!write(6,*)' ionion_realSpace : enter '
!stop
!ENDDEBUG

  ! Store xcart for each atom
  allocate(xcart(3, natom))
  call xredxcart(natom, 1, rprimd, xcart, xred)

  ! Allocate temporary array to store cartesian gradients.
  allocate(grew_cart(3, natom))

  ! Summing the interaction between ions.
  eew = 0.d0
  do ia1 = 1, natom, 1
    do ia2 = ia1 + 1, natom, 1
      r = sqrt((xcart(1, ia1) - xcart(1, ia2)) ** 2 + &
             & (xcart(2, ia1) - xcart(2, ia2)) ** 2 + &
             & (xcart(3, ia1) - xcart(3, ia2)) ** 2)
      eew = eew + zion(typat(ia1)) * zion(typat(ia2)) / r
    end do
  end do

  ! Summing the forces for each atom
  do ia1 = 1, natom, 1
    grew_cart(:, ia1) = 0.d0
    do ia2 = 1, natom, 1
      if (ia1 /= ia2) then
        r = (xcart(1, ia1) - xcart(1, ia2)) ** 2 + &
          & (xcart(2, ia1) - xcart(2, ia2)) ** 2 + &
          & (xcart(3, ia1) - xcart(3, ia2)) ** 2
        do igeo = 1, 3, 1
          grew_cart(igeo, ia1) = grew_cart(igeo, ia1) - (xcart(igeo, ia1) - xcart(igeo, ia2)) * &
                               & zion(typat(ia1)) * zion(typat(ia2)) / (r ** 1.5d0)
        end do
      end if
    end do
  end do

  deallocate(xcart)

  ! Transform cartesian gradients to reduced gradients.
  do iatom = 1, natom, 1
    do igeo = 1, 3, 1
      grewtn(igeo, iatom) = rprimd(1, igeo) * grew_cart(1, iatom) + &
                          & rprimd(2, igeo) * grew_cart(2, iatom) + &
                          & rprimd(3, igeo) * grew_cart(3, iatom)
    end do
  end do
  deallocate(grew_cart)

end subroutine ionion_realSpace
!!***
