!{\src2tex{textfont=tt}}
!!****f* ABINIT/cmevxclda
!! NAME
!! cmevxclda
!!
!! FUNCTION
!! Calculate < nk | Vxc | nk' > for the input wavefunctions.
!!
!! COPYRIGHT
!! Copyright (C) 1999-2006 ABINIT group (GMR, VO, LR, RWG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!  dtset <type(dataset_type)>=all input variables in this dataset
!!  ixc = choice for the exchange-correlation potential.
!!  mpi_enreg = informations about MPI parallelization.
!!  nbnds = number of bands.
!!  ngfft1,ngfft2,ngfft3 = size of the FFT grid.
!!  nkibz = number of k-points in the irreducible Brillouin zone.
!!  nr = total number of points on the FFT grid.
!!  nrb = effective number of points on the FFT grid.
!!  rho(nr) = the charge density on the FFT grid.
!!  rprimd(3,3) = dimensional real space primitive translations (bohr).
!!  wfr(nr,nbnds,nkibz) = wavefunctions on the FFT grid for nbnds bands at nkibz irreducible k-points.
!!
!! OUTPUT
!!  vxcme(nbnds,nkibz) = < nk | Vxc | nk' > for nbnds bands at nkibz irreducible k-points.
!!
!! TODO
!!  The < nk | Vxc | nk > could be computed during the KSS run and stored in the KSS file.
!!  They would then be calculated on the full FFT grid rather than on the usually coarse GW FFT grid.
!!
!! PARENTS
!!      sigma
!!
!! CHILDREN
!!      cvxclda
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine cmevxclda(dtset,ixc,mpi_enreg,nbnds,ngfft1,ngfft2,ngfft3,nkibz,nr,nrb,rho,rprimd,wfr,vxcme)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_05gw, except_this_one => cmevxclda
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: ixc,nbnds,ngfft1,ngfft2,ngfft3,nkibz,nr,nrb
 type(MPI_type),intent(inout) :: mpi_enreg
 type(dataset_type),intent(in) :: dtset
!arrays
 real(dp),intent(in) :: rho(nr),rprimd(3,3)
 complex,intent(in) :: wfr(nr,nbnds,nkibz)
 complex,intent(out) :: vxcme(nbnds,nbnds,nkibz)

!Local variables ------------------------------
!scalars
 integer :: ib,ik,ir,jb
 complex :: ct
 logical,parameter :: verbose=.false.
!arrays
 real(dp),allocatable :: vxclda(:)

!*************************************************************************

 vxcme(:,:,:)=0.0

!calculate Vxc in real space
 allocate(vxclda(nr))
 call cvxclda(dtset,ixc,mpi_enreg,ngfft1,ngfft2,ngfft3,nr,rho,rprimd,vxclda)

!calculate < nk | Vxc | nk >
 do ik=1,nkibz
  do ib=1,nbnds
   do jb=1,nbnds
    ct=0.0
    do ir=1,nr
     ct=ct+conjg(wfr(ir,ib,ik))*vxclda(ir)*wfr(ir,jb,ik)
    end do
    vxcme(ib,jb,ik)=ct/nrb
   end do
  end do
 end do

 if (verbose) then
  do ik=1,nkibz
   write(std_out,'(a,i3,/,a)') ' < nk | Vxc | nk > for k-point ',ik,'  n         <Vxc>'
   do ib=1,nbnds
     write(std_out,'(i3,2(e14.6,2x))') ib,vxcme(ib,ib,ik)*Ha_eV
   end do
  end do
 end if

 write(std_out,*)

 deallocate(vxclda)

end subroutine cmevxclda

!!***
