!{\src2tex{textfont=tt}}
!!****f* ABINIT/crho
!! NAME
!! crho
!!
!! FUNCTION
!! Calculate the charge density rho on the FFT grid.
!!
!! COPYRIGHT
!! Copyright (C) 1999-2006 ABINIT group (GMR, VO, LR, RWG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!  irottb(nr,nop) = symmetry operations on the FFT grid.
!!  nbnds = number of bands.
!!  ninv = 2 if inversion symmetry, 1 otherwise.
!!  nkbz = number of k-points in the full Brillouin zone.
!!  nkibz = number of k-points in the irreducible Brillouin zone.
!!  nkibzm = maximum number of k-points in the irreducible Brillouin zone.
!!  nop = number of symmetry operations.
!!  nr = total number of points on the FFT grid.
!!  nrb = effective number of points on the FFT grid.
!!  occ(nkibzm,nbnds) = occupation numbers for nbnds bands at nkibzm irreducible k-points.
!!  ucvol = unit cell volume.
!!  wfr(nr,nbnds,nkibz) = wavefunctions on the FFT grid for nbnds bands at nkibz irreducible k-points.
!!  wtk(nkibz) = irreducible k-points weights.
!!
!! OUTPUT
!!  omegaplasma = the plasma frequency.
!!  rho(nr) = the density on the FFT grid.
!!
!! PARENTS
!!      sigma
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine crho(irottb,nbnds,ninv,nkbz,nkibz,nkibzm,nop,nr,nrb,occ,omegaplasma,rho,ucvol,wfr,wtk)

 use defs_basis

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nbnds,ninv,nkbz,nkibz,nkibzm,nop,nr,nrb
 real(dp),intent(in) :: ucvol
 real(dp),intent(out) :: omegaplasma
!arrays
 integer,intent(in) :: irottb(nr,nop)
 real(dp),intent(in) :: occ(nkibzm,nbnds),wtk(nkibz)
 real(dp),intent(out) :: rho(nr)
 complex,intent(in) :: wfr(nr,nbnds,nkibz)

!Local variables ------------------------------
!scalars
 integer :: ib,ik,iop,ir
 real(dp) :: fact,rhoav,rs,tnepuc
!arrays
 real(dp),allocatable :: rho2(:)

!*************************************************************************

 allocate(rho2(nr))

!print info
 write(std_out,'(a)') ' calculating charge density...'

!zero charge density
 rho(:)=0.0
 rho2(:)=0.0

!loop over k-points in IBZ
 do ik=1,nkibz
  ib=1
! skip the higher bands if occupation is less than tol8
  do while ((abs(occ(ik,ib))>tol8).and.(ib<=nbnds))
   do ir=1,nr
    rho2(ir)=rho2(ir)+occ(ik,ib)*conjg(wfr(ir,ib,ik))*wfr(ir,ib,ik)*wtk(ik)
   end do
   ib=ib+1
  end do
 end do

!loop over symmetry operations, symmetrising rho
 fact=real(ninv)/(nkbz*ucvol)
 do iop=1,nop
  do ir=1,nr
   rho(ir)=rho(ir)+fact*rho2(irottb(ir,iop))
  end do
 end do

!write total charge
!open(unit=66,file='rho')
!write(66,'(f7.4)') (rho(ir),ir=1,nr)
!close(66)

!calculate total number of electrons as a check
 tnepuc=0.0
 do ir=1,nr
  tnepuc=tnepuc+rho(ir)
 end do
 tnepuc=tnepuc*ucvol/nrb
 write(std_out,'(a,f7.4)') ' total number of electrons per unit cell = ',tnepuc
 write(ab_out,'(a,f7.4)') ' total number of electrons per unit cell = ',tnepuc
 rhoav=tnepuc/ucvol
 write(std_out,'(a,f7.4)') ' average of density, n = ',rhoav
 write(ab_out,'(a,f7.4)') ' average of density, n = ',rhoav
 rs=(three/(four_pi*rhoav))**third
 write(std_out,'(a,f7.4)') ' r_s = ',rs
 write(ab_out,'(a,f7.4)') ' r_s = ',rs
 omegaplasma=sqrt(four_pi*rhoav)
 write(std_out,'(a,f7.4,a)') ' omega_plasma = ',omegaplasma*Ha_eV,' [eV]'
 write(ab_out,'(a,f7.4,a)') ' omega_plasma = ',omegaplasma*Ha_eV,' [eV]'
 write(std_out,*)
 write(ab_out,*)

 deallocate(rho2)

end subroutine crho

!!***
