!{\src2tex{textfont=tt}}
!!****f* ABINIT/density
!! NAME
!! density
!!
!! FUNCTION
!!  Compute the real-space density rho2, as well as
!!  the average density to initialize omegaplasma
!!
!!
!! COPYRIGHT
!! Copyright (C) 1999-2006 ABINIT group (GMR, VO, LR, RWG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!  irottb(nr,nop)=(to be described)
!!  nbnds=number of bands
!!  nkibzm=maximum number of k point in IBZ
!!  nkibz=number of k point in IBZ
!!  nkbz=number of k point in BZ
!!  ngfft1,ngfft1a,ngfft2,ngfft3=FFT grid dimensions
!!  ninv=2 if time-reversal symmetry is used, 1 if not
!!  nop=number of symmetry operations
!!  nr=number of point in FFT grid
!!  occ(nkibzm,nbnds)=occupation numbers, for each k point in IBZ, and each band
!!  ucvol=unit cell volume
!!  wtk(nkibz)=weights for k points (input variable)
!!
!! OUTPUT
!!  rho2(nr)=real-space density
!!  omegaplasma=Drude plasma frequency
!!
!! PARENTS
!!      screening
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine density(nbnds,nkibzm,nkibz,nkbz,nop,ninv,nr,ngfft1,ngfft1a,&
& ngfft2,ngfft3,irottb,ucvol,wtk,occ,rho2,omegaplasma)

 use defs_basis

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nbnds,ngfft1,ngfft1a,ngfft2,ngfft3,ninv,nkbz,nkibz
 integer,intent(in) :: nkibzm,nop,nr
 real(dp),intent(in) :: ucvol
 real(dp),intent(out) :: omegaplasma
!arrays
 integer,intent(in) :: irottb(nr,nop)
 real(dp),intent(in) :: occ(nkibzm,nbnds),wtk(nkibz)
 real(dp),intent(out) :: rho2(nr)

!Local variables-------------------------------
!scalars
 integer,parameter :: unitwfr=26
 integer :: ib,ik,iop,ir,istat
 real(dp) :: fact,rhoav,rs,tnepuc
!arrays
 real(dp),allocatable :: rho(:)
 complex,allocatable :: wfnk(:,:)

! *************************************************************************

 write(*,*) 'calculating charge density'

 allocate(rho(nr),stat=istat)
 if(istat/=0) stop 'out of memory'
 allocate(wfnk(nr,nbnds),stat=istat)
 if(istat/=0) stop 'out of memory'

!zero charge density
 do ir=1,nr
  rho(ir)=zero
  rho2(ir)=zero
 end do

!loop over k-points in IBZ
 do ik=1,nkibz
  read(unitwfr,rec=ik) ((wfnk(ir,ib),ir=1,nr),ib=1,nbnds)
  ib=1
! skip the higher bands if occupation is less than tol8
  do while ((abs(occ(ik,ib))>tol8).and.(ib<=nbnds))
   do ir=1,nr
    rho(ir)=rho(ir)+occ(ik,ib)*conjg(wfnk(ir,ib))*wfnk(ir,ib)*wtk(ik)
   end do
   ib=ib+1
  end do
 end do

 deallocate(wfnk)

!loop over symmetry operations, symmetrising rho
 if(ninv==1) then
  fact=1.0/(nkbz*ucvol)
 else
! factor of 2 is for inversion
  fact=2.0/(nkbz*ucvol)
 end if
 do iop=1,nop
  do ir=1,nr
   rho2(ir)=rho2(ir)+fact*rho(irottb(ir,iop))
  end do
 end do

 deallocate(rho)

!write total charge:
!open(unit=66,file='rho')
!write(66,'(f7.4)') (rho2(ir),ir=1,nr)
!close(66)

!calculate total number of electrons as a check
 tnepuc=0.0
 do ir=1,nr
  tnepuc=tnepuc+rho2(ir)
 end do
 tnepuc=tnepuc*ucvol/(ngfft1*ngfft2*ngfft3)
 write(6,'(a,f7.4)') ' total number of electrons per unit cell = ',tnepuc
 write(ab_out,'(a,f7.4)') ' total number of electrons per unit cell = ',tnepuc
 rhoav=tnepuc/ucvol
 write(6,'(a,f7.4)') ' average of density, n = ',rhoav
 write(ab_out,'(a,f7.4)') ' average of density, n = ',rhoav
 rs=(3.0/(4.0*(4.0*atan(1.0))*rhoav))**(1.0/3.0)
 write(6,'(a,f7.4)') ' r_s = ',rs
 write(ab_out,'(a,f7.4)') ' r_s = ',rs
 omegaplasma=sqrt(4.0*(4.0*atan(1.0))*rhoav)
 write(6,'(a,f7.4,a)') ' omega_plasma = ',omegaplasma*Ha_eV,'[eV]'
 write(ab_out,'(a,f7.4,a)') ' omega_plasma = ',omegaplasma*Ha_eV,'[eV]'
 write(6,*)
 write(ab_out,*)

end subroutine density
!!***
