!{\src2tex{textfont=tt}}
!!****f* ABINIT/rdkss
!! NAME
!! rdkss
!!
!! FUNCTION
!!  read _KSS file
!!
!! COPYRIGHT
!! Copyright (C) 1999-2006 ABINIT group (GMR, VO, LR, RWG, MM, XG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!  dtfil=filenames (here, only dtfil%filkss is used)
!!  mpsangx= 1+maximum angular momentum for nonlocal pseudopotential
!!  natomx=maximum number of atoms
!!  nbx=maximum number of bands
!!  ngx=maximum number of G vectors
!!  nkx=maximum number of k points
!!  nopx=maximum number of symmetry operations
!!  ntypatx=maximum number of types of atoms
!!  nwx=maximum number of planewaves
!!
!! OUTPUT
!!  a1(3), a2(3), a3(3)= real-space lattice vectors in au
!!  en(nkx,nbx)=KS energies for each k point and each band
!!  gvec(3,ngx)=integer coordinates of G vectors
!!  hdr=header of the KSS file
!!  kibz(3,nkx)=coordinates of k points in IBZ
!!  nel=number of electrons
!!  oc(nkx,nbx)=occupation numbers for each k point and each band
!!  op(3,3,nop)=symmetry operations in reciprocal space
!!  tit(2)(len=80)= (to be described)
!!  typat(natomx)=types of atoms
!!  vkb(nwx,ntypatx,mpsangx,nkx)=KB projector function
!!  vkbd(nwx,ntypatx,mpsangx,nkx)=derivative of the KB projector function in reciprocal space
!!  vkbsign(mpsangx,ntypatx)=sign of each KB dyadic product
!!  wf(nwx,nbx,nkx)=wavefunctions, for each band and each k point
!!  xred(3,natomx)=coordinates of atoms in real space
!!
!! PARENTS
!!      screening,sigma
!!
!! CHILDREN
!!      hdr_io,hdr_io_netcdf
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine rdkss(dtfil,hdr,nopx,nbx,nkx,ngx,nwx,tit,a1,a2,a3,op,gvec,kibz,en, &
& oc,wf,ntypatx,natomx,mpsangx,typat,xred,vkbsign,vkb,vkbd,nel)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01managempi
#else
 use defs_interfaces
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: mpsangx,natomx,nbx,ngx,nkx,nopx,ntypatx,nwx
 integer,intent(out) :: nel
 type(datafiles_type),intent(in) :: dtfil
 type(hdr_type),intent(out) :: hdr
!arrays
 integer,intent(out) :: gvec(3,ngx),typat(natomx)
 real(dp),intent(out) :: a1(3),a2(3),a3(3),en(nkx,nbx),kibz(3,nkx),oc(nkx,nbx)
 real(dp),intent(out) :: op(3,3,nopx),vkb(nwx,ntypatx,mpsangx,nkx)
 real(dp),intent(out) :: vkbd(nwx,ntypatx,mpsangx,nkx),vkbsign(mpsangx,ntypatx)
 real(dp),intent(out) :: xred(3,natomx)
 complex,intent(out) :: wf(nwx,nbx,nkx)
 character(len=80),intent(out) :: tit(2)

!Local variables-------------------------------
!scalars
 integer,parameter :: unitkss=23
 integer :: day,fform,i,ia,ib,ibp,ig,ik,il,ipa,is,ish,isym,isymend,isymin,it
 integer :: itypat,j,month,mpsang,natom,nbandkss,nela,nkpt,npwkss,nsh,nsym2
 integer :: ntypat,osl,rdwr,readnetcdf,year
 complex :: cdum
 logical,parameter :: DEBUG=.false.
!arrays
 integer,allocatable :: shlim(:),symrel2(:,:,:)
 real(dp),allocatable :: energyd(:),tnons2(:,:),vkbdb(:,:,:),vkbdd(:,:,:)
 real(dp),allocatable :: vkbsignd(:,:)
 complex,allocatable :: wfgs(:)
!no_abirules
 real :: cinf,csup,dum,einf,esup,sum
 real,allocatable :: vkbsigns(:,:)
 real,allocatable :: vkbds(:,:,:),vkbs(:,:,:)
 real,allocatable :: energys(:)
 complex (kind(0.0d0)), allocatable :: wfgd(:)

! *************************************************************************

!DEBUG
!write(6,*)' rdkss : enter '
!ENDDEBUG

 open(unit=unitkss,file=dtfil%filkss,form='unformatted')

!Read the header of the GS wavefunction file
 rdwr=1
 readnetcdf = 0 ! should become input parameter and make KSS a netcdf file as well
 if (readnetcdf == 0) then
  call hdr_io(fform,hdr,rdwr,unitkss)
 else if (readnetcdf == 1) then
  call hdr_io_netcdf(fform,hdr,rdwr,unitkss)
 end if


 if(fform/=502.and.fform/=602) then
  print *, 'ERROR: fform/=502,602', fform
  stop 'fform/=502,602'
 end if

 if(fform==502) then
  print *, 'Kohn Sham Structure double precision file'
 else if(fform==602) then
  print *, 'Kohn Sham Structure single precision file'
 end if

!Write the header of the GS wavefunction file
 rdwr=4
 call hdr_io(fform,hdr,rdwr,6)
      natom=hdr%natom
      ntypat=hdr%ntypat
      nkpt=hdr%nkpt

      a1(:) = hdr%rprimd(:,1)
      a2(:) = hdr%rprimd(:,2)
      a3(:) = hdr%rprimd(:,3)

      if(ntypat/=ntypatx) stop 'ntypat/=ntypatx'
      if(natom/=natomx) stop 'natom/=natomx'
      if(nkpt/=nkx) stop 'nkpt/=nkx'

      typat(1:natomx)=hdr%typat(1:natomx)
      kibz(:,1:nkx) = hdr%kptns(:,1:nkx)

      oc(:,:) = 0.0
      do ik = 1, nkpt
        do ib = 1, hdr%nband(ik)
          i = (ik-1)*hdr%nband(ik) + ib
          if(ib<=nbx) oc(ik,ib) = hdr%occ(i)
        end do
      end do

      do itypat = 1, ntypat
        if(hdr%pspso(itypat)/=1) then
          print *, '**ERROR: pspso value not programmed'
          print *, 'pspso = ', hdr%pspso(itypat)
          stop 'pspso /= 1'
        end if
      end do

 nel = 0
 do ia = 1, hdr%natom
  it = hdr%typat(ia)
  nela = hdr%znucltypat(it)  ! if it''s not a pseudoatom, add all electron
  do ipa = 1, hdr%npsp
    if(hdr%znuclpsp(ipa) == hdr%znucltypat(it)) nela = hdr%zionpsp(ipa)
  end do
  nel = nel + nela
 end do

 xred(:,1:natomx) =hdr%xred(:,1:natomx)

!End of treatment of the header

 read(unitkss) tit(1)
 read(unitkss) tit(2)
 write(*,'(1x,a79/1x,a79)') tit(1)(:79), tit(2)(:79)

 read(unitkss) nsym2, nbandkss, npwkss, nsh, mpsang
 print *, 'number of electrons ', nel
 print *, 'number of symmetries without inversion ', nsym2
 print *, 'number of bands ', nbandkss
 print *, 'number of plane waves ', npwkss
 print *, 'number of shells ', nsh
!      print *, 'mpsang ', mpsang
      if(mpsang/=mpsangx) stop 'mpsang/=mpsangx'
      if(nopx>nsym2) stop 'nopx>nsym2'
      if(ngx>npwkss) stop 'ngx>npwkss'
      if(nwx>npwkss) stop 'nwx>npwkss'
      if(nbx>nbandkss) stop 'nbx>nbandkss'

      allocate(symrel2(3,3,nsym2))
      read(unitkss) (((symrel2(j,i,isym),i=1,3),j=1,3),isym=1,nsym2)
      op(:,:,1:nopx) = symrel2(:,:,1:nopx)

      print *, 'symmetry operations [reciprocal lattice units]:'
      do isymin=1,nopx,8
        isymend=isymin+7
        if(isymend.gt.nopx) isymend=nopx
        do i=1,3
          write(6,2995) ((symrel2(i,j,isym),j=1,3),isym=isymin,isymend)
2995      format(7(3i3,1x),3i3)
        end do
        print *
      end do
      allocate(tnons2(3,nsym2))
      read(unitkss) ((tnons2(i,isym),i=1,3),isym=1,nsym2)
      print *, 'translations '
      write(*,'(10x,3f7.3)') tnons2

      allocate(shlim(nsh))

      read(unitkss) ((gvec(i,ig),i=1,3),ig=1,ngx)

      read(unitkss) (shlim(i),i=1,nsh)

      write(*,'(" shell limit G-vectors [reduced coordinates]")')
      osl = 1
      do ish = 1, nsh
        if(shlim(ish)>ngx) osl = shlim(ish)+1
        write(*,'(i6,i6,4(i5," (",3i3,")"),20(/12x,4(i5," (",3i3,")")))')&
&        ish, shlim(ish), (ig,(gvec(i,ig),i=1,3),ig=osl,shlim(ish))
        osl = shlim(ish)+1
      end do
      print *


      if(fform==502) then
        allocate(vkbsignd(mpsang,ntypat))
        read(unitkss) ((vkbsignd(il,is),il=1,mpsang),is=1,ntypat)
        vkbsign(1:mpsang,1:ntypat) = vkbsignd(1:mpsang,1:ntypat)
        deallocate(vkbsignd)
      else if(fform==602) then
        allocate(vkbsigns(mpsang,ntypat))
        read(unitkss) ((vkbsigns(il,is),il=1,mpsang),is=1,ntypat)
        vkbsign(1:mpsang,1:ntypat) = vkbsigns(1:mpsang,1:ntypat)
        deallocate(vkbsigns)
      end if

      write(*,'(" vkbsign: ",8f4.1)') vkbsign(:,:)
      print *

      if(fform==502) then
        allocate(vkbdb(npwkss,ntypat,mpsang))
        allocate(vkbdd(npwkss,ntypat,mpsang))
        allocate(energyd(nbandkss))
        allocate(wfgd(npwkss))
      else if(fform==602) then
        allocate(vkbs(npwkss,ntypat,mpsang))
        allocate(vkbds(npwkss,ntypat,mpsang))
        allocate(energys(nbandkss))
        allocate(wfgs(npwkss))
      end if

      write(ab_out,*)
      print *, ' k       eigenvalues [eV]'
      write(ab_out,*) ' k       eigenvalues [eV]'

      do ik = 1, nkpt
!        print *, 'ik ', ik
        do is = 1, ntypat
!          print *, 'is ', is
          do il = 1, mpsang
!            print *, 'il ', il
            if(fform==502) then
              read(unitkss) vkbdb(:,is,il)
              read(unitkss) vkbdd(:,is,il)
              vkb(1:nwx,is,il,ik) = vkbdb(1:nwx,is,il)
              vkbd(1:nwx,is,il,ik) = vkbdd(1:nwx,is,il)
            else if(fform==602) then
              read(unitkss) vkbs(:,is,il)
              read(unitkss) vkbds(:,is,il)
              vkb(1:nwx,is,il,ik) = vkbs(1:nwx,is,il)
              vkbd(1:nwx,is,il,ik) = vkbds(1:nwx,is,il)
            end if
            if(DEBUG) print *, 'vkb ', vkb(:,:,:,ik)
            if(DEBUG) print *, 'vkbd ', vkbd(:,:,:,ik)
          end do
        end do

        if(fform==502) then
          read(unitkss) energyd(1:nbandkss)
          en(ik,1:nbx) = energyd(1:nbx)
        else if(fform==602) then
          read(unitkss) energys(1:nbandkss)
          en(ik,1:nbx) = energys(1:nbx)
        end if

 write(*,'(i3,7x,10f7.2,/,50(10x,10f7.2,/))') ik, (Ha_eV*en(ik,ib),ib=1,nbx)
 write(ab_out,'(i3,7x,10f7.2,/,50(10x,10f7.2,/))') ik, (Ha_eV*en(ik,ib),ib=1,nbx)

        if(fform==502) then
          do ib = 1, nbx
            read(unitkss) wfgd(1:npwkss)
            wf(1:nwx,ib,ik) = wfgd(1:nwx)
          end do
          do ib = nbx+1 , nbandkss
            read(unitkss) wfgd(1:npwkss)
          end do
        else if(fform==602) then
          do ib = 1, nbx
            read(unitkss) wfgs(1:npwkss)
            wf(1:nwx,ib,ik) = wfgs(1:nwx)
          end do
          do ib = nbx+1 , nbandkss
            read(unitkss) wfgs(1:npwkss)
          end do
        end if
      end do

      print *
      write(ab_out,*)

      einf = 1.0e24; esup = 0.0
      cinf = 1.0e24; csup = 0.0
      do ik = 1, nkx
!       test on the normalization of wavefunctions
        do ib = 1, nbx
          dum = 0.0
          do ig = 1, nwx
            dum = dum + conjg(wf(ig,ib,ik))*wf(ig,ib,ik)
          end do
          if(dum<einf) einf = dum
          if(dum>esup) esup = dum
        end do
!       test on the orthogonalization of wavefunctions
        do ib = 1, nbx
          do ibp = ib+1, nbx
            cdum = 0.0
            do ig = 1, nwx
              cdum = cdum + conjg(wf(ig,ib,ik))*wf(ig,ibp,ik)
            end do
            if(abs(cdum)<cinf) cinf = abs(cdum)
            if(abs(cdum)>csup) csup = abs(cdum)
          end do
        end do
      end do


      print *, 'test on the normalization of the wavefunctions'
      write(ab_out,*) 'test on the normalization of the wavefunctions'
      write(6,5693) einf, esup
      write(ab_out,5693) einf, esup
5693  format(' min sum_G |a(n,k,G)| = ',f9.6/' max sum_G |a(n,k,G)| = ',f9.6)
      print *, 'test on the orthogonalization of the wavefunctions'
      write(ab_out,*) 'test on the orthogonalization of the wavefunctions'
      write(6,5694) cinf, csup
      write(ab_out,5694) cinf, csup
5694  format(' min sum_G a(n,k,G)* a(n",k,G) = ',f9.6/&
&       ' max sum_G a(n,k,G)* a(n",k,G) = ',f9.6)
      print *
      write(ab_out,*)

      close(unitkss)

! deallocations added by MM
      deallocate(symrel2,tnons2,shlim)
      if(fform==502)then
       deallocate(vkbdb)
       deallocate(vkbdd)
       deallocate(energyd)
       deallocate(wfgd)
      else if(fform==602)then
       deallocate(vkbs)
       deallocate(vkbds)
       deallocate(energys)
       deallocate(wfgs)
      end if

end subroutine rdkss
!!***
