!{\src2tex{textfont=tt}}
!!****f* ABINIT/get_fs_kpts
!!
!! NAME
!! get_fs_kpts
!!
!! FUNCTION
!! This routine determines the kpoints on the standard grid which belong
!!  to the Fermi surface
!!
!! COPYRIGHT
!! Copyright (C) 2004-2006 ABINIT group (MVer)
!! This file is distributed under the terms of the
!! GNU General Public Licence, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!    eigenGS = ground state eigenvalues
!!    hdr = header from input GS file
!!
!! OUTPUT
!!    FSkptflag = flag to use a given kpoint for the fermi-surface integration
!!    gaussig = width of gaussian energy window around fermi energy
!!       needed to get a good fraction of kpoints contributing to the FS
!!    maxFSband = last band which will contribute to the FS integration
!!    minFSband = lowest band which will contribute to the FS integration
!!    nFSkptirred = number of irreducible kpoints which contribute
!!    nFSband = maxFSband-minFSband+1
!!
!! NOTES
!!
!! PARENTS
!!      elphon
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine get_fs_kpts(eigenGS,FSkptflag,gaussig,hdr,maxFSband,minFSband,&
&  nFSkptirred,nFSband)

 use defs_basis
 use defs_datatypes

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(out) :: maxFSband,minFSband,nFSband,nFSkptirred
 real(dp),intent(out) :: gaussig
 type(hdr_type),intent(in) :: hdr
!arrays
 integer,intent(out) :: FSkptflag(hdr%nkpt)
 real(dp),intent(in) :: eigenGS(hdr%nband(1),hdr%nkpt)

!Local variables-------------------------------
!scalars
 integer :: iband,ikpt,nband
 real(dp) :: epsFS,gausstol

! *************************************************************************

  !  supposes nband is equal for all kpts
  nband = hdr%nband(1)

  !  gausstol = minimum weight value for integration weights on FS
  !     should be set to reproduce DOS at Ef (Ref. PRB 34, 5065 p. 5067)
  gausstol = 1.0d-10

  !  window of states around fermi Energy is contained in +/- epsFS
  !   should be adjusted to take into account a minimal but sufficient
  !   fraction of the kpoints: see the loop below.
  !   The 1000 is purely empirical!!!
  !   Should also take into account the density of kpoints.
  !  gaussig = width of gaussian for integration weights on FS
  gaussig = (maxval(eigenGS)-minval(eigenGS))/1000.0_dp

  !    NOTE: could loop back to here and change gaussig until we have
  !    a certain fraction of the kpoints in the FS region...
  nFSkptirred = 0
  !do while (dble(nFSkptirred) / dble(hdr%nkpt) < half)
  ! Do not use restricted fermi surface: include all kpts -> one
  do while (dble(nFSkptirred) / dble(hdr%nkpt) < one)
     gaussig = gaussig*1.05_dp
!DEBUG
!  write (*,*) ' now gaussig,nFSkptirred,hdr%nkpt = ',&
!&                   gaussig,nFSkptirred,hdr%nkpt
!ENDDEBUG

     !  as a consequence, we must take into account
     !    kpoints with states within epsFS:
     epsFS = gaussig*sqrt(log(one/(gaussig*sqrt(pi)*gausstol)))

     ! check if there are eigenvalues close to the Fermi surface
     !    (less than epsFS from it)
     FSkptflag(:) = 0
     maxFSband=1
     minFSband=nband
     do ikpt=1,hdr%nkpt
        do iband=1,nband
!DEBUG
!    write (*,'(a,3(E12.5,1x))') 'eigenGS(iband,ikpt),hdr%fermie,epsFS = ',&
!&                eigenGS(iband,ikpt)-hdr%fermie,epsFS
!ENDDEBUG
           if (abs(eigenGS(iband,ikpt) - hdr%fermie) < epsFS) then
              FSkptflag(ikpt) = 1
              if (iband > maxFSband) maxFSband = iband
              if (iband < minFSband) minFSband = iband
!DEBUG
!     write (*,*) 'iband,minFSband,maxFSband = ', iband,minFSband,maxFSband
!ENDDEBUG
           end if
        end do
     end do
     nFSband = maxFSband-minFSband+1

     !   number of irreducible kpoints (by all sym) contributing to the Fermi
     !   surface (to be completed by symops)
     nFSkptirred = sum(FSkptflag)
  end do

!DEBUG
! Do not use restricted fermi surface: include all bands
!minFSband = 6
!maxFSband = 9
!nFSband=maxFSband-minFSband+1
!ENDDEBUG

!DEBUG
! write (*,*) ' gausstol, gaussig, epsFS = ',&
!      &              gausstol, gaussig, epsFS
!ENDDEBUG

end subroutine get_fs_kpts
!!***
