!{\src2tex{textfont=tt}}
!!****f* ABINIT/respfn
!! NAME
!! respfn
!!
!! FUNCTION
!! Primary routine for conducting DFT calculations of Response functions.
!!
!! COPYRIGHT
!! Copyright (C) 1999-2006 ABINIT group (XG, DRH)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!  codvsn=code version
!!  cpui=initial cpu time
!!  dtfil <type(datafiles_type)>=variables related to files
!!  dtset <type(dataset_type)>=all input variables for this dataset
!!  mband=maximum number of bands
!!  mgfft=maximum single fft dimension
!!  mkmem=maximum number of k points which can fit in core memory
!!  mkmems(3)=array containing the tree values of mkmem (see above) (k-GS, k+q-GS and RF)
!!  mpi_enreg=informations about MPI parallelization
!!  mpw=maximum number of planewaves in basis sphere (large number)
!!  natom=number of atoms in unit cell
!!  nfft=(effective) number of FFT grid points (for this processor)
!!  nkpt=number of k points
!!  npwtot(nkpt)=number of planewaves in basis and boundary at each k point
!!  nspden=number of spin-density components
!!  nspinor=number of spinorial components of the wavefunctions
!!  nsppol=number of channels for spin-polarization (1 or 2)
!!  nsym=number of symmetry elements in space group
!!  walli=initial wall clock time
!!  xred(3,natom)=reduced dimensionless atomic coordinates
!!
!! OUTPUT
!!  etotal=total energy (sum of 7 contributions) (hartree)
!!
!! SIDE EFFECTS
!!  iexit=index of "exit" on first line of file (0 if not found)
!!  occ(mband*nkpt*nsppol)=occup number for each band (often 2) at each k point
!!    Occupations number may have been read from a previous dataset...
!!  pawrad(ntypat*usepaw) <type(pawrad_type)>=paw radial mesh and related data
!!    Some dimensions in pawrad have been set in driver.f
!!  pawtab(ntypat*usepaw) <type(pawtab_type)>=paw tabulated starting data
!!    Some dimensions in pawtab have been set in driver.f
!!  psps <type(pseudopotential_type)>=variables related to pseudopotentials
!!    Before entering the first time in respfn, a significant part of psps
!!    has been initialized: the integers dimekb,lmnmax,lnmax,mpssang,mpssoang,
!!    mpsso,mgrid,ntypat,n1xccc,usepaw,useylm, and the arrays dimensioned to npsp
!!    All the remaining components of psps are to be initialized in the call
!!    to pspini.  The next time the code enters respfn, psps might be identical
!!    to the one of the previous dtset, in which case, no reinitialisation
!!    is scheduled in pspini.f .
!!
!! PARENTS
!!      driver
!!
!! CHILDREN
!!      bstruct_clean,bstruct_init,chkexi,chkph3,d2sym3,distrb2,dyfnl3,dyfro3
!!      dyout3,dyxc13,eltfrhar3,eltfrkin3,eltfrloc3,eltfrnl3,eltfrxc3,ewald3
!!      ewald4,fourdp,gath3,getcut,getph,hdr_clean,hdr_init,hdr_update
!!      initmpi_fft,initylmg,int2char4,inwffil,ioarr,ioddb8,kpgio,leave_new
!!      loper3,mkcore,mklocl,mkrho,mpi_barrier,mpi_bcast,newocc,phfrq3,prtph3
!!      psddb8,pspini,q0dy3,rhohxc,setsym,setup1,status,symkchk,symq3,symzat
!!      syper3,timab,wffclose,wffdelete,wings3,wrtloctens,wrtout,xcomm_world
!!      xme_whoiam,xsum_master
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine respfn(codvsn,cpui,dtfil,dtset,etotal,iexit,&
&  mband,mgfft,mkmem,mkmems,mpi_enreg,mpw,natom,nfft,nkpt,npwtot,nspden,&
&  nspinor,nsppol,nsym,occ,pawrad,pawtab,psps,walli,xred)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_00basis
 use interfaces_01managempi
 use interfaces_01util
 use interfaces_02ffts
 use interfaces_02geometry
 use interfaces_02psp
 use interfaces_03recipspace
 use interfaces_03xc
 use interfaces_04iowfdenpot
 use interfaces_04occeig
 use interfaces_05common
 use interfaces_06response
 use interfaces_08seqpar, except_this_one => respfn
#else
 use defs_xfuncmpi
 use defs_xc
#endif
!End of the abilint section

 implicit none

#if defined MPI
           include 'mpif.h'
#endif
!Arguments ------------------------------------
 integer,intent(in) :: mband,mgfft,mkmem,mpw,nfft
 integer,intent(inout) :: iexit,natom,nkpt,nspden,nspinor,nsppol,nsym
 real(dp),intent(in) :: cpui,walli
 real(dp),intent(out) :: etotal
 character(len=6),intent(in) :: codvsn
 type(MPI_type),intent(inout) :: mpi_enreg
 type(datafiles_type),intent(in) :: dtfil
 type(dataset_type),intent(inout) :: dtset
 type(pseudopotential_type),intent(inout) :: psps
 integer,intent(in) :: mkmems(3)
 integer,intent(inout) :: npwtot(nkpt)
 real(dp),intent(inout) :: xred(3,natom)
 real(dp),intent(inout) :: occ(mband*nkpt*nsppol)
 type(pawrad_type),intent(inout) :: pawrad(psps%ntypat,psps%usepaw)
 type(pawtab_type),intent(inout) :: pawtab(psps%ntypat,psps%usepaw)

!Local variables-------------------------------
!---- Local variables : integer scalars
#if !defined MPI
 integer,parameter :: nkpt_max=50
#endif
#if defined MPI
 integer,parameter :: nkpt_max=-1
#endif
!Define file format for different type of files. Presently,
!only one file format is supported for each type of files, but this might
!change soon ...
!1   for wavefunction file, old format (version prior to 2.0)
!2   for wavefunction file, new format (version 2.0 and after)    (fform)
!(51 or 52   for density rho(r)       (fformr)
!101 or 102 for potential V(r) file. (fformv)
 integer,parameter :: fform=2,fformv=102,formeig=0,level=13
 integer,parameter :: response=1,syuse=0
 integer :: fformr=52
 integer :: accessfil,analyt,ask_accurate,band_index,bantot,choice,flag,fullinit,gencond
 integer :: gscase,iapp,iatom,iband,idir,idir2,ierr,ifft,ii,ikpt,index
 integer :: initialized,ios,ipert,ipert2,ir,ireadwf0,iscf,iscf_eff,isppol
 integer :: itime,itypat,jband,jj,master,me,mk1mem,mkqmem,mpert,mpsang,mu
 integer :: n1,n2,n3,n3xccc,nband_k,nblok,nkpt_eff,nkxc,ntypat,openexit,option,optorth
 integer :: outd2,prtbbb,prtdos,prtvol,qzero,rdwr,rdwrpaw,rfasr,rfelfd,rfphon,rfstrs
 integer :: rfuser,spaceworld,sumg0,tim_mkrho,timrev,tmkmem,vrsddb
 real(dp) :: boxcut,cpus,ecore,ecut_eff,eei,eew,ehart,eii,ek,enl,entropy,enxc
 real(dp) :: etot,fermie,gsqcut,gsqcut_eff,gsqcutdg_eff,qphnrm,residm
 real(dp) :: rhosum,tmp,tolmxf,tolwfr
 real(dp) :: ucvol,vxcavg
 logical :: test_mpi
 character(len=fnlen) :: ddbnm,dscrpt
 character(len=4) :: tag
 character(len=500) :: message
 type(bandstructure_type) :: bstruct
 type(dens_sym_operator_type) :: densymop_gs
 type(hdr_type) :: hdr
 type(wffile_type) :: wffgs,wfftgs
 integer :: ddkfil(3),rfdir(3)
 integer,allocatable :: atindx(:),atindx1(:),blkflg(:,:,:,:),blkflg1(:,:,:,:)
 integer,allocatable :: blkflg2(:,:,:,:),carflg(:,:,:,:),indsym(:,:,:)
 integer,allocatable :: irrzon(:,:,:),kg(:,:),kgq(:,:),nattyp(:),npwarr(:)
 integer,allocatable :: pertsy(:,:),rfpert(:),symq(:,:,:),symrec(:,:,:)
 real(dp) :: corstr(6),dummy2(6),gmet(3,3),gprimd(3,3),k0(3),lpsstr(6),qphon(3)
 real(dp) :: rmet(3,3),rprimd(3,3),strsxc(6),tsec(2)
 real(dp),allocatable :: amass(:),cg(:,:),d2bbb(:,:,:,:,:,:),d2cart(:,:,:,:,:)
 real(dp),allocatable :: d2cart_bbb(:,:,:,:,:,:),d2eig0(:,:,:,:,:)
 real(dp),allocatable :: d2k0(:,:,:,:,:),d2lo(:,:,:,:,:),d2loc0(:,:,:,:,:)
 real(dp),allocatable :: d2matr(:,:,:,:,:),d2nfr(:,:,:,:,:),d2nl(:,:,:,:,:)
 real(dp),allocatable :: d2nl0(:,:,:,:,:),d2nl1(:,:,:,:,:),d2tmp(:,:,:,:,:)
 real(dp),allocatable :: d2vn(:,:,:,:,:),displ(:),doccde(:),dyew(:,:,:,:,:)
 real(dp),allocatable :: dyewq0(:,:,:),dyfrlo(:,:,:),dyfrlo_indx(:,:,:)
 real(dp),allocatable :: dyfrnl(:,:,:),dyfrwf(:,:,:),dyfrx1(:,:,:,:,:)
 real(dp),allocatable :: dyfrx2(:,:,:),eigen0(:),eigval(:),eigvec(:)
 real(dp),allocatable :: eltcore(:,:),elteew(:,:),eltfrhar(:,:),eltfrkin(:,:)
 real(dp),allocatable :: eltfrloc(:,:),eltfrnl(:,:),eltfrxc(:,:),grtn_indx(:,:)
 real(dp),allocatable :: grxc(:,:),kxc(:,:),ph1d(:,:),phfrq(:),phnons(:,:,:)
 real(dp),allocatable :: rhog(:,:),rhor(:,:),vhartr(:),vpsp(:),vtrial(:,:)
 real(dp),allocatable :: vxc(:,:),work(:),xccc3d(:),ylm(:,:),ylmgr(:,:,:)
 character(len=fnlen) :: tmpfil(12)
 type(atmrhoij_type),allocatable :: atmrhoij(:)
 integer :: idtmpfil(12)

! ***********************************************************************

!DEBUG
!write(6,*)' respfn : enter'
!stop
!ENDDEBUG

 call timab(132,1,tsec)
 call timab(133,1,tsec)

 call status(0,dtfil%filstat,iexit,level,'enter         ')

 mpi_enreg%paralbd=1
 mpi_enreg%parareel=0
 mpi_enreg%me_fft=0
 mpi_enreg%nproc_fft=1
 mpi_enreg%paral_fft=0
 mpi_enreg%paral_level=2

#if defined MPI
           allocate(mpi_enreg%proc_distrb(nkpt,mband,nsppol))
           call distrb2(mband, dtset%nband, nkpt, nsppol, mpi_enreg)
#endif


 call initmpi_fft(dtset,mpi_enreg)

!
!  If dtset%accesswff == 2 set all array outputs to netcdf format
!
 accessfil = 0
 if (dtset%accesswff == 2) then
   accessfil = 1
 end if

!Structured debugging if prtvol==-level
 prtvol=dtset%prtvol
 if(prtvol==-level)then
  write(message,'(80a,a,a)')  ('=',ii=1,80),ch10,&
&   ' respfn : enter , debug mode '
  call wrtout(06,message,'COLL')
 end if

!Option input variables
 iscf=dtset%iscf


!Respfn input variables
 rfasr=dtset%rfasr   ; rfdir(1:3)=dtset%rfdir(1:3)
 rfelfd=dtset%rfelfd ; rfphon=dtset%rfphon
 rfstrs=dtset%rfstrs ; rfuser=dtset%rfuser

!mkmem variables (mkmem is already argument)
 mkqmem=mkmems(2) ; mk1mem=mkmems(3)

 ntypat=psps%ntypat

 call status(0,dtfil%filstat,iexit,level,'call setup1   ')

 ecore=0.0d0

!LIKELY TO BE TAKEN AWAY
 initialized=0
 ek=0.0d0 ; ehart=0.0d0 ; enxc=0.0d0 ; eei=0.0d0 ; enl=0.0d0
 eii=0.0d0 ; eew=0.0d0

!Set up for iterations
 ecut_eff= (dtset%ecut) * (dtset%dilatmx) **2
 allocate(amass(natom))
!DEBUG
!write(6,*)' respfn : ntypat=',ntypat
!ENDDEBUG
 call setup1(dtset%acell_orig,amass,dtset%amu,bantot,&
& ecut_eff,ecut_eff,gmet,gprimd,gsqcut_eff,gsqcut_eff,dtset%iboxcut,dtset%intxc,dtset%ionmov,&
& natom,dtset%nband,dtset%ngfft,dtset%ngfft,nkpt,dtset%nqpt,nsppol,nsym,ntypat,&
& dtset%qptn,response,rmet,dtset%rprim_orig,rprimd,dtset%typat,ucvol,psps%usepaw)

!Define the set of admitted perturbations
 mpert=natom+6

!Initialize the list of perturbations rfpert
 allocate(rfpert(mpert))
 rfpert(:)=0
 if(rfphon==1)rfpert(dtset%rfatpol(1):dtset%rfatpol(2))=1

 if(rfelfd==1.or.rfelfd==2)rfpert(natom+1)=1
 if(rfelfd==1.or.rfelfd==3)rfpert(natom+2)=1
 if(rfstrs==1.or.rfstrs==3)rfpert(natom+3)=1
 if(rfstrs==2.or.rfstrs==3)rfpert(natom+4)=1
 if(rfuser==1.or.rfuser==3)rfpert(natom+5)=1
 if(rfuser==2.or.rfuser==3)rfpert(natom+6)=1

!Create names for the temporary files based on dtfil%filnam_ds(5)
!by appending adequate string.
!'_1WF1' -> dtfil%unwft1
!'_1WF2' -> dtfil%unwft2
!'_KG'   -> dtfil%unkg
!'_KGQ'  -> dtfil%unkgq (not used for the time being)
!'_KG1'  -> dtfil%unkg1
!'_DUM'  -> tmp_unit (real dummy name)
!'_WFGS' -> dtfil%unwftgs
!'_WFKQ' -> dtfil%unwftkq
!'_YLM'  -> dtfil%unylm
!'_YLM1' -> dtfil%unylm1
 tmpfil(1) =trim(dtfil%filnam_ds(5))//'_1WF1'
 tmpfil(2) =trim(dtfil%filnam_ds(5))//'_1WF2'
 tmpfil(3) =trim(dtfil%filnam_ds(5))//'_KG'
 tmpfil(4) =trim(dtfil%filnam_ds(5))//'_KGQ'
 tmpfil(5) =trim(dtfil%filnam_ds(5))//'_KG1'
 tmpfil(6) =trim(dtfil%filnam_ds(5))//'_DUM'
 tmpfil(7) =trim(dtfil%filnam_ds(5))//'_WFGS'
 tmpfil(8) =trim(dtfil%filnam_ds(5))//'_WFKQ'
 tmpfil(10)=trim(dtfil%filnam_ds(5))//'_YLM'
 tmpfil(11)=trim(dtfil%filnam_ds(5))//'_YLM1'

!Default for sequential use
 master=0
 call xme_whoiam(me)
!Init spaceworld

!BEGIN TF_CHANGES
 call xcomm_world(mpi_enreg,spaceworld)
!END TF_CHANGES


!There is a definite problem for the treatment
!of // within CPP sections...
#if defined MPI
           do ii=1,11
              idtmpfil(ii)=ii
           end do
           if (mpi_enreg%paral_compil_mpio==1.and.dtset%accesswff==1)then
              idtmpfil(1)=0              !_1wf1
              idtmpfil(2)=0              ! s1wf2
              idtmpfil(7)=0              !  WFGS
              idtmpfil(8)=0              !  WFKQ
           endif
           call int2char4(me,tag)
           do ii=1,11
           if(idtmpfil(ii) /= 0) tmpfil(ii)=trim(tmpfil(ii))//'_P-'//tag
           end do
#endif

 call status(0,dtfil%filstat,iexit,level,'call kpgio(1) ')

!Set up the basis sphere of planewaves
 allocate(kg(3,mpw*mkmem),npwarr(nkpt))
 call kpgio(ecut_eff,dtset%exchn2n3,gmet,dtset%istwfk,kg,tmpfil(3),&
& dtset%kptns,mkmem,dtset%nband,nkpt,'PERS',mpi_enreg,mpw,npwarr,npwtot,&
& nsppol,dtfil%unkg)

!Set up the Ylm for each k point
 mpsang=psps%mpsang
 allocate(ylm(mpw*mkmem,mpsang*mpsang*psps%useylm))
 if (rfstrs/=0) then
  allocate(ylmgr(mpw*mkmem,9,mpsang*mpsang*psps%useylm))
 else
  allocate(ylmgr(1,1,psps%useylm))
 end if
 if (psps%useylm==1) then
  if(mkmem==0) open(dtfil%unylm,file=tmpfil(10),form='unformatted',status='unknown')
  call status(0,dtfil%filstat,iexit,level,'call initylmg ')
  option=0;if (rfstrs/=0) option=2
  call initylmg(gprimd,kg,dtset%kptns,mkmem,mpi_enreg,mpsang,mpw,dtset%nband,nkpt,&
&               npwarr,nsppol,option,rprimd,dtfil%unkg,dtfil%unylm,ylm,ylmgr)
 end if

 call timab(133,2,tsec)
 call timab(134,1,tsec)

!Determine cut-off for g^2 sphere
!TO BE ADAPTED TO PAW LATER...
 gsqcutdg_eff=gsqcut_eff

!Open and read pseudopotential files

 call status(0,dtfil%filstat,iexit,level,'call pspini(1)')

 call pspini(ecore,gencond,gsqcut_eff,gsqcutdg_eff,iscf,dtset%ixc,level,&
& natom,pawrad,pawtab,dtset%pawxcdev,prtvol,psps,dtset%typat,rprimd)

 call timab(134,2,tsec)
 call timab(135,1,tsec)

!Initialize band structure datatype
 allocate(doccde(bantot),eigen0(bantot))
 doccde(:)=zero ; eigen0(:)=zero
 call bstruct_init(bantot,bstruct,doccde,eigen0,dtset%istwfk,dtset%kptns,&
& dtset%nband,nkpt,npwarr,nsppol,dtset%occ_orig,dtset%wtk)
 deallocate(doccde,eigen0)

!Initialize header
 gscase=0
 call hdr_init(bstruct,codvsn,dtset,hdr,pawtab,gscase,psps)

!Update header, with evolving variables, when available
!Here, rprimd, xred and occ are available
 etot=hdr%etot ; fermie=hdr%fermie ; residm=hdr%residm
 call hdr_update(atmrhoij,bantot,etot,fermie,hdr,natom,&
& residm,rprimd,occ,psps%usepaw,xred)

!Clean band structure datatype (should use it more in the future !)
 call bstruct_clean(bstruct)

 call status(0,dtfil%filstat,iexit,level,'call inwffil(1')

!Initialize wavefunction files and wavefunctions.
 ireadwf0=1

 allocate(cg(2,mpw*nspinor*mband*mkmem*nsppol))
 allocate(eigen0(mband*nkpt*nsppol))
 eigen0(:)=0.0d0 ; ask_accurate=1
 optorth=1;if (psps%usepaw==1) optorth=0

 call inwffil(ask_accurate,cg,dtset,dtset%ecut,ecut_eff,eigen0,dtset%exchn2n3,&
& formeig,gmet,hdr,ireadwf0,dtset%istwfk,kg,dtset%kptns,&
& dtset%localrdwf,mband,mkmem,mpi_enreg,mpw,&
& dtset%nband,dtset%ngfft,nkpt,npwarr,nspden,nspinor,nsppol,nsym,&
& occ,optorth,prtvol,rprimd,dtset%symafm,dtset%symrel,dtset%tnons,&
& dtfil%unkg,wffgs,wfftgs,dtfil%unwffgs,dtfil%unwftgs,dtfil%fnamewffk,tmpfil(7))

 call timab(135,2,tsec)
 call timab(136,1,tsec)

!Close wffgs, if it was ever opened (in inwffil)
 if (ireadwf0==1) then
  call WffClose(wffgs,ierr)
 end if

!Report on eigen0 values
 write(message, '(a,a)' )ch10,' respfn : eigen0 array'
 call wrtout(6,message,'COLL')
 nkpt_eff=nkpt
 if( (prtvol==0.or.prtvol==1.or.prtvol==2) .and. nkpt>nkpt_max ) nkpt_eff=nkpt_max
 band_index=0
 do isppol=1,nsppol
  do ikpt=1,nkpt
   nband_k=dtset%nband(ikpt+(isppol-1)*nkpt)
   if(ikpt<=nkpt_eff)then
    write(message, '(a,i2,a,i5)' )&
&    '  isppol=',isppol,', k point number',ikpt
    call wrtout(6,message,'COLL')
    do iband=1,nband_k,4
     write(message, '(a,4es16.6)')&
&     '  ',eigen0(iband+band_index:min(iband+3,nband_k)+band_index)
     call wrtout(6,message,'COLL')
    end do
   else if(ikpt==nkpt_eff+1)then
    write(message,'(a,a)' )&
&    '  respfn : prtvol=0, 1 or 2, stop printing eigen0.',ch10
    call wrtout(6,message,'COLL')
   end if
   band_index=band_index+nband_k
  end do
 end do

!Allocation for forces and atomic positions (should be taken away, also argument ... )
 allocate(grxc(3,natom))

!Do symmetry stuff
 call status(0,dtfil%filstat,iexit,level,'call setsym   ')
 allocate(irrzon(dtset%nfft**(1-1/nsym),2,nspden/nsppol))
 allocate(phnons(2,dtset%nfft**(1-1/nsym),nspden/nsppol))
!DEBUG
!write(6,*)' respfn : stop before GS setsym'
!stop
!ENDDEBUG
 allocate(indsym(4,nsym,natom),symrec(3,3,nsym))
 iscf_eff=0
!If the density is to be computed by mkrho, need irrzon and phnons
 if(dtset%getden==0)iscf_eff=1
 call setsym(densymop_gs,indsym,irrzon,iscf_eff,natom,&
& nfft,dtset%ngfft,nspden,nsppol,nsym,&
& phnons,dtset%symafm,symrec,dtset%symrel,dtset%tnons,dtset%typat,xred)

!DEBUG
!write(6,*)' respfn : stop after GS setsym'
!stop
!ENDDEBUG

!Symmetrize atomic coordinates over space group elements:
 call status(0,dtfil%filstat,iexit,level,'call symzat   ')
 call symzat(indsym,natom,nsym,dtset%symrel,dtset%tnons,xred)

!Generate an index table of atoms, in order for them to be used
!type after type.

 allocate(atindx(natom),atindx1(natom),nattyp(ntypat))
 index=1
 do itypat=1,ntypat
  nattyp(itypat)=0
  do iatom=1,natom
   if(dtset%typat(iatom)==itypat)then
    atindx(iatom)=index
    atindx1(index)=iatom
    index=index+1
    nattyp(itypat)=nattyp(itypat)+1
   end if
  end do
 end do

!Compute structure factor phases for current atomic pos:
 allocate(ph1d(2,3*(2*mgfft+1)*natom))
 n1=dtset%ngfft(1) ; n2=dtset%ngfft(2) ; n3=dtset%ngfft(3)
 call status(0,dtfil%filstat,iexit,level,'call getph    ')
 call getph(atindx,natom,n1,n2,n3,ph1d,xred)

!Compute occupation numbers and fermi energy, in case
!occupation scheme is metallic.
 allocate(doccde(mband*nkpt*nsppol))
 if( dtset%occopt>=3.and.dtset%occopt<=7 ) then
  call status(0,dtfil%filstat,iexit,level,'call newocc   ')
  call newocc(doccde,eigen0,entropy,fermie,dtset%fixmom,mband,dtset%nband,&
&  dtset%nelect,nkpt,nspinor,nsppol,occ,dtset%occopt,prtvol,dtset%stmbias,&
&  dtset%tphysel,dtset%tsmear,dtset%wtk)

! Update fermie and occ
  etot=hdr%etot ; residm=hdr%residm
  call hdr_update(atmrhoij,bantot,etot,fermie,hdr,natom,&
&  residm,rprimd,occ,psps%usepaw,xred)

 else
! doccde is irrelevant in this case
  doccde(:)=0.0d0

 end if

 allocate(rhog(2,nfft),rhor(nfft,nspden))
!Read ground-state charge density from diskfile in case getden /= 0
!or compute it from wfs that were
!read previously : rhor as well as rhog

 if (dtset%getden /= 0) then

  if (me == 0) then
   rdwr = 1 ; rdwrpaw = 0
   call status(0,dtfil%filstat,iexit,level,'call ioarr    ')
   call ioarr (accessfil,rhor,etotal,fformr,dtfil%fildensin,hdr,nfft,nspden,rdwr,rdwrpaw)
  end if



#if defined MPI
          call MPI_BARRIER(spaceworld,ierr)
          call MPI_BCAST(rhor,nfft*nspden,&
&            MPI_DOUBLE_PRECISION,0,spaceworld,ierr)
#endif

! Compute up+down rho(G) by fft
  call status(0,dtfil%filstat,iexit,level,'call fourdp   ')
  allocate(work(nfft))
  work(:)=rhor(:,1)
  call fourdp(1,rhog,work,-1,mpi_enreg,nfft,dtset%ngfft,0)
  deallocate(work)

 else

  call status(0,dtfil%filstat,iexit,level,'call mkrho    ')
  tim_mkrho=4
  call mkrho(cg,densymop_gs,dtset,irrzon,dtset%istwfk,kg,mband,mgfft,mkmem,mpi_enreg,&
&  mpw,dtset%nband,nfft,dtset%ngfft,nkpt,npwarr,nspden,nspinor,nsppol,nsym,&
&  occ,phnons,rhog,rhor,dtset%symafm,&
&  tim_mkrho,ucvol,dtfil%unkg,wfftgs,dtset%wtk)

 end if    ! getden

!The GS irrzon and phnons were only needed to symmetrize the GS density
 deallocate(irrzon,phnons)

!Debugging : print the different parts of rhor
 if(prtvol==-level)then
  write(message,'(a)') '   ir     rhor(ir)     '
  call wrtout(06,message,'COLL')
  do ir=1,nfft
   if(ir<=11 .or. mod(ir,301)==0 )then
    write(message,'(i5,a,es13.6)')ir,' ',rhor(ir,1)
    call wrtout(06,message,'COLL')
    if(nspden==2)then
     write(message,'(a,es13.6)')'      ',rhor(ir,2)
     call wrtout(06,message,'COLL')
    end if
   end if
  end do
 end if

!Will compute now the total potential
 allocate(vhartr(nfft),vpsp(nfft),vtrial(nfft,nspden),vxc(nfft,nspden))

!Recompute first large sphere cut-off gsqcut,
!without taking into account dilatmx
 k0(:)=0.0d0
 call status(0,dtfil%filstat,iexit,level,'call getcut   ')
 call getcut(boxcut,dtset%ecut,gmet,gsqcut,dtset%iboxcut,6,k0,dtset%ngfft)

!Compute local ionic pseudopotential vpsp:
 option=1
!These are used as dummy variables
 allocate(dyfrlo_indx(3,3,natom),grtn_indx(3,natom))
 call status(0,dtfil%filstat,iexit,level,'call mklocl   ')
 call mklocl(dtset,dyfrlo_indx,eei,gmet,gprimd,&
& grtn_indx,gsqcut,lpsstr,mgfft,mpi_enreg,natom,nattyp,&
& nfft,dtset%ngfft,nspden,ntypat,option,ph1d,psps,&
& dtset%qprtrb,rhog,rhor,rmet,rprimd,ucvol,dtset%vprtrb,vpsp,xred)
 deallocate(dyfrlo_indx,grtn_indx)

!Compute core electron density xccc3d
 n3xccc=0
 if (psps%n1xccc/=0) n3xccc=nfft
 allocate(xccc3d(n3xccc))
 if (psps%n1xccc/=0) then
  option=1
  call status(0,dtfil%filstat,iexit,level,'call mkcore   ')
  allocate(dyfrx2(3,3,natom))
  call mkcore(dummy2,dyfrx2,grxc,mpi_enreg,natom,nfft,nspden,ntypat,&
&  n1,psps%n1xccc,n2,n3,option,rprimd,dtset%typat,ucvol,vxc,&
&  psps%xcccrc,psps%xccc1d,xccc3d,xred)
  deallocate(dyfrx2)
 end if

!Set up hartree and xc potential. Compute kxc here.
 option=2
 nkxc=2*nspden-1
 if(dtset%ixc>=11 .and. dtset%ixc<=15)nkxc=23
 allocate(kxc(nfft,nkxc))

 call status(0,dtfil%filstat,iexit,level,'call rhohxc   ')
 call rhohxc(dtset,enxc,gsqcut,psps%usepaw,kxc,mpi_enreg,nfft,dtset%ngfft,&
& work,0,work,0,nkxc,nspden,n3xccc,option,rhog,rhor,rprimd,strsxc,1,vhartr,vxc,vxcavg,xccc3d)

!Compute local + Hxc potential, and subtract mean potential.
 do isppol=1,nsppol
  do ifft=1,nfft
   vtrial(ifft,isppol)=vhartr(ifft)+vxc(ifft,isppol)+vpsp(ifft)
  end do
 end do
 deallocate(vhartr,vpsp)

 call status(0,dtfil%filstat,iexit,level,'end respfn(1) ')

 if(prtvol==-level)then
  write(message,'(a,a)') ch10,&
&   ' respfn : ground-state density and potential set up. '
  call wrtout(06,message,'COLL')
 end if

! -----2. Frozen-wavefunctions and Ewald(q=0) parts of 2DTE

 allocate(eltcore(6,6),elteew(6+3*natom,6),eltfrhar(6,6),eltfrnl(6+3*natom,6))
 allocate(eltfrloc(6+3*natom,6),eltfrkin(6,6),eltfrxc(6+3*natom,6))
 eltcore(:,:)=0.0d0
 elteew(:,:)=0.0d0
 eltfrnl(:,:)=0.0d0
 eltfrloc(:,:)=0.0d0
 eltfrkin(:,:)=0.0d0
 eltfrhar(:,:)=0.0d0
 eltfrxc(:,:)=0.0d0

 allocate(dyew(2,3,natom,3,natom),dyewq0(3,3,natom))
 allocate(dyfrnl(3,3,natom),dyfrlo(3,3,natom),dyfrwf(3,3,natom))
 allocate(dyfrx2(3,3,natom))
 dyew(:,:,:,:,:)=0.0d0
 dyewq0(:,:,:)=0.0d0
 dyfrnl(:,:,:)=0.0d0
 dyfrlo(:,:,:)=0.0d0
 dyfrwf(:,:,:)=0.0d0
 dyfrx2(:,:,:)=0.0d0

 if (rfphon==1) then

! (independent of the phonon wavelength, diagonal with respect
!  to the atoms, real )

  call dyfnl3(atindx,atindx1,cg,dyfrnl,dtset%ecut,dtset%ecutsm,&
&  fform,dtset%istwfk,&
&  kg,dtset%kptns,mband,mgfft,mkmem,mpi_enreg,mpsang,&
&  mpw,natom,nattyp,dtset%nband,&
&  nkpt,dtset%ngfft,dtset%nloalg,npwarr,&
&  nspinor,nsppol,nsym,ntypat,occ,ph1d,&
&  prtvol,psps,rprimd,dtfil%unkg,wfftgs,dtfil%unylm,&
&  dtset%wtk,xred,ylm)

! dyfrnl has not yet been symmetrized, but will be in the next routine
  call status(0,dtfil%filstat,iexit,level,'call dyfro3    ')
  call dyfro3(atindx,dyfrnl,dyfrlo,dyfrwf,dyfrx2,&
&  gmet,gprimd,gsqcut,indsym,mgfft,mpi_enreg,psps%mqgrid_vl,natom,nattyp,&
&  nfft,dtset%ngfft,nspden,nsym,ntypat,psps%n1xccc,n3xccc,ph1d,psps%qgrid_vl,&
&  rhog,rprimd,symrec,dtset%typat,ucvol,psps%vlspl,&
&  vxc,psps%xcccrc,psps%xccc1d,xccc3d,xred)

! The frozen-wavefunction part of the dynamical matrix is now:
!  dyfrnl:  non-local contribution
!  dyfrlo:  local contribution
!  dyfrx2:  2nd order xc core correction contribution
!  dyfrwf:  all      contributions

! Compute Ewald (q=0) contribution
  qphon(:)=zero
  sumg0=0
  call status(0,dtfil%filstat,iexit,level,'call ewald3(1)')
  call ewald3(dyew,gmet,natom,qphon,rmet,sumg0,dtset%typat,ucvol,xred,psps%ziontypat)
  option=1
  call q0dy3(natom,dyewq0,dyew,option)

!End of the frozen-wavefunction and Ewald(q=0) parts of the dynamical matrix
 end if

!Section for the strain perturbation - frozen-wavefunction, Ewald, etc.
! parts of the elastic tensor

 if(rfstrs/=0) then

! Verify that k-point set has full space-group symmetry; otherwise exit
  call status(0,dtfil%filstat,iexit,level,'call symkchk ')
  timrev=1
  call symkchk(gmet,dtset%kptns,nkpt,nsym,symrec,timrev)

! Calculate the nonlocal part of the elastic tensor
  call status(0,dtfil%filstat,iexit,level,'call eltfrnl3 ')
  call eltfrnl3(atindx,atindx1,cg,eltfrnl,dtset%ecut,dtset%ecutsm,&
&  fform,dtset%istwfk,&
&  kg,dtset%kptns,mband,mgfft,mkmem,mpi_enreg,mpsang,&
&  mpw,natom,nattyp,dtset%nband,&
&  nkpt,dtset%ngfft,dtset%nloalg,npwarr,&
&  nspinor,nsppol,nsym,ntypat,occ,ph1d,&
&  prtvol,psps,rprimd,dtset%typat,dtfil%unkg,wfftgs,dtfil%unylm,&
&  dtset%useylm,dtset%wtk,xred,ylm,ylmgr)
! Calculate the kinetic part of the elastic tensor
  call status(0,dtfil%filstat,iexit,level,'call eltfrkin3')
  call eltfrkin3(cg,eltfrkin,dtset%ecut,dtset%ecutsm,dtset%effmass,&
&  fform,dtset%istwfk,&
&  kg,dtset%kptns,mband,mgfft,mkmem,mpi_enreg,&
&  mpw,dtset%nband,&
&  nkpt,dtset%ngfft,npwarr,&
&  nspinor,nsppol,nsym,occ,&
&  prtvol,rprimd,dtfil%unkg,wfftgs,&
&  dtset%wtk)

! Calculate the hartree part of the elastic tensor
  call status(0,dtfil%filstat,iexit,level,'call eltfrhar3')
  call eltfrhar3(eltfrhar,rprimd,gsqcut,mpi_enreg,nfft,dtset%ngfft,rhog)

! Calculate the xc part of the elastic tensor
  call status(0,dtfil%filstat,iexit,level,'call eltfrxc3 ')
  call eltfrxc3(eltfrxc,enxc,gsqcut,kxc,mpi_enreg,natom,&
& nfft,dtset%ngfft,nkxc,nspden,ntypat,psps%n1xccc,n3xccc,rhor,rprimd,&
& dtset%typat,vxc,psps%xcccrc,psps%xccc1d,xccc3d,xred)

! Calculate the local potential part of the elastic tensor
  call status(0,dtfil%filstat,iexit,level,'call eltfrloc3')
  call eltfrloc3(atindx,eltfrloc,gmet,gprimd,gsqcut,mgfft,mpi_enreg,psps%mqgrid_vl,&
&  natom,nattyp,nfft,dtset%ngfft,ntypat,ph1d,psps%qgrid_vl,rhog,&
&  ucvol,psps%vlspl)

! Calculate the Ewald part of the elastic tensor
  call status(0,dtfil%filstat,iexit,level,'call ewald4')
  call ewald4(elteew,gmet,gprimd,natom,ntypat,rmet,rprimd,&
&   dtset%typat,ucvol,xred,psps%ziontypat)

! Calculate the psp core energy part of elastic tensor (trivial)
  eltcore(1:3,1:3)=ecore/ucvol

 end if !rfstrs/=0
!End section for strain perturbation

 deallocate(ph1d,vxc,xccc3d)

 if(prtvol==-level)then
  write(message,'(a,a)') ch10,&
&   ' respfn : frozen wavef. and Ewald(q=0) part of 2DTE done. '
  call wrtout(06,message,'COLL')
 end if

 call timab(136,2,tsec)

! -----3. Initialisation of 1st response, taking into account the q vector.

 call timab(137,1,tsec)

 write(message, '(a,a,a)' )ch10,&
& ' ==>  initialize data related to q vector <== ',ch10
 call wrtout(6,message,'COLL')
 call wrtout(ab_out,message,'COLL')

 qphon(:)=dtset%qptn(:)
 sumg0=1

!Treat the case of q=0 or q too close to 0
 qzero=0
 if(qphon(1)**2+qphon(2)**2+qphon(3)**2 < 1.d-14)then
  qphon(:)=0.0d0
  write(message, '(a,a,a)' )&
&  ' respfn : the norm of the phonon wavelength (as input) was small (<1.d-7).',&
&  ch10,'  q has been set exactly to (0 0 0)'
  call wrtout(6,message,'COLL')
  sumg0=0
  qzero=1
 else
  if(rfelfd/=0 .or. rfstrs/=0)then
!  Temporarily, ...
   write(message, '(a,a,a,a,a,3es16.6,a,a,a,i2,a,i2,a,a,a)' )ch10,&
&   ' respfn : ERROR -',ch10,&
&   '  The treatment of non-zero wavevector q is restricted to phonons.',&
&   '  However, the input normalized qpt is',qphon(:),',',ch10,&
&   '  while rfelfd=',rfelfd,', and rfstrs=',rfstrs,'.',ch10,&
&   '  Action : change qpt, or rfelfd or rfstrs in the input file.'
   call wrtout(6,message,'COLL')
   call leave_new('COLL')
  else if(rfasr.eq.2)then
   write(message, '(a,a,a,a)' )ch10,&
&   ' respfn : WARNING -',ch10,&
&   '  rfasr=2 not allowed with q/=0 => rfasr was reset to 0.'
   call wrtout(6,message,'COLL')
   rfasr=0
  end if
 end if

!Examine the symmetries of the q wavevector
 allocate(symq(4,2,nsym))
 call status(0,dtfil%filstat,iexit,level,'call symq3    ')
 call symq3(nsym,qphon,symq,symrec,timrev)

!Determine the symmetrical perturbations
 allocate(pertsy(3,mpert))
 call status(0,dtfil%filstat,iexit,level,'call syper3   ')
 call syper3(indsym,mpert,natom,nsym,pertsy,rfdir,rfpert,symq,symrec,dtset%symrel)
 write(message, '(a)' ) &
& ' The list of irreducible perturbations for this q vector is:'
 call wrtout(ab_out,message,'COLL')
 ii=1
 do ipert=1,mpert
  do idir=1,3
   if(rfpert(ipert)==1.and.rfdir(idir)==1)then
    if( pertsy(idir,ipert)==1 )then
     write(message, '(i5,a,i2,a,i4)' )&
&     ii,')    idir=',idir,'    ipert=',ipert
     call wrtout(ab_out,message,'COLL')
     ii=ii+1
    end if
   end if
  end do
 end do

!Contribution to the dynamical matrix from ion-ion energy
 if(rfphon==1)then
  call status(0,dtfil%filstat,iexit,level,'call ewald3(2)')
  call ewald3(dyew,gmet,natom,qphon,rmet,sumg0,dtset%typat,ucvol,xred,psps%ziontypat)
  option=0
  call q0dy3(natom,dyewq0,dyew,option)
 end if

!1-order contribution of the xc core correction to the
!dynamical matrix
 allocate(dyfrx1(2,3,natom,3,natom))
 dyfrx1(:,:,:,:,:)=0.0d0
 if(rfphon==1.and.psps%n1xccc/=0)then
  call dyxc13(dyfrx1,gmet,gsqcut,kxc,mpi_enreg,natom,nfft,dtset%ngfft,nkxc,nspden,&
&  ntypat,psps%n1xccc,qphon,rprimd,timrev,dtset%typat,&
&  ucvol,psps%xcccrc,psps%xccc1d,xred)
 end if

!Deallocate the arrays that were needed only for the frozen
!wavefunction part
 deallocate(cg,eigen0,kg,npwarr)

!Close the unneeded temporary data files, if any
 if (mkmem==0) then
  close (unit=dtfil%unkg,status='delete')
  if (psps%useylm==1) close (unit=dtfil%unylm,status='delete')
  call WffDelete(wfftgs,ierr)
 end if

#if defined MPI
           deallocate(mpi_enreg%proc_distrb)
#endif

 allocate(blkflg(3,mpert,3,mpert))
 allocate(d2eig0(2,3,mpert,3,mpert))
 allocate(d2k0(2,3,mpert,3,mpert))
 allocate(d2lo(2,3,mpert,3,mpert))
 allocate(d2loc0(2,3,mpert,3,mpert))
 allocate(d2nfr(2,3,mpert,3,mpert))
 allocate(d2nl(2,3,mpert,3,mpert))
 allocate(d2nl0(2,3,mpert,3,mpert))
 allocate(d2nl1(2,3,mpert,3,mpert))
 allocate(d2vn(2,3,mpert,3,mpert))
 blkflg(:,:,:,:)=0
 d2eig0(:,:,:,:,:)=0.0d0 ; d2k0(:,:,:,:,:)=0.0d0
 d2lo(:,:,:,:,:)=0.0d0   ; d2loc0(:,:,:,:,:)=0.0d0
 d2nfr(:,:,:,:,:)=0.0d0  ; d2nl(:,:,:,:,:)=0.0d0
 d2nl0(:,:,:,:,:)=0.0d0  ; d2nl1(:,:,:,:,:)=0.0d0
 d2vn(:,:,:,:,:)=0.0d0

 prtbbb=dtset%prtbbb
 allocate(d2bbb(2,3,3,mpert,mband,mband*prtbbb))
 allocate(d2cart_bbb(2,3,3,mpert,mband,mband*prtbbb))
 if(prtbbb==1)then
  d2cart_bbb(:,:,:,:,:,:)=0.0d0
  d2bbb(:,:,:,:,:,:)=0.0d0
 end if

!Check whether exiting was required by the user.
!If found then do not start minimization steps
!At this first call to chkexi, initialize cpus
 cpus=dtset%cpus
 if(abs(cpus)>1.0d-5)cpus=cpus+cpui
 openexit=1 ; if(dtset%chkexit==0) openexit=0
 call chkexi(cpus,dtfil%filnam_ds(1),iexit,ab_out,mpi_enreg,openexit)
 if (iexit==0) then

! #######################################################################

  write(message,'(a,80a)')ch10,('=',mu=1,80)
  call wrtout(ab_out,message,'COLL')
  call wrtout(06,message,'COLL')

  call status(0,dtfil%filstat,iexit,level,'call loper3   ')

  ddkfil(:)=0
! Note that kg, cg, eigen0, mpw and npwarr are NOT passed to loper3 :
! they will be reinitialized for each perturbation, with an eventually
! reduced set of k point, thanks to the use of symmetry operations.
  call loper3(amass,atindx,atindx1,blkflg,codvsn,cpui,cpus,doccde,&
&  ddkfil,dtfil,dtset,dyew,dyfrlo,dyfrnl,dyfrx1,dyfrx2,d2bbb,d2lo,d2nl,&
&  eltcore,elteew,eltfrhar,eltfrkin,eltfrloc,eltfrnl,eltfrxc,&
&  etotal,fermie,gsqcut_eff,iexit,indsym,kxc,mband,mgfft,&
&  mkmem,mkqmem,mk1mem,mpert,mpi_enreg,mpsang,natom,nattyp,&
&  nfft,nkpt,nkxc,nspden,nspinor,nsppol,nsym,occ,pawtab,pertsy,prtbbb,&
&  psps,rfpert,rhog,rhor,symq,symrec,timrev,tmpfil,&
&  vtrial,vxcavg,walli,xred)

! #####################################################################

!End of the check of hasty exit
 end if

 write(message, '(80a,a,a,a,a)' ) ('=',mu=1,80),ch10,ch10,&
&  ' ---- first-order wavefunction calculations are completed ----',&
&  ch10
 call wrtout(ab_out,message,'COLL')
 call wrtout(06,  message,'COLL')

 deallocate(doccde)


!BEGIN TF_CHANGES
#if defined MPI
! in paral_respfn-case the masters have to reconstruct some array's
 if((mpi_enreg%paral_compil_respfn == 1 .and. mpi_enreg%respfn_master_comm /= MPI_COMM_NULL)) then
  !gather arrays from all cpu's
  call xsum_master(blkflg,0,mpi_enreg%respfn_master_comm,ierr)
  call xsum_master(d2lo,0,mpi_enreg%respfn_master_comm,ierr)
  call xsum_master(d2nl,0,mpi_enreg%respfn_master_comm,ierr)
  call xsum_master(vtrial,0,mpi_enreg%respfn_master_comm,ierr)
end if

!DEBUG
!write(6,*) "blkflg",mpi_enreg%paral_compil_respfn,":", blkflg
!write(6,*) "ddkfil",mpi_enreg%paral_compil_respfn,":", ddkfil
!write(6,*) "d2bbb-array",mpi_enreg%paral_compil_respfn,":", d2bbb
!write(6,*) "d2lo-array",mpi_enreg%paral_compil_respfn,":", d2lo
!write(6,*) "d2nl-array",mpi_enreg%paral_compil_respfn,":", d2nl
!write(6,*) "etotal",mpi_enreg%paral_compil_respfn,":", etotal
!write(6,*) "nspinor",mpi_enreg%paral_compil_respfn,":", nspinor
!write(6,*) "fermie",mpi_enreg%paral_compil_respfn,":", fermie
!write(6,*) "vtrial",mpi_enreg%paral_compil_respfn,":", vtrial
!write(6,*) "xred",mpi_enreg%paral_compil_respfn,":", xred
!ENDDEBUG

#endif
!END TF_CHANGES

!Output of the localization tensor
 if ((rfpert(natom+1) /= 0).and.(me == 0).and. dtset%occopt<=2) then
  call wrtloctens(blkflg,d2bbb,d2nl,mband,mpert,natom,dtset%prtbbb,rprimd)
 end if

!The perturbation  natom+1 was only an auxiliary perturbation,
!needed to construct the electric field response, so its flag
!is now set to 0.
! rfpert(natom+1)=0

!Were 2DTE computed ?
 if(rfphon==0 .and. rfelfd==2 .and. rfstrs==0 .and. rfuser==0)then

  write(message,'(a,a)' )ch10,&
&  ' respfn : d/dk was computed, but no 2DTE, so no DDB output.'
  call wrtout(6,message,'COLL')
  call wrtout(ab_out,message,'COLL')

!If 2DTE were computed, only one processor must output them and compute
!frequencies.
 else if(me==0)then

  write(message,'(a,a)' )ch10,&
&  ' ==> Compute Derivative Database <== '
  call wrtout(6,message,'COLL')
  call wrtout(ab_out,message,'COLL')

! Open the formatted derivative database file, and write the
! preliminary information
  call status(0,dtfil%filstat,iexit,level,'call ioddb8   ')
  vrsddb=010929
  dscrpt=' Note : temporary (transfer) database '
  choice=2
  ddbnm=trim(dtfil%filnam_ds(4))//'_DDB'
! tolwfr must be initialized here, but it is a dummy value
  tolwfr=1.0d0
!DEBUG
! write(6,*)'respfn:ntypat=',ntypat
!ENDDEBUG
  call ioddb8 (choice,dscrpt,ddbnm,natom,mband,&
&  nkpt,nsym,ntypat,dtfil%unddb,vrsddb,&
&  dtset%acell_orig,dtset%amu,dtset%dilatmx,dtset%ecut,dtset%ecutsm,&
&  dtset%intxc,iscf,dtset%ixc,dtset%kpt,dtset%kptnrm,&
&  natom,dtset%nband,dtset%ngfft,nkpt,nspden,nspinor,&
&  nsppol,nsym,ntypat,occ,dtset%occopt,&
&  dtset%rprim_orig,dtset%sciss,dtset%spinat,dtset%symafm,dtset%symrel,&
&  dtset%tnons,tolwfr,dtset%tphysel,dtset%tsmear,&
&  dtset%typat,dtset%wtk,xred,psps%ziontypat,dtset%znucl)

  nblok=1 ; fullinit=1
  call psddb8 (choice,psps%dimekb,psps%ekb,fullinit,psps%indlmn,&
&  psps%lmnmax,psps%lnmax,nblok,&
&  ntypat,dtfil%unddb,psps%pspso,psps%usepaw,psps%useylm,vrsddb)

! In the RESPFN code, nstdy3 and stady3 were called here
  d2nfr(:,:,:,:,:)=d2lo(:,:,:,:,:)+d2nl(:,:,:,:,:)


! In case of bbb decomposition
  if(prtbbb==1)then
   allocate(blkflg1(3,mpert,3,mpert))
   allocate(blkflg2(3,mpert,3,mpert))
   blkflg2(:,:,:,:) = blkflg(:,:,:,:)
   do ipert = 1, mpert
    do ipert2 = 1, mpert
     if ((ipert /= natom + 2).and.(ipert>natom).and.(ipert2/=natom+2)) then
      blkflg2(:,ipert2,:,ipert) = 0
     end if
    end do
   end do
   allocate(d2tmp(2,3,mpert,3,mpert))
   do iband = 1,mband
    d2tmp(:,:,:,:,:)=0.0d0
    blkflg1(:,:,:,:) = blkflg2(:,:,:,:)
    d2tmp(:,:,natom+2,:,:) = d2bbb(:,:,:,:,iband,iband)
    call d2sym3(blkflg1,d2tmp,indsym,mpert,natom,nsym,qphon,symq,&
&           symrec,dtset%symrel,timrev)
    d2bbb(:,:,:,:,iband,iband) = d2tmp(:,:,natom+2,:,:)
   end do
   deallocate(blkflg1,blkflg2,d2tmp)
  end if

! Complete the d2nfr matrix by symmetrization of the existing elements
  call d2sym3(blkflg,d2nfr,indsym,mpert,natom,nsym,qphon,symq,symrec,dtset%symrel,timrev)

! Note that there is a bug in d2sym3 which will set some elements of
!  blkflg to 1 even when no corresponding symmetry-related element
!  has been computed.  This has the effect of producing spurious extra
!  output lines in the 2nd-order matrix listing in the .out file
!  and in the DDB file. The suprious matrix elements are all zero,
!  so this is primarily an annoyance.(DRH)


! Add the frozen-wf (dyfrwf) part to the ewald part (dyew),
! the part 1 of the frozen wf part of the xc core correction
! (dyfrx1) and the non-frozen part (dynfr) to get the second-order
! derivative matrix (d2matr), then
! take account of the non-cartesian coordinates (d2cart).
  allocate(d2cart(2,3,mpert,3,mpert))
  allocate(carflg(3,mpert,3,mpert))
  allocate(d2matr(2,3,mpert,3,mpert))
  outd2=1
  call status(0,dtfil%filstat,iexit,level,'call gath3    ')
  call gath3(blkflg,carflg,&
&  dyew,dyfrwf,dyfrx1,d2bbb,d2cart,d2cart_bbb,d2matr,d2nfr,&
&  eltcore,elteew,eltfrhar,eltfrkin,eltfrloc,eltfrnl,eltfrxc,&
&  gprimd,mband,mpert,natom,ntypat,outd2,dtset%prtbbb,&
&  rfasr,rfdir,rfpert,rprimd,dtset%typat,ucvol,psps%ziontypat)

! Output of the dynamical matrix
! (Note : remember, previously, the processor me=0 has been selected)
  call status(0,dtfil%filstat,iexit,level,'call dyout3   ')
  call dyout3(blkflg,carflg,dtfil%unddb,ddkfil,dyew,dyfrlo,&
&  dyfrnl,dyfrx1,dyfrx2,d2cart,d2cart_bbb,d2eig0,d2k0,d2lo,d2loc0,d2matr,&
&  d2nl,d2nl0,d2nl1,d2vn,&
&  eltcore,elteew,eltfrhar,eltfrkin,eltfrloc,eltfrnl,eltfrxc,&
&  ab_out,mband,mpert,natom,ntypat,&
&  outd2,dtset%prtbbb,prtvol,qphon,qzero,dtset%typat,rfdir,rfpert,rfphon,&
&  rfstrs,psps%ziontypat)

  close(dtfil%unddb)

! In case of phonons, diagonalize the dynamical matrix
  if(rfphon==1)then

!  First, suppress the 'wings' elements,
!  for which the diagonal element is not known
   call wings3(carflg,d2cart,mpert,natom)

!  Check the analyticity of the dynamical matrix
   analyt=0
   if (rfpert(natom+2)==0 .or. rfpert(natom+2)==2 .or. sumg0==1 ) analyt=1

!  Diagonalize the analytic part
   allocate(displ(2*3*natom*3*natom))
   allocate(eigval(3*natom),eigvec(2*3*natom*3*natom))
   allocate(phfrq(3*natom))
   qphnrm=1.0d0
   call phfrq3(dtset%amu,displ,d2cart,eigval,eigvec,indsym,mpert,&
&   nsym,natom,nsym,ntypat,phfrq,qphnrm,qphon,dtset%rprimd_orig,0,dtset%symrel,dtset%typat,ucvol,xred)

!  Print the phonon frequencies
   call prtph3(displ,0,dtset%enunit,-1,ab_out,natom,phfrq,qphnrm,qphon)

!  Check the completeness of the dynamical matrix and eventually send a warning
   call chkph3(carflg,0,mpert,natom)

!  In case of a non-analytical part,
!  get the phonon frequencies for three different directions
!  (in cartesian coordinates)
   if(analyt==0)then
    qphnrm=0.0d0
    do idir=1,3
!    Need to know the corresponding dielectric constant
     if(carflg(idir,natom+2,idir,natom+2)==1)then
      qphon(:)=0.0d0 ; qphon(idir)=1.0d0
!     Get the phonon frequencies
      call phfrq3(dtset%amu,displ,d2cart,eigval,eigvec,indsym,mpert,&
&      nsym,natom,nsym,ntypat,phfrq,qphnrm,qphon,dtset%rprimd_orig,0,dtset%symrel,dtset%typat,ucvol,xred)
!     Print the phonon frequencies
      call prtph3(displ,0,dtset%enunit,-1,ab_out,natom,phfrq,qphnrm,qphon)
!     Check the completeness of the dynamical matrix
!     and eventually send a warning
      call chkph3(carflg,idir,mpert,natom)
     end if
    end do
   if (idir < 4) then
    qphon(idir)=0.0d0
   end if
   end if

   deallocate(displ,eigval,eigvec)
   deallocate(phfrq)

! End condition on phonons
  end if

  deallocate(carflg,d2cart,d2matr)

!End of the DDB output part
 end if

!Deallocate arrays
 deallocate(amass,atindx,atindx1,blkflg)
 deallocate(dyew,dyewq0,dyfrlo,dyfrnl,dyfrwf,dyfrx1,dyfrx2)
 deallocate(d2bbb,d2cart_bbb)
 deallocate(d2eig0,d2k0,d2lo,d2loc0,d2nfr,d2nl,d2nl0,d2nl1,d2vn)
 deallocate(eltcore,elteew,eltfrhar,eltfrnl,eltfrloc,eltfrkin,eltfrxc)
 deallocate(grxc,indsym,kxc,nattyp,pertsy)
 deallocate(rfpert,rhog,rhor,symq,symrec,vtrial,ylm,ylmgr)

!Clean the header
 call hdr_clean(hdr)

 write(message, '(a,a)' ) ch10,' respfn : exiting '
 call wrtout(06,message,'COLL')

 call status(0,dtfil%filstat,iexit,level,'exit          ')

 call timab(137,2,tsec)
 call timab(132,2,tsec)

end subroutine respfn
!!***
