!{\src2tex{textfont=tt}}
!!****f* ABINIT/subdiago
!! NAME
!! subdiago
!!
!! FUNCTION
!! This routine diagonalizes the Hamiltionian is the eigenfunctions subspace
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (DCA, XG, GMR, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.
!!
!! INPUTS
!!  filstat=name of the status file
!!  icg=shift to be applied on the location of data in the array cg
!!  ikpt=number of the k-point
!!  inonsc=index of non self-consistent loop
!!  istwf_k=input parameter that describes the storage of wfs
!!  mcg=second dimension of the cg array
!!  mpi_enreg=informations about MPI parallelization
!!  nband_k=number of bands at this k point for that spin polarization
!!  npw_k=number of plane waves at this k point
!!  nspinor=number of spinorial components of the wavefunctions
!!  prtvol=control print volume and debugging output
!!  subham(nband_k*(nband_k+1))=Hamiltonian expressed in sthe WFs subspace
!!  subovl(nband_k*(nband_k+1)*use_subovl)=overlap matrix expressed in sthe WFs subspace
!!  use_subovl=1 if the overlap matrix is not identity in WFs subspace
!!  usepaw= 0 for non paw calculation; =1 for paw calculation
!!
!! OUTPUT
!!  eig_k(nband_k)=array for holding eigenvalues (hartree)
!!  evec(2*nband_k,nband_k)=array for holding eigenvectors
!!
!! SIDE EFFECTS
!!  cg(2,mcg)=wavefunctions
!!  gsc(2,mcg*usepaw)=<g|S|c> matrix elements (S=overlap)
!!
!! PARENTS
!!      vtowfk
!!
!! CHILDREN
!!      chpev,chpgv,hermit,leave_new,mpi_bcast,normev,sdirot,status,timab
!!      wrtout,zhpev,zhpgv
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine subdiago(cg,filstat,eig_k,evec,gsc,icg,ikpt,inonsc,istwf_k,&
&                    mcg,mpi_enreg,nband_k,npw_k,nspinor,prtvol,&
&                    subham,subovl,use_subovl,usepaw)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_00basis
 use interfaces_01managempi
 use interfaces_01util
 use interfaces_04wfs
#endif
!End of the abilint section

 implicit none

#if defined MPI || defined MPI_FFT
          include 'mpif.h'
#endif
!Arguments ------------------------------------
 integer,intent(in) :: icg,ikpt,inonsc,istwf_k,mcg,nband_k,npw_k
 integer,intent(in) :: nspinor,prtvol,use_subovl,usepaw
 character(len=fnlen),intent(in) :: filstat
 type(MPI_type),intent(inout) :: mpi_enreg
 real(dp),intent(inout) :: subham(nband_k*(nband_k+1)),subovl(nband_k*(nband_k+1)*use_subovl)
 real(dp),intent(out) :: eig_k(nband_k),evec(2*nband_k,nband_k)
 real(dp),intent(inout) :: cg(2,mcg),gsc(2,mcg*usepaw)

!Local variables-------------------------------
 integer,parameter :: level=8
 integer :: iband,ii,ierr,iexit
 character(len=500) :: message
 real(dp) :: tsec(2)
 real(dp),allocatable :: zhpev1(:,:),zhpev2(:)

! *********************************************************************

#ifdef VMS
!DEC$ ATTRIBUTES ALIAS:'ZHPEV' :: zhpev
!DEC$ ATTRIBUTES ALIAS:'ZHPGV' :: zhpgv
#endif

!Impose Hermiticity on diagonal elements of subham (and subovl, if needed)
 call status(inonsc,filstat,iexit,level,'call hermit   ')
 call hermit(subham,subham,ierr,nband_k)
 if (use_subovl==1) call hermit(subovl,subovl,ierr,nband_k)

! Diagonalize the Hamitonian matrix
 call status(inonsc,filstat,iexit,level,'call zhpev    ')
 allocate(zhpev1(2,2*nband_k-1),zhpev2(3*nband_k-2))

#if defined T3E
#if defined MPI
 if ((mpi_enreg%paralbd <=1) .or. ((mpi_enreg%paralbd >1) .and. &
&    (mpi_enreg%me_group==0))) then
#endif
  if (use_subovl==1) then
   call CHPGV(1,'V','U',nband_k,subham,subovl,eig_k,evec,nband_k,&
&             zhpev1,zhpev2,ierr)
  else
   call CHPEV ('V','U',nband_k,subham,eig_k,evec,nband_k,zhpev1,&
&              zhpev2,ierr)
#if defined MPI
  end if
 end if
 if (mpi_enreg%paralbd >1) then
  call timab(48,1,tsec)
  call MPI_BCAST(evec,2*nband_k*nband_k, &
&   MPI_DOUBLE_PRECISION,0,mpi_enreg%kpt_comm(mpi_enreg%num_group),ierr)
  call timab(48,2,tsec)
 end if
#endif
#else
#if defined MPI
 if ((mpi_enreg%paralbd <=1) .or. ((mpi_enreg%paralbd >1) .and. &
&    (mpi_enreg%me_group==0))) then
#endif
  if (use_subovl==1) then
   call ZHPGV(1,'V','U',nband_k,subham,subovl,eig_k,evec,nband_k,&
&             zhpev1,zhpev2,ierr)
  else

   call ZHPEV ('V','U',nband_k,subham,eig_k,evec,nband_k,zhpev1,&
&               zhpev2,ierr)
  end if
#if defined MPI
 end if

 if (mpi_enreg%paralbd >1) then
  call timab(48,1,tsec)
  call MPI_BCAST(evec,2*nband_k*nband_k, &
&   MPI_DOUBLE_PRECISION,0,mpi_enreg%kpt_comm(mpi_enreg%num_group),ierr)
  call timab(48,2,tsec)
 end if
#endif
#endif

 deallocate(zhpev1,zhpev2)
!DEBUG
!write(6,*)' subdiago : after zhpev '
!stop
!ENDDEBUG

!Normalize each eigenvector and set phase:
 call status(inonsc,filstat,iexit,level,'call normev   ')
 call normev(evec,nband_k,nband_k)

 if(prtvol==-level)then
  write(message,'(a)')&
&  ' subdiago : iband band  evec(re:im)'
  call wrtout(06,message,'PERS')
  do iband=1,nband_k
   do ii=1,nband_k
    write(message,'(2i5,2es16.6)')&
&    iband,ii,evec(2*ii-1,iband),evec(2*ii,iband)
    call wrtout(06,message,'PERS')
   end do
  end do
 end if

 if(istwf_k==2)then
  do iband=1,nband_k
   do ii=1,nband_k
    if(abs(evec(2*ii,iband))>1.0d-10)then
     write(message,'(a,a,a,a,2i5,2es16.6,a,a)')ch10,&
&     ' subdiago : BUG ',&
&     '  For istwf_k=2, observed the following element of evec :',ch10,&
&     iband,ii,evec(2*ii-1,iband),evec(2*ii,iband),ch10,&
&     '  with a non-negligible imaginary part.'
     call wrtout(06,message,'PERS')
     call leave_new('PERS')
    end if
   end do
  end do
 end if

!Carry out rotation of bands C(G,n) according to evecs:
 call status(inonsc,filstat,iexit,level,'call sdirot   ')
 call sdirot(cg,evec,icg,mcg,nband_k,nband_k,npw_k*nspinor)
!If paw, musb also rotate S.C(G,n):
 if (usepaw==1) call sdirot(gsc,evec,icg,mcg,nband_k,nband_k,npw_k*nspinor)
! DEBUG
!  write(6,*)' subdiago : cg(1:2) for different bands (3) '
!  do iband=1,nband_k
!   iwavef=(iband-1)*npw_k+icg
!   write(6, '(4es16.6)' )cg(1:2,1+iwavef:2+iwavef)
!  end do
! ENDDEBUG

end subroutine subdiago
