!{\src2tex{textfont=tt}}
!!****f* ABINIT/tddft
!!    
!! NAME
!! tddft
!!
!! FUNCTION
!! Compute the excitation energies within TDLDA
!! from input wavefunctions, eigenenergies, and band occupations.
!!
!! COPYRIGHT
!! Copyright (C) 1999-2004 ABINIT group (XG, JYR, MB, MBELAND, SHAMEL)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!  cg(2,mpw*nspinor*mband*mkmem*nsppol)=wf in G space
!!  dtfil <type(datafiles_type)>=variables related to files
!!  dtset <type(dataset_type)>=all input variables for this dataset
!!  eigen(mband*nkpt*nsppol)=array for holding eigenvalues (hartree)
!!  etotal=total energy of the ground-state (Ha)
!!  gmet(3,3)=metrix tensor in G space in Bohr**-2.
!!  gprimd(3,3)=dimensional reciprocal space primitive translations
!!  gsqcut=cutoff on (k+G)^2 (bohr^-2)
!!  kg(3,mpw*mkmem)=reduced planewave coordinates.
!!  kxc(nfft,nkxc)=exchange-correlation kernel
!!  mband=maximum number of bands
!!  mgfftdiel=maximum size of 1D FFTs, for the computation of the dielectric matrix
!!  mkmem=maximum number of k points in core memory
!!  mpi_enreg=informations about MPI parallelization
!!  mpw=maximum allowed value for npw
!!  nfft=(effective) number of FFT grid points (for this processor)
!!       WARNING about parallelization: see below
!!  ngfftdiel(18)=contain all needed information about 3D FFT, for dielectric matrix,
!!                see ~abinit/doc/input_variables/vargs.htm#ngfft

!!  nkpt=number of k points
!!  nkxc=second dimension of the array kxc (see rhohxc for a description)
!!  npwarr(nkpt)=number of planewaves at each k point
!!  nspinor=number of spinorial components of the wavefunctions
!!  nsppol=1 for unpolarized, 2 for spin-polarized
!!  occ(mband*nkpt*nsppol)=
!!          occupation numbers for each band (usually 2.0) at each k point
!!  ucvol=unit cell volume (Bohr**3)
!!  wffnew=unit number for current wf disk file
!!
!! OUTPUT
!!  (only writing)
!!
!! WARNING:
!! This routine should not be parallelized on space for the time being,
!!    because the already existing parallelisation is not the usual one, found
!!    in the majority of ABINIT routines.
!! 
!! NOTES
!! * Only accept nspinor=1, nsppol=1, nkpt=1 (Gamma point), and occopt<3
!!   (insulating occupation numbers).
!!   It is expected to make it work for nsppol=2 in the future.
!!
!! * For the oscillator strengths, see the paper
!!   ''Time-Dependent Density Functional Response Theory of Molecular
!!     systems: Theory, Computational Methods, and Functionals'', by M.E. Casida,
!!   in Recent Developments and Applications of Modern Density Functional
!!   Theory, edited by J.M. Seminario (Elsevier, Amsterdam, 1996).
!!
!! TODO
!! The temporary files should be deleted at the end of this routine
!!
!! PARENTS
!!      vtorho
!!
!! CHILDREN
!!      chpev,fourdp,fourwf,hartre,hdr_skip,int2char4,leave_new,matr3inv
!!      mpi_barrier,mpi_bcast,mpi_gatherv,mpi_reduce,mpi_scatterv,rdnpw
!!      read_wfrspa,rwwf,sort_dp,sphereboundary,timab,wrtout,xcomm_world
!!      xme_whoiam,xproc_max,xsum_master,zhpev
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine tddft(cg,dtfil,dtset,eigen,etotal,gmet,gprimd,gsqcut,&
&  kg,kxc,mband,mgfftdiel,mkmem,mpi_enreg,mpw,nfft,ngfftdiel,nkpt,nkxc,&
&  npwarr,nspinor,nsppol,occ,ucvol,wffnew)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_00basis
 use interfaces_01managempi
 use interfaces_01util
 use interfaces_02ffts
 use interfaces_03xc
 use interfaces_04iowfdenpot
#else
 use defs_xfuncmpi
 use defs_interfaces
#endif
!End of the abilint section

 implicit none

#         if defined MPI || defined MPI_FFT
          include 'mpif.h'
#         endif
!Arguments ------------------------------------
 integer, intent(in) :: mband,mgfftdiel,mkmem,mpw,nfft,nkpt,nkxc,nsppol
 integer, intent(inout) :: nspinor
 real(dp), intent(in) :: etotal,gsqcut,ucvol
 type(datafiles_type), intent(in) :: dtfil
 type(dataset_type), intent(in) :: dtset
 type(MPI_type), intent(inout) :: mpi_enreg
 type(wffile_type), intent(inout) :: wffnew
 integer, intent(in) :: kg(3,mpw*mkmem),ngfftdiel(18),npwarr(nkpt)
 real(dp), intent(in) :: cg(2,mpw*nspinor*mband*mkmem*nsppol),eigen(mband*nkpt*nsppol)
 real(dp), intent(in) :: gmet(3,3),gprimd(3,3),kxc(nfft,nkxc),occ(mband*nkpt*nsppol)

!Local variables-------------------------------
 integer,parameter :: nexcitout=20
 integer :: cplex,i,i1,i2,i3,iband,idir,ier,ierr,iexcit,iexcit1,iexcit2,ifft,ii
 integer :: old_iexcit
 integer :: ikpt,index,iocc,iocc1,iocc2,iocc_l,iocc_n,ipw,isppol
 integer :: istwf_k,iunocc,iunocc1,iunocc2,iunocc_l,iunocc_n,jband,jexcit
 integer :: jexcit1,jexcit_cbase,master,mcg_disk,me_loc,mu,nband1,nband_k
 integer :: nband_occ,nband_unocc,nband_win,ndiel,ndiel1,ndiel2,ndiel3,ndiel4
 integer :: ndiel5,ndiel6,nexcit,nexcit_max,nexcit_win,nfftdiel,nlargest,nnext
 integer :: nproc_loc,npw1,npw_k,pole_approx,sing_trip,spaceComm,tim_fourwf
 integer :: tim_rwwf,save_accesswff
 integer :: rec,recl,idummy,jdummy
 real(dp) :: buffer,buffer_inv,cim,cre,diffeig,eigbnd,eigunocc,emax_win
 real(dp) :: f_singlet,f_triplet,factor,ff,flargest,fnext,fr_invsquare,fr_power
 real(dp) :: normint,product,root1,root2,saa,sab,sbb,si1,si2,si3,sum_exc
 real(dp) :: sum_exc_i,sum_exc_r,sum_singlet,sum_triplet,sumsqr,sumx,sumy,sumz
 real(dp) :: theta,thppi,weight,xx,yy
 logical :: am_master,file_exist
 logical, allocatable :: done_excit(:,:),done_sexc(:),done_sexc2(:)
 character(len=fnlen) :: fname_tdexcit,fname_wf
 character(len=4) :: tag
 character(len=500) :: message
 integer,allocatable :: flagb_win(:),gbound(:,:),indarr(:),index_band(:)
 integer,allocatable :: kg_dum(:,:),kg_k(:,:)
 integer,allocatable :: excit_coords(:,:)
 integer :: count_to_do, count, displ, countmax, displmax
 integer :: ijexcit, ijexcit2, sendcount
 real(dp) :: sendbuf(4)
 real(dp) :: cauchy(7),poscart(3),qphon(3),rprimd(3,3),tsec(2),dummy(2,1)
 real(dp),allocatable :: cg_disk(:,:),cwavef(:,:),eexcit(:)
 real(dp),allocatable :: eexcit2(:),eig_dum(:),eigsqr_singlet(:,:,:,:)
 real(dp),allocatable :: eigsqr_triplet(:,:,:,:),kxc_singlet(:,:,:)
 real(dp),allocatable :: kxc_triplet(:,:,:),matr(:),occ_dum(:)
 real(dp),allocatable :: osc_str(:,:),pos(:,:),rhoaug(:,:,:),rhog(:,:)
 real(dp),allocatable :: sexc(:,:),sqrtks(:),vec(:,:,:),vhartr(:),wfprod(:,:,:)
 real(dp) :: eigsqr_singlet_dummy, eigsqr_triplet_dummy
 real(dp),allocatable :: wfraug(:,:,:,:),wfrspa(:,:,:,:),work(:),zhpev1(:,:)
 real(dp),allocatable :: zhpev2(:)
!no_abirules
#          if defined MPI
           integer :: iproc,jband_loc,nband_per_proc,nbuf
           integer,allocatable :: counts(:),displs(:),recvcounts(:)
           real(dp), allocatable :: recvbuf(:,:)
#          endif 

! *************************************************************************

#ifdef VMS
!DEC$ ATTRIBUTES ALIAS:'ZHPEV' :: zhpev
#endif

!DEBUG
!write(6,*)' tddft : enter '
!call flush(6)
!if(mkmem==0)stop
!ENDDEBUG

!Init mpi_comm
!BEGIN TF_CHANGES
 call xcomm_world(mpi_enreg,spaceComm)
!END TF_CHANGES

 am_master=.true.
 master = 0
!Init ntot proc max
 call xproc_max(nproc_loc,ierr)

!Define who i am
 call xme_whoiam(me_loc)

#if defined MPI

           if (me_loc/=0) then
            am_master=.FALSE.
           endif
           write(message, '(a,i3,a)' ) ' TDDFT ',nproc_loc,' CPU synchronized'
           call wrtout(6,message,'COLL')
           write(message, '(a,3D12.5,a,3D12.5,a,3D12.5)' ) ' gmet ',&
&                             gmet(1,1),gmet(1,2),gmet(1,3),ch10,&
&                             gmet(2,1),gmet(2,2),gmet(2,3),ch10,&
&                             gmet(3,1),gmet(3,2),gmet(3,3)                               
           call wrtout(6,message,'COLL')
#endif 


!COMMENT these values should become arguments
!the two first define the energy window

!XG 020209 : Jean-Yves, note that the definition 
!of td_maxene is in Hartree ...
 emax_win = dtset%td_maxene
!nexcit_max is defined later
!nexcit_max = 600   

 call timab(95,1,tsec)

 istwf_k=dtset%istwfk(1)

 if(nkpt/=1 .or. &
&    abs(dtset%kptns(1,1))+abs(dtset%kptns(2,1))+abs(dtset%kptns(3,1))>1.0d-6 )then
  write(message, '(a,a,a,a,a,a,a,a,i4,a,3es14.6,a,a,a,a,a)' )ch10,&
&  ' tddft : ERROR -',ch10,&
&  '  The computation of excited states using TDDFT is only allowed',ch10,&
&  '  with nkpt=1, kpt=(0 0 0), but the following values are input:',ch10,&
&  '  nkpt=',nkpt,', kpt=',dtset%kptns(1:3,1),'.',ch10,&
&  '  Action : in the input file, set nkpt to 1 and kpt to 0 0 0 ,',ch10,&
&  '  or change iscf.'
  call wrtout(6,message,'COLL')
  call leave_new('COLL')
 endif

 if(nsppol/=1)then
  write(message, '(a,a,a,a,a,a,a,a,a,a)' ) ch10,&
&  ' tddft :  ERROR -',ch10,&
&  '  The computation of excited states using TDDFT is restricted',ch10,&
&  '  for the time being to nsppol=1, while input nsppol=2.',ch10,&
&  '  Action : if you want to compute excited states within TDDFT,',ch10,&
&  '  set nsppol to 1 in the input file. Otherwise, do not use iscf=-1.'
  call wrtout(6,message,'COLL')
  call leave_new('COLL')
 endif

 if(nspinor/=1)then
  write(message, '(a,a,a,a,a,a,a,a,a,a)' ) ch10,&
&  ' tddft :  ERROR -',ch10,&
&  '  The computation of excited states using TDDFT is restricted',ch10,&
&  '  for the time being to nspinor=1, while input nspinor=2.',ch10,&
&  '  Action : if you want to compute excited states within TDDFT,',ch10,&
&  '  set nsppol to 1 in the input file. Otherwise, do not use iscf=-1.'
  call wrtout(6,message,'COLL')
  call leave_new('COLL')
 endif

 if(dtset%occopt>2)then
  write(message, '(a,a,a,a,a,a,i2,a,a,a,a,a)' ) ch10,&
&  ' tddft :  ERROR -',ch10,&
&  '  The computation of excited states using TDDFT is only allowed',ch10,&
&  '  with occopt=0, 1, or 2, while input occopt=',dtset%occopt,'.',ch10,&
&  '  Action : if you want to compute excited states within TDDFT,',ch10,&
&  '  set occopt=0, 1, or 2 in the input file. Otherwise, do not use iscf=-1.'
  call wrtout(6,message,'COLL')
  call leave_new('COLL')
 endif


!Examine the occupation numbers, and determine the number of 
!occupied and unoccupied states.
 nband_k=dtset%nband(1)
 npw_k=npwarr(1)
 nband_occ=0
 do iband=1,nband_k
  if(abs(occ(iband)-2.0d0)<1.0d-6)nband_occ=nband_occ+1
 enddo
 nband_unocc=nband_k-nband_occ
 nexcit=nband_occ*nband_unocc

 if(dtset%td_mexcit==0)then
  nexcit_max=nexcit
 else
  nexcit_max = dtset%td_mexcit
 endif

 write(message, '(a,a,a,a,i4,a,i4,a,a,i4,a,a,a,i6,a)' )ch10,&
& ' *** TDDFT : computation of excited states *** ',ch10,&
& ' Splitting of',dtset%nband(1),' bands in',nband_occ,' occupied bands,',&
&     ' and',nband_unocc,' unoccupied bands,',ch10,&
& ' giving',nexcit,' excitations.'
 call wrtout(6,message,'COLL')
 call wrtout(ab_out,message,'COLL')

!Allocate the matrices to be diagonalized. 
!Use a simple storage mode, to be improved in the future.
 allocate(eigsqr_singlet(nband_occ,nband_unocc,nband_occ,nband_unocc))
 allocate(eigsqr_triplet(nband_occ,nband_unocc,nband_occ,nband_unocc))
 allocate(eexcit(nexcit),sqrtks(nexcit))
 allocate(flagb_win(nband_k))
 eigsqr_singlet(:,:,:,:)=0.0d0
 eigsqr_triplet(:,:,:,:)=0.0d0

!Fill the diagonal elements with square of differences of KS eigenvalues
!(also not very efficient, but OK for the present first coding)
!Also compute the square root of Kohn-Sham eigenvalue differences
 do iunocc=1,nband_unocc
  eigunocc=eigen(iunocc+nband_occ)
  do iocc=1,nband_occ
   iexcit=iocc+nband_occ*(iunocc-1)
   diffeig=eigunocc-eigen(iocc)
   eigsqr_singlet(iocc,iunocc,iocc,iunocc)=diffeig**2
   eigsqr_triplet(iocc,iunocc,iocc,iunocc)=diffeig**2
   eexcit(iexcit)=diffeig
   sqrtks(iexcit)=sqrt(diffeig)
  enddo
 enddo

!Sort the excitation energies : note that the array eexcit is reordered
 allocate(indarr(nexcit))
 indarr(:)=(/ (ii,ii=1,nexcit) /)
 call sort_dp(nexcit,eexcit,indarr,tol14)

!Determine an energy window for the excitations
!to take into account. This is necessary for large systems

 nexcit_win = 0
 do iexcit = 1, nexcit
  if ((eexcit(iexcit) < emax_win ).and.(nexcit_win < nexcit_max)) then
   nexcit_win = nexcit_win + 1 
  endif 
 enddo

!identification of the bands contributing to the 
!nexcit_win  excitations within the window

 nband_win = 0
 flagb_win(:) = 0
 do iexcit = 1, nexcit_win
  iexcit1 = indarr(iexcit)
  iunocc1 = (iexcit1-1)/nband_occ+1
  iocc1   = iexcit1-(iunocc1-1)*nband_occ
  if (flagb_win(nband_occ+iunocc1)==0) flagb_win(nband_occ+iunocc1) =1
  if (flagb_win(iocc1)==0) flagb_win(iocc1) =1 
 enddo

 do iband=1,nband_k
  nband_win=nband_win+flagb_win(iband)
 enddo


 write(message,'(a,i5)') 'Nr of bands to Fourier transform : ',nband_win      
 call wrtout(6,message,'COLL')

 ndiel1=ngfftdiel(1) ; ndiel2=ngfftdiel(2) ; ndiel3=ngfftdiel(3)
!ndiel4,ndiel5,ndiel6 are FFT dimensions, modified to avoid cache trashing
 ndiel4=ngfftdiel(4) ; ndiel5=ngfftdiel(5) ; ndiel6=ngfftdiel(6)

!The evaluation of integrals, later, needs the following factor
 normint=1.d0/(ucvol*dble(ndiel1*ndiel2*ndiel3))

!Setup the positions in real space for later integration
 call matr3inv(gprimd,rprimd)

 allocate(pos(max(ndiel1,ndiel2,ndiel3),3))

!Select the reduced position of the point with respect to the box center,
!in the interval ]-0.5,0.5]. 
 buffer=0.05d0 ; buffer_inv=one/buffer
 do idir=1,3
  if(idir==1)ndiel=ndiel1
  if(idir==2)ndiel=ndiel2
  if(idir==3)ndiel=ndiel3
  do ii=1,ndiel
!  dtset%boxcenter(3)=reduced coordinates of the center of the box,
!  in view of the computation of the oscillator strength
   pos(ii,idir)=(ii-1)/(1.0d0*ndiel)-dtset%boxcenter(idir)
   pos(ii,idir)=pos(ii,idir)-nint(pos(ii,idir)-tol12)
!  The linear behaviour is cut-off when one becomes
!  close to the boundaries : the buffer allows to match smoothly
!  one side of the cell to the other. This is important
!  to get rid of small breakings of symmetry, that are
!  confusing in accurate tests
   if(abs(pos(ii,idir))>half-buffer)then
!   xx is always positive, and goes linearly from 1 to 0
!   in the buffer region
    xx=(half-abs(pos(ii,idir)))*buffer_inv
!   The cut-off is applied to pos(:,:)
    pos(ii,idir)=pos(ii,idir)*xx*(two-xx)
!DEBUG
!   if (idir==1)then
!    write(6,'(a,i2,a,F12.5,F12.5)')'idiel : ',ii,'   x : ',pos(ii,idir),&
!&    dtset%boxcenter(idir)
!   endif 
!ENDDEBUG
   endif
  enddo
 enddo

!  need to run in MPI I/O case   
  if (wffnew%accesswff == 1 ) then
   save_accesswff=wffnew%accesswff
   wffnew%accesswff = 0
  else
!  Do not store value but set to have save_accesswff /= 1
   save_accesswff = 0
  end if 


 if(am_master)then

!-----------------------------------------------------------
!Do i/o as needed, mkmem==0 means wf and kg info on disk file
!The disk access is only done by master...

  allocate(gbound(2*mgfftdiel+8,2),kg_k(3,npw_k))

  if (mkmem==0) then

!  Skip wffnew header
   call hdr_skip(wffnew,ierr)
!  rewind the kpgsph data file on unit unkg
   rewind (dtfil%unkg)
   mcg_disk=mpw*nspinor*mband
   allocate(cg_disk(2,mcg_disk))

   ikpt=1 ; isppol=1
   call rdnpw(ikpt,isppol,nband_k,npw_k,nspinor,0,dtfil%unkg)
!  Read k+g data
   read (dtfil%unkg) kg_k(:,1:npw_k)
   call sphereboundary(gbound,istwf_k,kg_k,mgfftdiel,npw_k)

  else   !so, mkmem/=0

   ikpt=1 ; isppol=1
!  Only one k point
   kg_k(:,1:npw_k)=kg(:,1:npw_k)
   call sphereboundary(gbound,istwf_k,kg_k,mgfftdiel,npw_k)

  end if !mkmem==0

  if(mkmem==0)then
!  Read the wavefunction block for ikpt,isppol
   tim_rwwf=8
   allocate(eig_dum(mband),kg_dum(3,0),occ_dum(mband))
   call rwwf(cg_disk,eig_dum,0,0,0,ikpt,isppol,kg_dum,mband,mcg_disk,nband_k,nband_k,&
&  npw_k,nspinor,occ_dum,-2,0,tim_rwwf,wffnew)
   deallocate(eig_dum,kg_dum,occ_dum)

  end if !mkmem==0

 endif ! am_master

!  need to run in MPI I/O case   
  if ( save_accesswff == 1 )  wffnew%accesswff = 1

!    end of read
!-------------------------------------------------------------


!DEBUG
!write(6,'(a)') 'After reading the wavefunction'      
!call flush(6)
!call wrtout(6,message,'COLL')
!ENDDEBUG


!Use a simple implementation for the computation of the kernel elements
 if (am_master) then
  allocate(cwavef(2,mpw))
  allocate(rhoaug(ndiel4,ndiel5,ndiel6),wfraug(2,ndiel4,ndiel5,ndiel6))
 endif
 allocate(index_band(nband_k)) 

!If mkmem is not 0, all real-space states are kept in memory
!                0, only 2   are in memory at the same time
 if (mkmem/=0) then
  allocate(wfrspa(ndiel4,ndiel5,ndiel6,nband_win))
 else
  allocate(wfrspa(ndiel4,ndiel5,ndiel6,2))
 endif

!DEBUG
!write(message,'(a)') 'After allocating wfrspa'      
!call wrtout(6,message,'COLL')
!ENDDEBUG

 weight=0.0d0

!Generate states in real space, only for bands contributing to excitations in window
 jband=0
 do iband=1,nband_k

  if(flagb_win(iband) == 1) then
   jband=jband+1
   index_band(iband)=jband

   if (am_master) then
!   Obtain Fourier transform in fft box
    if(mkmem/=0)cwavef(:,1:npw_k)=cg(:,1+(iband-1)*npw_k:iband*npw_k)
    if(mkmem==0)cwavef(:,1:npw_k)=cg_disk(:,1+(iband-1)*npw_k:iband*npw_k)

!DEBUG
!   write(6,*)' iband : ',iband,'  -> index ',jband,index_band(iband)
!ENDDEBUG


    tim_fourwf=14
!   This call should be made by master, and then the results be sent to the other procs

    call fourwf(1,rhoaug,cwavef,dummy,wfraug,gbound,gbound,&
     & istwf_k,kg_k,kg_k,mgfftdiel,mpi_enreg,1,ngfftdiel,npw_k,1,ndiel4,ndiel5,ndiel6,&
     & 0,tim_fourwf, weight)

!DEBUG
!   write(message, '(a,i5)')' After Fourier proc ',me_loc
!ENDDEBUG

!   Fix the phase, and checks that the wavefunction is real
!   (should be merged with routine fxphas)
    saa=zero ; sab=zero ; sbb=zero
    do i3=1,ndiel3
     do i2=1,ndiel2
      do i1=1,ndiel1
       saa=saa+wfraug(1,i1,i2,i3)**2
       sbb=sbb+wfraug(2,i1,i2,i3)**2
       sab=sab+wfraug(1,i1,i2,i3)*wfraug(2,i1,i2,i3)
      enddo
     enddo
    enddo

    if(sbb>5.0d-9)then

     write(message, '(a,a,a,a,a,a,es20.10,a,i4,a)' )ch10,&
&     ' tddft : WARNING -',ch10,&
&     '  The imaginary part of wavefunctions should be practically zero.',&
&     ch10,'  This is not the case, since sbb=',sbb,' for iband=',iband,'.'
     call wrtout(6,message,'PERS')
     if(sbb>1.0d-7)then
      call leave_new('COLL')
     endif
    endif

!   Possibility of writing to disk

    if (mkmem/=0) then
     wfrspa(:,:,:,jband)=wfraug(1,:,:,:)
    else
     call int2char4(jband,tag) 
     fname_wf=trim(dtfil%filnam_ds(5))//'_TDWF'//tag
     write(6,*)'fileout : ',fname_wf
     open(tmp_unit,file=fname_wf,form='unformatted',status='unknown')
     write(tmp_unit)iband,eigen(iband)
     write(tmp_unit)wfraug(1,:,:,:)
     close(tmp_unit)
    endif
   endif !am_master 

!  if mkmem=0 the eigenstates (in r space) with eigenvalues inside
!  the energy window have been stored to disk
!  may cause enormous disk usage, slow down computation. Use only
!  if limited memory is available. This option allows a restart.

  endif

!End loop on iband
 enddo

 if (am_master) then
  deallocate(gbound,kg_k)
  deallocate(cwavef)
  deallocate(rhoaug,wfraug)
  if(mkmem==0)deallocate(cg_disk)
 endif
 deallocate(flagb_win)

!Case mkmem /= 0 , send wfrspa from master to world
#if defined MPI
           if (mkmem/=0) then
            nbuf=ndiel4*ndiel5*ndiel6*nband_win
            call MPI_BCAST(wfrspa,nbuf,MPI_DOUBLE_PRECISION,master,spaceComm,ierr)
           endif
#endif


!DEBUG
!#if defined MPI
!          call MPI_BARRIER(MPI_COMM_WORLD,ierr)
!          write(message,'(a)')' after iband loop synchronization done...'
!          call  wrtout(6,message,'COLL')
!#endif
!ENDDEBUG

!Compute the xc kernel, in the form needed for the singlet or triplet
!excitation energy. 
!In the case ixc=20, kxc vanishes, but no change is made here, for simplicity.

!DEBUG
!write(6,*)' tddft : xc kernel '
!do ifft=1,nkxc,41
! write(6,*)ifft,kxc(ifft,1),kxc(ifft,2),kxc(ifft,3)
!enddo
!stop
!ENDDEBUG

 allocate(kxc_singlet(ndiel1,ndiel2,ndiel3))
 allocate(kxc_triplet(ndiel1,ndiel2,ndiel3))
 index=1
 if(dtset%ixc/=22)then
  do i3=1,ndiel3
   do i2=1,ndiel2
    do i1=1,ndiel1
     kxc_singlet(i1,i2,i3)=two*kxc(index,1)
     kxc_triplet(i1,i2,i3)=two*(kxc(index,1)-kxc(index,2))
     index=index+1
    enddo
   enddo
  enddo
 else
! This is for the Burke-Petersilka-Gross hybrid, with ixc=22
  do i3=1,ndiel3
   do i2=1,ndiel2
    do i1=1,ndiel1
     kxc_singlet(i1,i2,i3)= kxc(index,2)
     kxc_triplet(i1,i2,i3)=-kxc(index,2)
     index=index+1
    enddo
   enddo
  enddo
 endif

 pole_approx=0

 allocate(excit_coords(nexcit_win**2,2))

!DEBUG
!write(6,*)'before first loop'
!ENDDEBUG



!0000000000000000000000000000000000000000000000000000000
!check if matrix file fname_tdexcit exists on disk
!if the file is present,calculation is a continuation 
 if (am_master) then
  fname_tdexcit=trim(dtfil%filnam_ds(5))//'_TDEXCIT'
  inquire(file=fname_tdexcit,exist=file_exist)
  ! for direct access to excitation file
  inquire(iolength=recl) eigsqr_singlet(1,1,1,1),eigsqr_triplet(1,1,1,1), &
&                        iexcit,jexcit
  open(tmp_unit2, file=fname_tdexcit,form='unformatted', recl=recl, &
&      access='DIRECT')

  allocate(done_excit(nexcit_win,nexcit_win))

  if(file_exist)then
   write(6,*)'TDDFT continues from a previous run'
   rec=0
   do iexcit=1,nexcit_win
    iexcit2 = indarr(iexcit)
    iunocc2 = (iexcit2 - 1 )/nband_occ + 1
    iocc2 =iexcit2 - (iunocc2 - 1) * nband_occ

    do jexcit=1,nexcit_win
     iexcit1 = indarr(jexcit)
     iunocc1 = (iexcit1 -1 )/nband_occ + 1
     iocc1 = iexcit1 - (iunocc1 -1) * nband_occ
     rec=rec+1 ! record of the entry in the excitation file
     read(tmp_unit2,rec=rec) eigsqr_singlet_dummy, &
&                            eigsqr_triplet_dummy, &
&                            idummy, jdummy
     done_excit(jexcit,iexcit)= ( idummy /= -1 .and. jdummy /= -1 ) ! if true, eigsqr_singlet and eigsqr_triplet are ok
                                                                    ! and a true is marked in the logical array done_excit
     if (done_excit(jexcit,iexcit)) then
      eigsqr_singlet(iocc1,iunocc1,iocc2,iunocc2)=eigsqr_singlet_dummy
      eigsqr_triplet(iocc1,iunocc1,iocc2,iunocc2)=eigsqr_triplet_dummy
     endif
    enddo
   enddo
  else
   write(*,*)'no excitation matrix on disk'
   write(6,*)'TDDFT starts from scratch'
   done_excit = .false. ! initialize the logical array to false
   rec=0
   do iexcit=1,nexcit_win
    do jexcit=1,nexcit_win
     rec=rec+1
     write(tmp_unit2,rec=rec) 0_dp,0_dp,-1,-1
    enddo
   enddo
  endif

! Need to list the elements to compute, taking the symmetry into account: valid only for Gamma point but this is already the case
  count_to_do=0
  do iexcit=1,nexcit_win
   do jexcit=1,iexcit
    if (.not. done_excit(jexcit,iexcit)) then
     count_to_do=count_to_do+1
     excit_coords(count_to_do,1)=iexcit
     excit_coords(count_to_do,2)=jexcit
    endif
   enddo
  enddo

  deallocate(done_excit)

#           if defined MPI
            allocate(counts(0:nproc_loc-1))
            allocate(displs(0:nproc_loc-1))
            allocate(recvcounts(0:nproc_loc-1))
            allocate(recvbuf(4,nproc_loc-1))

!           Compute limits for load balancing
            do iproc=0,nproc_loc-1
             displs(iproc)=(iproc*count_to_do)/nproc_loc
             counts(iproc)=min(((iproc+1)*count_to_do)/nproc_loc,count_to_do)-displs(iproc)
            enddo
#           endif

 endif ! am_master

#          if defined MPI
           call MPI_BCAST(count_to_do,1,MPI_INTEGER,master,spaceComm,ierr)
#          endif

 displ=(me_loc*count_to_do)/nproc_loc
 count=min(((me_loc+1)*count_to_do)/nproc_loc,count_to_do)-displ
 displmax=((nproc_loc-1)*count_to_do)/nproc_loc
 countmax=count_to_do-displmax

 write(message,'(A,I6)') 'Maximum number of matrix elements per processor = ',countmax
 call wrtout(6,message,'COLL')

#          if defined MPI
!          Need to dispatch the elements to compute to the different processes
           call MPI_Scatterv(excit_coords(1,1),counts,displs,MPI_INTEGER,excit_coords(1,1),count, &
&                            MPI_INTEGER,0,spaceComm,ierr)
           call MPI_Scatterv(excit_coords(1,2),counts,displs,MPI_INTEGER,excit_coords(1,2),count, &
&                  MPI_INTEGER,0,spaceComm,ierr)
#          endif

 nfftdiel=ndiel1*ndiel2*ndiel3
 allocate(wfprod(ndiel1,ndiel2,ndiel3),work(nfftdiel))
 allocate(sexc(3,nexcit_win))
 allocate(done_sexc(nexcit_win))
 allocate(done_sexc2(nexcit_win))
 allocate(rhog(nfftdiel,2),vhartr(nfftdiel))

  sexc(:,:)=zero 
  done_sexc(:)=.false.

!----------------------------------------------------------
!Main double loop

 old_iexcit=0
 do ijexcit=1,countmax
  ! we really loop only through count, but we need to go through countmax
  ! to make sure that all processes execute MPI_Gatherv below
  if (ijexcit <= count) then
   iexcit=excit_coords(ijexcit,1)
   jexcit=excit_coords(ijexcit,2)

   iexcit2 = indarr(iexcit)
   iunocc2 = (iexcit2 - 1 )/nband_occ + 1
   iocc2 =iexcit2 - (iunocc2 - 1) * nband_occ

   iexcit1 = indarr(jexcit)
   iunocc1 = (iexcit1 -1 )/nband_occ + 1
   iocc1 = iexcit1 - (iunocc1 -1) * nband_occ

   if (old_iexcit /= iexcit) then
!   We start a new column of the matrix
!DEBUG
!   write(message,'(a,i5,a,i3)')'treating  iexcit =  ',iexcit,&
!                              &'  with proc ',me_loc
!   call wrtout(6,message,'PERS') 
!ENDDEBUG

!   If mkmem ==0, the real-space filled and empty states are read from disk  
    if (mkmem==0) then
!    Read occupied band using external subroutine
       call read_wfrspa(index_band(iocc2),dtfil,eigbnd,iband,1,ndiel4,ndiel5,ndiel6,wfrspa(:,:,:,1))
!    Read unoccupied band
     call read_wfrspa(index_band(iunocc2+nband_occ),dtfil,eigbnd,iband,2,&
          & ndiel4,ndiel5,ndiel6,wfrspa(:,:,:,2))
    endif

!DEBUG
!   write(message,'(a,i3)')'Multiplicating phi1 phi2, on proc ',me_loc
!   call wrtout(6,message,'PERS')
!ENDDEBUG 

    ifft=1
    do i3=1,ndiel3
     do i2=1,ndiel2
      do i1=1,ndiel1
       if (mkmem/=0) then
        wfprod(i1,i2,i3)=wfrspa(i1,i2,i3,index_band(iocc2)) &
&                       *wfrspa(i1,i2,i3,index_band(iunocc2+nband_occ))
       else
        wfprod(i1,i2,i3)=wfrspa(i1,i2,i3,1)*wfrspa(i1,i2,i3,2)
       endif
       work(ifft)=wfprod(i1,i2,i3)
       ifft=ifft+1
      enddo
     enddo
    enddo
    if (jexcit == 1) then
     do i3=1,ndiel3
      do i2=1,ndiel2
       do i1=1,ndiel1
        do idir=1,3
         poscart(idir)=rprimd(idir,1)*pos(i1,1)+&
&                      rprimd(idir,2)*pos(i2,2)+&
&                      rprimd(idir,3)*pos(i3,3)
         sexc(idir,iexcit)=sexc(idir,iexcit)+poscart(idir)*wfprod(i1,i2,i3)
        enddo
       enddo
      enddo
      done_sexc(iexcit)=.true.
     enddo
    endif

!   For the singlet correction, must compute the hartre potential created
!   by the product of wavefunctions
    cplex=1 ; qphon(:)=0.0d0

!DEBUG
!   write(message,'(a,i3)')'Before Fourdp, on proc ',me_loc
!   call wrtout(6,message,'PERS')
!ENDDEBUG 

    call fourdp(cplex,rhog,work,-1,mpi_enreg,nfftdiel,ngfftdiel,0)

!DEBUG
!   write(message,'(a,i3)')'Before Hartree, on proc ',me_loc
!   call wrtout(6,message,'PERS')
!   write(6,*)'CPU ',me_loc,ch10,&
!&            '   cplex : ',cplex,ch10,&
!&            '   gmet(3,3)  : ',gmet(3,3),ch10,&
!&            '   gsqcut : ',gsqcut,ch10,&
!&            '   qphon(1) : ',qphon(1),ch10,&
!&            '   rhog(1,1) :,',rhog(1,1),ch10,&
!&            '   vhartr(1) :,',vhartr(1)  
!ENDDEBUG 

    call hartre(cplex,gmet,gsqcut,0,mpi_enreg,nfftdiel,ngfftdiel,qphon,rhog,vhartr)

!DEBUG
!   write(message,'(a,i3)')'After Hartree, on proc ',me_loc
!   call wrtout(6,message,'PERS')
!ENDDEBUG
   endif
   old_iexcit=iexcit

!DEBUG
!  write(6,*)'  treating  iexcit =  ',jexcit
!  write(6,*)'   indarr(iexcit) =',iexcit1,iocc1,iunocc1
!  write(6,*)'   index ',index_band(iocc1),index_band(iunocc1+nband_occ)
!ENDDEBUG

!  if mkmem==0 the filled and empty states for excitation iexcit1 are read from disk  
   if (mkmem==0) then
!    Read occupied band using external subroutine
       call read_wfrspa(index_band(iocc1),dtfil,eigbnd,iband,1,ndiel4,ndiel5,ndiel6,wfrspa(:,:,:,1))
!    Read unoccupied band
     call read_wfrspa(index_band(iunocc1+nband_occ),dtfil,eigbnd,iband,2,&
          & ndiel4,ndiel5,ndiel6,wfrspa(:,:,:,2))
   endif !mkmem==0

   if(pole_approx==0 .or. (iunocc1==iunocc2 .and. iocc1==iocc2))then
    sum_triplet=0.0d0
    sum_singlet=0.0d0
    f_triplet=0.0d0
    f_singlet=2.0d0
!   For Fermi-Amaldi kxc, the xc contribution is -1/2 the Hartree contribution
!   to the triplet state. The following factors combines both contributions.
    if(dtset%ixc==20 .or. dtset%ixc==22)then
     f_triplet=-1.0d0
     f_singlet= 1.0d0
    endif
    ifft=1
    do i3=1,ndiel3
     do i2=1,ndiel2
      do i1=1,ndiel1
       if (mkmem/=0) then
        product=wfrspa(i1,i2,i3,index_band(iocc1)) &
&              *wfrspa(i1,i2,i3,index_band(iunocc1+nband_occ))
       else
        product=wfrspa(i1,i2,i3,1)*wfrspa(i1,i2,i3,2)
       endif      
       sum_triplet=sum_triplet+&
&        product*(f_triplet*vhartr(ifft)+kxc_triplet(i1,i2,i3) &
&               * wfprod(i1,i2,i3))
!      The factor of two before vhartr is derived from Eqs(1) and (2) of
!      Vasiliev, specialized to the spin-unpolarized case
       sum_singlet=sum_singlet+&
&        product*(f_singlet*vhartr(ifft)+kxc_singlet(i1,i2,i3) &
&               *wfprod(i1,i2,i3))
       ifft=ifft+1
      enddo ! i1
     enddo ! i2
    enddo ! i3

!   The factor two is coherent with the formulas of Vasiliev et al
    factor=2.0d0*sqrtks(iexcit1)*sqrtks(iexcit2)*normint
    eigsqr_triplet(iocc1,iunocc1,iocc2,iunocc2)=    &
&    eigsqr_triplet(iocc1,iunocc1,iocc2,iunocc2)+factor*sum_triplet
    eigsqr_singlet(iocc1,iunocc1,iocc2,iunocc2)=    &
&    eigsqr_singlet(iocc1,iunocc1,iocc2,iunocc2)+factor*sum_singlet

!  End condition of being diagonal element if pole approximation
   endif

!  Continue writing excitation matrix 
   if (am_master) then
    ! the master writes its results to disk
    write(tmp_unit2, rec=(iexcit-1)*nexcit_win+jexcit ) &
&    eigsqr_singlet(iocc1,iunocc1,iocc2,iunocc2),&
&    eigsqr_triplet(iocc1,iunocc1,iocc2,iunocc2), iexcit, jexcit
!DEBUG
!   write(6,*)'singlet: ',eigsqr_singlet(iocc1,iunocc1,iocc2,iunocc2),'iexcit: ',iexcit,'jexcit :',jexcit
!   write(6,*)'triplet: ',eigsqr_triplet(iocc1,iunocc1,iocc2,iunocc2),'iexcit: ',iexcit,'jexcit :',jexcit
!ENDDEBUG
    sendcount=0
   else
    sendcount=4
    sendbuf=(/ eigsqr_singlet(iocc1,iunocc1,iocc2,iunocc2), eigsqr_triplet(iocc1,iunocc1,iocc2,iunocc2),&
            &real(iexcit,dp), real(jexcit,dp) /)
   endif ! am_master
  else
   ! ijexcit > count

   ! done with local work, so send message of zero length
   sendcount=0

  endif ! ijexcit <= count

#           if defined MPI
            if (am_master) then

!            Compute displacements and counts for the gathering of the results
             displs(0)=0
             recvcounts(0)=0
             do iproc=1,nproc_loc-1
              recvcounts(iproc)=min(((iproc+1)*count_to_do)/nproc_loc,count_to_do)-(iproc*count_to_do)/nproc_loc
              if (recvcounts(iproc) < countmax .and. ijexcit==countmax) then
               recvcounts(iproc)=0
              else
               recvcounts(iproc)=4
              endif
              displs(iproc)=displs(iproc-1)+recvcounts(iproc-1)
             enddo
            endif

            call MPI_Gatherv(sendbuf,sendcount,MPI_DOUBLE_PRECISION,recvbuf,recvcounts,displs, &
&                            MPI_DOUBLE_PRECISION,0,spaceComm,ierr)

            if (am_master) then

             ! Extract eigsqr_singlet, eigsqr_triplet, iexcit, jexcit from receive buffer and
             ! write to file
             do ijexcit2=1,sum(recvcounts)/4
              iexcit=int(recvbuf(3,ijexcit2))
              jexcit=int(recvbuf(4,ijexcit2))

              iexcit2 = indarr(iexcit)
              iunocc2 = (iexcit2 - 1 )/nband_occ + 1
              iocc2 =iexcit2 - (iunocc2 - 1) * nband_occ

              iexcit1 = indarr(jexcit)
              iunocc1 = (iexcit1 -1 )/nband_occ + 1
              iocc1 = iexcit1 - (iunocc1 -1) * nband_occ

              eigsqr_singlet(iocc1,iunocc1,iocc2,iunocc2)=recvbuf(1,ijexcit2)
              eigsqr_triplet(iocc1,iunocc1,iocc2,iunocc2)=recvbuf(2,ijexcit2)
!DEBUG
!             write(6,*)'singlet: ',eigsqr_singlet(iocc1,iunocc1,iocc2,iunocc2),'iexcit: ',iexcit,'jexcit :',jexcit
!             write(6,*)'triplet: ',eigsqr_triplet(iocc1,iunocc1,iocc2,iunocc2),'iexcit: ',iexcit,'jexcit :',jexcit
!ENDDEBUG
              write(tmp_unit2, rec=(iexcit-1)*nexcit_win+jexcit ) &
         &        eigsqr_singlet(iocc1,iunocc1,iocc2,iunocc2),&
         &        eigsqr_triplet(iocc1,iunocc1,iocc2,iunocc2), iexcit, jexcit
             enddo

            endif
#           endif

!End indices loops
 enddo ! ijexcit

!End of the main double loop 
!--------------------------------------------------------------------

#          if defined MPI
!          sexc needs to be summed here since it used only by master
           call MPI_BARRIER(spaceComm,ierr)
           call xsum_master(sexc,master,spaceComm,ierr)
           done_sexc2=done_sexc
           call MPI_Reduce(done_sexc2,done_sexc,nexcit_win,MPI_LOGICAL,MPI_LOR,&
&                          master,spaceComm,ierr)
#          endif

 if (am_master) then
! We compute sexc again if it was not done. Will only be executed if
! there was a restart from values read from logical unit tmp_unit2.
  do iexcit=1,nexcit_win
   if (.not.done_sexc(iexcit)) then
    iexcit2 = indarr(iexcit)
    iunocc2 = (iexcit2 - 1 )/nband_occ + 1
    iocc2 =iexcit2 - (iunocc2 - 1) * nband_occ
    if (mkmem==0) then
!    Read occupied band using external subroutine
       call read_wfrspa(index_band(iocc2),dtfil,eigbnd,iband,1,ndiel4,ndiel5,ndiel6,wfrspa(:,:,:,1))
!    Read unoccupied band
     call read_wfrspa(index_band(iunocc2+nband_occ),dtfil,eigbnd,iband,2,&
          & ndiel4,ndiel5,ndiel6,wfrspa(:,:,:,2))
    endif
    do i3=1,ndiel3
     do i2=1,ndiel2
      do i1=1,ndiel1
       if (mkmem/=0) then
        wfprod(i1,i2,i3)=wfrspa(i1,i2,i3,index_band(iocc2)) &
&                       *wfrspa(i1,i2,i3,index_band(iunocc2+nband_occ))
       else
        wfprod(i1,i2,i3)=wfrspa(i1,i2,i3,1)*wfrspa(i1,i2,i3,2)
       endif
       do idir=1,3
        poscart(idir)=rprimd(idir,1)*pos(i1,1)+&
&                     rprimd(idir,2)*pos(i2,2)+&
&                     rprimd(idir,3)*pos(i3,3)
        sexc(idir,iexcit)=sexc(idir,iexcit)+poscart(idir)*wfprod(i1,i2,i3)
       enddo
      enddo
     enddo
    enddo
   endif
  enddo
 endif

 deallocate(work,rhog,pos)
 deallocate(vhartr,kxc_singlet,kxc_triplet,wfprod)
 deallocate(index_band,excit_coords)  
 deallocate(wfrspa)

!Write the first excitation energies
 write(message, '(a,a,es18.8,a,a,a,a,a,a,a,a,a)' )ch10,&
& '  Ground state total energy (Ha) :',etotal,ch10,ch10,&
& '  Kohn-Sham energy differences,',ch10,&
& '  corresponding total energies and oscillator strengths (X,Y,Z and average)-',&
& ch10,&
& '  (oscillator strengths smaller than 1.e-6 are set to zero)',ch10,&
& '  Transition  (Ha)  and   (eV)   Tot. Ene. (Ha)  Aver     XX       YY       ZZ'
 call wrtout(ab_out,message,'COLL')
 call wrtout(6,message,'COLL')

 if (am_master) then

#           if defined MPI
            deallocate(counts,displs)
            deallocate(recvbuf,recvcounts)
#           endif

  allocate(osc_str(7,nexcit))

  do iexcit=1,nexcit_win

   iexcit2=indarr(iexcit)
   iunocc=(iexcit2-1)/nband_occ+1
   iocc=iexcit2-(iunocc-1)*nband_occ
   osc_str(1,iexcit)=zero
   do idir=1,3
!   One of the factor of two comes from the spin degeneracy,
!   the other comes from Eq.(40) of Casida
    osc_str(idir+1,iexcit)=&
&    (sexc(idir,iexcit)*sqrtks(iexcit2)*normint*ucvol)**2*two*two
    osc_str(1,iexcit)=osc_str(1,iexcit)&
&    +osc_str(idir+1,iexcit)*third    
   enddo
   do ii=1,4
    if(abs(osc_str(ii,iexcit))<tol6)osc_str(ii,iexcit)=zero
   enddo
!  Changed, the whole spectrum is written
!  The array eexcit has been reordered previously, the others also                 
   write(message, '(i4,a,i3,2es12.5,es13.5,es11.4,3es9.2)' ) &
&     iocc,'->',iunocc+nband_occ,eexcit(iexcit),&
&     eexcit(iexcit)*Ha_eV,eexcit(iexcit)+etotal,&
!     XG 020209 : Jean-Yves, I assume that the printout of sexc is for debugging ?!
!&     osc_str(1:4,iexcit),sexc(1:3,iexcit)
 &     osc_str(1:4,iexcit)
   call wrtout(ab_out,message,'COLL')
   call wrtout(6,message,'COLL')

  enddo

! Check of the Sum rule for Casida eq.47,
! only exact if complete basis of excitations, as well as local potentials only.
  sumx=zero
  do iexcit=1,nexcit_win
   sumx=sumx+osc_str(1,iexcit)
  enddo
  write(message, '(a,es16.6)' ) &
&  '  Sum of osc. strength : ',sumx
  call wrtout(ab_out,message,'COLL')
  call wrtout(6,message,'COLL')

!-Diagonalize the excitation matrices----------------------------

  allocate(eexcit2(nexcit_win),vec(2,nexcit_win,nexcit_win))

  do sing_trip=1,2

   if(pole_approx==0)then

    allocate(matr(nexcit_win*(nexcit_win+1)))
    allocate(zhpev1(2,2*nexcit_win-1),zhpev2(3*nexcit_win-2))
    matr(:)=zero
    ier=0
!DEBUG
!   write(6,*)' after allocation matrices     '
!ENDDEBUG


!   Store the matrix in proper mode before calling zhpev
    index=1
    do iexcit=1,nexcit_win
     iexcit2 = indarr(iexcit)
     iunocc2=(iexcit2-1)/nband_occ+1
     iocc2=iexcit2-(iunocc2-1)*nband_occ

     do jexcit=1,iexcit
      iexcit1=indarr(jexcit)
      iunocc1=(iexcit1-1)/nband_occ+1
      iocc1=iexcit1-(iunocc1-1)*nband_occ
!     if (mkmem/=0) then
       if(sing_trip==1)matr(index)=eigsqr_singlet(iocc1,iunocc1,iocc2,iunocc2)
       if(sing_trip==2)matr(index)=eigsqr_triplet(iocc1,iunocc1,iocc2,iunocc2)
!     endif
      matr(index+1)=0.0d0
      index=index+2
     enddo

    enddo

!DEBUG
!   write(6,*)' after filling    matrices     '
!ENDDEBUG

#if defined T3E
    call CHPEV ('V','U',nexcit_win,matr,eexcit2,vec,nexcit_win,zhpev1,&
&    zhpev2,ier)
#else
    call ZHPEV ('V','U',nexcit_win,matr,eexcit2,vec,nexcit_win,zhpev1,&
&    zhpev2,ier)
#endif

    deallocate(matr,zhpev1,zhpev2)
!DEBUG
!   write(6,*)' after deallocating matrices     '
!ENDDEBUG


   else 

    vec(:,:,:)=0.0d0
    do iunocc=1,nband_k 
     do iocc=1,nband_k 
      index=iocc+nband_k*(iunocc-1)
      if(sing_trip==1)eexcit2(index)=eigsqr_singlet(iocc,iunocc,iocc,iunocc)
      if(sing_trip==2)eexcit2(index)=eigsqr_triplet(iocc,iunocc,iocc,iunocc)
      vec(1,index,index)=1.0d0
     enddo
    enddo

   endif

!  Compute the excitation energies from the square root of eexcit2
!  eexcit(:)=sqrt(eexcit2(:)

   deallocate(eexcit)
   allocate(eexcit(nexcit_win))

   eexcit(:)=sqrt(dabs(eexcit2(:))+1.0D-20)
!  Write the first excitation energies
   if(sing_trip==1)write(message, '(a,a,a,a,a,a)' )ch10,&
&   '  TDDFT singlet excitation energies (at most 20 of them are printed),',&
&   ch10,'  and corresponding total energies.                ',ch10,&
&   '  Excit#   (Ha)    and    (eV)    total energy (Ha)    major contributions '
   if(sing_trip==2)write(message, '(a,a,a,a,a,a)' )ch10,&
&   '  TDDFT triplet excitation energies (at most 20 of them are printed),',&
&   ch10,'  and corresponding total energies.                ',ch10,&
&   '  Excit#   (Ha)    and    (eV)    total energy (Ha)    major contributions '
   call wrtout(ab_out,message,'COLL')
   call wrtout(6,message,'COLL')
   do iexcit=1,min(nexcit_win,nexcitout)
!   Select largest and next contributions
    flargest=0.0d0 ; fnext=0.0d0
    nlargest=0.0d0 ; nnext=0.0d0
    do jexcit=1,nexcit_win
     ff=vec(1,jexcit,iexcit)**2+vec(2,jexcit,iexcit)**2
     if(ff>flargest+tol12)then
      nnext=nlargest ; fnext=flargest
      nlargest=indarr(jexcit) ; flargest=ff
     else if(ff>fnext+tol12)then
      nnext=indarr(jexcit) ; fnext=ff
     endif
    enddo
    iunocc_l=(nlargest-1)/nband_occ+1
    iocc_l=nlargest-(iunocc_l-1)*nband_occ
    iunocc_n=(nnext-1)/nband_occ+1
    iocc_n=nnext-(iunocc_n-1)*nband_occ
    write(message, '(i4,es15.5,es14.5,es16.6,f8.2,a,i3,a,i3,a,f6.2,a,i3,a,i3,a)' ) &
&     iexcit,eexcit(iexcit),&
&     eexcit(iexcit)*Ha_eV,eexcit(iexcit)+etotal,&
&     flargest,'(',iocc_l,'->',iunocc_l+nband_occ,')',&
&     fnext,'(',iocc_n,'->',iunocc_n+nband_occ,')'
    call wrtout(ab_out,message,'COLL')
    call wrtout(6,message,'COLL')
   enddo

!  For each iexcit excitation, compute the oscillator strength (Casida, eq 47)
   write(message, '(a,a,a,a)' )ch10,&
&   '  Oscillator strengths :  (elements smaller than 1.e-6 are set to zero)',ch10,&
&   '  Excit#   (Ha)   Average    XX        YY        ZZ         XY        XZ        YZ'
   call wrtout(ab_out,message,'COLL')
   call wrtout(6,message,'COLL') 

   do iexcit=1,nexcit_win

!   One of the factor of two comes from the spin degeneracy,
!   the other comes from Eq.(40) of Casida
    factor=(normint*ucvol)**2*two*two

    osc_str(:,iexcit)=zero

    factor=(normint*ucvol)**2*two*two

    do jexcit=1,nexcit_win
     jexcit_cbase=indarr(jexcit)
     do idir=1,3
      osc_str(idir+1,iexcit)=osc_str(idir+1,iexcit)+ &
&     sexc(idir,jexcit)*sqrtks(jexcit_cbase)*sqrt(factor)* &
&     vec(1,jexcit,iexcit)
     enddo ! idir
    enddo ! jexcit

!   The "standard" definition of the oscillator strength is the square
!   of the matrix elements. 
!   So, instead of the coding 
!   do idir=1,3
!    osc_str(1,iexcit)=osc_str(1,iexcit)+osc_str(idir+1,iexcit)**2*third      
!   enddo
!   I think that the following is more "standard"
!   Now, osc_str(2:4,iexcit) are the X, Y and Z matrix elements, not
!   yet the oscillator strengths
    osc_str(5,iexcit)=osc_str(2,iexcit)*osc_str(3,iexcit)   ! off diag XY
    osc_str(6,iexcit)=osc_str(2,iexcit)*osc_str(4,iexcit)   ! off diag XZ
    osc_str(7,iexcit)=osc_str(3,iexcit)*osc_str(4,iexcit)   ! off diag ZZ
    do idir=1,3
!    Here the X,Y, and Z matrix elements are combined to give diagonal osc. strengths
     osc_str(idir+1,iexcit)=osc_str(idir+1,iexcit)**2        
     osc_str(1,iexcit)=osc_str(1,iexcit)+osc_str(idir+1,iexcit)*third ! compute the trace
    enddo
!   At this stage, osc_str(1,iexcit) is exactly the same as from your coding
!***End of section to be checked

    do ii=1,7
     if(abs(osc_str(ii,iexcit))<tol6)osc_str(ii,iexcit)=zero
    enddo
!   XG 020209 : Jean-Yves, the off-diagonal oscillator strengths
!   can become negative. It is important for automatic
!   checking that the numbers are separated by a blank, even
!   if they are negative. So replace the following format, to have at least one blank.
!   write(message, '(i4,es12.5,es10.3,3es10.3,3es10.3)' ) &
    write(message, '(i4,es12.5,es10.3,3es10.3,3es10.2)' ) &
&    iexcit,eexcit(iexcit),osc_str(1:7,iexcit)
    call wrtout(ab_out,message,'COLL')
    call wrtout(6,message,'COLL')
   enddo

!  Check of the Sum rule for Casida eq.47, 
!  only exact if complete basis of excitations, as well as local potentials only.
   sumx=zero
   do iexcit=1,nexcit_win
    sumx=sumx+osc_str(1,iexcit)
   enddo
   write(message, '(a,es16.6)' ) &
&   '  Sum of osc. strength : ',sumx
   call wrtout(ab_out,message,'COLL')
   call wrtout(6,message,'COLL')

!  If singlet, compute Cauchy coefficients
   if(sing_trip==1)then 
    cauchy(:)=zero   
    do iexcit=1,nexcit_win
     fr_invsquare=one/(eexcit(iexcit)**2)
     fr_power=one
     do ii=1,7
      fr_power=fr_power*fr_invsquare
      cauchy(ii)=cauchy(ii)+osc_str(1,iexcit)*fr_power
     enddo
    enddo
    write(message, '(a,es11.3,a,es11.3,a,es11.3,a,a,es11.3,a,es11.3,a,es11.3,a,es11.3)' ) &
&    '  Cauchy coeffs (au) : ( -2)->',cauchy(1),&
&      ', ( -4)->',cauchy(2),', ( -6)->',cauchy(3),ch10,&
&    '    (-8)->',cauchy(4),', (-10)->',cauchy(5),', (-12)->',cauchy(6),', (-14)->',cauchy(7)
    call wrtout(ab_out,message,'COLL')
    call wrtout(6,message,'COLL')
   endif

! End the loop on singlet or triplet
  enddo

  deallocate(eexcit2,vec)
  deallocate(osc_str)

  close(tmp_unit2,status='delete')

  call timab(95,2,tsec)

 endif  ! end of am_master

 deallocate(eigsqr_singlet,eigsqr_triplet,eexcit,sqrtks,sexc,done_sexc)
 deallocate(indarr)
 deallocate(done_sexc2)

 return

end subroutine tddft
!!***
