!{\src2tex{textfont=tt}}
!!****f* ABINIT/vtorhotf
!! NAME
!! vtorhotf
!!
!! FUNCTION
!! This routine computes the new density from a fixed potential (vtrial)
!! using the Thomas-Fermi functional
!!
!! COPYRIGHT
!! Copyright (C) 1998-2006 ABINIT group (DCA, XG, GMR, MF, AR, MM)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!  densymop_gs <type(dens_sym_operator_type)>=the density symmetrization
!!   operator (ground-state symmetries)
!!  dtfil <type(datafiles_type)>=variables related to files
!!  dtset <type(dataset_type)>=all input variables for this dataset
!!  irrzon(nfft**(1-1/nsym),2,nspden/nsppol)=irreducible zone data
!!  mpi_enreg=informations about MPI parallelization
!!  natom=number of atoms in cell.
!!  nfft=number of fft grid points
!!  nspden=number of spin-density components
!!  nsppol=1 for unpolarized, 2 for spin-polarized
!!  nsym=number of symmetry elements in space group
!!  phnons(2,nfft**(1-1/nsym),nspden/nsppol)=nonsymmorphic translation phases
!!  ucvol=unit cell volume in bohr**3.
!!  vtrial(nfft,nspden)=INPUT Vtrial(r).
!!
!! OUTPUT
!!  ek=kinetic energy part of total energy.
!!  enl=nonlocal pseudopotential part of total energy.
!!  entropy=entropy due to the occupation number smearing (if metal)
!!  fermie=fermi energy (Hartree)
!!  grnl(3*natom)=stores grads of nonlocal energy wrt length scales
!!   (3x3 tensor) and grads wrt atomic coordinates (3*natom)
!!
!! SIDE EFFECTS
!!  rhog(2,nfft)=array for Fourier transform of electron density
!!  rhor(nfft,nspden)=array for electron density in electrons/bohr**3.
!!
!! PARENTS
!!      scfcv
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine vtorhotf(densymop_gs,dtfil,dtset,ek,enl,entropy,fermie,grnl,&
&  irrzon,mpi_enreg,natom,nfft,nspden,nsppol,nsym,phnons,rhog,rhor,ucvol,vtrial)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_00basis
 use interfaces_01managempi
 use interfaces_01util
 use interfaces_11drive, except_this_one => vtorhotf
#endif
!End of the abilint section

 implicit none

!Arguments -------------------------------
!do not break the two previous lines 
!scalars
 integer,intent(in) :: natom,nfft,nspden,nsppol,nsym
 real(dp),intent(in) :: ucvol
 real(dp),intent(out) :: ek,enl,entropy,fermie
 type(MPI_type),intent(inout) :: mpi_enreg
 type(datafiles_type),intent(in) :: dtfil
 type(dataset_type),intent(in) :: dtset
 type(dens_sym_operator_type),intent(in) :: densymop_gs
!arrays
 integer,intent(in) :: irrzon((dtset%ngfft(1)*dtset%ngfft(1)*dtset%ngfft(1))**(1-1/nsym),2,nspden/nsppol)
 real(dp),intent(in) :: phnons(2,(dtset%ngfft(1)*dtset%ngfft(1)*dtset%ngfft(1))**(1-1/nsym),nspden/nsppol)
 real(dp),intent(in) :: vtrial(nfft,nspden)
 real(dp),intent(inout) :: rhog(2,nfft),rhor(nfft,nspden)
 real(dp),intent(out) :: grnl(3*natom)

!Local variables-------------------------------
!Thomas-Fermi local variables
#ifndef HAVE_FORTRAN_INTERFACES
 external zfermi12,zfermi32,ifermi12,zfermim12
 external fm12a1,fp12a1,xp12a1
#endif
 !external fp32a1
! real(dp) :: fp32a1,fm12a1,fp12a1,ifermi12,xp12a1,zfermi12,zfermi32,zfermim12
!scalars
 integer,parameter :: jdichomax=20,level=7
 integer :: bantot_glob,dimsum,i1,i2,i3,ierr,iexit,ifft,ii,ir,iscf,jdicho
 integer :: me_fft,n1,n2,n3,nbuf,nfftot,nproc_fft,option,prtvol,spaceComm,tag
 integer :: tmpdim
 real(dp),save :: cktf,fermie_tol,nelect_mid
 real(dp) :: dnelect_mid_dx,dxrtnewt,eektemp,eektf,feektemp,feektf,nelect_middx
 real(dp) :: rtnewt,rtnewtdx,sum_rhor_mid,sum_rhor_middx
 logical,save :: lfirst_time_tf=.true.
 logical :: lnewtonraphson
 character(len=500) :: message
!arrays
 real(dp) :: tsec(2)
 real(dp),allocatable :: betamumoinsV(:),rhor_mid(:),rhor_middx(:)

! *********************************************************************

!DEBUG
!  write(6,*)' vtorhotf : enter'
!ENDDEBUG
!Keep track of total time spent in vtorho
 call timab(21,1,tsec)


 call status(0,dtfil%filstat,iexit,level,'enter         ')

!Structured debugging if prtvol==-level
 prtvol=dtset%prtvol
 if(prtvol==-level)then
  write(message,'(80a,a,a)') ('=',ii=1,80),ch10,' vtorho : enter '
  call wrtout(06,message,'COLL')
 end if

 n1=dtset%ngfft(1) ; n2=dtset%ngfft(2) ; n3=dtset%ngfft(3)
 me_fft=dtset%ngfft(11) ; nproc_fft=dtset%ngfft(10)
 iscf=dtset%iscf
!Debugging : print vtrial and rhor
 if(prtvol==-level)then
  write(message,'(a)') '   ir              vtrial(ir)     rhor(ir) '
  call wrtout(06,message,'COLL')
  do ir=1,nfft
!  if(ir<=11 .or. mod(ir,301)==0 )then
    i3=(ir-1)/n1/(n2/nproc_fft)
    i2=(ir-1-i3*n1*n2/nproc_fft)/n1
    i1=ir-1-i3*n1*n2/nproc_fft-(i2-me_fft)*n1
    write(message,'(i5,3i3,a,2d13.6)')ir,i1,i2,i3,' ',vtrial(ir,1),rhor(ir,1)
    call wrtout(06,message,'COLL')
    if(nspden>=2)then
     write(message,'(a,2d13.6)')'               ',vtrial(ir,2),rhor(ir,2)
     call wrtout(06,message,'COLL')
    end if
!  end if
  end do
 end if

 ek=0.0d0
 enl=0.0d0
 grnl(:)=0.0d0

!Initialize rhor if needed
 if(iscf>0) rhor(:,:)=0.0d0

!call Thomas-Fermi for the density
 call tf
!Compute energy terms
 call tfek
 mpi_enreg%paral_level=2
 call timab(21,2,tsec)
 return
!End thomas fermi

 contains

subroutine tf
!This is the Thomas-Fermi approach

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_00basis
 use interfaces_01managempi
 use interfaces_01util
 use interfaces_05common
 use interfaces_11drive
#else
 use defs_xfuncmpi
#endif
!End of the abilint section

 implicit none

 real(dp):: rs,kin
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 double precision :: zfermim12
 double precision :: zfermi12
#endif
!End of the abilint section

 allocate(rhor_mid(nfft),rhor_middx(nfft))
 fermie_tol=1.d-10
 cktf=one/two/pi**2*(two*dtset%tsmear)**1.5_dp

! Should be made an input variable, if TF really needed for production
! rtnewt=dtset%userra
  rtnewt=zero

!Newton Raphson
 if (lfirst_time_tf) then
  lfirst_time_tf=.false.
 end if
 jdicho=0
 lnewtonraphson=.false.
 do while (.not.lnewtonraphson)
  jdicho=jdicho+1
!  do ifft=1,nfft
!   rhor_mid(ifft)=cktf*zfermi12((rtnewt-vtrial(ifft,1))/dtset%tsmear)
!   rhor_middx(ifft)=cktf*zfermim12((rtnewt-vtrial(ifft,1))/dtset%tsmear)
!  end do
  call fm12a1t(cktf,rtnewt,dtset%tsmear,vtrial(:,1),rhor_middx,rhor_mid,&
&              nfft)
  sum_rhor_mid=sum(rhor_mid(:))
  sum_rhor_middx=sum(rhor_middx(:))
  mpi_enreg%paral_level=3
  call xcomm_init(mpi_enreg,spaceComm)
  call xsum_mpi(sum_rhor_mid,spaceComm ,ierr)
  call xsum_mpi(sum_rhor_middx,spaceComm ,ierr)
  nelect_mid=sum_rhor_mid*ucvol/(nfft*nproc_fft)-dtset%nelect
  dnelect_mid_dx=sum_rhor_middx*ucvol/(nfft*nproc_fft)/dtset%tsmear/two
  dxrtnewt=nelect_mid/dnelect_mid_dx
  rtnewt=rtnewt-dxrtnewt
  if (abs(nelect_mid) < fermie_tol/2._dp) then
    lnewtonraphson=.true.
  end if
  if (jdicho > jdichomax) then
    write(6,*) 'NEWTON RAPHSON NOT CONVERGED'
    stop
  end if
 end do
 fermie=rtnewt
 rhor(:,1)=rhor_mid(:)
 deallocate(rhor_mid,rhor_middx)

!DEBUG
!write(6,*)'fmid,nmid,jdicho',fermie,nelect_mid,jdicho
!ENDDEBUG

!Compute rhog
 call timab(70,1,tsec)

 call status(0,dtfil%filstat,iexit,level,'compute rhog  ')
 nfftot=dtset%ngfft(1)*dtset%ngfft(2)*dtset%ngfft(3)
 call symrhg(1,densymop_gs,irrzon,mpi_enreg,nfft,nfftot,dtset%ngfft,nspden,nsppol,nsym,phnons,&
&   rhog,rhor,dtset%symafm)

!  We now have both rho(r) and rho(G), symmetrized, and if nsppol=2
!  we also have the spin-up density, symmetrized, in rhor(:,2).

 call timab(70,2,tsec)
end subroutine tf

subroutine tfek
!This is the calculation of the kinetic energy for Thomas Fermi
!Energy and free energy must be distinguished

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_00basis
 use interfaces_01managempi
 use interfaces_11drive
#else
 use defs_xfuncmpi
#endif
!End of the abilint section

 implicit none

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 double precision :: zfermi32
 double precision :: fp32a1
 double precision :: ifermi12
 double precision :: zfermi12
#endif
!End of the abilint section

 allocate(betamumoinsV(nfft))
 cktf=one/two/pi**2*(two*dtset%tsmear)**1.5_dp
 eektf=0.0d0
 feektf=0.0d0
 do ifft=1,nfft

!  betamumoinsv(ifft)=ifermi12(rhor(ifft,1)/cktf)
  betamumoinsv(ifft)=(rtnewt-vtrial(ifft,1))/dtset%tsmear
!  eektemp=zfermi32(betamumoinsV(ifft))/zfermi12(betamumoinsV(ifft))
  eektemp=fp32a1(betamumoinsV(ifft))/rhor(ifft,1)*cktf
  feektemp=betamumoinsV(ifft)-two/three*eektemp
  feektf=feektf+feektemp*rhor(ifft,1)
  eektf=eektf+eektemp*rhor(ifft,1)
 end do
 !Init mpi_comm
   mpi_enreg%paral_level=3
   call xcomm_init(mpi_enreg,spaceComm)
   call timab(48,1,tsec)
   call xsum_mpi(eektf,spaceComm ,ierr)
   call xsum_mpi(feektf,spaceComm ,ierr)
   call timab(48,2,tsec)
 eektf=eektf*dtset%tsmear
 eektf=eektf*ucvol/dble(nfft*nproc_fft)
 feektf=feektf*dtset%tsmear
 feektf=feektf*ucvol/dble(nfft*nproc_fft)
!DEBUG
!write(6,*)'eektf',eektf
!stop ('vtorhotf')
!ENDDEBUG
 ek=eektf
 entropy=(eektf-feektf)/dtset%tsmear
 deallocate(betamumoinsV)
end subroutine tfek

end subroutine vtorhotf

!..
!..file contains fermi-dirac integral routines:
!..
!..function zfermim12 does a rational function fit for the order -1/2 integral
!..function zfermi12 does a rational function fit for the order 1/2 integral
!..function zfermi1 does a rational function fit for the order 1 integral
!..function zfermi32 does a rational function fit for the order 3/2 integral
!..function zfermi2 does a rational function fit for the order 2 integral
!..function zfermi52 does a rational function fit for the order 5/2 integral
!..function zfermi3 does a rational function fit for the order 3 integral
!..
!..function ifermim12 is a rational function fit for the inverse of order -1/2
!..function ifermi12 is a rational function fit for the inverse of order 1/2
!..function ifermi32 is a rational function fit for the inverse of order 3/2
!..function ifermi52 is a rational function fit for the inverse of order 5/2





      double precision function zfermim12(xx)
!..
!..this routine applies a rational function expansion to get the fermi-dirac
!..integral of order -1/2 evaluated at x. maximum error is 1.23d-12.
!..reference: antia apjs 84,101 1993
!..
!..declare

 implicit none

!Arguments -------------------------------
      double precision, intent(in) :: xx

!Local variables-------------------------------
      integer          ii,m1,k1,m2,k2
      double precision an,a1(12),b1(12),a2(12),b2(12),rn,den,xx1
!..load the coefficients of the expansion
      data  an,m1,k1,m2,k2 /-0.5d0, 7, 7, 11, 11/
      data  (a1(ii),ii=1,8)/ 1.71446374704454d7,    3.88148302324068d7,&
     &                     3.16743385304962d7,    1.14587609192151d7,&
     &                     1.83696370756153d6,    1.14980998186874d5,&
     &                     1.98276889924768d3,    1.0d0/
      data  (b1(ii),ii=1,8)/ 9.67282587452899d6,    2.87386436731785d7,&
     &                     3.26070130734158d7,    1.77657027846367d7,&
     &                     4.81648022267831d6,    6.13709569333207d5,&
     &                     3.13595854332114d4,    4.35061725080755d2/
      data (a2(ii),ii=1,12)/-4.46620341924942d-15, -1.58654991146236d-12,&
     &                    -4.44467627042232d-10, -6.84738791621745d-8,&
     &                    -6.64932238528105d-6,  -3.69976170193942d-4,&
     &                    -1.12295393687006d-2,  -1.60926102124442d-1,&
     &                    -8.52408612877447d-1,  -7.45519953763928d-1,&
     &                     2.98435207466372d0,    1.0d0/
      data (b2(ii),ii=1,12)/-2.23310170962369d-15, -7.94193282071464d-13,&
     &                    -2.22564376956228d-10, -3.43299431079845d-8,&
     &                    -3.33919612678907d-6,  -1.86432212187088d-4,&
     &                    -5.69764436880529d-3,  -8.34904593067194d-2,&
     &                    -4.78770844009440d-1,  -4.99759250374148d-1,&
     &                     1.86795964993052d0,    4.16485970495288d-1/


      if (xx .lt. 2.0d0) then
       xx1 = exp(xx)
       rn = xx1 + a1(m1)
       do ii=m1-1,1,-1
        rn = rn*xx1 + a1(ii)
       end do
       den = b1(k1+1)
       do ii=k1,1,-1
        den = den*xx1 + b1(ii)
       end do
       zfermim12 = xx1 * rn/den
!..
      else
       xx1 = 1.0d0/(xx*xx)
       rn = xx1 + a2(m2)
       do ii=m2-1,1,-1
        rn = rn*xx1 + a2(ii)
       end do
       den = b2(k2+1)
       do ii=k2,1,-1
        den = den*xx1 + b2(ii)
       end do
       zfermim12 = sqrt(xx)*rn/den
      end if
      return
      end function zfermim12






      double precision function zfermi12(xx)
!..
!..this routine applies a rational function expansion to get the fermi-dirac
!..integral of order 1/2 evaluated at x. maximum error is 5.47d-13.
!..reference: antia apjs 84,101 1993
!..
!..declare

 implicit none

!Arguments -------------------------------
      double precision, intent(in) :: xx

!Local variables-------------------------------
      integer          ii,m1,k1,m2,k2
      double precision an,a1(12),b1(12),a2(12),b2(12),rn,den,xx1

!..load the coefficients of the expansion
      data  an,m1,k1,m2,k2 /0.5d0, 7, 7, 10, 11/
      data  (a1(ii),ii=1,8)/5.75834152995465d6,   1.30964880355883d7,&
     &                    1.07608632249013d7,   3.93536421893014d6,&
     &                    6.42493233715640d5,   4.16031909245777d4,&
     &                    7.77238678539648d2,   1.0d0/
      data  (b1(ii),ii=1,8)/6.49759261942269d6,   1.70750501625775d7,&
     &                    1.69288134856160d7,   7.95192647756086d6,&
     &                    1.83167424554505d6,   1.95155948326832d5,&
     &                    8.17922106644547d3,   9.02129136642157d1/
      data (a2(ii),ii=1,11)/4.85378381173415d-14, 1.64429113030738d-11,&
     &                    3.76794942277806d-9,  4.69233883900644d-7,&
     &                    3.40679845803144d-5,  1.32212995937796d-3,&
     &                    2.60768398973913d-2,  2.48653216266227d-1,&
     &                    1.08037861921488d0,   1.91247528779676d0,&
     &                    1.0d0/
      data (b2(ii),ii=1,12)/7.28067571760518d-14, 2.45745452167585d-11,&
     &                    5.62152894375277d-9,  6.96888634549649d-7,&
     &                    5.02360015186394d-5,  1.92040136756592d-3,&
     &                    3.66887808002874d-2,  3.24095226486468d-1,&
     &                    1.16434871200131d0,   1.34981244060549d0,&
     &                    2.01311836975930d-1, -2.14562434782759d-2/


      if (xx .lt. 2.0d0) then
       xx1 = exp(xx)
       rn = xx1 + a1(m1)
       do ii=m1-1,1,-1
        rn = rn*xx1 + a1(ii)
       end do
       den = b1(k1+1)
       do ii=k1,1,-1
        den = den*xx1 + b1(ii)
       end do
       zfermi12 = xx1 * rn/den

      else
       xx1 = 1.0d0/(xx*xx)
       rn = xx1 + a2(m2)
       do ii=m2-1,1,-1
        rn = rn*xx1 + a2(ii)
       end do
       den = b2(k2+1)
       do ii=k2,1,-1
        den = den*xx1 + b2(ii)
       end do
       zfermi12 = xx*sqrt(xx)*rn/den
      end if
      return
      end function zfermi12





      double precision function zfermi1(xx)
!..
!..this routine applies a rational function expansion to get the fermi-dirac
!..integral of order 1 evaluated at x. maximum error is 1.0e-8.
!..reference: antia  priv comm. 11sep94
!..
!..declare

 implicit none

!Arguments -------------------------------
      double precision, intent(in) :: xx

!Local variables-------------------------------
      integer          ii,m1,k1,m2,k2
      double precision an,a1(12),b1(12),a2(12),b2(12),rn,den,xx1

!..load the coefficients of the expansion
      data  an,m1,k1,m2,k2 /1.0, 7, 4, 9, 5/
      data  (a1(ii),ii=1,8)/-7.606458638543d7,  -1.143519707857d8,&
     &                    -5.167289383236d7,  -7.304766495775d6,&
     &                    -1.630563622280d5,   3.145920924780d3,&
     &                    -7.156354090495d1,   1.0d0/
      data  (b1(ii),ii=1,5)/-7.606458639561d7,  -1.333681162517d8,&
     &                    -7.656332234147d7,  -1.638081306504d7,&
     &                    -1.044683266663d6/
      data (a2(ii),ii=1,10)/-3.493105157219d-7, -5.628286279892d-5,&
     &                    -5.188757767899d-3, -2.097205947730d-1,&
     &                    -3.353243201574d0,  -1.682094530855d1,&
     &                    -2.042542575231d1,   3.551366939795d0,&
     &                    -2.400826804233d0,   1.0d0/
      data  (b2(ii),ii=1,6)/-6.986210315105d-7, -1.102673536040d-4,&
     &                    -1.001475250797d-2, -3.864923270059d-1,&
     &                    -5.435619477378d0,  -1.563274262745d1/


      if (xx .lt. 2.0d0) then
       xx1 = exp(xx)
       rn = xx1 + a1(m1)
       do ii=m1-1,1,-1
        rn = rn*xx1 + a1(ii)
       end do
       den = b1(k1+1)
       do ii=k1,1,-1
        den = den*xx1 + b1(ii)
       end do
       zfermi1 = xx1 * rn/den

      else
       xx1 = 1.0d0/(xx*xx)
       rn = xx1 + a2(m2)
       do ii=m2-1,1,-1
        rn = rn*xx1 + a2(ii)
       end do
       den = b2(k2+1)
       do ii=k2,1,-1
        den = den*xx1 + b2(ii)
       end do
       zfermi1 = xx*xx*rn/den
      end if
      return
      end function zfermi1





      double precision function zfermi32(xx)
!..
!..this routine applies a rational function expansion to get the fermi-dirac
!..integral of order 3/2 evaluated at x. maximum error is 5.07d-13.
!..reference: antia apjs 84,101 1993
!..
!..declare

 implicit none

!Arguments -------------------------------
      double precision, intent(in) :: xx

!Local variables-------------------------------
      integer          ii,m1,k1,m2,k2
      double precision an,a1(12),b1(12),a2(12),b2(12),rn,den,xx1

!..load the coefficients of the expansion
      data  an,m1,k1,m2,k2 /1.5d0, 6, 7, 9, 10/
      data  (a1(ii),ii=1,7)/4.32326386604283d4,   8.55472308218786d4,&
     &                    5.95275291210962d4,   1.77294861572005d4,&
     &                    2.21876607796460d3,   9.90562948053193d1,&
     &                    1.0d0/
      data  (b1(ii),ii=1,8)/3.25218725353467d4,   7.01022511904373d4,&
     &                    5.50859144223638d4,   1.95942074576400d4,&
     &                    3.20803912586318d3,   2.20853967067789d2,&
     &                    5.05580641737527d0,   1.99507945223266d-2/
      data (a2(ii),ii=1,10)/2.80452693148553d-13, 8.60096863656367d-11,&
     &                    1.62974620742993d-8,  1.63598843752050d-6,&
     &                    9.12915407846722d-5,  2.62988766922117d-3,&
     &                    3.85682997219346d-2,  2.78383256609605d-1,&
     &                    9.02250179334496d-1,  1.0d0/
      data (b2(ii),ii=1,11)/7.01131732871184d-13, 2.10699282897576d-10,&
     &                    3.94452010378723d-8,  3.84703231868724d-6,&
     &                    2.04569943213216d-4,  5.31999109566385d-3,&
     &                    6.39899717779153d-2,  3.14236143831882d-1,&
     &                    4.70252591891375d-1, -2.15540156936373d-2,&
     &                    2.34829436438087d-3/


      if (xx .lt. 2.0d0) then
       xx1 = exp(xx)
       rn = xx1 + a1(m1)
       do ii=m1-1,1,-1
        rn = rn*xx1 + a1(ii)
       end do
       den = b1(k1+1)
       do ii=k1,1,-1
        den = den*xx1 + b1(ii)
       end do
       zfermi32 = xx1 * rn/den

      else
       xx1 = 1.0d0/(xx*xx)
       rn = xx1 + a2(m2)
       do ii=m2-1,1,-1
        rn = rn*xx1 + a2(ii)
       end do
       den = b2(k2+1)
       do ii=k2,1,-1
        den = den*xx1 + b2(ii)
       end do
       zfermi32 = xx*xx*sqrt(xx)*rn/den
      end if
      return
      end function zfermi32




      double precision function zfermi2(xx)
!..
!..this routine applies a rational function expansion to get the fermi-dirac
!..integral of order 2 evaluated at x. maximum error is 1.0e-8.
!..reference: antia  priv comm. 11sep94
!..
!..declare

 implicit none

!Arguments -------------------------------
      double precision, intent(in) :: xx

!Local variables-------------------------------
      integer          ii,m1,k1,m2,k2
      double precision an,a1(12),b1(12),a2(12),b2(12),rn,den,xx1

!..load the coefficients of the expansion
      data  an,m1,k1,m2,k2 /2.0, 7, 4, 5, 9/
      data  (a1(ii),ii=1,8)/-1.434885992395d8,  -2.001711155617d8,&
     &                    -8.507067153428d7,  -1.175118281976d7,&
     &                    -3.145120854293d5,   4.275771034579d3,&
     &                    -8.069902926891d1,   1.0d0/
      data  (b1(ii),ii=1,5)/-7.174429962316d7,  -1.090535948744d8,&
     &                    -5.350984486022d7,  -9.646265123816d6,&
     &                    -5.113415562845d5/
      data  (a2(ii),ii=1,6)/ 6.919705180051d-8,  1.134026972699d-5,&
     &                     7.967092675369d-4,  2.432500578301d-2,&
     &                     2.784751844942d-1,  1.0d0/
      data (b2(ii),ii=1,10)/ 2.075911553728d-7,  3.197196691324d-5,&
     &                     2.074576609543d-3,  5.250009686722d-2,&
     &                     3.171705130118d-1, -1.147237720706d-1,&
     &                     6.638430718056d-2, -1.356814647640d-2,&
     &                    -3.648576227388d-2,  3.621098757460d-2/


      if (xx .lt. 2.0d0) then
       xx1 = exp(xx)
       rn = xx1 + a1(m1)
       do ii=m1-1,1,-1
        rn = rn*xx1 + a1(ii)
       end do
       den = b1(k1+1)
       do ii=k1,1,-1
        den = den*xx1 + b1(ii)
       end do
       zfermi2 = xx1 * rn/den

      else
       xx1 = 1.0d0/(xx*xx)
       rn = xx1 + a2(m2)
       do ii=m2-1,1,-1
        rn = rn*xx1 + a2(ii)
       end do
       den = b2(k2+1)
       do ii=k2,1,-1
        den = den*xx1 + b2(ii)
       end do
       zfermi2 = xx*xx*xx*rn/den
      end if
      return
      end function zfermi2






      double precision function zfermi52(xx)
!..
!..this routine applies a rational function expansion to get the fermi-dirac
!..integral of order 5/2 evaluated at x. maximum error is 2.47d-13.
!..reference: antia apjs 84,101 1993
!..
!..declare

 implicit none

!Arguments -------------------------------
      double precision, intent(in) :: xx

!Local variables-------------------------------
      integer          ii,m1,k1,m2,k2
      double precision an,a1(12),b1(12),a2(12),b2(12),rn,den,xx1

!..load the coefficients of the expansion
      data  an,m1,k1,m2,k2 /2.5d0, 6, 7, 10, 9/
      data  (a1(ii),ii=1,7)/6.61606300631656d4,   1.20132462801652d5,&
     &                    7.67255995316812d4,   2.10427138842443d4,&
     &                    2.44325236813275d3,   1.02589947781696d2,&
     &                    1.0d0/
      data  (b1(ii),ii=1,8)/1.99078071053871d4,   3.79076097261066d4,&
     &                    2.60117136841197d4,   7.97584657659364d3,&
     &                    1.10886130159658d3,   6.35483623268093d1,&
     &                    1.16951072617142d0,   3.31482978240026d-3/
      data (a2(ii),ii=1,11)/8.42667076131315d-12, 2.31618876821567d-9,&
     &                    3.54323824923987d-7,  2.77981736000034d-5,&
     &                    1.14008027400645d-3,  2.32779790773633d-2,&
     &                    2.39564845938301d-1,  1.24415366126179d0,&
     &                    3.18831203950106d0,   3.42040216997894d0,&
     &                    1.0d0/
      data (b2(ii),ii=1,10)/2.94933476646033d-11, 7.68215783076936d-9,&
     &                    1.12919616415947d-6,  8.09451165406274d-5,&
     &                    2.81111224925648d-3,  3.99937801931919d-2,&
     &                    2.27132567866839d-1,  5.31886045222680d-1,&
     &                    3.70866321410385d-1,  2.27326643192516d-2/


      if (xx .lt. 2.0d0) then
       xx1 = exp(xx)
       rn = xx1 + a1(m1)
       do ii=m1-1,1,-1
        rn = rn*xx1 + a1(ii)
       end do
       den = b1(k1+1)
       do ii=k1,1,-1
        den = den*xx1 + b1(ii)
       end do
       zfermi52 = xx1 * rn/den

      else
       xx1 = 1.0d0/(xx*xx)
       rn = xx1 + a2(m2)
       do ii=m2-1,1,-1
        rn = rn*xx1 + a2(ii)
       end do
       den = b2(k2+1)
       do ii=k2,1,-1
        den = den*xx1 + b2(ii)
       end do
       zfermi52 = xx*xx*xx*sqrt(xx)*rn/den
      end if
      return
      end function zfermi52





      double precision function zfermi3(xx)
!..
!..this routine applies a rational function expansion to get the fermi-dirac
!..integral of order 3 evaluated at x. maximum error is 1.0e-8.
!..reference: antia  priv comm. 11sep94
!..
!..declare

 implicit none

!Arguments -------------------------------
      double precision, intent(in) :: xx

!Local variables-------------------------------
      integer          ii,m1,k1,m2,k2
      double precision an,a1(12),b1(12),a2(12),b2(12),rn,den,xx1

!..load the coefficients of the expansion
      data  an,m1,k1,m2,k2 /3.0, 4, 6, 7, 7/
      data  (a1(ii),ii=1,5)/ 6.317036716422d2,    7.514163924637d2,&
     &                     2.711961035750d2,    3.274540902317d1,&
     &                     1.0d0/
      data  (b1(ii),ii=1,7)/ 1.052839452797d2,    1.318163114785d2,&
     &                     5.213807524405d1,    7.500064111991d0,&
     &                     3.383020205492d-1,   2.342176749453d-3,&
     &                    -8.445226098359d-6/
      data  (a2(ii),ii=1,8)/ 1.360999428425d-8,   1.651419468084d-6,&
     &                     1.021455604288d-4,   3.041270709839d-3,&
     &                     4.584298418374d-2,   3.440523212512d-1,&
     &                     1.077505444383d0,    1.0d0/
      data  (b2(ii),ii=1,8)/ 5.443997714076d-8,   5.531075760054d-6,&
     &                     2.969285281294d-4,   6.052488134435d-3,&
     &                     5.041144894964d-2,   1.048282487684d-1,&
     &                     1.280969214096d-2,  -2.851555446444d-3/


      if (xx .lt. 2.0d0) then
       xx1 = exp(xx)
       rn = xx1 + a1(m1)
       do ii=m1-1,1,-1
        rn = rn*xx1 + a1(ii)
       end do
       den = b1(k1+1)
       do ii=k1,1,-1
        den = den*xx1 + b1(ii)
       end do
       zfermi3 = xx1 * rn/den

      else
       xx1 = 1.0d0/(xx*xx)
       rn = xx1 + a2(m2)
       do ii=m2-1,1,-1
        rn = rn*xx1 + a2(ii)
       end do
       den = b2(k2+1)
       do ii=k2,1,-1
        den = den*xx1 + b2(ii)
       end do
       zfermi3 = xx*xx*xx*xx*rn/den
      end if
      return
      end function zfermi3





      double precision function ifermim12(ff)
!..
!..this routine applies a rational function expansion to get the inverse
!..fermi-dirac integral of order -1/2 when it is equal to f.
!..maximum error is 3.03d-9.   reference: antia apjs 84,101 1993
!..
!..declare

 implicit none

!Arguments -------------------------------
      double precision, intent(in) :: ff

!Local variables-------------------------------
      integer          ii,m1,k1,m2,k2
      double precision an,a1(12),b1(12),a2(12),b2(12),rn,den,ff1

!..load the coefficients of the expansion
      data  an,m1,k1,m2,k2 /-0.5d0, 5, 6, 6, 6/
      data  (a1(ii),ii=1,6)/-1.570044577033d4,   1.001958278442d4,&
     &                    -2.805343454951d3,   4.121170498099d2,&
     &                    -3.174780572961d1,   1.0d0/
      data  (b1(ii),ii=1,7)/-2.782831558471d4,   2.886114034012d4,&
     &                    -1.274243093149d4,   3.063252215963d3,&
     &                    -4.225615045074d2,   3.168918168284d1,&
     &                    -1.008561571363d0/
      data  (a2(ii),ii=1,7)/ 2.206779160034d-8,  -1.437701234283d-6,&
     &                     6.103116850636d-5,  -1.169411057416d-3,&
     &                     1.814141021608d-2,  -9.588603457639d-2,&
     &                     1.0d0/
      data  (b2(ii),ii=1,7)/ 8.827116613576d-8,  -5.750804196059d-6,&
     &                     2.429627688357d-4,  -4.601959491394d-3,&
     &                     6.932122275919d-2,  -3.217372489776d-1,&
     &                     3.124344749296d0/

      if (ff .lt. 4.0d0) then
       rn = ff + a1(m1)
       do ii=m1-1,1,-1
        rn = rn*ff + a1(ii)
       end do
       den = b1(k1+1)
       do ii=k1,1,-1
        den = den*ff + b1(ii)
       end do
       ifermim12 = log(ff * rn/den)

      else
       ff1 = 1.0d0/ff**(1.0d0/(1.0d0 + an))
       rn = ff1 + a2(m2)
       do ii=m2-1,1,-1
        rn = rn*ff1 + a2(ii)
       end do
       den = b2(k2+1)
       do ii=k2,1,-1
        den = den*ff1 + b2(ii)
       end do
       ifermim12 = rn/(den*ff1)
      end if
      return
      end function ifermim12






      double precision function ifermi12(ff)
!..
!..this routine applies a rational function expansion to get the inverse
!..fermi-dirac integral of order 1/2 when it is equal to f.
!..maximum error is 4.19d-9.   reference: antia apjs 84,101 1993
!..
!..declare

 implicit none

!Arguments -------------------------------
      double precision, intent(in) :: ff

!Local variables-------------------------------
      integer          ii,m1,k1,m2,k2
      double precision an,a1(12),b1(12),a2(12),b2(12),rn,den,ff1

!..load the coefficients of the expansion
      data  an,m1,k1,m2,k2 /0.5d0, 4, 3, 6, 5/
      data  (a1(ii),ii=1,5)/ 1.999266880833d4,   5.702479099336d3,&
     &                     6.610132843877d2,   3.818838129486d1,&
     &                     1.0d0/
      data  (b1(ii),ii=1,4)/ 1.771804140488d4,  -2.014785161019d3,&
     &                     9.130355392717d1,  -1.670718177489d0/
      data  (a2(ii),ii=1,7)/-1.277060388085d-2,  7.187946804945d-2,&
     &                    -4.262314235106d-1,  4.997559426872d-1,&
     &                    -1.285579118012d0,  -3.930805454272d-1,&
     &                     1.0d0/
      data  (b2(ii),ii=1,6)/-9.745794806288d-3,  5.485432756838d-2,&
     &                    -3.299466243260d-1,  4.077841975923d-1,&
     &                    -1.145531476975d0,  -6.067091689181d-2/


      if (ff .lt. 4.0d0) then
       rn = ff + a1(m1)
       do ii=m1-1,1,-1
        rn = rn*ff + a1(ii)
       end do
       den = b1(k1+1)
       do ii=k1,1,-1
        den = den*ff + b1(ii)
       end do
       ifermi12 = log(ff * rn/den)

      else
       ff1 = 1.0d0/ff**(1.0d0/(1.0d0 + an))
       rn = ff1 + a2(m2)
       do ii=m2-1,1,-1
        rn = rn*ff1 + a2(ii)
       end do
       den = b2(k2+1)
       do ii=k2,1,-1
        den = den*ff1 + b2(ii)
       end do
       ifermi12 = rn/(den*ff1)
      end if
      return
      end function ifermi12






      double precision function ifermi32(ff)
!..
!..this routine applies a rational function expansion to get the inverse
!..fermi-dirac integral of order 3/2 when it is equal to f.
!..maximum error is 2.26d-9.   reference: antia apjs 84,101 1993
!..
!..declare

 implicit none

!Arguments -------------------------------
      double precision, intent(in) :: ff

!Local variables-------------------------------
      integer          ii,m1,k1,m2,k2
      double precision an,a1(12),b1(12),a2(12),b2(12),rn,den,ff1

!..load the coefficients of the expansion
      data  an,m1,k1,m2,k2 /1.5d0, 3, 4, 6, 5/
      data  (a1(ii),ii=1,4)/ 1.715627994191d2,   1.125926232897d2,&
     &                     2.056296753055d1,   1.0d0/
      data  (b1(ii),ii=1,5)/ 2.280653583157d2,   1.193456203021d2,&
     &                     1.167743113540d1,  -3.226808804038d-1,&
     &                     3.519268762788d-3/
      data  (a2(ii),ii=1,7)/-6.321828169799d-3, -2.183147266896d-2,&
     &                    -1.057562799320d-1, -4.657944387545d-1,&
     &                    -5.951932864088d-1,  3.684471177100d-1,&
     &                     1.0d0/
      data  (b2(ii),ii=1,6)/-4.381942605018d-3, -1.513236504100d-2,&
     &                    -7.850001283886d-2, -3.407561772612d-1,&
     &                    -5.074812565486d-1, -1.387107009074d-1/


      if (ff .lt. 4.0d0) then
       rn = ff + a1(m1)
       do ii=m1-1,1,-1
        rn = rn*ff + a1(ii)
       end do
       den = b1(k1+1)
       do ii=k1,1,-1
        den = den*ff + b1(ii)
       end do
       ifermi32 = log(ff * rn/den)

      else
       ff1 = 1.0d0/ff**(1.0d0/(1.0d0 + an))
       rn = ff1 + a2(m2)
       do ii=m2-1,1,-1
        rn = rn*ff1 + a2(ii)
       end do
       den = b2(k2+1)
       do ii=k2,1,-1
        den = den*ff1 + b2(ii)
       end do
       ifermi32 = rn/(den*ff1)
      end if
      return
      end function ifermi32





      double precision function ifermi52(ff)
!..
!..this routine applies a rational function expansion to get the inverse
!..fermi-dirac integral of order 5/2 when it is equal to f.
!..maximum error is 6.17d-9.   reference: antia apjs 84,101 1993
!..
!..declare

 implicit none

!Arguments -------------------------------
      double precision, intent(in) :: ff

!Local variables-------------------------------
      integer          ii,m1,k1,m2,k2
      double precision an,a1(12),b1(12),a2(12),b2(12),rn,den,ff1

!..load the coefficients of the expansion
      data  an,m1,k1,m2,k2 /2.5d0, 2, 3, 6, 6/
      data  (a1(ii),ii=1,3)/ 2.138969250409d2,   3.539903493971d1,&
     &                     1.0d0/
      data  (b1(ii),ii=1,4)/ 7.108545512710d2,   9.873746988121d1,&
     &                     1.067755522895d0,  -1.182798726503d-2/
      data  (a2(ii),ii=1,7)/-3.312041011227d-2,  1.315763372315d-1,&
     &                    -4.820942898296d-1,  5.099038074944d-1,&
     &                     5.495613498630d-1, -1.498867562255d0,&
     &                     1.0d0/
      data  (b2(ii),ii=1,7)/-2.315515517515d-2,  9.198776585252d-2,&
     &                    -3.835879295548d-1,  5.415026856351d-1,&
     &                    -3.847241692193d-1,  3.739781456585d-2,&
     &                    -3.008504449098d-2/


      if (ff .lt. 4.0d0) then
       rn = ff + a1(m1)
       do ii=m1-1,1,-1
        rn = rn*ff + a1(ii)
       end do
       den = b1(k1+1)
       do ii=k1,1,-1
        den = den*ff + b1(ii)
       end do
       ifermi52 = log(ff * rn/den)

      else
       ff1 = 1.0d0/ff**(1.0d0/(1.0d0 + an))
       rn = ff1 + a2(m2)
       do ii=m2-1,1,-1
        rn = rn*ff1 + a2(ii)
       end do
       den = b2(k2+1)
       do ii=k2,1,-1
        den = den*ff1 + b2(ii)
       end do
       ifermi52 = rn/(den*ff1)
      end if
      return
      end function ifermi52

      double precision function fp12a1 (x)
! ************************************************************* Nohel **
!
      implicit real*8 (a-h,o-z)
!
      intrinsic exp,sqrt
!
! **********************************************************************
! *                                                                    *
! *               Integrale de Fermi d'ordre 1/2                       *
! *    Fp12(x) = somme de 0 a l'infini de (dt*t**1/2)/(1+exp(t-x))     *
! *                                                                    *
! **********************************************************************
!
! H. M. Antia, Astrophys. J. Suppl. 84, 101 (1993)
! Erreur relative maximum annoncee 5.54 e-5
! Erreur relative maximum constatee : -5.53e-5 pour eta = 2
!
      if (x.lt.2.) then
        y=exp(x)
        fp12a1=y*(21.8168+y*(13.1693+y))&
     &    /(24.6180+y*(23.5546+y*(4.76290+y*0.134481)))
      else
        y=1./(x*x)
        fp12a1=x*sqrt(x)*(0.0473011+y*(0.548433+y))&
     &    /(0.0709478+y*(0.737041+y*0.382065))
      end if
!
! **********************************************************************
end function fp12a1
      double precision function fp32a1 (x)
! ************************************************************* Nohel **
!
      implicit real*8 (a-h,o-z)
!
      intrinsic exp,sqrt
!
! **********************************************************************
! *                                                                    *
! *               Integrale de Fermi d'ordre 3/2                       *
! *    Fp32(x) = somme de 0 a l'infini de (dt*t**3/2)/(1+exp(t-x))     *
! *                                                                    *
! **********************************************************************
!
! H. M. Antia, Astrophys. J. Suppl. 84, 101 (1993)
! Erreur relative maximum annoncee 6.54 e-5
! Erreur relative maximum constatee : -5.84e-5 pour eta = -5
!
      if (x.lt.2.) then
        y=exp(x)
        fp32a1=y*(135.863+y*(49.2764+y))/(102.210+y*(55.0312+y*4.23365))
      else
        x2=x*x
        y=1./x2
        fp32a1=x2*sqrt(x)*(0.154699+y*(1.20037+y))&
     &    /(0.386765+y*(0.608119-y*0.165665))
      end if
!
! **********************************************************************
end function fp32a1
      double precision function xp12a1 (y)
! ************************************************************* Nohel **
!
      implicit real*8 (a-h,o-z)
!
      intrinsic log
      parameter (deux=2.,deuxs3=deux/3.)
!
! **********************************************************************
! *                                                                    *
! *              Calcul de eta tel que fp12 (eta) = y                  *
! *          ou fp12 est l'integrale de Fermi d'ordre +1/2             *
! *                                                                    *
! **********************************************************************
!
! H. M. Antia, Astrophys. J. Suppl. 84, 101 (1993)
! Erreur relative maximum annoncee sur exp(eta) : 3.02 e-5
!
      if (y.lt.4.) then
        top=44.593646+y*(11.288764+y)
        den=39.519346+y*(-5.7517464+y*0.26594291)
        xp12a1=log(y*top/den)
      else
        z=y**(-deuxs3)
        top=34.873722+z*(-26.922515+z)
        den=26.612832+z*(-20.452930+z*11.808945)
        xp12a1=top/(z*den)
      end if
!
! **********************************************************************
end function xp12a1
      double precision function fm12a1 (x)
! ************************************************************* Nohel **
!
      implicit real*8 (a-h,o-z)
!
      intrinsic exp,sqrt
!
! **********************************************************************
! *                                                                    *
! *               Integrale de Fermi d'ordre -1/2                      *
! *    Fm12(x) = somme de 0 a l'infini de (dt*t**-1/2)/(1+exp(t-x))    *
! *                                                                    *
! **********************************************************************
!
! H. M. Antia, Astrophys. J. Suppl. 84, 101 (1993)
! Erreur relative maximum annoncee 4.75 e-5
!
      if (x.lt.2.) then
        y=exp(x)
        fm12a1=y*(23.1456+y*(13.7820+y))&
     &    /(13.0586+y*(17.0048+y*(5.07527+y*0.236620)))
      else
        y=1./(x*x)
        fm12a1=sqrt(x)*(0.0153602+y*(0.146815+y))&
     &    /(0.00768015+y*(0.0763700+y*0.570485))
      end if
!
! **********************************************************************
end function fm12a1
subroutine fm12a1t (cktf,rtnewt,tsmear,vtrial,rhor_middx,rhor_mid,&
&                         nfft)
! ************************************************************* Nohel **
!
      implicit real*8 (a-h,o-z)
      real*8::vtrial(nfft),rhor_middx(nfft),rhor_mid(nfft)
!
      intrinsic exp,sqrt
!
! **********************************************************************
! *                                                                    *
! *               Integrale de Fermi d'ordre -1/2                      *
! *    Fm12(x) = somme de 0 a l'infini de (dt*t**-1/2)/(1+exp(t-x))    *
! *                      ....                                              *
! **********************************************************************
!
! H. M. Antia, Astrophys. J. Suppl. 84, 101 (1993)
! Erreur relative maximum annoncee 4.75 e-5
!
      do ifft=1,nfft
      x=(rtnewt-vtrial(ifft))/tsmear
      if (x.lt.2.) then
        y=exp(x)
        rhor_middx(ifft)=cktf*y*(23.1456d0+y*(13.7820d0+y))&
     &    /(13.0586d0+y*(17.0048d0+y*(5.07527d0+y*0.236620d0)))
        rhor_mid(ifft)=cktf*y*(21.8168+y*(13.1693+y))&
     &    /(24.6180+y*(23.5546+y*(4.76290+y*0.134481)))
      else
        y=1./(x*x)
        sqrtx=sqrt(x)
        rhor_middx(ifft)=cktf*sqrtx*(0.0153602d0+y*(0.146815d0+y))&
     &    /(0.00768015d0+y*(0.0763700d0+y*0.570485d0))
        rhor_mid(ifft)=cktf*x*sqrtx*(0.0473011+y*(0.548433+y))&
     &    /(0.0709478+y*(0.737041+y*0.382065))
      end if
      end do
!
! **********************************************************************
end subroutine fm12a1t
!!***
