!{\src2tex{textfont=tt}}
!!****m* ABINIT/defs_elphon
!!
!! NAME
!! defs_elphon
!!
!! FUNCTION
!! This module contains the datastructures for elphon
!!  the different (huge) matrices will either be allocated and
!!  used, or be written to disk. All combinations should be feasible.
!!
!! COPYRIGHT
!! Copyright (C) 2004-2006 ABINIT group (MVer)
!! This file is distributed under the terms of the
!! GNU General Public Licence, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! NOTES
!!  Contains two datastructures:
!!   elph_type contains data and dimensions for the kpoints near the
!!    fermi surface and the $g_{k k+q}$ matrix elements
!!   phon_type contains the necessary data to interpolate the
!!    phonon bandstructure and eigenvectors in reciprocal space (ie.
!!    interatomic force constants and corresponding real space grid info).
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

module defs_elphon

  use defs_basis

  implicit none

!!***

!----------------------------------------------------------------------

!!****t* defs_elphon/elph_type
!! NAME
!! elph_type
!!
!! FUNCTION
!! elph_type contains data and dimensions for the kpoints near the
!! fermi surface and the $g_{k k+q}$ matrix elements
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

  type elph_type
     ! Matrix dimensions (natom is redundant)
     integer :: natom,nbranch,nFSband,nFSkpt,nFSkptirred,nqpt
     ! index of lower and upper bands used for the FS
     integer :: minFSband,maxFSband
     ! Number of bands kept in final gkk matrix elements:
     !   either 1 if sum is performed immediately
     !   or = nband if all elements are kept based on flag tkeepbands
     integer :: ngkkband
     !MG20060531
     integer :: nqptirred   !number of irred q-points

     !units for file output
     integer :: unita2f,unit_gkk2,unit_gkk_rpt,unitgkq,unitphfrq
     !flags for existence and RAM/disk status for each matrix below
     integer :: gkqexist,gkqwrite
     integer :: gkk2exist,gkk2write
     integer :: gkk_rptexist,gkk_rptwrite
     integer :: phfrqwrite,phfrqexist

     ! flags
     integer :: doscalprod
     ! flag to symmetrize gkq matrix elements
     integer :: tsymgkq
     ! flag to sum over bands or not
     integer :: tkeepbands

     ! dimensions and increments for a2F function
     integer :: na2f
     real(dp) :: omega_min,omega_max,a2fsmear,domega

!MG20060531
     real(dp) :: n0        !DOS at the Fermi level (states/Ha/spin)
     real(dp) :: mustar    !mustar parameter
     real(dp) :: fermie    !Fermi energy (Ha), either comes from wfk file or from anaddb input file
!ENDMG20060531

     character(len=fnlen) :: elph_base_name !base name for output files

!MG20060531
     integer, pointer :: qirredtofull(:) !mapping between the qpoints found in the GGK file
                                         !and the array of qpoints generated by the code

     real(dp),pointer :: wtq(:) !weight for each qpoint in the full grid spqt
                                !if a point is not in the IBZ ==>  wtq=0 !MG20050911 we can also use indqpt

     real(dp),pointer :: spqpt(:,:)           !special q points obtained by the Monkhorst & Pack method,
                                              !in reduced coordinates.These vectors have to be multiplied
                                              !by the reciprocal basis vectors

    real(dp),pointer :: FSkpt(:,:)            !coordinates of the full kpoints close to the FS

    real(dp),pointer :: FSintweight(:,:)     !integration weights (see also gkk_intweight)
    real(dp),pointer :: FSkptirred(:,:)
!ENDMG

     !  integration weights for gkk matrix elements on FS:
     !   if tkeepbands == 0 all are 1
     !   if tkeepbands == 1 then = to FSintweight in elphon
     !  DOES NOT INCLUDE FACTOR OF 1/nFSkpt
     real(dp),pointer :: gkk_intweight(:,:)

     !real(dp),pointer :: gkk_qpt(:,:,:,:,:,:)
     !  Now gkq contains gkk2 matrices on basic qpts, summed over bands if ngkkband==1
     real(dp),pointer :: gkk_qpt(:,:,:,:,:,:,:)

     ! For the moment, gkk_rpt in memory is out of the question
     !real(dp),pointer :: gkk_rpt(:,:,:,:,:,:)
     !  Same for real space:
     real(dp),pointer :: gkk_rpt(:,:,:,:,:,:,:)
     real(dp),pointer :: gkk2(:,:,:,:,:)

     !  gamma matrices integrated over kpoint coeff and bands: still depends on qpt
     real(dp),pointer :: gamma_qpt(:,:,:,:)
     real(dp),pointer :: gamma_rpt(:,:,:,:)

     !  phonon frequencies
     real(dp),pointer :: phfrq(:,:)
     !  a2f function
     real(dp),pointer :: a2f(:,:,:)

!MG20060531
     real(dp),pointer :: qgrid_data(:,:,:) !e-ph values calculated over the irreducible part of the q-grid:
                                           !first entry  =  index of the q-point,
                                           !second index =  branch index
                                           !the third slice contains the frequency, the linewidth and lambda(q,nu)
                                           !for that particular phonon mode

     end type elph_type
!!***

!----------------------------------------------------------------------

!!****t* defs_elphon/phon_type
!! NAME
!! phon_type
!!
!! FUNCTION
!! phon_type contains the necessary data to interpolate the
!! phonon bandstructure and eigenvectors in reciprocal space (ie.
!! interatomic force constants and corresponding real space grid info).
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

  type phon_type

     integer :: dipdip
     integer :: mpert,nsym
     integer :: natom,ntypat,nrpt
     integer, pointer :: indsym(:,:,:),symrel(:,:,:),typat(:)
     real(dp) :: ucvol
     real(dp), pointer :: acell(:),amu(:),atmfrc(:,:,:,:,:,:)
     real(dp), pointer :: dielt(:,:),dyewq0(:,:,:),gprim(:,:),gmet(:,:)
     real(dp), pointer :: xred(:,:),zeff(:,:,:)
     real(dp), pointer :: rcan(:,:),rmet(:,:),rprim(:,:),rprimd(:,:),rpt(:,:)
     real(dp), pointer :: trans(:,:),wghatm(:,:,:)

   end type phon_type

end module defs_elphon
!!***
